#!/usr/bin/env python
# -*- coding: utf-8 -*-
# pylint: disable-msg=W0614
'''
Wrapper module for a more Pythonic usage experienced using the automatically
generated C{liblcms} Python bindings from module L{_lcms}.
'''

## Created: 2008-10-12 Guy K. Kloss <G.Kloss@massey.ac.nz>
## Changed: $Id:$
##
## Copyright 2008 Guy K. Kloss
## All rights reserved.
##
## This software is distributed under the terms of the
## GNU Lesser General Public License
##
## http://www.massey.ac.nz/~gkloss/

__author__ = 'Guy K. Kloss <G.Kloss@massey.ac.nz>'
__date__ = '$Date: $'
__version__ = '$Revision: $'[11:-2]

# Import all generated code, so we can override things.
import ctypes
from littlecms.generated._lcms import * # pylint: disable-msg=W0614
from littlecms.generated._lcms import _cmsSaveProfile

class PropertyContainer(object):
    """
    Container class for simple property objects.
    """
    def __init__(self, **attributes):
        self.__dict__ = attributes
        
    def __repr__(self):
        """Print fields of the object."""
        res = []
        for attribute, value in self.__dict__.items():
            res.append('%s=%s' % (attribute, value.__repr__()))
        return '%s(%s)' % (self.__class__.__name__, ', '.join(res))


class ColourSpace(PropertyContainer):
    """
    A colour space descriptor.
    """
    pass
    

# Colour space type descriptors for lcms.h and icc34.h.
COLOUR_TYPE_FROM_NAME = {
    'GRAY':  ColourSpace(name='GRAY',  lcms=PT_GRAY,  ICC=icSigGrayData),
    'RGB':   ColourSpace(name='RGB',   lcms=PT_RGB,   ICC=icSigRgbData),
    'CMY':   ColourSpace(name='CMY',   lcms=PT_CMY,   ICC=icSigCmyData),
    'CMYK':  ColourSpace(name='CMYK',  lcms=PT_CMYK,  ICC=icSigCmykData),
    'YCbCr': ColourSpace(name='YCbCr', lcms=PT_YCbCr, ICC=icSigYCbCrData),
    'YUV':   ColourSpace(name='YUV',   lcms=PT_YUV,   ICC=icSigLuvData),
    'XYZ':   ColourSpace(name='XYZ',   lcms=PT_XYZ,   ICC=icSigXYZData),
    'Lab':   ColourSpace(name='Lab',   lcms=PT_Lab,   ICC=icSigLabData),
    'HSV':   ColourSpace(name='HSV',   lcms=PT_HSV,   ICC=icSigHsvData),
    'HLS':   ColourSpace(name='HLS',   lcms=PT_HLS,   ICC=icSigHlsData),
    'Yxy':   ColourSpace(name='Yxy',   lcms=PT_Yxy,   ICC=icSigYxyData)}
#   # These colour space types are not really supported by the
#   # Python bindings, yet.
#     'ANY':    ColourSpace(name='ANY',    lcms=PT_ANY,    ICC=None),
#     'YUVK':   ColourSpace(name='YUVK',   lcms=PT_YUVK,   ICC=None),
#     'HiFi':   ColourSpace(name='HiFi',   lcms=PT_HiFi,   ICC=None),
#     'HiFi7':  ColourSpace(name='HiFi7',  lcms=PT_HiFi7,  ICC=icSig7colorData),
#     'HiFi8':  ColourSpace(name='HiFi8',  lcms=PT_HiFi8,  ICC=icSig8colorData),
#     'HiFi9':  ColourSpace(name='HiFi9',  lcms=PT_HiFi9,  ICC=icSig9colorData),
#     'HiFi10': ColourSpace(name='HiFi10', lcms=PT_HiFi10, ICC=icSig10colorData),
#     'HiFi11': ColourSpace(name='HiFi11', lcms=PT_HiFi11, ICC=icSig11colorData),
#     'HiFi12': ColourSpace(name='HiFi12', lcms=PT_HiFi12, ICC=icSig12colorData),
#     'HiFi13': ColourSpace(name='HiFi13', lcms=PT_HiFi13, ICC=icSig13colorData),
#     'HiFi14': ColourSpace(name='HiFi14', lcms=PT_HiFi14, ICC=icSig14colorData),
#     'HiFi15': ColourSpace(name='HiFi15', lcms=PT_HiFi15, ICC=icSig15colorData),
#   # Unmapped ICC types.
#     icSig2colorData
#     icSig3colorData
#     icSig4colorData
#     icSig5colorData
#     icSig6colorData

# Generate "COLOUR_TYPE_FROM_LCMS" and "COLOUR_TYPE_FROM_ICC" dictionaries
COLOUR_TYPE_FROM_LCMS = {}
COLOUR_TYPE_FROM_ICC = {}
for name, content in COLOUR_TYPE_FROM_NAME.items():
    COLOUR_TYPE_FROM_LCMS[content.lcms] = content
    COLOUR_TYPE_FROM_ICC[content.ICC] = content


# A flag generating C macro reimplemented as a function.
def cmsFLAGS_GRIDPOINTS(number):
    """
    Returns a flag to specify the desired number of grid points 
    in the device link.

    @param number: Number of grid points.
    @type number: C{int}

    @return: Flag usable for the L{cmsCreateTransform} function.
    @rtype: C{int}
    """
    return (number & 0xFF) << 16


# SWIG wrapper backwards compatibility definitions.
def __array_repr(self):
    return '%s(%s)' % (self.__class__.__name__, [x for x in self])

COLORB = ctypes.c_ubyte * 3
COLORB.__repr__ = __array_repr
COLORW = ctypes.c_uint16 * 3
COLORW.__repr__ = __array_repr

cmsSaveProfile = _cmsSaveProfile # pylint: disable-msg=C0103
