#!/usr/bin/env python
# -*- coding: utf-8 -*-
# pylint: disable-msg=
'''
General things for the LCMS wrapper API and colour space handling.
'''

## Created: 2009-01-17 Guy K. Kloss <G.Kloss@massey.ac.nz>
## Changed: $Id:$
##
## Copyright 2000 Guy K. Kloss
## All rights reserved.
##
## This software is distributed under the terms of the
## GNU Lesser General Public License
##
## http://www.massey.ac.nz/~gkloss/

__author__ = 'Guy K. Kloss <G.Kloss@massey.ac.nz>'
__date__ = '$Date: $'
__version__ = '$Revision: $'[11:-2]

from littlecms import c_lcms

CHANNELS = 3


class LcmsException(Exception):
    """
    Indicates that an Exception in the Lcms module has occurred.
    """
    pass



def __handleLcmsError(errorCode, errorText):
    """
    Handle an error from native liblcms.
    """
    if errorCode == c_lcms.LCMS_ERRC_WARNING:
        errorLevel = 'Warning'
    elif errorCode == c_lcms.LCMS_ERRC_RECOVERABLE:
        errorLevel = 'Recoverable Error'
    elif errorCode == c_lcms.LCMS_ERRC_ABORTED:
        errorLevel = 'Aborting Error'
    else:
        errorLevel = 'THIS SHOULD NOT HAVE HAPPENED!'
    message = '%s: %s!' % (errorLevel, errorText)
    raise LcmsException(message)


c_lcms.cmsErrorAction(c_lcms.LCMS_ERROR_SHOW)
c_lcms.cmsSetErrorHandler(c_lcms.cmsErrorHandlerFunction(__handleLcmsError))


def tagSignature2text(tag_sig):
    """
    Converts the integer ICC tag signature value to the corresponding four
    character text representation.
    
    @param tag_sig: Numeric tag signature.
    @type tag_sig: C{int}
    
    @return: Text representation.
    @rtype: C{string}  
    """
    result = []
    for _ in range(4):
        item = tag_sig % 0x100
        result.insert(0, chr(item))
        tag_sig = (tag_sig - item) / 0x100
    return ''.join(result)
