#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''Setup for generated code to alter some behaviour.'''

## Created: 2008-10-19 Guy K. Kloss <G.Kloss@massey.ac.nz>
## Changed: $Id:$
##
## Copyright 2008 Guy K. Kloss
## All rights reserved.
##
## This software is distributed under the terms of the
## GNU Lesser General Public License
##
## http://www.massey.ac.nz/~gkloss/

__author__ = 'Guy K. Kloss <G.Kloss@massey.ac.nz>'
__date__ = '$Date: $'
__version__ = '$Revision: $'[11:-2]


import ctypes
from ctypes.util import find_library
import platform

# One of Gary Bishop's ctypes tricks:
# http://wwwx.cs.unc.edu/~gb/wp/blog/2007/02/11/ctypes-tricks/
# Hack the ctypes.Structure class to include printing the fields.
class Structure(ctypes.Structure):
    def __repr__(self):
        """Print fields of the object."""
        res = []
        for field in self._fields_:
            res.append('%s=%s' % (field[0], repr(getattr(self, field[0]))))
        return '%s(%s)' % (self.__class__.__name__, ', '.join(res))

    @classmethod
    def from_param(cls, obj):
        """Magically construct from a tuple."""
        if isinstance(obj, cls):
            return obj
        if isinstance(obj, tuple):
            return cls(*obj)
        raise TypeError


def _initUNIX():
    return ctypes.cdll.LoadLibrary(find_library('lcms'))
    
    
def _initWindows():
    # Still missing: Modification on where to look for the "lcmsvc7.dll" file,
    # that might be provided with the Windows installer.
    return ctypes.windll.LoadLibrary('lcms.dll')


def _init():
    """
    Loads the shared library through ctypes and returns a library
    L{ctypes.CDLL} instance 
    """
    if platform.system() == 'Windows':
        return _initWindows()
    else:
        return _initUNIX()
