#!/usr/bin/env python
'''Reading of ICC profile header and tags' content.'''

import littlecms
from ctypes import POINTER, c_ubyte, byref

Device_Class = {littlecms.icSigInputClass : 'Input',
                littlecms.icSigDisplayClass : 'Display',
                littlecms.icSigOutputClass : 'Output',
                littlecms.icSigLinkClass : 'Link',
                littlecms.icSigAbstractClass : 'Abstract',
                littlecms.icSigColorSpaceClass : 'ColorSpace',
                littlecms.icSigNamedColorClass : 'NamedColor',
                littlecms.icMaxEnumClass : 'MaxEnum'}

Color_Space = {littlecms.icSigXYZData : 'XYZ',
                littlecms.icSigLabData : 'Lab',
                littlecms.icSigLuvData : 'Luv',
                littlecms.icSigYCbCrData : 'YCbCr',
                littlecms.icSigYxyData : 'Yxy',
                littlecms.icSigRgbData : 'Rgb',
                littlecms.icSigGrayData : 'Gray',
                littlecms.icSigHsvData : 'Hsv',
                littlecms.icSigHlsData : 'Hls',
                littlecms.icSigCmykData : 'Cmyk',
                littlecms.icSigCmyData : 'Cmy',
                littlecms.icSig2colorData : '2color',
                littlecms.icSig3colorData : '3color',
                littlecms.icSig4colorData : '4color',
                littlecms.icSig5colorData : '5color',
                littlecms.icSig6colorData : '6color',
                littlecms.icSig7colorData : '7color',
                littlecms.icSig8colorData : '8color',
                littlecms.icSig9colorData : '9color',
                littlecms.icSig10colorData : '10color',
                littlecms.icSig11colorData : '11color',
                littlecms.icSig12colorData : '12color',
                littlecms.icSig13colorData : '13color',
                littlecms.icSig14colorData : '14color',
                littlecms.icSig15colorData : '15color'}

class ReadTags:
    def __init__(self, profile, raw):
        self.raw = raw
        self.profile = profile
        self.all_tags = profile.tags
        self.text = profile.productInfo + '\n\n'
        self.read_header()
        self.read_tags()

    def read_tags(self):
        text = '======== TAGS ========\n\n'
        text += 'Total number of tags: %d\n\n' % len(self.all_tags)
        if len(self.all_tags):
            l=[]
            for k, v in self.all_tags.iteritems():
                if isinstance(v, littlecms.tag.TextTag) and k not in l:
                    text += '  tag "%s":\n    %s\n\n' % (littlecms.tagSignature2text(k), v.payload)
                    l.append(k)
            text += '\n'
            for k,v in self.all_tags.iteritems():
                if isinstance(v, littlecms.tag.ColourTag) and k not in l:
                    text += '  tag "%s":\n    XYZ = %s\n' % (littlecms.tagSignature2text(k), v.payload.XYZ)
                    text += '    Lab = %s\n\n' % v.payload.Lab.round(3)
                    l.append(k)
            text += '\n'
            for k,v in self.all_tags.iteritems():
                if isinstance(v, littlecms.LutTag) and k not in l:
                    tag_id_string = littlecms.tagSignature2text(k)
                    try:
                        text += self.get_CLUT_tag_content(tag_id_string, v)
                    except:
                        pass
                    l.append(k)
            text += '\n'
            for k,v in self.all_tags.iteritems():
                if isinstance(v, littlecms.tag.Tag) and k not in l:
                    tag_str = littlecms.tagSignature2text(k)
                    text += '  tag "%s":\n    %s\n' % (tag_str, v.payload)
                    if tag_str in ['rTRC', 'gTRC', 'bTRC']:
                        text += '    gamma = %s\n' % self.estimate_gamma(k)
                    text += '\n'
                    l.append(k)
        self.text += text

    def estimate_gamma(self, signature):
        gammatable = littlecms.cmsReadICCGamma(self.profile.lcmsProfile, signature)
        gamma = littlecms.cmsEstimateGamma(gammatable)
        return '%0.2f' % round(gamma, 2)

    def get_report(self):
        return self.text

    def get_CLUT_tag_content(self, tag_id_string, tag):
        # Content of "ugly" or in print irrelevant entries has been commented.
        output = '  tag "%s":\n' % tag_id_string
        output += '    wFlags = %s\n' % tag.content.wFlags
        # output += '    Matrix = %s\n' % tag.content.Matrix
        output += '    InputChan = %s\n' % tag.content.InputChan
        output += '    OutputChan = %s\n' % tag.content.OutputChan
        output += '    InputEntries = %s\n' % tag.content.InputEntries
        output += '    OutputEntries = %s\n' % tag.content.OutputEntries
        output += '    cLutPoints = %s\n' % tag.content.cLutPoints
        # output += '    L1 = %s\n' % tag.content.L1
        # output += '    L2 = %s\n' % tag.content.L2
        # output += '    T = %s\n' % tag.content.T
        output += '    Tsize = %s\n' % tag.content.Tsize
        # output += '    In16params = %s\n' % tag.content.In16params
        # output += '    Out16params = %s\n' % tag.content.Out16params
        # output += '    CLut16params = %s\n' % tag.content.CLut16params
        output += '    Intent = %s\n' % tag.content.Intent
        # output += '    Mat3 = %s\n' % tag.content.Mat3
        # output += '    Ofs3 = %s\n' % tag.content.Ofs3
        # output += '    L3 = %s\n' % tag.content.L3
        # output += '    L3params = %s\n' % tag.content.L3params
        # output += '    L3Entries = %s\n' % tag.content.L3Entries
        # output += '    Mat4 = %s\n' % tag.content.Mat4
        # output += '    Ofs4 = %s\n' % tag.content.Ofs4
        # output += '    L4 = %s\n' % tag.content.L4
        # output += '    L4params = %s\n' % tag.content.L4params
        # output += '    L4Entries = %s\n' % tag.content.L4Entries
        output += '    FixGrayAxes = %s\n' % tag.content.FixGrayAxes
        # output += '    LCurvesSeed = %s\n' % tag.content.LCurvesSeed
        return output + '\n'

    def read_header(self):
        text = '======== HEADER INFO ========\n\n'
        text += 'Size: %s\n' % self.raw.size
        text += 'CMM Type: %s\n' % self.get_cmmtype()
        text += 'Version: %s\n' % self.get_version()
        text += 'Device Class: %s\n' % self.get_device_class()
        text += 'Color Space: %s\n' % self.get_colorspace()
        text += 'PCS: %s\n' % self.get_pcs()
        text += 'Date: %s\n' % self.get_date()
        text += 'Magick: %s\n' % self.get_magick()
        text += 'Platform: %s\n' % self.get_platform()
        text += 'Flags: %s\n' % self.get_flags()
        text += 'Manufacturer: %s\n' % self.get_manufacturer()
        text += 'Model: %s\n' % self.get_model()
        text += 'Attributes: %s\n' % self.get_attributes()
        text += 'Intent: %s\n' % self.get_intent()
        text += 'Illuminant: %s\n' % self.get_illuminant()
        text += 'Creator: %s\n' % self.get_creator()
        text += 'ProfileID: %s\n\n' % self.get_profileID()
        self.text += text

    def get_version(self):
        return hex(littlecms.cmsGetProfileICCversion(self.profile.lcmsProfile))[:-1]

    def get_cmmtype(self):
        return self.raw[4:8].tostring()

    def get_device_class(self):
        dc = littlecms.cmsGetDeviceClass(self.profile.lcmsProfile) or None
        if dc:
            return Device_Class[dc]
        return ''

    def get_colorspace(self):
        cs = littlecms.cmsGetColorSpace(self.profile.lcmsProfile) or None
        if cs:
            return Color_Space[cs]
        return ''

    def get_pcs(self):
        pcs = littlecms.cmsGetPCS(self.profile.lcmsProfile) or None
        if pcs:
            return Color_Space[pcs]
        return ''

    def get_date(self):
        date = littlecms.tm()
        tm = byref(date)
        bul = littlecms.cmsTakeCreationDateTime(tm, self.profile.lcmsProfile)
        if bul:
            return '%s/%s/%s %s:%s:%s' % (1900 + date.tm_year, date.tm_mon, date.tm_mday, date.tm_hour, date.tm_min, date.tm_sec)
        return ''

    def get_magick(self):
        return self.raw[36:40].tostring()

    def get_platform(self):
        return self.raw[40:44].tostring()

    def get_flags(self):
        flags = littlecms.cmsTakeHeaderFlags(self.profile.lcmsProfile )
        if flags == 0:
            return 'not embedded'
        return 'embedded'

    def get_manufacturer(self):
        return littlecms.cmsTakeManufacturer(self.profile.lcmsProfile) or ''
    
    def get_model(self):
        return littlecms.cmsTakeModel(self.profile.lcmsProfile) or ''

    def get_attributes(self):
        atr = littlecms.cmsTakeHeaderAttributes(self.profile.lcmsProfile)
        if atr == 0:
            return 'Reflective, glossy'
        else:
            return 'Transparency, matte'

    def get_intent(self):
        intent = littlecms.cmsTakeRenderingIntent(self.profile.lcmsProfile)
        if intent == 0:
            return 'Perceptual'
        elif intent == 1:
            return 'Relative Colorimetric'
        elif intent == 2:
            return 'Saturation'
        else:
            return 'Absolute Colorimetric'

    def get_illuminant(self):
        xyz = self.profile.illuminant.XYZ
        return 'X=%0.5f Y=%0.5f Z=%0.5f' % (xyz[0], xyz[1], xyz[2])
        
    def get_creator(self):
        return self.raw[80:84].tostring()

    def get_profileID(self):
        p = POINTER(c_ubyte)()
        p = littlecms.cmsTakeProfileID(self.profile.lcmsProfile)
        return str(p.contents.value)

    
