#description: GTKRawGallery library loader 
import os.path
import sys
import traceback
from subprocess import *


def messagebox_error(label, title_="Import Error", close_dialog=True):
    from Tkinter import Tk, Label, Button
    window = Tk()
    window.bell()
    window.title(title_)
    window.minsize(350, 100)
    left = (window.winfo_screenwidth() - 350) / 2
    top = (window.winfo_screenheight() - 100) / 2
    geometry = '%dx%d+%d+%s' % (350, 100, left, top)
    window.geometry(geometry)
    Label(window, text=label).pack(fill='both', expand=True)
    Button(window, text='close', command=window.destroy).pack(side='bottom')
    window.mainloop()
    if close_dialog:
            sys.exit(1)
    
def check_python_version():
        if sys.version[:3] < "2.6":
                messagebox_error("GTKRawGallery requires Python 2.6 or newer!")

def check_pygtk_numpy_support():
        import gtk, pygtk, warnings
        try:
                pixbuf = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB, 0, 8, 1, 1)
                with warnings.catch_warnings():
                    warnings.simplefilter("ignore")
                    pixel = pixbuf.get_pixels_array()
                return True
        except:
                return False

def load_magickwand():
	try:
		from pythonmagickwand.api import version
		version_ =  version()
                if version_:
                        version_ = version_[12:17]
                else:
                        try:
                                p = Popen(['convert', '-version'], shell=True, stdout=PIPE, stdin=PIPE, stderr=STDOUT)
                                output = p.communicate()[0]
                                p.stdout.close()
                                p.stdin.close()
                                version_ = output.split('ImageMagick ')[1][:5]
                        except:
                                version_ = None
                if version_ and version_ <  "6.4.9" :
                        messagebox_error("GTKRawGallery requires ImageMagick 6.4.9 or newer!")
		from pythonmagickwand.image import Image as MagickImage
		from pythonmagickwand.color import Color
		from pythonmagickwand import api
		from ctypes import POINTER, c_ulong
		PI = POINTER(c_ulong)
		PI = c_ulong()
		try:
                        QUANTUM = int(api.GetMagickQuantumDepth(PI)[1:]) 
			Q_RANGE = int(api.GetMagickQuantumRange(PI))
                except:                 # assumes you have a Q16 version installed
                        QUANTUM = 16 
                        Q_RANGE = 65535
		return MagickImage, api, Color, QUANTUM, Q_RANGE
	except:
    		traceback.print_exc()
    		messagebox_error("GTKRawGallery requires ImageMagick 6.4.9 or newer!")

def get_delegates():
        from pythonmagickwand import api
	from ctypes import c_char_p
	delegates = c_char_p()
	delegates = api.MagickQueryConfigureOption("DELEGATES")
	return delegates or ''
		
def load_drawing():
	try:
		from pythonmagickwand import drawing
		return drawing
	except:
    		traceback.print_exc()
		messagebox_error("Cannot load drawing module!")

def binary_exists(binary):
        for item in sys.path:
                if os.path.isfile(os.path.join(item, binary)):
                        return True
        return False

def load_module(name):
        found = True
        if name == 'gtk':
                try:
                	import gtk
                        if gtk.gtk_version < (2, 12, 0):
                                found = False
                except:
			traceback.print_exc()
                        found = False
                finally:
                        if not found:
                                messagebox_error("GTKRawGallery requires GTK+ 2.12.0 or newer!")
                        return gtk
        elif name == 'cairo':
                try:
                        import cairo
                        return cairo
                except:
                        traceback.print_exc()
                        messagebox_error("GTKRawGallery requires Cairo!")
        elif name == 'pygtk':
                try:
                        import pygtk
			import gtk
                        if gtk.pygtk_version < (2, 12, 0):
                                found = False
                except:
			traceback.print_exc()
                        found = False
                finally:
                        if not found:
                                messagebox_error("GTKRawGallery requires PyGTK 2.12.0 or newer!")
                        return pygtk
        elif name == 'numpy':
                try:
                        import numpy 
                        if numpy.version.version < "1.0.4":
                                found = False	
                except:
			traceback.print_exc()
                        found = False
                finally:
                        if not found:
                                messagebox_error("GTKRawGallery requires numpy 1.0.4 or newer!")
                        return numpy
        elif name == 'Image':
                try:
                        import Image 
                        if Image.VERSION < "1.1.6":
                                found = False	
                except:
			traceback.print_exc()
                        found = False
                finally:
                        if not found:
                                messagebox_error("GTKRawGallery requires Imaging Python 1.1.6 or newer!")
                        return Image
        elif name == 'exiftool':
                if all([not os.path.isfile('/usr/bin/exiftool'), not os.path.isfile('/usr/local/bin/exiftool'), not binary_exists('exiftool.exe')]):
                        messagebox_error("GTKRawGallery requires Exiftool 7.41 or newer!")
                p = Popen(['exiftool', '-ver'], shell=True, stdin=PIPE, stdout=PIPE, stderr=STDOUT)
                version = p.communicate()[0][:4]
                p.stdin.close()
                p.stdout.close()
		if version < '7.41':    # It might be edited to work with an older version of Exiftool. Not tested!
			messagebox_error("GTKRawGallery requires Exiftool 7.41 or newer!")
                return True
	elif name == 'dcraw':
                if all([not os.path.isfile('/usr/bin/dcraw'), not os.path.isfile('/usr/local/bin/dcraw'), not binary_exists('dcraw.exe')]):
                        messagebox_error("GTKRawGallery requires Dcraw 8.93 or newer!")
                p = Popen(['dcraw'], shell=True, stdin=PIPE, stdout=PIPE, stderr=STDOUT, universal_newlines=True, env={'LANG':'en_US'})
                output = p.communicate()[0]
                p.stdin.close()
                p.stdout.close()
		if '-p <file>' in output:
			lcms = True
		else:
			lcms = False
		version = output.split('\n')[1][-4:]
		if version < '8.93':
			messagebox_error("GTKRawGallery requires Dcraw 8.93 or newer!")
##		if not lcms:
##                        messagebox_error("Dcraw is compiled without Lcms support.\nColor management will be disabled!", "Warning", False)
                return version, lcms
        elif name == 'cdrecord':
                if all([not os.path.isfile('/usr/bin/cdrecord'), not os.path.isfile('/usr/local/bin/cdrecord'), not binary_exists('cdrecord.exe')]):
                        return False
                return True
        elif name == 'mkisofs':
                if all([not os.path.isfile('/usr/bin/mkisofs'), not os.path.isfile('/usr/local/bin/mkisofs'), not binary_exists('mkisofs.exe')]):
                        return False
                return True
        elif name == 'sqlite':
                try:	
                        return getattr(__import__('gtkrg_sqlite'), 'Sqlite')
                except:
			traceback.print_exc()
                        messagebox_error("GTKRawGallery requires Sqlite3!") # should not be raised from Python 2.6
        elif name == 'gio':
                try:
                        import gio
                except:
                        gio = False
                return gio
        elif name == 'littlecms':
                try:
                        import littlecms
                except:
                        traceback.print_exc()
                        littlecms = False
                return littlecms

def load_splash_screen(datadir, VERSION, COPYRIGHT):
        import gtk, pygtk
        splash_tree = gtk.Builder()
        splash_tree.add_from_file(os.path.join(datadir, 'splash.ui'))
        image = splash_tree.get_object('image1')
        image.set_from_file(os.path.join(datadir, 'icons', 'splash.png'))
        eventbox1 = splash_tree.get_object('eventbox1')
        label1 = splash_tree.get_object('label1')
        label1.set_text('GTKRawGallery ' + VERSION + '\t' + COPYRIGHT)
        eventbox1.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse('black'))
        label1.modify_fg(gtk.STATE_NORMAL, gtk.gdk.color_parse('white'))
        splash = splash_tree.get_object('window1')
        progressbar = splash_tree.get_object('progressbar1')
        return splash, progressbar

    
