#!/usr/bin/env python
#description: GTKRawGallery module for tags database management.

import sqlite3         
import os
import os.path

sqlite3.register_adapter(str, lambda s: s.decode('utf-8'))

class Sqlite:
        
        def __init__(self):
                self.ark = os.path.expanduser('~/.GTKRawGallery')
                self.database = os.path.join(self.ark, 'tags.db')
                if not os.path.exists(self.ark):
                        os.mkdir(self.ark)
                connection = sqlite3.connect(self.database)
                sql = 'create table if not exists TAGGED (filename text unique, keywords text, dir text)'
                connection.execute(sql)
                connection.commit()
                connection.close()

        def get_tagged_from_database(self, tag):
                connection = sqlite3.connect(self.database)
                connection.text_factory = str
                cursor = connection.cursor()
                sql = 'select filename, dir from TAGGED where keywords like %s' % '(?)'
                tuple_ = cursor.execute(sql, ('%;' + tag + ';%',)).fetchall()
                connection.close()
                return tuple_

        def get_tagged_from_dir(self, directory):
                connection = sqlite3.connect(self.database)
                connection.text_factory = str
                cursor = connection.cursor()
                sql = 'select dir, filename from TAGGED where dir=%s' % '(?)'
                tuple_ = cursor.execute(sql,(directory,)).fetchall() 
                connection.close()
                return tuple_

        def has_filename(self, filename, tag):
                connection = sqlite3.connect(self.database)
                connection.text_factory = str
                cursor = connection.cursor()
                sql = 'select filename, keywords from TAGGED where filename like %s and keywords like %s' % ('(?)', '(?)')
                tuple_ = cursor.execute(sql, ('%' + filename + '%', '%;' + tag + ';%')).fetchone()
                connection.close()
                if tuple_:
                        return True
                return False
                
        def get_keywords(self, filename):       
                connection = sqlite3.connect(self.database)
                connection.text_factory = str
                cursor = connection.cursor()
                sql = 'select filename, keywords from TAGGED where filename=%s' % '(?)'
                tuple_ = cursor.execute(sql,(filename,)).fetchone() 
                connection.close()
                if tuple_:
                        return map(str, tuple_[1].split(';'))[1:-1] 
                else:
                        return []
                        
        def insert_row(self, filename, tags, dirname):
                if isinstance(tags, list):
                        keys = ';'
                        for item in tags:
                                keys += item + ';'
                        tags = keys
                else:
                        tags = ';' + tags + ';'
                try:
                        connection = sqlite3.connect(self.database)
                        try:
                                with connection:
                                        sql = 'insert into TAGGED values %s' % '(?,?,?)'
                                        connection.execute(sql, (filename, tags, dirname))
                                        connection.commit()
                        except sqlite3.IntegrityError:
                                print "couldn't add %s twice" % row[0]
                        connection.close()
                except:
                        print 'transaction failed with database!'
                                
        def remove_row(self, filename):
                try:
                        connection = sqlite3.connect(self.database)
                        cursor = connection.cursor()
                        sql = 'delete from TAGGED where filename=%s' % '(?)'
                        connection.execute(sql, (filename,))
                        connection.commit()
                        connection.close()
                except:
                        print 'transaction failed with database!'
                
        def clear_database(self):
                connection = sqlite3.connect(self.database)
                connection.execute('drop table TAGGED')
                connection.execute('create table TAGGED (filename text unique, keywords text, dir text)')
                connection.commit()
                connection.close()

        def update_filename(self, row):
                row = tuple([s.decode('utf-8') for s in row])
                connection = sqlite3.connect(self.database)
                sql = 'update TAGGED set filename=%s where filename=%s' % ('(?)', '(?)')
                connection.execute(sql, row)
                connection.commit()
                connection.close()
                
        def update_keywords(self, filename, keywords):
                connection = sqlite3.connect(self.database)
                sql = 'update TAGGED set keywords=%s where filename=%s' % ('(?)', '(?)')
                keys = ';'
                for item in keywords:
                        keys += item + ';'
                connection.execute(sql, (keys, filename))
                connection.commit()
                connection.close()

        def update_row(self, row):
                connection = sqlite3.connect(self.database)
                sql = 'update or ignore TAGGED set filename=%s, dir=%s where filename=%s' % ('(?)', '(?)', '(?)')
                connection.execute(sql, row)
                connection.commit()
                connection.close()
                                
                
