from ctypes import c_ulong
from pythonmagickwand import api


## Pixel resolutions
PIXELS_PER_INCH = api.PixelsPerInchResolution
PIXELS_PER_CENTIMETER = api.PixelsPerCentimeterResolution

## Compression types
NO_COMPRESSION = api.NoCompression

## Composite operations
OVER_COMPOSITE_OP = api.OverCompositeOp

def _check_wand_error(wand, func):
    if not func:
        severity = api.ExceptionType()
        description = api.MagickGetException(wand, severity)
        raise Exception(description)

class MagickWand(object):

    def __init__(self, image=None):
        self._wand = api.NewMagickWand()

        if hasattr(image, 'read'):
            c = image.read()
            self._check_wand_error(api.MagickReadImageBlob(self._wand, c, len(c)))

    def __del__(self):
        if self._wand and api:
            self._wand = api.DestroyMagickWand(self._wand)

    def _check_wand_error(self, func):
        _check_wand_error(self._wand, func)

    def _get_size(self):
        x = y = c_ulong()
        self._check_wand_error(api.MagickGetSize(self._wand, x, y))
        return (x.value, y.value)

    def _set_size(self, value):
        self._check_wand_error(api.MagickSetSize(self._wand, value[0], value[1]))

    size = property(_get_size, _set_size, None, 'A tuple containing the size associated with the magick wand.')
