#!/usr/bin/env python
# -*- coding: cp1252 -*-
'''
gtkrawgallery.py: GTKRawGallery main script;
comment: A workflow oriented photo manager for digital camera raw image development;
author: Daniele Isca <bit123 AT users DOT sourceforge DOT net>;
license: GNU GPL v.2 (see file COPYING);
This software comes with no warranty of any kind, use at your own risk!
'''
__VERSION__ = "0.9.61"
__COPYRIGHT__ = u' Daniele Isca'

import os.path
import os
import sys
import time
import locale

###sets environ PATH for stand-alone package##
if sys.platform.startswith('win'):
        os.environ['PATH'] += ';' + os.path.join(os.getcwd(), 'Gtk', 'bin')+ ';' + os.path.join(os.getcwd(), 'Gtk', 'lib')
        os.environ['PATH'] += ';' + os.path.join(os.getcwd(), 'ImageMagick')
        os.environ['MAGICK_CONFIGURE_PATH'] = os.path.join(os.getcwd(), 'ImageMagick\config')
        os.environ['MAGICK_CODER_MODULE_PATH'] = os.path.join(os.getcwd(), 'ImageMagick\modules\coders')

homepath = '/'
dcraw = 'dcraw'
exiftool = 'exiftool'

if os.path.isdir('/usr/share/locale'):
        locale_path = '/usr/share/locale'
elif os.path.isdir('/usr/local/share/locale'):
        locale_path = '/usr/local/share/locale'
        
if os.path.isdir('/usr/local/share/gtkrawgallery'):
        gtkrg_datadir = '/usr/local/share/gtkrawgallery'
elif os.path.isdir('/usr/share/gtkrawgallery'):
        gtkrg_datadir = '/usr/share/gtkrawgallery'
elif sys.platform.startswith('win'):
        gtkrg_datadir = os.getcwd()
        homepath = '\\'
        locale_path = 'locale'
else:
        print "/usr/local/share/gtkrawgallery; /usr/share/gtkrawgallery: directory not found!"
        sys.exit(1)

if gtkrg_datadir not in sys.path:
        sys.path.append(gtkrg_datadir)

try:
        import gobject
except:
        from gtkrg_loader import messagebox_error
        messagebox_error("No module named gobject!")
gobject.threads_init()

import gtkrg_loader as loader
loader.check_python_version()
gtk = loader.load_module('gtk')
pygtk = loader.load_module('pygtk')

def pulse_splash(pb, fraction, text=None):
        pb.set_fraction(fraction)
        if text:
                pb.set_text(text)
        while gtk.events_pending():
                gtk.main_iteration(False)
                
splash, splash_pb = loader.load_splash_screen(gtkrg_datadir, __VERSION__, __COPYRIGHT__)
while gtk.events_pending():
        gtk.main_iteration(False)
        
pulse_splash(splash_pb, 0.05, 'Loading MagickWand...')
MagickImage, api, Color, QUANTUM, Q_RANGE = loader.load_magickwand()
pulse_splash(splash_pb, 0.1, 'Loading DrawingWand...')
drawing = loader.load_drawing()
pulse_splash(splash_pb, 0.15, 'Loading Cairo...')
cairo = loader.load_module('cairo')
pulse_splash(splash_pb, 0.2, 'Loading Numpy...')
numpy = loader.load_module('numpy')
pygtk_has_numpy_support = loader.check_pygtk_numpy_support()
if not pygtk_has_numpy_support:
        print '''DeprecationWarning: Pygtk is not compiled with Numpy support.
Consider to recompile Pygtk with Numpy support for best performance.\nTrying to load PIL...'''
        pulse_splash(splash_pb, 0.25, 'Loading Imaging Python...')
        Image = loader.load_module('Image')
        print 'PIL loaded!'
pulse_splash(splash_pb, 0.3, 'Looking for Exiftool...')
loader.load_module('exiftool')
pulse_splash(splash_pb, 0.35, 'Looking for Dcraw...')
dcraw_version, has_lcms = loader.load_module('dcraw')
Sqlite = loader.load_module('sqlite')()
pulse_splash(splash_pb, 0.4, 'Looking for Cdrtools...')
cdrecord = loader.load_module('cdrecord')
mkisofs = loader.load_module('mkisofs')
pulse_splash(splash_pb, 0.45, 'Loading GIO...')
gio = loader.load_module('gio')
pulse_splash(splash_pb, 0.5, 'Looking for Lcms...')
DELEGATES = loader.get_delegates()
littlecms = loader.load_module('littlecms')
pulse_splash(splash_pb, 0.55, 'Loading builtin modules...')
        
from subprocess import *
from tempfile import mkdtemp
from string import ascii_uppercase
import threading
import glob
import math
import cPickle
import gc
import shutil
import traceback
import warnings
import gettext
import json
import exif
import gtkrg_profiles as Profiles

pulse_splash(splash_pb, 0.6, 'Initializing GTKRawGallery...')
        
encoding = locale.getpreferredencoding()
utf8conv = lambda x : unicode(x, encoding, errors='replace').encode('utf8')
unistr = lambda x : unicode(x, encoding, errors='replace')

locale.setlocale(locale.LC_ALL, 'en_US')        #since there is not other translation
gettext.bindtextdomain('gtkrawgallery', locale_path)
gettext.textdomain('gtkrawgallery')
_ = gettext.gettext
gettext.install('gtkrawgallery', locale_path, unicode=1)

class ErrorLog:
        
        def __init__(self, textview, window):
                self.window = window
                self.textview = textview
                self.logfile = os.path.join(os.path.expanduser('~/.GTKRawGallery'), 'error.log') 
                
        def write(self, string):
                now = "[" + time.strftime("%Y-%m-%d %H:%M:%S")+ "]: "
                buffer = self.textview.get_buffer()
                iter = buffer.get_end_iter()
                buffer.place_cursor(iter)
                buffer.insert(iter, utf8conv(now + string))
                self.textview.scroll_to_mark(buffer.get_insert(), 0.1)
                fd = open(self.logfile, "a")
                fd.write(now + string)
                fd.close()
                self.window.set_title("Error Log <!>")
                
        def flush(self):        # called when stderr=STDOUT
                sys.stdout.flush()
                

class Gallery:
        
        def __init__(self):
                self.wtree = gtk.Builder()
                self.wtree.add_from_file(os.path.join(gtkrg_datadir, 'gtkrg.ui'))
                self.load_widgets()
                self.setup_signal_handlers()
                self._id = None
                self.i = 0
                self.radius = 20
                self.drawing_area = 0
                self.combobox12_id = 0
                self.pid = 0
                self.n_columns = 0
                self.magick_size = 0
                self.wand_list = []
                self.blocked_list = []
                self.wb_hist = ['white balance', []]
                self.size_history = [0] * 100
                self.wand_index = 0
                self.crop_count = 0
                self.border_count = 0
                self.wand = None
                self.removed_from_win2 = False
                self.window7_was_iconified = False
                self.window16_was_iconified = False
                self.window2_closed = False
                self.overexposed = False
                self.crop_ratio = 1.5
                self.cdrecord_error = False
                self.x_offset = self.y_offset = 0  
                self.surface = None
                self.pixbuf1 = None
                self.drawing_area_id = None             
                self.stop_batch_process = None
                self.batch_iter = None
                self.sample = None
                self.close_panel = None
                self.scrolledwindow6 = None
                self.raw_preview = None
                self.embedded_profile = None
                self.watch = gtk.gdk.Cursor(gtk.gdk.WATCH)
                self.stop_thread = threading.Event()
                self.stop_thread2 = threading.Event()
                self.stop_thread3 = threading.Event()
                self.stop_gallery_thread = threading.Event()
                self.stop_thumb_generator = threading.Event()
                self.stop_progressbar5 = threading.Event()
                self.mail_event = threading.Event()
                self.lock = threading.Lock()
                self.lock2 = threading.Lock()
                self.count = 0
                self.n = 0
                self.indice = 0
                self.total_size = 0
                self.delay = 0.00
                self.x = 0
                self.y = 0
                self.x1 = 0
                self.y1 = 0
                self.infile = 0
                self.w = 0
                self.h = 0
                self.RGB_levels = {0 : [0,1.0,100], 1 : [0,1.0,100], 2 : [0,1.0,100], 3 : [0,1.0,100]}
                self.mixer_levels = {0 : [100,0,0] , 1 : [0,100,0], 2 : [0,0,100]}
                self.vector = {0 : (), 1 : (), 2 : (), 3 : () }
                self.c_balance = [0,0,0]
                self.mid_contrast = []
                self.kelvin2RGB = Profiles.kelvin2RGB
                self.sRGB_profile = numpy.array(Profiles.sRGB_profile, 'u1')
                self.AdobeRGB1998_profile = numpy.array(Profiles.AdobeRGB1998_profile, 'u1')
                self.expander_list = (self.expander2, self.expander3, self.expander4, self.expander5, self.expander6,
                                self.expander7, self.expander8, self.expander9, self.expander10, self.expander11,
                                self.expander12, self.expander13, self.expander14, self.expander15, self.expander16,
                                self.expander17, self.expander18, self.expander19, self.expander20, self.expander21,
                                self.expander22, self.expander23, self.expander24, self.expander25, self.expander26,
                                self.expander27, self.expander29, self.expander30, self.expander31, self.expander32,
                                self.expander33, self.expander34, self.expander35, self.expander36, self.expander37)
                self.contrast = self.ratio = self.fine_degree = 0
                self.cache_size = 50
                self.border_color = 0xffffffff
                self.gradi = (gtk.gdk.PIXBUF_ROTATE_NONE, gtk.gdk.PIXBUF_ROTATE_COUNTERCLOCKWISE,
                gtk.gdk.PIXBUF_ROTATE_UPSIDEDOWN, gtk.gdk.PIXBUF_ROTATE_CLOCKWISE)
                self.width = 150
                self.height = 100
                self.rotate = False
                self.image_was_changed = False
                self.workflow_started = False
                self.overwrite_all = False
                self.overwrite = False
                self.stop_metadata_toggle = False
                self.history = []
                self.workflow_copy = []
                self.level_hist = []
                self.bright_sat = [0,0,0]
                self.pila = []
                self.list = []
                self.selected = []
                self.image = []
                self.e = []
                self.event = []
                self.favorite_tags = []
                self.default_tags = ['-DateTimeOriginal', '-ISO', '-ExposureTime', '-FNumber', '-FocalLength', '-ExposureProgram', '-ExposureCompensation',
                                '-ImageDescription', '-CameraModelName', '-Software', '-ExifImageWidth', '-ExifImageLength', '-ExifImageHeight',
                                '-BitsPerSample', '-Compression', '-Flash', '-FlashExposureCompensation', '-Lens', '-WhiteBalance',
                                '-WhiteBalanceTemperature','-FocusMode', '-FocusDistance', '-FocusProcess', '-Keywords']
                self.pblist = range(3)
                self.pblist[1] = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB, 0, 8, 1, 1)
                self.fullpblist = range(3)
                self.b_pixlist = range(3)
                self.image_size = '0x0'
                self.fname = 'filetemp.tiff'
                self.pixindex = 1
                self.pbindex = 1
                self.fullpbindex = 1
                self.loop = 1
                self.r = 1
                self.g = 1
                self.b = 1
                self.tag_group = {0 : '-EXIF', 1 : '-IPTC', 2 : '-XMP-exif', 3 : '-XMP-iptcCore', 4 : '-XMP-iptcExt', 5 : '-XMP-tiff',
                                  6 : '-XMP-dc', 7 : '-XMP-cc', 8 : '-XMP-crs', 9 : '-XMP-photoshop', 10 : '-XMP-microsoft',
                                  11 : '-XMP-xmp', 12 : '-XMP-xmpRights'}
                self.print_settings = None
                self.page_setup = None
                self.previous_index = 1
                self.oldadj = 2
                self.pause = 2
                self.frames = 25
                self.album = ''
                self.added_album = ''
                self.cartella = os.path.expanduser('~')
                self.rawpath = {}
                self.rotated = {}
                self.batch_queue = {}
                self.monitors = {}
                self.gallery_monitor = None
                self.presets = {}
                self.auto_wb_presets = {}
                self.ark = os.path.expanduser('~/.GTKRawGallery') 
                self.cache = os.path.join(self.ark, '.Thumbnails')
                self.profile_dir = os.path.join(self.ark, 'profiles')
                self.batch_thumbs = os.path.join(self.ark, '.Batch_Thumbs')
                self.temp = mkdtemp(prefix="gtkrg-raw")
                self.thumbdir = mkdtemp(prefix="gtkrg-thumbnails")
                self.version = 'GTKRawGallery ' + __VERSION__  + '    '
                self.formats = []
                for item in gtk.gdk.pixbuf_get_formats():
                        self.formats += item['extensions']
                self.formats = tuple(['.' + i for i in self.formats])
                self.raw_formats = ('.cr2', '.nef', '.crw', '.raf', '.kdc', '.dcr', '.mrw', '.sr2',
                                    '.orf', '.ptx', '.pef', '.arw', '.srf', '.x3f', '.erf', '.dng',
                                    '.bay', '.bmq', '.cs1', '.dc2', '.fff', '.k25', '.mos', '.rdc', '.rw2')
                if not os.path.isdir(self.ark):
                        os.mkdir(self.ark)
                if not os.path.isdir(self.cache):
                        os.mkdir(self.cache)
                if not os.path.isdir(self.batch_thumbs):
                        os.mkdir(self.batch_thumbs)
                if not os.path.isdir(self.profile_dir):
                        os.mkdir(self.profile_dir)
                self.cache_list = os.listdir(self.cache)
                self.make_distort_grid()
                
                self.boxmodel = gtk.ListStore(str)
                self.boxmodel3 = gtk.ListStore(str)
                self.boxmodel5 = gtk.ListStore(str)
                self.boxmodel7 = gtk.ListStore(str)
                self.boxmodel14 = gtk.ListStore(str)
                self.boxmodel21 = gtk.ListStore(str)
                self.boxmodel22 = gtk.ListStore(str)
                self.boxmodel24 = gtk.ListStore(str)
                self.boxmodel30 = gtk.ListStore(str)
                self.boxmodel31 = gtk.ListStore(str)
                self.boxmodel33 = gtk.ListStore(str)
                self.boxmodel34 = gtk.ListStore(str)
                self.boxmodel35 = gtk.ListStore(str)

#=============  self.treeview1 struct =========================================================
                                                                                                                        
                self.model = gtk.ListStore(gtk.gdk.Pixbuf, str)                                         
                cell = gtk.CellRendererText()
                pb_renderer = gtk.CellRendererPixbuf()
                column = gtk.TreeViewColumn()
                column.set_title('Albums')
                column.pack_start(pb_renderer, expand=False)
                column.add_attribute(pb_renderer, 'pixbuf', 0)
                column.pack_start(cell, expand=True)
                column.add_attribute(cell, 'text', 1)
                column.set_resizable(True)                                                                              
                column.set_sort_column_id(1)                                                    
                self.treeview1.append_column(column)                            
                self.treeview1.set_model(self.model)                                    
                        
#=============  self.treeview2 struct =========================================================
                        
                self.treestore = gtk.TreeStore(gtk.gdk.Pixbuf, str)
                column0 = gtk.TreeViewColumn("Directory Tree")
                self.treeview2.append_column(column0)
                cellpb = gtk.CellRendererPixbuf()
                cell = gtk.CellRendererText()
                column0.pack_start(cellpb, False)
                column0.add_attribute(cellpb, 'pixbuf', 0)
                column0.pack_start(cell, True)
                column0.add_attribute(cell, 'text', 1)
                self.folderpb = gtk.gdk.pixbuf_new_from_file(os.path.join(gtkrg_datadir, 'icons', 'stock_directory_24.png'))
                if sys.platform.startswith('win'):
                        self.rootpb = gtk.gdk.pixbuf_new_from_file(os.path.join(gtkrg_datadir, 'icons', 'stock_harddisk_24.png'))
                        for item in ascii_uppercase:
                                if os.path.exists(item + ':' + homepath):
                                        iter_ = self.treestore.append(None, [self.rootpb, item + ':'])
                                        self.treestore.append(iter_)
                else:
                        self.rootpb = self.folderpb
                        iter_ = self.treestore.append(None, [self.rootpb, '/'])
                        self.treestore.append(iter_)
                self.treeview2.set_model(self.treestore)
                
#============== self.treeview3 struct =========================================================

                self.exifmodel = gtk.ListStore(str, str)
                cell = gtk.CellRendererText()                                                                           
                column_item = gtk.TreeViewColumn("Tag", cell, text=0)                   
                column_item.set_resizable(True)                                                                                         
                self.treeview3.append_column(column_item)                               
                cell = gtk.CellRendererText()                                                                           
                column_value = gtk.TreeViewColumn("Value", cell, text=1)                        
                column_value.set_resizable(True)
                self.treeview3.append_column(column_value)              
                self.treeview3.set_model(self.exifmodel)
                
#============= self.treeview4 struct ==========================================================
                
                self.batchmodel = gtk.ListStore(str, gtk.gdk.Pixbuf, 'gboolean', str, str)
                (COL_N, COL_PIX, COL_TOGGLE, COL_FILENAME, COL_COLOR) = range(5)
                renderer_number = gtk.CellRendererText()
                column_number = gtk.TreeViewColumn("Run", renderer_number, text=COL_N)
                column_number.set_cell_data_func (renderer_number, self.cell_data_function, None)
                self.treeview4.append_column(column_number)
                renderer_pix = gtk.CellRendererPixbuf()
                column_pix = gtk.TreeViewColumn("Preview", renderer_pix, pixbuf=COL_PIX)
                self.treeview4.append_column(column_pix)
                renderer_toggle = gtk.CellRendererToggle()
                renderer_toggle.set_property('activatable', True)
                renderer_toggle.connect('toggled', self.on_renderer_toggle_toggled, self.batchmodel)
                column_toggle = gtk.TreeViewColumn("Enabled", renderer_toggle)
                column_toggle.add_attribute(renderer_toggle, "activatable", 2)
                column_toggle.add_attribute(renderer_toggle, "active", 2)
                column_toggle.set_attributes(renderer_toggle, active=2)
                column_toggle.set_cell_data_func (renderer_toggle, self.cell_data_function, None)
                self.treeview4.append_column(column_toggle)
                renderer_filename = gtk.CellRendererText()
                column_filename = gtk.TreeViewColumn("Filename", renderer_filename, text=COL_FILENAME)
                column_filename.set_cell_data_func (renderer_filename, self.cell_data_function, None)
                self.treeview4.append_column(column_filename)
                renderer_color = gtk.CellRendererText()
                column_color = gtk.TreeViewColumn("Status", renderer_color, text=COL_COLOR)
                column_color.set_cell_data_func (renderer_color, self.cell_data_function, None)
                self.treeview4.append_column(column_color)
                self.treeview4.set_model(self.batchmodel)

#=============== self.treeview5 struct ========================================================
                
                self.tagmodel = gtk.ListStore('gboolean', str)
                tag_toggle = gtk.CellRendererToggle()
                tag_toggle.set_property('activatable', True)
                tag_toggle.connect('toggled', self.on_tag_toggle_toggled, self.tagmodel)
                column_tag_toggle = gtk.TreeViewColumn(None, tag_toggle)
                column_tag_toggle.set_attributes(tag_toggle, active=0)
                self.treeview5.append_column(column_tag_toggle)
                renderer_tag_name = gtk.CellRendererText()
                column_tag_name = gtk.TreeViewColumn("Keywords", renderer_tag_name, text=1)
                self.treeview5.append_column(column_tag_name)
                self.treeview5.set_model(self.tagmodel)

#=============== self.treeview6 struct=========================================================
                
                self.tagmodel2 = gtk.ListStore('gboolean', str)
                self.tag_toggle2 = gtk.CellRendererToggle()
                self.tag_toggle2.set_property('activatable', True)
                self.id81 = self.tag_toggle2.connect('toggled', self.on_tag_toggle_toggled, self.tagmodel2)
                column_tag_toggle2 = gtk.TreeViewColumn(None, self.tag_toggle2)
                column_tag_toggle2.set_attributes(self.tag_toggle2, active=0)
                self.treeview6.append_column(column_tag_toggle2)
                renderer_tag_name2 = gtk.CellRendererText()
                column_tag_name2 = gtk.TreeViewColumn("Keywords", renderer_tag_name2, text=1)
                self.treeview6.append_column(column_tag_name2)
                self.treeview6.set_model(self.tagmodel2)
                
#=============== other combobox structs =======================================================
                self.tagmodel3 = gtk.ListStore(str)
                cell = gtk.CellRendererText()
                self.combobox5.set_model(self.boxmodel5)
                self.combobox5.pack_start(cell, True)
                self.combobox5.add_attribute(cell, 'text', 0)
                self.boxmodel7.append(['jpg'])
                self.boxmodel7.append(['png'])
                self.boxmodel7.append(['tiff 8 bit'])
                self.boxmodel7.append(['tiff 16 bit'])
                self.boxmodel7.append(['tiff best'])
                self.boxmodel7.append(['bmp'])
                self.combobox7.set_model(self.boxmodel7)
                self.combobox7.pack_start(cell, True)
                self.combobox7.add_attribute(cell, 'text', 0)
                self.combobox14.set_model(self.boxmodel14)
                self.combobox14.pack_start(cell, True)
                self.combobox14.add_attribute(cell, 'text', 0)
                self.combobox20.set_model(self.boxmodel14)
                self.combobox20.pack_start(cell, True)
                self.combobox20.add_attribute(cell, 'text', 0)
                self.combobox16.set_model(self.boxmodel3)
                self.combobox16.pack_start(cell, True)
                self.combobox16.add_attribute(cell, 'text', 0)
                self.combobox15.set_model(self.tagmodel3)
                self.combobox15.pack_start(cell, True)
                self.combobox15.add_attribute(cell, 'text', 0)
                self.combobox21.set_model(self.boxmodel21)
                self.combobox21.pack_start(cell, True)
                self.combobox21.add_attribute(cell, 'text', 0)
                self.combobox22.set_model(self.boxmodel22)
                self.combobox22.pack_start(cell, True)
                self.combobox22.add_attribute(cell, 'text', 0)
                self.combobox23.set_model(self.boxmodel14)
                self.combobox23.pack_start(cell, True)
                self.combobox23.add_attribute(cell, 'text', 0)
                self.combobox24.set_model(self.boxmodel24)
                self.combobox24.pack_start(cell, True)
                self.combobox24.add_attribute(cell, 'text', 0)
                self.combobox28.set_model(self.boxmodel)
                self.combobox28.pack_start(cell, True)
                self.combobox28.add_attribute(cell, 'text', 0)
                self.combobox29.set_model(self.boxmodel)
                self.combobox29.pack_start(cell, True)
                self.combobox29.add_attribute(cell, 'text', 0)
                self.combobox30.set_model(self.boxmodel30)
                self.combobox30.pack_start(cell, True)
                self.combobox30.add_attribute(cell, 'text', 0)
                self.combobox31.set_model(self.boxmodel31)
                self.combobox31.pack_start(cell, True)
                self.combobox31.add_attribute(cell, 'text', 0)
                self.combobox32.set_model(self.boxmodel31)
                self.combobox32.pack_start(cell, True)
                self.combobox32.add_attribute(cell, 'text', 0)
                self.combobox33.set_model(self.boxmodel33)
                self.combobox33.pack_start(cell, True)
                self.combobox33.add_attribute(cell, 'text', 0)
                self.combobox34.set_model(self.boxmodel34)
                self.combobox34.pack_start(cell, True)
                self.combobox34.add_attribute(cell, 'text', 0)
                self.combobox35.set_model(self.boxmodel35)
                self.combobox35.pack_start(cell, True)
                self.combobox35.add_attribute(cell, 'text', 0)
                self.boxmodel30.append(['No WB Presets'])
                self.boxmodel30.append(['Auto WB Presets'])
                self.boxmodel5.append(['Camera WB'])
                self.boxmodel5.append(['Auto WB'])
                self.boxmodel5.append(['Manual WB'])
                self.boxmodel33.append(['sRGB'])
                self.boxmodel33.append(['AdobeRGB 1998'])
                self.boxmodel34.append(['sRGB'])
                self.boxmodel34.append(['AdobeRGB 1998'])
                self.boxmodel35.append(['sRGB'])
                
#=============== self.treeview7 struct ========================================================

                self.tv7_model = gtk.ListStore(str, str)
                tv7_cell1 = gtk.CellRendererText()
                tv7_column1 = gtk.TreeViewColumn('Filename', tv7_cell1, text=0)
                self.treeview7.append_column(tv7_column1)
                tv7_cell2 = gtk.CellRendererText()
                tv7_column2 = gtk.TreeViewColumn('Tags', tv7_cell2, text=1)
                self.treeview7.append_column(tv7_column2)
                self.treeview7.set_model(self.tv7_model)
                
#================self.treeview8 struct=========================================================

                self.custom_model = gtk.ListStore(str, str, str)
                cell = gtk.CellRendererText()                                                                           
                column_item = gtk.TreeViewColumn("Group", cell, text=0)                 
                column_item.set_resizable(True)                                                                                         
                self.treeview8.append_column(column_item)
                cell = gtk.CellRendererText()                                                                           
                column_item = gtk.TreeViewColumn("Tag", cell, text=1)                   
                column_item.set_resizable(True)                                                                                         
                self.treeview8.append_column(column_item)                               
                cell = gtk.CellRendererText()   
                cell.set_property("editable", True)
                cell.set_property("cell-background-gdk", gtk.gdk.color_parse('#ffffff'))
                cell.connect("edited", self.on_custom_model_edited, self.custom_model, 2)
                column_value = gtk.TreeViewColumn("Value", cell, text=2)                        
                column_value.set_resizable(True)
                self.treeview8.append_column(column_value)              
                self.treeview8.set_model(self.custom_model)
                
#================ treeview9 structure =========================================================
                
                self.attachment_model = gtk.ListStore(gtk.gdk.Pixbuf, str)
                renderer_pix = gtk.CellRendererPixbuf()
                column_pix = gtk.TreeViewColumn("Preview", renderer_pix, pixbuf=0)
                self.treeview9.append_column(column_pix)
                cell = gtk.CellRendererText()                                                                           
                column_item = gtk.TreeViewColumn("Attachment", cell, text=1)                    
                column_item.set_resizable(True)                                                                                         
                self.treeview9.append_column(column_item)
                self.treeview9.set_model(self.attachment_model)
                
#=============== self.treeview10 struct =======================================================

                self.exif_tab_model = gtk.ListStore('gboolean', str, str)
                tag_toggle = gtk.CellRendererToggle()
                tag_toggle.set_property('activatable', True)
                tag_toggle.connect('toggled', self.on_metadata_tab_toggled, self.exif_tab_model)
                column_tag_toggle = gtk.TreeViewColumn("Favorites", tag_toggle)
                column_tag_toggle.set_attributes(tag_toggle, active=0)
                self.treeview10.append_column(column_tag_toggle)
                renderer_tag_name = gtk.CellRendererText()
                column_tag_name = gtk.TreeViewColumn("Tag", renderer_tag_name, text=1)
                self.treeview10.append_column(column_tag_name)
                renderer_tag_field = gtk.CellRendererText()
                column_tag_field = gtk.TreeViewColumn("Value", renderer_tag_field, text=2)
                self.treeview10.append_column(column_tag_field)
                self.treeview10.set_model(self.exif_tab_model)

#=============== self.treeview11 struct =======================================================

                self.iptc_tab_model = gtk.ListStore('gboolean', str, str)
                tag_toggle = gtk.CellRendererToggle()
                tag_toggle.set_property('activatable', True)
                tag_toggle.connect('toggled', self.on_metadata_tab_toggled, self.iptc_tab_model)
                column_tag_toggle = gtk.TreeViewColumn("Favorites", tag_toggle)
                column_tag_toggle.set_attributes(tag_toggle, active=0)
                self.treeview11.append_column(column_tag_toggle)
                renderer_tag_name = gtk.CellRendererText()
                column_tag_name = gtk.TreeViewColumn("Tag", renderer_tag_name, text=1)
                self.treeview11.append_column(column_tag_name)
                renderer_tag_field = gtk.CellRendererText()
                column_tag_field = gtk.TreeViewColumn("Value", renderer_tag_field, text=2)
                self.treeview11.append_column(column_tag_field)
                self.treeview11.set_model(self.iptc_tab_model)

#=============== self.treeview12 struct =======================================================

                self.xmp_tab_model = gtk.ListStore('gboolean', str, str)
                tag_toggle = gtk.CellRendererToggle()
                tag_toggle.set_property('activatable', True)
                tag_toggle.connect('toggled', self.on_metadata_tab_toggled, self.xmp_tab_model)
                column_tag_toggle = gtk.TreeViewColumn("Favorites", tag_toggle)
                column_tag_toggle.set_attributes(tag_toggle, active=0)
                self.treeview12.append_column(column_tag_toggle)
                renderer_tag_name = gtk.CellRendererText()
                column_tag_name = gtk.TreeViewColumn("Tag", renderer_tag_name, text=1)
                self.treeview12.append_column(column_tag_name)
                renderer_tag_field = gtk.CellRendererText()
                column_tag_field = gtk.TreeViewColumn("Value", renderer_tag_field, text=2)
                self.treeview12.append_column(column_tag_field)
                self.treeview12.set_model(self.xmp_tab_model)

#=============== self.treeview13 struct =======================================================

                self.makernotes_tab_model = gtk.ListStore('gboolean', str, str)
                tag_toggle = gtk.CellRendererToggle()
                tag_toggle.set_property('activatable', True)
                tag_toggle.connect('toggled', self.on_metadata_tab_toggled, self.makernotes_tab_model)
                column_tag_toggle = gtk.TreeViewColumn("Favorites", tag_toggle)
                column_tag_toggle.set_attributes(tag_toggle, active=0)
                self.treeview13.append_column(column_tag_toggle)
                renderer_tag_name = gtk.CellRendererText()
                column_tag_name = gtk.TreeViewColumn("Tag", renderer_tag_name, text=1)
                self.treeview13.append_column(column_tag_name)
                renderer_tag_field = gtk.CellRendererText()
                column_tag_field = gtk.TreeViewColumn("Value", renderer_tag_field, text=2)
                self.treeview13.append_column(column_tag_field)
                self.treeview13.set_model(self.makernotes_tab_model)
                
#===============self.treeview14 struct ========================================================

                self.custom_model2 = gtk.ListStore(str, str, str)
                cell = gtk.CellRendererText()                                                                           
                column_item = gtk.TreeViewColumn("Group", cell, text=0)                 
                column_item.set_resizable(True)                                                                                         
                self.treeview14.append_column(column_item)
                cell = gtk.CellRendererText()                                                                           
                column_item = gtk.TreeViewColumn("Tag", cell, text=1)                   
                column_item.set_resizable(True)                                                                                         
                self.treeview14.append_column(column_item)                              
                cell = gtk.CellRendererText()   
                cell.set_property("editable", True)
                cell.set_property("cell-background-gdk", gtk.gdk.color_parse('#ffffff'))
                cell.connect("edited", self.on_custom_model_edited, self.custom_model2, 2)
                column_value = gtk.TreeViewColumn("Value", cell, text=2)                        
                column_value.set_resizable(True)
                self.treeview14.append_column(column_value)             
                self.treeview14.set_model(self.custom_model2)
                
#================self.treeview15 struct ======================================================
                
                self.cdmodel = gtk.ListStore('gboolean', gtk.gdk.Pixbuf, str)
                toggle = gtk.CellRendererToggle()
                toggle.set_property('activatable', True)
                toggle.connect('toggled', self.on_cdmodel_toggled, self.cdmodel)
                column_toggle = gtk.TreeViewColumn("Enabled", toggle)
                column_toggle.set_attributes(toggle, active=0)
                self.treeview15.append_column(column_toggle)
                renderer_pix = gtk.CellRendererPixbuf()
                column_pix = gtk.TreeViewColumn("Preview", renderer_pix, pixbuf=1)
                self.treeview15.append_column(column_pix)
                cell = gtk.CellRendererText()                                                                           
                column_item = gtk.TreeViewColumn("Filename", cell, text=2)                      
                column_item.set_resizable(True)                                                                                         
                self.treeview15.append_column(column_item)
                self.treeview15.set_model(self.cdmodel)
                
#================ self.treeview16 struct ======================================================
                
                self.preset_model = gtk.ListStore(str, str, str, str, str)
                cell = gtk.CellRendererText()
                column_item = gtk.TreeViewColumn("ID", cell, text=0)                 
                column_item.set_resizable(True)
                column_item.set_min_width(20)
                self.treeview16.append_column(column_item)
                cell = gtk.CellRendererText()
                cell.set_property("editable", True)
                cell.connect("edited", self.on_custom_model_edited, self.preset_model, 1)
                column_item1 = gtk.TreeViewColumn("WB Name", cell, text=1)                 
                column_item1.set_resizable(True)
                column_item1.set_expand(True) 
                self.treeview16.append_column(column_item1)
                cell1 = gtk.CellRendererText()
                cell1.set_property("editable", True)
                cell1.connect("edited", self.on_custom_model_edited, self.preset_model, 2)
                column_item2 = gtk.TreeViewColumn("Red multiplier", cell1, text=2)                 
                column_item2.set_resizable(True)
                column_item2.set_min_width(100)
                self.treeview16.append_column(column_item2)
                cell2 = gtk.CellRendererText()
                cell2.set_property("editable", True)
                cell2.connect("edited", self.on_custom_model_edited, self.preset_model, 3)
                column_item3 = gtk.TreeViewColumn("Green multiplier", cell2, text=3)               
                column_item3.set_resizable(True)
                column_item3.set_min_width(100)
                self.treeview16.append_column(column_item3)
                cell3 = gtk.CellRendererText()
                cell3.set_property("editable", True)
                cell3.connect("edited", self.on_custom_model_edited, self.preset_model, 4)
                column_item4 = gtk.TreeViewColumn("Blue multiplier", cell3, text=4)                 
                column_item4.set_resizable(True)
                column_item4.set_min_width(100)
                self.treeview16.append_column(column_item4)
                self.treeview16.set_model(self.preset_model)
                pulse_splash(splash_pb, 0.7)
#==============================================================================================
                self.get_default_style()
                self.load_settings_database()
                self.load_album_database()
                self.load_batch_database()
                self.load_mtimes_database()
                self.load_templates()
                pulse_splash(splash_pb, 0.75)
                self.widgets_settings()
                pulse_splash(splash_pb, 1)
                self.scrolledwindow2.set_size_request(int(self.window1.get_allocation().width * 0.83), -1)
                self.window1.maximize()
                self.window1.show()
                splash.hide()
                self.button5.hide()
                self.window1.window.set_cursor(self.watch)
                while gtk.events_pending():
                        gtk.main_iteration(False)
                if len(sys.argv) > 1:
                        if os.path.isfile(sys.argv[1]) and sys.argv[1].lower().endswith(self.formats + self.raw_formats):
                                self.open_with_gtkrawgallery(sys.argv[1])
                        elif os.path.isdir(sys.argv[1]):
                                self.open_last_folder(sys.argv[1], None)
                else:
                        self.load_last_path()
                self.window1.window.set_cursor(None)
                self.add_patterns()
                self.hpaned1.handler_unblock(self.id28)
                self.cache_control_thread = threading.Thread(target=self.cache_control, args=())
                self.cache_control_thread.start()
                sys.stderr = ErrorLog(self.textview1, self.window14)
                self.make_surface(True)
                self.draw_gradient(self.r, self.g, self.b)
                self.rect = self.viewport1.allocation
                
#======= Gallery Methods ======================================================================

        def initialize_widgets(self, tree):
                for widget in tree.get_objects():
                        try:
                                name = gtk.Buildable.get_name(widget)
                                if name and not hasattr(self, name):
                                        setattr(self, name, widget)
                        except:
                                pass

        def messagedialog(self, title=None, primary_text=None, secondary_text=None, buttons=gtk.BUTTONS_NONE, parent=None, flags=0, type_=gtk.MESSAGE_INFO, image=None, size=None):
                dialog = gtk.MessageDialog(parent=parent, flags=flags, type=type_, buttons=buttons, message_format=primary_text)
                if size:
                        dialog.set_size_request(*size)
                if secondary_text:
                        dialog.format_secondary_text(secondary_text)
                if image:
                        dialog.set_image(image)
                if title:
                        dialog.set_title(title)
                dialog.set_icon_from_file(self.logo)
                dialog.set_position(gtk.WIN_POS_CENTER)
                dialog.show()
                return dialog
                                        
                        
        def load_widgets(self):
                self.initialize_widgets(self.wtree)
                self.display = gtk.gdk.display_get_default()
                self.screen0 = self.display.get_screen(0)
                self.filter0 = gtk.FileFilter()
                self.filter1 = gtk.FileFilter()
                self.filter2 = gtk.FileFilter()
                self.eb = gtk.EventBox()
                self.grey = gtk.gdk.pixbuf_new_from_file(self.get_icon_path('layer1.png'))
                self.make_background_pixbuf()
                self.make_aboutdialog()
                self.adj = self.hscale1.get_adjustment()
                self.jpg_quality = self.hscale2.get_adjustment()

        def widgets_settings(self):
                self.dialog11.set_default_response(gtk.RESPONSE_OK)
                self.dialog19.set_default_response(gtk.RESPONSE_OK)
                self.dialog13.set_default_response(gtk.RESPONSE_OK)
                self.dialog23.set_default_response(gtk.RESPONSE_OK)
                self.dialog1.set_default_response(gtk.RESPONSE_OK)
                self.window17.set_keep_above(True)
                self.dialog1.set_keep_above(True)
                self.dialog3.set_keep_above(True)
                self.dialog7.set_keep_above(True)
                self.dialog11.set_keep_above(True)
                self.dialog13.set_keep_above(True)
                self.window14.set_keep_above(True)
                self.dialog13.set_transient_for(self.window2)
                self.dialog16.set_keep_above(True)
                self.dialog19.set_keep_above(True)
                self.dialog21.set_keep_above(True)
                self.dialog22.set_keep_above(True)
                self.dialog23.set_keep_above(True)
                self.dialog24.set_keep_above(True)
                self.dialog28.set_keep_above(True)
                self.dialog30.set_keep_above(True)
                self.dialog31.set_keep_above(True)
                self.dialog33.set_keep_above(True)
                self.filechooserdialog2.set_keep_above(True)
                self.filechooserdialog3.set_keep_above(True)
                self.filechooserdialog4.set_keep_above(True)
                self.window5.set_keep_above(True)
                self.window8.set_keep_above(True)
                self.window11.set_keep_above(True)
                self.context_id1 = self.statusbar1.get_context_id('win2_percentage')
                self.context_id2 = self.statusbar2.get_context_id('dimensions')
                self.context_id3 = self.statusbar3.get_context_id('tot. photos')
                self.context_id4 = self.statusbar4.get_context_id('win1_percentage')
                self.image9.set_from_file(self.get_icon_path('layout-gallery.png'))
                self.image10.set_from_file(self.get_icon_path('layout-bottom.png'))
                self.image7.set_from_file(self.get_icon_path('sound.png'))
                self.image8.set_from_file(self.get_icon_path('video.png'))
                self.image33.set_from_file(self.get_icon_path('object-flip-horizontal.png'))
                self.image40.set_from_file(self.get_icon_path('object-flip-vertical.png'))
                self.image43.set_from_file(self.get_icon_path('object-rotate-right.png'))
                self.image46.set_from_file(self.get_icon_path('object-rotate-left.png'))
                self.image49.set_from_file(self.get_icon_path('resize.png'))
                self.image72.set_from_file(self.get_icon_path('stock_contact.png'))
                open_image = gtk.image_new_from_file(self.get_icon_path('open_photo.png'))
                self.open_image.set_image(open_image)
                pulse_splash(splash_pb, 0.8)
                imagemenuitem6 = gtk.image_new_from_file(self.get_icon_path('open_photo.png'))
                self.imagemenuitem6.set_image(imagemenuitem6)
                imagemenuitem19 = gtk.image_new_from_file(self.get_icon_path('mime-image.png'))
                self.imagemenuitem19.set_image(imagemenuitem19)
                imagemenuitem20 = gtk.image_new_from_file(self.get_icon_path('mime-image.png'))
                self.imagemenuitem20.set_image(imagemenuitem20)
                imagemenuitem2 = gtk.image_new_from_file(self.get_icon_path('hide-panel.png'))
                self.imagemenuitem2.set_image(imagemenuitem2)
                imagemenuitem3 = gtk.image_new_from_file(self.get_icon_path('show-panel.png'))
                pulse_splash(splash_pb, 0.85)
                self.imagemenuitem3.set_image(imagemenuitem3)
                imagemenuitem5 = gtk.image_new_from_file(self.get_icon_path('bottom-panel.png'))
                self.imagemenuitem5.set_image(imagemenuitem5)
                menuitem13 = gtk.image_new_from_file(self.get_icon_path('hide-pane.png'))
                self.menuitem13.set_image(menuitem13)
                open_album1 = gtk.image_new_from_file(self.get_icon_path('open_album.png'))
                self.open_album1.set_image(open_album1)
                menuitem32 = gtk.image_new_from_file(self.get_icon_path('stock_contact.png'))
                self.menuitem32.set_image(menuitem32)
                image10 = gtk.Image()
                image10.set_from_file(self.get_icon_path('rotate_ccw.png'))
                self.toolbutton10.set_icon_widget(image10)
                image10.show()
                image11 = gtk.Image()
                image11.set_from_file(self.get_icon_path('rotate_cw.png'))
                self.toolbutton11.set_icon_widget(image11)
                image11.show()
                image18 = gtk.Image()
                image18.set_from_file(self.get_icon_path('select_all.png'))
                self.toolbutton18.set_icon_widget(image18)
                image18.show()
                image19 = gtk.Image()
                image19.set_from_file(self.get_icon_path('unselect.png'))
                self.toolbutton19.set_icon_widget(image19)
                image19.show()
                image20 = gtk.Image()
                image20.set_from_file(self.get_icon_path('inverted.png'))
                self.toolbutton20.set_icon_widget(image20)
                image20.show()
                image26 = gtk.Image()
                image26.set_from_file(self.get_icon_path('hide-panel.png'))
                self.toolbutton26.set_icon_widget(image26)
                image26.show()
                image27 = gtk.Image()
                image27.set_from_file(self.get_icon_path('show-panel.png'))
                self.toolbutton27.set_icon_widget(image27)
                image27.show()
                image28 = gtk.Image()
                image28.set_from_file(self.get_icon_path('hide-pane.png'))
                self.toolbutton28.set_icon_widget(image28)
                image28.show()
                image29 = gtk.Image()
                image29.set_from_file(self.get_icon_path('bottom-panel.png'))
                self.toolbutton29.set_icon_widget(image29)
                image29.show()
                image30 = gtk.Image()
                image30.set_from_file(self.get_icon_path('linear-hist.png'))
                self.togglebutton1.add(image30)
                image30.show()
                image31 = gtk.Image()
                image31.set_from_file(self.get_icon_path('log-hist.png'))
                self.togglebutton2.add(image31)
                image31.show()
                image32 = gtk.Image()
                image32.set_from_file(self.get_icon_path('rotate_ccw.png'))
                self.toolbutton42.set_icon_widget(image32)
                image32.show()
                image33 = gtk.Image()
                image33.set_from_file(self.get_icon_path('rotate_cw.png'))
                self.toolbutton43.set_icon_widget(image33)
                image33.show()
                image35 = gtk.Image()
                image35.set_from_file(self.get_icon_path('playlist.png'))
                self.toolbutton46.set_icon_widget(image35)
                image35.show()
                image36 = gtk.Image()
                image36.set_from_file(self.get_icon_path('cms.png'))
                self.toolbutton12.set_icon_widget(image36)
                image36.show()
                menuitem35_image= gtk.Image()
                menuitem35_image.set_from_file(self.get_icon_path('console.png'))
                self.menuitem35.set_image(menuitem35_image)
                menuitem35_image.show()
                menuitem29_image= gtk.Image()
                menuitem29_image.set_from_file(self.get_icon_path('shortcuts.png'))
                self.menuitem29.set_image(menuitem29_image)
                menuitem29_image.show()
                menuitem25_image= gtk.Image()
                menuitem25_image.set_from_stock(gtk.STOCK_APPLY, 1)
                self.menuitem25.set_image(menuitem25_image)
                menuitem25_image.show()
                menuitem12_image= gtk.Image()
                menuitem12_image.set_from_stock(gtk.STOCK_FIND, 1)
                self.menuitem12.set_image(menuitem12_image)
                menuitem12_image.show()
                self.nero = gtk.gdk.color_parse('#000000')
                self.selection_color = self.colorbutton5.get_color()
                self.colorbase = gtk.gdk.color_parse('#EFEFEF')
                self.viewport1.modify_bg(gtk.STATE_NORMAL, self.image_background_color)
                self.viewport2.modify_bg(gtk.STATE_NORMAL, self.treeview_color)
                self.window3.modify_bg(gtk.STATE_NORMAL, self.nero)
                self.window4.modify_bg(gtk.STATE_NORMAL, self.nero)
                self.window1.modify_bg(gtk.STATE_NORMAL, self.toolbar_color)
                self.window2.modify_bg(gtk.STATE_NORMAL, self.toolbar_color)
                self.toolbar1.modify_bg(gtk.STATE_NORMAL, self.toolbar_color)
                self.toolbar2.modify_bg(gtk.STATE_NORMAL, self.toolbar_color)
                self.eventbox1.modify_bg(gtk.STATE_NORMAL, self.gallery_color)
                self.progressbar3.modify_bg(gtk.STATE_NORMAL, self.toolbar_color)
                self.progressbar8.modify_bg(gtk.STATE_NORMAL, self.toolbar_color)
                self.button5.modify_bg(gtk.STATE_NORMAL, self.toolbar_color)
                self.treeview1.modify_base(gtk.STATE_NORMAL, self.treeview_color)
                self.treeview2.modify_base(gtk.STATE_NORMAL, self.treeview_color)
                self.treeview3.modify_base(gtk.STATE_NORMAL, self.treeview_color)
                self.treeview6.modify_base(gtk.STATE_NORMAL, self.treeview_color)
                self.treeview10.modify_base(gtk.STATE_NORMAL, self.treeview_color)
                self.treeview11.modify_base(gtk.STATE_NORMAL, self.treeview_color)
                self.treeview12.modify_base(gtk.STATE_NORMAL, self.treeview_color)
                self.treeview13.modify_base(gtk.STATE_NORMAL, self.treeview_color)
                self.treeview14.modify_base(gtk.STATE_NORMAL, self.treeview_color)
                self.eventbox3.modify_bg(gtk.STATE_NORMAL, self.treeview_color)
                self.eventbox5.modify_bg(gtk.STATE_NORMAL, self.treeview_color)
                self.eventbox6.modify_bg(gtk.STATE_NORMAL, self.treeview_color)
                self.eventbox7.modify_bg(gtk.STATE_NORMAL, self.treeview_color)
                self.eventbox39.modify_bg(gtk.STATE_NORMAL, self.treeview_color)
                self.eventbox40.modify_bg(gtk.STATE_NORMAL, self.treeview_color)
                self.logo = self.get_icon_path('gtkrawgallery.png')
                for value in  self.__dict__.itervalues():
                        if isinstance(value, (gtk.Dialog, gtk.Window)):
                                value.set_icon_from_file(self.logo)
                self.aboutdialog.set_icon_from_file(self.logo)
                self.filechooserbutton1.set_current_folder(self.ark)
                self.filechooserbutton2.set_current_folder(os.path.expanduser('~'))
                self.filechooserbutton8.set_current_folder(os.path.expanduser('~'))
                self.filechooserbutton9.set_current_folder(os.path.expanduser('~'))
                self.filechooserdialog2.set_current_folder(os.path.expanduser('~'))
                self.filechooserdialog3.set_current_folder(os.path.expanduser('~'))
                self.entry4.set_text(self.ark)
                self.entry5.set_text(os.path.expanduser('~'))
                self.entry8.set_text(os.path.expanduser('~'))
                self.window1.set_title(self.version)
                self.window1.move(0, 0)
                n_screens = self.display.get_n_screens()
                screen1 = self.display.get_screen(n_screens -1)
                self.window2.set_default_size(int(screen1.get_width() * 0.9), int(screen1.get_height() * 0.9))
                self.scrolledwindow3.set_size_request(int(screen1.get_width() * 0.9 * 0.60), -1)
                self.hpaned1.set_position(int(self.screen0.get_width() * 0.20))
                self.filter0.set_name('Sound files')
                self.filter1.set_name('Image files') 
                self.filter2.set_name('ICC Profile')
                self.combobox1.handler_block(self.id50)
                self.combobox1.set_active(0)
                self.combobox1.handler_unblock(self.id50)
                self.togglebutton1.handler_block(self.id63)
                self.togglebutton1.set_active(True)
                self.togglebutton1.handler_unblock(self.id63)
                self.combobox5.handler_block(self.id42)
                self.combobox5.set_active(0)
                self.combobox5.handler_unblock(self.id42)
                self.combobox6.set_active(0)
                self.combobox7.set_active(0)
                self.combobox8.set_active(0)
                self.combobox9.set_active(0)
                self.combobox11.set_active(0)
                self.combobox12.set_active(1)
                self.combobox13.set_active(0)
                self.combobox18.set_active(0)
                self.combobox16.set_active(0)
                pulse_splash(splash_pb, 0.9)
                self.combobox15.set_active(0)
                self.combobox19.set_active(0)
                self.combobox30.handler_block(self.id95)
                self.combobox36.handler_block(self.id104)
                self.combobox37.handler_block(self.id105)
                self.combobox30.set_active(0)
                self.combobox36.set_active(0)
                self.combobox37.set_active(0)
                pulse_splash(splash_pb, 0.95)
                self.combobox36.handler_unblock(self.id104)
                self.combobox37.handler_unblock(self.id105)
                self.combobox30.handler_unblock(self.id95)
                self.togglebutton6.handler_block(self.id97)
                self.togglebutton6.set_active(True)
                self.image92.set_from_stock(gtk.STOCK_CONNECT, gtk.ICON_SIZE_BUTTON)
                self.image92.show()
                self.togglebutton6.set_tooltip_text(_('Disconnect GUI'))
                self.togglebutton6.handler_unblock(self.id97)
                self.spinbutton30.handler_block(self.id77)
                self.spinbutton38.handler_block(self.id78)
                self.reset_curve(None, False)
                warnings.simplefilter('ignore', numpy.RankWarning)
                if not has_lcms:
                        self.checkbutton9.set_sensitive(False)
                        self.checkbutton10.set_sensitive(False)
                        self.filechooserbutton4.set_sensitive(False)
                        self.filechooserbutton5.set_sensitive(False)
                if dcraw_version >= "8.94":
                        self.radiobutton11.set_sensitive(True)
                if not cdrecord and not mkisofs:
                        self.menuitem33.set_sensitive(False)
                else:
                        self.scanbus()
                self.boxmodel14.append(['None'])
                self.boxmodel14.append(['Template1'])
                for key in sorted(self.templates.iterkeys()):
                        if key not in ['None', 'Template1']:
                                self.boxmodel14.append([key])
                self.combobox14.set_active(0)
                self.combobox20.set_active(0)
                self.combobox23.set_active(0)
                self.curve2.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse('#ffc8d2'))
                self.curve3.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse('#d2ffd2'))
                self.curve4.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse('#c8c8ff'))
                self.image85.set_from_pixbuf(gtk.gdk.pixbuf_new_from_file(self.get_icon_path('stock-color-picker-black-18.png')))
                self.image86.set_from_pixbuf(gtk.gdk.pixbuf_new_from_file(self.get_icon_path('stock-color-picker-gray-18.png')))
                self.image87.set_from_pixbuf(gtk.gdk.pixbuf_new_from_file(self.get_icon_path('stock-color-picker-white-18.png')))
                self.image85.show()
                self.image86.show()
                self.image87.show()
                if "lcms" not in DELEGATES:
                        self.toolbutton12.set_sensitive(False)
                if not littlecms:
                        self.toolbutton12.set_sensitive(False)
                        self.button166.set_sensitive(False)
                        self.button167.set_sensitive(False)
                        self.button168.set_sensitive(False)
                if self.checkbutton41.get_active() and self.combobox4.get_active() == 1:
                        self.combobox4.handler_block(self.id62)
                        self.combobox4.set_active(0)
                        self.combobox4.handler_unblock(self.id62)
                self.entry26.set_sensitive(self.checkbutton35.get_active())
                self.entry27.set_sensitive(self.checkbutton35.get_active())
                self.toolbutton25.set_icon_widget(gtk.image_new_from_stock(gtk.STOCK_INDENT, gtk.ICON_SIZE_BUTTON))
                self.toolbutton25.show_all()
                self.toolbutton25.set_tooltip_text('Hide Sidebar')
                self.hide_exif_panel = False
                self.hscale28.get_adjustment().value = 2.22
                self.spinbutton45.set_value(2.22)
                self.hscale34.get_adjustment().value = 4.5
                self.spinbutton42.set_value(4.5)
                self.hscale3.get_adjustment().value = 1.0
                self.spinbutton11.set_value(1.0)
                self.hscale14.get_adjustment().value = 50
                self.spinbutton22.set_value(50)

        def load_batch_database(self):
                if os.path.isfile(os.path.join(self.ark, 'batch.db')):
                        f = open(os.path.join(self.ark, 'batch.db'))
                        self.batch_list = cPickle.load(f)
                        f.close()
                        self.batch_queue = self.batch_list[1]
                        for item in self.batch_list[0]:
                                self.batchmodel.append([item[0], gtk.gdk.pixbuf_new_from_file(os.path.join(self.batch_thumbs, \
                                                        os.path.basename(item[2]) + '.jpg')), item[1], item[2], item[3]])
                else:
                        self.batch_list = [0,0]

        def load_mtimes_database(self):
                if os.path.isfile(os.path.join(self.ark, 'thumbs_mtimes.db')):
                        f = open(os.path.join(self.ark, 'thumbs_mtimes.db'))
                        self.mtimes = cPickle.load(f)
                        f.close()
                else:
                        self.mtimes = {}

        def load_templates(self):
                if os.path.isfile(os.path.join(self.ark, 'templates.db')):
                        f = open(os.path.join(self.ark, 'templates.db'))
                        self.templates = cPickle.load(f)
                        f.close()
                else:
                        self.templates = {'None' : [['','','']], 'Template1' : [['XMP-dc', 'Creator', ''],
                                        ['IPTC', 'Keywords', ''], ['EXIF', 'Copyright', ''], ['EXIF', 'UserComment', '']]}
                    
        def load_presets_database(self):
                for key in sorted(self.presets.iterkeys()):
                        self.boxmodel31.append([key])
                        self.boxmodel30.append([key])
                else:
                        self.boxmodel31.append([None])
                        
        def load_settings_database(self):
                if os.path.isfile(os.path.join(self.ark, 'settings')):
                        f = open(os.path.join(self.ark, 'settings'))
                        self.settings = cPickle.load(f)
                        f.close()
                        if self.settings.has_key('presets'):
                                self.presets = self.settings['presets']
                                self.load_presets_database()
                        else:
                                self.boxmodel31.append([None])
                        if self.settings.has_key('server'):
                                self.entry24.set_text(self.settings['server'])
                        if self.settings.has_key('port'):
                                self.entry25.set_text(self.settings['port'])
                        if self.settings.has_key('user'):
                                self.entry26.set_text(self.settings['user'])
                        if self.settings.has_key('password'):
                                self.entry27.set_text(self.settings['password'])
                        if self.settings.has_key('from_addr'):
                                self.entry28.set_text(self.settings['from_addr'])
                        if self.settings.has_key('rcpt_to'):
                                self.entry29.set_text(self.settings['rcpt_to'])
                        if self.settings.has_key('authenticate'):
                                self.checkbutton35.set_active(self.settings['authenticate'])
                        if self.settings.has_key('favorite_tags'):
                                self.favorite_tags = self.settings['favorite_tags']
                        else:
                                self.favorite_tags = self.default_tags[:]
                        if self.settings.has_key('window1_size'):
                                window1_size = self.settings['window1_size']
                                if window1_size[0] == self.screen0.get_width() and window1_size[1] > self.screen0.get_height() - 50:
                                        self.window1.maximize()
                                else:
                                        self.window1.resize(window1_size[0], window1_size[1])
                        else:
                                self.window1.maximize()
                        if self.settings.has_key('gallery_color'):
                                self.gallery_color = gtk.gdk.Color(self.settings['gallery_color'])      
                                self.colorbutton1.set_color(self.gallery_color)
                        else:
                                self.gallery_color = self.colorbutton1.get_color()
                        if self.settings.has_key('label_color'):
                                self.label_color = gtk.gdk.Color(self.settings['label_color'])  
                                self.colorbutton2.set_color(self.label_color)
                        else:
                                self.label_color = self.colorbutton2.get_color()
                        if self.settings.has_key('treeview_color'):
                                self.treeview_color = gtk.gdk.Color(self.settings['treeview_color'])
                                self.colorbutton3.set_color(self.treeview_color)
                        else:
                                self.treeview_color = self.colorbutton3.get_color()
                        if self.settings.has_key('toolbar_color'):
                                self.toolbar_color = gtk.gdk.Color(self.settings['toolbar_color'])
                                self.colorbutton4.set_color(self.toolbar_color)
                        else:
                                self.toolbar_color = self.colorbutton4.get_color()
                        if self.settings.has_key('selection_color'):
                                self.selection_color = gtk.gdk.Color(self.settings['selection_color'])
                                self.colorbutton5.set_color(self.selection_color)
                        else:
                                self.selection_color = self.colorbutton5.get_color()
                        if self.settings.has_key('image_background_color'):
                                self.image_background_color = gtk.gdk.Color(self.settings['image_background_color'])
                                self.colorbutton8.set_color(self.image_background_color)
                        else:
                                self.image_background_color = self.colorbutton8.get_color()
                        if self.settings.has_key('adj_value'):
                                self.adj.set_value(self.settings['adj_value'])
                        else:
                                self.adj.set_value(2)
                                self.settings['adj_value'] = 2
                        self.pause = self.settings['adj_value']
                        if self.settings.has_key('jpg_quality'):
                                self.jpg_quality.set_value(self.settings['jpg_quality'])
                        if self.settings.has_key('border_color'):
                                self.border_color = self.settings['border_color']
                                if self.border_color == 0xffffffff:
                                        self.combobox27.set_active(0)
                                elif self.border_color == 0x00000000:
                                        self.combobox27.set_active(2)
                                else:
                                        self.combobox27.set_active(1)
                        if self.settings.has_key('frames'):
                                self.frames = self.settings['frames']
                                self.spinbutton2.set_value(self.frames)
                        if self.settings.has_key('radiobutton8') and self.settings['radiobutton8']:
                                self.radiobutton8.set_active(True)
                                self.width = 100
                                self.height = 67
                        if self.settings.has_key('radiobutton1') and self.settings['radiobutton1']:
                                self.radiobutton1.set_active(True)
                                self.width = 150
                                self.height = 100
                        if self.settings.has_key('radiobutton2') and self.settings['radiobutton2']:
                                self.radiobutton2.set_active(True)
                                self.width = 201
                                self.height = 134
                        if self.settings.has_key('radiobutton3') and self.settings['radiobutton3']:
                                self.radiobutton3.set_active(True)
                                self.width = 260
                                self.height = 174
                        if self.settings.has_key('radiobutton6') and self.settings['radiobutton6']:
                                self.radiobutton6.set_active(True)
                                self.toolbuttons(False, True)
                        elif self.settings.has_key('radiobutton7') and self.settings['radiobutton7']:
                                self.radiobutton7.set_active(True)
                                self.toolbuttons(True, False)
                        if self.settings.has_key('checkbutton2') and self.settings['checkbutton2']:
                                self.checkbutton2.set_active(True)
                        if self.settings.has_key('checkbutton3') and self.settings['checkbutton3']:
                                self.checkbutton3.set_active(True)
                        if self.settings.has_key('checkbutton4') and self.settings['checkbutton4']:
                                self.checkbutton4.set_active(True)
                        if self.settings.has_key('checkbutton5') and self.settings['checkbutton5']:
                                self.checkbutton5.set_active(True)
                        if self.settings.has_key('checkbutton6') and self.settings['checkbutton6']:
                                self.checkbutton6.set_active(True)
                        else:
                                self.checkbutton6.set_active(False)
                        if self.settings.has_key('checkbutton7') and self.settings['checkbutton7']:
                                self.checkbutton7.set_active(True)
                        else:
                                self.checkbutton7.set_active(False)
                        if self.settings.has_key('checkbutton17') and self.settings['checkbutton17']:
                                self.checkbutton17.set_active(True)
                        else:
                                self.checkbutton17.set_active(False)
                        if self.settings.has_key('checkbutton28') and self.settings['checkbutton28']:
                                self.checkbutton28.set_active(True)
                        if self.settings.has_key('checkbutton33') and self.settings['checkbutton33']:
                                self.checkbutton33.set_active(True)
                        if self.settings.has_key('checkbutton360') and self.settings['checkbutton360']:
                                self.checkbutton360.set_active(True)
                        self.ark = self.settings['ark']
                        self.filechooserbutton1.set_current_folder(self.ark)
                        self.entry4.set_text(self.ark)
                        if self.settings.has_key('filechooserbutton7'):
                                self.filechooserbutton7.set_current_folder(self.settings['filechooserbutton7'])
                        else:   
                                self.filechooserbutton7.set_current_folder(os.path.expanduser('~'))
                                self.settings['filechooserbutton7'] = os.path.expanduser('~')
                        if self.settings.has_key('combobox10'): 
                                self.combobox10.set_active(self.settings['combobox10'])
                        else:                           
                                self.combobox10.set_active(0)
                                self.settings['combobox10'] = 0
                        if self.settings.has_key('hscale23'):   
                                self.hscale23.get_adjustment().value = self.settings['hscale23']
                        else: 
                                self.hscale23.get_adjustment().value = 80
                                self.settings['hscale23'] = 80
                        if self.settings.has_key('sample') and self.settings['sample']:
                                self.sample = self.settings['sample']
                                self.filechooserbutton3.set_filename(self.sample)
                        if self.settings.has_key('delay'):
                                self.delay = self.settings['delay']
                                self.spinbutton3.set_value(self.delay)
                        if self.settings.has_key('depth'):
                                self.radiobutton11.handler_block(self.id52)
                                if self.settings['depth'] == 8:
                                        self.radiobutton10.set_active(True)
                                else:
                                        if dcraw_version >= "8.94":
                                                self.radiobutton11.set_active(True)
                                self.radiobutton11.handler_unblock(self.id52)
                        else:
                                if dcraw_version >= "8.94":
                                        self.settings['depth'] = 16
                                        self.radiobutton11.handler_block(self.id52)
                                        self.radiobutton11.set_active(True)
                                        self.radiobutton11.handler_unblock(self.id52)
                        if self.settings.has_key('-f') and self.settings['-f']:
                                self.checkbutton8.handler_block(self.id53)
                                self.checkbutton8.set_active(True)
                                self.checkbutton8.handler_unblock(self.id53)
                        self.combobox2.handler_block(self.id60)
                        self.combobox2.set_active(0)
                        self.combobox2.handler_unblock(self.id60)
                        self.combobox3.handler_block(self.id61)
                        self.combobox3.set_active(0)
                        self.combobox3.handler_unblock(self.id61)
                        self.combobox39.handler_block(self.id108)
                        self.combobox39.set_active(0)
                        self.combobox39.handler_unblock(self.id108)
                        if self.settings.has_key('output_colorspace'):
                                self.combobox4.handler_block(self.id62)
                                self.combobox4.set_active(self.settings['output_colorspace'])
                                self.combobox4.handler_unblock(self.id62)
                        if self.settings.has_key('camera_profile') and self.settings['camera_profile']:
                                self.checkbutton9.handler_block(self.id54)
                                self.checkbutton9.set_active(True)
                                self.checkbutton9.handler_unblock(self.id54)
                        if self.settings.has_key('camera_ICC') and self.settings['camera_ICC']:
                                self.filechooserbutton4.handler_block(self.id20)
                                self.filechooserbutton4.set_filename(self.settings['camera_ICC'])
                                self.filechooserbutton4.handler_unblock(self.id20)
                        if self.settings.has_key('output_profile') and self.settings['output_profile']:
                                self.checkbutton10.handler_block(self.id55)
                                self.checkbutton10.set_active(True)
                                self.checkbutton10.handler_unblock(self.id55)
                        if self.settings.has_key('output_ICC') and self.settings['output_ICC']:
                                self.filechooserbutton5.handler_block(self.id21)
                                self.filechooserbutton5.set_filename(self.settings['output_ICC'])
                                self.filechooserbutton5.handler_unblock(self.id21)
                        if self.settings.has_key('+M') and self.settings['+M']:
                                self.checkbutton11.handler_block(self.id56)
                                self.checkbutton11.set_active(True)
                                self.checkbutton11.handler_unblock(self.id56)
                        if self.settings.has_key('dark_frame') and self.settings['dark_frame']:
                                self.checkbutton13.handler_block(self.id58)
                                self.checkbutton13.set_active(True)
                                self.checkbutton13.handler_unblock(self.id58)
                        if self.settings.has_key('dark_filename') and self.settings['dark_filename']:
                                self.filechooserbutton6.set_filename(self.settings['dark_filename'])
                        if self.settings.has_key('-j') and self.settings['-j']:
                                self.checkbutton14.handler_block(self.id59)
                                self.checkbutton14.set_active(True)
                                self.checkbutton14.handler_unblock(self.id59)
                        if self.settings.has_key('-h') and self.settings['-h']:
                                self.checkbutton16.handler_block(self.id67)
                                self.checkbutton16.set_active(True)
                                self.checkbutton16.handler_unblock(self.id67)
                        if self.settings.has_key('-D') and self.settings['-D']:
                                self.checkbutton1.handler_block(self.id96)
                                self.checkbutton1.set_active(True)
                                self.checkbutton1.handler_unblock(self.id96)
                        if self.settings.has_key('tags') and self.settings['tags']:
                                self.populate_tag_managers()
                        self.combobox33.handler_block(self.id101)
                        self.combobox34.handler_block(self.id102)
                        self.combobox35.handler_block(self.id103)
                        self.combobox36.handler_block(self.id104)
                        self.combobox37.handler_block(self.id105)
                        self.combobox38.handler_block(self.id106)
                        if self.settings.has_key('in_profile_icc'):
                                self.input_profile_name = self.settings['in_profile_icc']
                                if self.input_profile_name == 'sRGB':
                                        self.combobox33.set_active(0)
                                        self.input_profile = self.sRGB_profile
                                elif self.input_profile_name == 'AdobeRGB 1998':
                                        self.combobox33.set_active(1)
                                        self.input_profile = self.AdobeRGB1998_profile
                                else:
                                        if os.path.isfile(self.input_profile_name):
                                                self.boxmodel33.append([os.path.basename(self.input_profile_name)])
                                                self.combobox33.set_active(2)
                                                self.input_profile = numpy.fromfile(self.input_profile_name, 'u1')
                                        else:
                                                self.combobox33.set_active(0)
                                                self.input_profile = self.sRGB_profile
                                                self.input_profile_name = 'sRGB'
                        else:
                                self.combobox33.set_active(0)
                                self.input_profile_name = 'sRGB'
                                self.input_profile = self.sRGB_profile
                        if self.settings.has_key('out_profile_icc'):
                                self.output_profile_name = self.settings['out_profile_icc']
                                if self.output_profile_name == 'sRGB':
                                        self.combobox34.set_active(0)
                                        self.output_profile = self.sRGB_profile
                                elif self.output_profile_name == 'AdobeRGB 1998':
                                        self.combobox34.set_active(1)
                                        self.output_profile = self.AdobeRGB1998_profile
                                else:
                                        if os.path.isfile(self.output_profile_name):
                                                self.boxmodel34.append([os.path.basename(self.output_profile_name)])
                                                self.combobox34.set_active(2)
                                                self.output_profile = numpy.fromfile(self.output_profile_name, 'u1')
                                        else:
                                                self.combobox34.set_active(0)
                                                self.output_profile = self.sRGB_profile
                                                self.output_profile_name = 'sRGB'
                        else:
                                self.combobox34.set_active(0)
                                self.output_profile_name = 'sRGB'
                                self.output_profile = self.sRGB_profile
                        if self.settings.has_key('display_profile_icc'):
                                self.display_profile_name = self.settings['display_profile_icc']
                                if self.display_profile_name == 'sRGB':
                                        self.combobox35.set_active(0)
                                        self.display_profile = self.sRGB_profile
                                else:
                                        if os.path.isfile(self.display_profile_name):
                                                self.boxmodel35.append([os.path.basename(self.display_profile_name)])
                                                self.combobox35.set_active(1)
                                                self.display_profile = numpy.fromfile(self.display_profile_name, 'u1')
                                        else:
                                                self.combobox35.set_active(0)
                                                self.display_profile = self.sRGB_profile
                                                self.display_profile_name = 'sRGB'
                        else:
                                self.combobox35.set_active(0)
                                self.display_profile_name = 'sRGB'
                                self.display_profile = self.sRGB_profile
                        if self.settings.has_key('embedded_profile'):
                                self.combobox38.set_active(self.settings['embedded_profile'])
                                self.embedded_profile_behaviour = self.settings['embedded_profile']
                        else:
                                self.combobox38.set_active(2)
                                self.settings['embedded_profile'] = self.embedded_profile_behaviour = 2
                        if self.settings.has_key('cms') and self.settings['cms']:
                                self.checkbutton41.set_active(self.settings['cms'])
                                self.checkbutton9.set_sensitive(not self.settings['cms'])
                                self.checkbutton10.set_sensitive(not self.settings['cms'])
                                self.combobox4.handler_block(self.id62)
                                self.combobox4.set_active(2)
                                self.combobox4.handler_unblock(self.id62)
                        self.combobox33.handler_unblock(self.id101)
                        self.combobox34.handler_unblock(self.id102)
                        self.combobox35.handler_unblock(self.id103)
                        self.combobox36.handler_unblock(self.id104)
                        self.combobox37.handler_unblock(self.id105)
                        self.combobox38.handler_unblock(self.id106)
                        
                else:
                        self.settings = {}
                        self.default_settings()
                
        def default_settings(self, event=None):
                self.disconnect_dcraw_gui()
                self.window1.move(0, 0)
                self.hide_exif_panel = False
                self.toolbutton25.set_icon_widget(gtk.image_new_from_stock(gtk.STOCK_INDENT, gtk.ICON_SIZE_BUTTON))
                self.toolbutton25.show_all()
                self.toolbutton25.set_tooltip_text('Hide Sidebar')
                self.settings['jpg_quality'] = self.jpg_quality.value = 75
                self.settings['checkbutton2'] = False
                self.settings['checkbutton3'] = True
                self.settings['checkbutton4'] = False
                self.settings['checkbutton5'] = True
                self.settings['checkbutton6'] = False
                self.settings['checkbutton7'] = False
                self.settings['checkbutton17'] = False
                self.settings['checkbutton28'] = False
                self.settings['checkbutton33'] = False
                self.settings['radiobutton1'] = True
                self.settings['radiobutton2'] = False
                self.settings['radiobutton3'] = False
                self.settings['radiobutton8'] = False
                self.settings['radiobutton6'] = True   
                self.settings['hide_exif'] = False
                self.settings['cms'] = False
                self.settings['ark'] = self.ark
                if dcraw_version >= "8.94":
                        self.settings['depth'] = 16
                        self.radiobutton11.handler_block(self.id52)
                        self.radiobutton11.set_active(True)
                        self.radiobutton11.handler_unblock(self.id52)
                self.radiobutton1.set_active(True)
                self.radiobutton6.set_active(True)
                self.radiobutton7.set_active(False)
                self.checkbutton2.set_active(False)
                self.checkbutton3.set_active(True)
                self.checkbutton4.set_active(False)
                self.checkbutton5.set_active(True)
                self.checkbutton6.set_active(False)
                self.checkbutton17.set_active(False)
                self.checkbutton28.set_active(False)
                self.checkbutton33.set_active(False)
                self.checkbutton360.set_active(False)
                self.checkbutton41.handler_block(self.id100)
                self.checkbutton41.set_active(False)
                self.checkbutton41.handler_unblock(self.id100)
                self.default_colors()
                self.gallery_color = self.colorbutton1.get_color()
                self.label_color = self.colorbutton2.get_color()
                self.treeview_color = self.colorbutton3.get_color()
                self.toolbar_color = self.colorbutton4.get_color()
                self.selection_color = self.colorbutton5.get_color()
                self.image_background_color = self.colorbutton8.get_color()
                self.settings['border_color'] = 0xffffffff
                self.combobox27.set_active(0)
                self.frames = 25
                self.delay = 0.00
                self.sample = None
                self.settings['frames'] = 25
                self.settings['sample'] = self.sample
                self.settings['delay'] = 0.00
                self.spinbutton3.set_value(0.00)
                self.spinbutton2.set_value(self.frames)
                self.adj.set_value(2)
                self.filechooserbutton1.set_current_folder(self.ark)
                self.entry9.set_text('50')
                self.settings['window1_size'] = self.screen0.get_width(), self.screen0.get_height()
                self.filechooserbutton7.set_current_folder(os.path.expanduser('~'))
                self.settings['filechooserbutton7'] = os.path.expanduser('~')
                self.combobox10.set_active(0)
                self.settings['combobox10'] = 0
                self.hscale23.get_adjustment().value = 80
                self.settings['hscale23'] = 80
                self.combobox2.set_active(0)
                self.combobox3.set_active(0)
                self.combobox4.set_active(1)
                self.combobox5.set_active(0)
                self.combobox39.handler_block(self.id108)
                self.combobox39.set_active(0)
                self.combobox39.handler_unblock(self.id108)
                self.combobox33.handler_block(self.id101)
                self.combobox34.handler_block(self.id102)
                self.combobox35.handler_block(self.id103)
                self.combobox33.set_active(0)
                self.combobox34.set_active(0)
                self.combobox35.set_active(0)
                self.combobox33.handler_unblock(self.id101)
                self.combobox34.handler_unblock(self.id102)
                self.combobox35.handler_unblock(self.id103)
                self.combobox30.handler_block(self.id95)
                self.combobox36.handler_block(self.id104)
                self.combobox37.handler_block(self.id105)
                self.combobox38.handler_block(self.id106)
                self.combobox30.set_active(0)
                self.combobox36.set_active(0)
                self.combobox37.set_active(0)
                self.combobox38.set_active(2)
                self.combobox36.handler_unblock(self.id104)
                self.combobox37.handler_unblock(self.id105)
                self.combobox38.handler_unblock(self.id106)
                self.combobox30.handler_unblock(self.id95)
                self.checkbutton11.set_active(True)
                self.checkbutton16.set_active(True)
                self.checkbutton35.set_active(False) 
                self.settings['tags'] = ['Family', 'Portraits', 'Panoramics', 'Macro', 'Sport', 'Nature']
                self.favorite_tags = self.default_tags[:]
                self.entry25.set_text('25')
                self.input_profile_name = 'sRGB'
                self.input_profile = self.sRGB_profile
                self.output_profile_name = 'sRGB'
                self.output_profile = self.sRGB_profile
                self.display_profile_name = 'sRGB'
                self.display_profile = self.sRGB_profile
                self.populate_tag_managers()
                self.connect_dcraw_gui()
                self.store_settings_db()
                
        def populate_tag_managers(self):
                for item in self.settings['tags']:
                        self.tagmodel.append([False, item])
                        self.tagmodel2.append([False, item])
                        self.tagmodel3.append([item])

        def load_album_database(self):
                if os.path.isfile(os.path.join(self.ark, 'albums.db')):
                        f = open(os.path.join(self.ark, 'albums.db'))
                        self.albums = cPickle.load(f)
                        f.close()
                        self.boxmodel3.append(['All Albums'])
                        for key in sorted(self.albums.iterkeys()):
                                try:
                                        self.model.append([gtk.gdk.pixbuf_new_from_file(os.path.join(self.ark, key, '_preview_.png')), key])
                                except:
                                        self.model.append([gtk.gdk.pixbuf_new_from_file(self.get_icon_path('no_preview.png')), key])
                                self.boxmodel.append([key])
                                self.boxmodel3.append([key])
                else:
                        self.albums = {}
                    
        def setup_signal_handlers(self):
                self.window1.connect('delete_event', self.destroy)
                self.toolbutton2.connect('clicked', self.first_photo)
                self.toolbutton3.connect('clicked', self.last_photo)
                self.toolbutton4.connect('clicked', self.previous_photo)
                self.toolbutton5.connect('clicked', self.next_photo)
                self.toolbutton12.connect('clicked', self.show_color_manager)#1914
                self.toolbutton44.connect('clicked', self.add_to_batch_queue)
                self.toolbutton45.connect('clicked', self.delete_image_from_window2)
                self.toolbutton46.connect('clicked', self.show_batch_queue)
                self.treeview2.connect('row_expanded', self.expand_rows)
                self.treeview2.connect('row_collapsed', self.collapse_rows)
                self.add_to_album1.connect('activate', self.add_selection)
                self.open_image.connect('activate', self.display_photo)
                self.menuitem36.connect('activate', self.open_with_gimp_)
                self.menuitem37.connect('activate', self.open_with_other_)
                self.menuitem9.connect('activate', self.open_with_gimp_)
                self.menuitem10.connect('activate', self.open_with_other_)
                self.menuitem13.connect('activate', self.hide_pane)
                self.convert.connect('activate', self.show_convert)
                self.about1.connect('activate', self.about_gtkrawgallery)
                self.exit2.connect('activate', self.destroy)
                self.rename1.connect('activate', self.rename_album)
                self.delete3.connect('activate', self.Delete)
                self.menuitem5.connect('activate', self.get_options)
                self.window1.connect('window_state_event', self.window_state_event)
                self.window2.connect('window_state_event', self.window_state_event)
                self.button5.connect('clicked', self.stop_thumbnails)
                self.save_as.connect('activate', self.save_as_dialog)
                self.win6_okbutton.connect('clicked', self.multiple_conversion)
                self.win6_cancelbutton.connect('clicked', self.hide_win6)
                self.toolbutton24.connect('toggled', self.full_exif)
                self.toolbutton25.connect('clicked', self.hide_exif)
                self.toolbutton37.connect('clicked', self.save_as_dialog)
                self.menuitem6.connect('activate', self.open_with_gimp_)
                self.imagemenuitem5.connect('activate', self.hide_pane, True)
                self.imagemenuitem8.connect('activate', self.add_selection)
                self.imagemenuitem10.connect('activate', self.save_as_dialog)
                self.imagemenuitem19.connect('activate', self.set_album_preview)
                self.imagemenuitem20.connect('activate', self.set_album_preview)
                self.menuitem22.connect('activate', self.show_tag_manager)
                self.menuitem24.connect('activate', self.show_tag_manager)
                self.menuitem25.connect('activate', self.show_tag_manager)
                self.menuitem35.connect('activate', self.show_error_console)
                self.button4.connect('clicked', self.default_colors)
                self.button22.connect('clicked', self.clear_cache)
                self.notebook5.connect('switch_page', self.on_notebook5_switched)
                self.combobox5.connect('changed', self.wb_changed)
                self.combobox6.connect('changed', self.on_combobox6_changed)
                self.combobox8.connect('changed', self.on_combobox8_changed)
                self.combobox10.connect('changed', self.on_combobox10_changed)
                self.combobox13.connect('changed', self.on_combobox13_changed)
                self.combobox14.connect('changed', self.on_combobox20_changed)
                self.combobox18.connect('changed', self.on_combobox18_changed)
                self.combobox19.connect('changed', self.on_combobox18_changed)
                self.combobox20.connect('changed', self.on_combobox20_changed)
                self.combobox25.connect('changed', self.on_combobox25_changed)
                self.combobox31.connect('changed', self.on_combobox31_changed)
                self.button17.connect('clicked', self.set_default_dcraw_options)
                self.hscale3.connect('scroll_event', self.on_hscale_scrolled)
                self.hscale4.connect('scroll_event', self.on_hscale_scrolled)
                self.hscale5.connect('scroll_event', self.on_hscale_scrolled)
                self.hscale6.connect('scroll_event', self.on_hscale_scrolled)
                self.hscale7.connect('scroll_event', self.on_hscale_scrolled)
                self.hscale8.connect('scroll_event', self.on_hscale_scrolled)
                self.hscale9.connect('scroll_event', self.on_hscale_scrolled)
                self.hscale10.connect('scroll_event', self.on_hscale_scrolled)
                self.hscale11.connect('scroll_event', self.on_hscale_scrolled)
                self.hscale12.connect('scroll_event', self.on_hscale_scrolled)
                self.hscale13.connect('scroll_event', self.on_hscale_scrolled)
                self.hscale14.connect('scroll_event', self.on_hscale_scrolled)
                self.hscale15.connect('scroll_event', self.on_hscale_scrolled)
                self.hscale16.connect('scroll_event', self.on_hscale_scrolled)
                self.hscale17.connect('scroll_event', self.on_hscale_scrolled)
                self.hscale18.connect('scroll_event', self.on_hscale_scrolled)
                self.hscale19.connect('scroll_event', self.on_hscale_scrolled)
                self.hscale20.connect('scroll_event', self.on_hscale_scrolled)
                self.hscale21.connect('scroll_event', self.on_hscale_scrolled)
                self.hscale22.connect('scroll_event', self.on_hscale_scrolled)
                self.hscale24.connect('scroll_event', self.on_hscale_scrolled)
                self.hscale25.connect('scroll_event', self.on_hscale_scrolled)
                self.hscale26.connect('scroll_event', self.on_hscale_scrolled)
                self.hscale27.connect('scroll_event', self.on_hscale_scrolled)
                self.hscale28.connect('scroll_event', self.on_hscale_scrolled)
                self.hscale29.connect('scroll_event', self.on_hscale_scrolled)
                self.hscale30.connect('scroll_event', self.on_hscale_scrolled)
                self.hscale31.connect('scroll_event', self.on_hscale_scrolled)
                self.hscale32.connect('scroll_event', self.on_hscale_scrolled)
                self.hscale33.connect('scroll_event', self.on_hscale_scrolled)
                self.hscale34.connect('scroll_event', self.on_hscale_scrolled)
                self.hscale35.connect('scroll_event', self.on_hscale_scrolled)
                self.hscale36.connect('scroll_event', self.on_hscale_scrolled)
                self.hscale37.connect('scroll_event', self.on_hscale_scrolled)
                self.hscale38.connect('scroll_event', self.on_hscale_scrolled)
                self.hscale39.connect('scroll_event', self.on_hscale_scrolled)
                self.hscale40.connect('scroll_event', self.on_hscale_scrolled)
                self.hscale41.connect('scroll_event', self.on_hscale_scrolled)
                self.hscale42.connect('scroll_event', self.on_hscale_scrolled)
                self.hscale43.connect('scroll_event', self.on_hscale_scrolled)
                self.hscale44.connect('scroll_event', self.on_hscale_scrolled)
                self.hscale45.connect('scroll_event', self.on_hscale_scrolled)
                self.hscale46.connect('scroll_event', self.on_hscale_scrolled)
                self.hscale47.connect('scroll_event', self.on_hscale_scrolled)
                self.hscale48.connect('scroll_event', self.on_hscale_scrolled)
                self.hscale49.connect('scroll_event', self.on_hscale_scrolled)
                self.hscale50.connect('scroll_event', self.on_hscale_scrolled)
                self.hscale51.connect('scroll_event', self.on_hscale_scrolled)
                self.hscale52.connect('scroll_event', self.on_hscale_scrolled)
                self.hscale53.connect('scroll_event', self.on_hscale_scrolled)
                self.combobox2.connect('scroll_event', self.on_hscale_scrolled)
                self.combobox3.connect('scroll_event', self.on_hscale_scrolled)
                self.combobox4.connect('scroll_event', self.on_hscale_scrolled)
                self.combobox5.connect('scroll_event', self.on_hscale_scrolled)
                self.combobox6.connect('scroll_event', self.on_hscale_scrolled)
                self.combobox8.connect('scroll_event', self.on_hscale_scrolled)
                self.combobox11.connect('scroll_event', self.on_hscale_scrolled)
                self.combobox12.connect('scroll_event', self.on_hscale_scrolled)
                self.combobox13.connect('scroll_event', self.on_hscale_scrolled)
                self.hscale4.connect('button_release_event', self.on_level_changed)
                self.hscale5.connect('button_release_event', self.on_level_changed)
                self.hscale6.connect('button_release_event', self.on_level_changed)
                self.hscale7.connect('button_release_event', self.on_brightness_and_saturation_changed)
                self.hscale8.connect('button_release_event', self.on_brightness_and_saturation_changed)
                self.hscale18.connect('button_release_event', self.on_brightness_and_saturation_changed)
                self.hscale9.connect('button_release_event', self.color_balance)
                self.hscale10.connect('button_release_event', self.color_balance)
                self.hscale11.connect('button_release_event', self.color_balance)
                self.hscale12.connect('button_release_event', self.on_contrast_changed)
                self.hscale13.connect('button_release_event', self.on_midtones_contrast_changed)
                self.hscale14.connect('button_release_event', self.on_midtones_contrast_changed)
                self.hscale31.connect('button_release_event', self.on_channel_mixer_changed)
                self.hscale32.connect('button_release_event', self.on_channel_mixer_changed)
                self.hscale33.connect('button_release_event', self.on_channel_mixer_changed)
                self.hscale38.connect('button_release_event', self.apply_curve)
                self.hscale39.connect('button_release_event', self.apply_curve)
                self.hscale40.connect('button_release_event', self.apply_curve)
                self.hscale41.connect('button_release_event', self.apply_curve)
                self.hscale50.connect('button_release_event', self.apply_colortemp)
                self.hscale51.connect('button_release_event', self.apply_colortemp)
                self.spinbutton1.connect('scroll_event', self.on_spinbutton_scrolled)
                self.spinbutton4.connect('scroll_event', self.on_spinbutton_scrolled)
                self.spinbutton5.connect('scroll_event', self.on_spinbutton_scrolled)
                self.spinbutton6.connect('scroll_event', self.on_spinbutton_scrolled)
                self.spinbutton7.connect('scroll_event', self.on_spinbutton_scrolled)
                self.spinbutton8.connect('scroll_event', self.on_spinbutton_scrolled)
                self.spinbutton9.connect('scroll_event', self.on_spinbutton_scrolled)
                self.spinbutton10.connect('scroll_event', self.on_spinbutton_scrolled)
                self.spinbutton11.connect('scroll_event', self.on_spinbutton_scrolled)
                self.spinbutton12.connect('scroll_event', self.on_spinbutton_scrolled)
                self.spinbutton13.connect('scroll_event', self.on_spinbutton_scrolled)
                self.spinbutton14.connect('scroll_event', self.on_spinbutton_scrolled)
                self.spinbutton15.connect('scroll_event', self.on_spinbutton_scrolled)
                self.spinbutton16.connect('scroll_event', self.on_spinbutton_scrolled)
                self.spinbutton17.connect('scroll_event', self.on_spinbutton_scrolled)
                self.spinbutton18.connect('scroll_event', self.on_spinbutton_scrolled)
                self.spinbutton19.connect('scroll_event', self.on_spinbutton_scrolled)
                self.spinbutton20.connect('scroll_event', self.on_spinbutton_scrolled)
                self.spinbutton21.connect('scroll_event', self.on_spinbutton_scrolled)
                self.spinbutton22.connect('scroll_event', self.on_spinbutton_scrolled)
                self.spinbutton23.connect('scroll_event', self.on_spinbutton_scrolled)
                self.spinbutton24.connect('scroll_event', self.on_spinbutton_scrolled)
                self.spinbutton25.connect('scroll_event', self.on_spinbutton_scrolled)
                self.spinbutton26.connect('scroll_event', self.on_spinbutton_scrolled)
                self.spinbutton27.connect('scroll_event', self.on_spinbutton_scrolled)
                self.spinbutton28.connect('scroll_event', self.on_spinbutton_scrolled)
                self.spinbutton29.connect('scroll_event', self.on_spinbutton_scrolled)
                self.spinbutton30.connect('scroll_event', self.on_spinbutton_scrolled)
                self.spinbutton31.connect('scroll_event', self.on_spinbutton_scrolled)
                self.spinbutton32.connect('scroll_event', self.on_spinbutton_scrolled)
                self.spinbutton33.connect('scroll_event', self.on_spinbutton_scrolled)
                self.spinbutton34.connect('scroll_event', self.on_spinbutton_scrolled)
                self.spinbutton35.connect('scroll_event', self.on_spinbutton_scrolled)
                self.spinbutton36.connect('scroll_event', self.on_spinbutton_scrolled)
                self.spinbutton37.connect('scroll_event', self.on_spinbutton_scrolled)
                self.spinbutton38.connect('scroll_event', self.on_spinbutton_scrolled)
                self.spinbutton39.connect('scroll_event', self.on_spinbutton_scrolled)
                self.spinbutton40.connect('scroll_event', self.on_spinbutton_scrolled)
                self.spinbutton41.connect('scroll_event', self.on_spinbutton_scrolled)
                self.spinbutton42.connect('scroll_event', self.on_spinbutton_scrolled)
                self.spinbutton43.connect('scroll_event', self.on_spinbutton_scrolled)
                self.spinbutton44.connect('scroll_event', self.on_spinbutton_scrolled)
                self.spinbutton45.connect('scroll_event', self.on_spinbutton_scrolled)
                self.spinbutton46.connect('scroll_event', self.on_spinbutton_scrolled)
                self.spinbutton47.connect('scroll_event', self.on_spinbutton_scrolled)
                self.spinbutton48.connect('scroll_event', self.on_spinbutton_scrolled)
                self.spinbutton49.connect('scroll_event', self.on_spinbutton_scrolled)
                self.spinbutton50.connect('scroll_event', self.on_spinbutton_scrolled)
                self.spinbutton51.connect('scroll_event', self.on_spinbutton_scrolled)
                self.spinbutton52.connect('scroll_event', self.on_spinbutton_scrolled)
                self.spinbutton53.connect('scroll_event', self.on_spinbutton_scrolled)
                self.spinbutton54.connect('scroll_event', self.on_spinbutton_scrolled)
                self.spinbutton55.connect('scroll_event', self.on_spinbutton_scrolled)
                self.spinbutton56.connect('scroll_event', self.on_spinbutton_scrolled)
                self.spinbutton57.connect('scroll_event', self.on_spinbutton_scrolled)
                self.spinbutton58.connect('scroll_event', self.on_spinbutton_scrolled)
                self.spinbutton59.connect('scroll_event', self.on_spinbutton_scrolled)
                self.spinbutton64.connect('scroll_event', self.on_spinbutton_scrolled)
                self.spinbutton65.connect('scroll_event', self.on_spinbutton_scrolled)
                self.spinbutton66.connect('scroll_event', self.on_spinbutton_scrolled)
                self.spinbutton67.connect('scroll_event', self.on_spinbutton_scrolled)
                self.spinbutton68.connect('scroll_event', self.on_spinbutton_scrolled)
                self.spinbutton69.connect('scroll_event', self.on_spinbutton_scrolled)
                self.spinbutton70.connect('scroll_event', self.on_spinbutton_scrolled)
                self.spinbutton71.connect('scroll_event', self.on_spinbutton_scrolled)
                self.spinbutton72.connect('scroll_event', self.on_spinbutton_scrolled)
                self.spinbutton73.connect('scroll_event', self.on_spinbutton_scrolled)
                self.spinbutton49.connect('value_changed', self.on_spinbutton49_value_changed)
                self.spinbutton50.connect('value_changed', self.on_spinbutton50_value_changed)
                self.spinbutton52.connect('value_changed', self.on_spinbutton52_value_changed)
                self.spinbutton54.connect('value_changed', self.on_spinbutton54_value_changed)
                self.spinbutton10.connect('button_release_event', self.on_color_spinbutton_released)
                self.spinbutton12.connect('button_release_event', self.on_color_spinbutton_released)
                self.spinbutton13.connect('button_release_event', self.on_color_spinbutton_released)
                self.spinbutton14.connect('button_release_event', self.on_color_spinbutton_released)
                self.spinbutton15.connect('button_release_event', self.on_color_spinbutton_released)
                self.spinbutton16.connect('button_release_event', self.on_color_spinbutton_released)
                self.spinbutton17.connect('button_release_event', self.on_color_spinbutton_released)
                self.spinbutton18.connect('button_release_event', self.on_color_spinbutton_released)
                self.spinbutton20.connect('button_release_event', self.on_color_spinbutton_released)
                self.spinbutton21.connect('button_release_event', self.on_color_spinbutton_released)
                self.spinbutton22.connect('button_release_event', self.on_color_spinbutton_released)
                self.spinbutton27.connect('button_release_event', self.on_color_spinbutton_released)
                self.spinbutton28.connect('button_release_event', self.on_color_spinbutton_released)
                self.spinbutton37.connect('button_release_event', self.on_color_spinbutton_released)
                self.spinbutton46.connect('button_release_event', self.on_color_spinbutton_released)
                self.spinbutton47.connect('button_release_event', self.on_color_spinbutton_released)
                self.spinbutton48.connect('button_release_event', self.on_color_spinbutton_released)
                self.spinbutton55.connect('button_release_event', self.on_color_spinbutton_released)
                self.spinbutton57.connect('button_release_event', self.on_color_spinbutton_released)
                self.spinbutton58.connect('button_release_event', self.on_color_spinbutton_released)
                self.spinbutton59.connect('button_release_event', self.on_color_spinbutton_released)
                self.spinbutton72.connect('button_release_event', self.on_color_spinbutton_released)
                self.spinbutton73.connect('button_release_event', self.on_color_spinbutton_released)
                self.toolbutton23.connect('clicked',self.undo_workflow)
                self.toolbutton39.connect('clicked', self.redo_workflow)
                self.toolbutton47.connect('clicked', self.on_toolbutton47_clicked)
                self.button25.connect('clicked', self.find_tagged)
                self.button26.connect('clicked', self.on_window9_delete_event)
                self.button33.connect('clicked', self.undo_handler)
                self.button34.connect('clicked', self.undo_handler)
                self.button36.connect('clicked', self.on_button36_clicked)
                self.button42.connect('clicked', self.on_button42_clicked)
                self.button45.connect('clicked', self.on_button45_clicked)
                self.button40.connect('clicked', self.show_dialog31)
                self.button54.connect('clicked', self.undo_handler)
                self.button55.connect('clicked', self.show_tagmanager_info)
                self.button57.connect('clicked', self.undo_handler)
                self.button58.connect('clicked', self.apply_sharpen)
                self.button61.connect('clicked', self.undo_handler)
                self.button63.connect('clicked', self.apply_noise)
                self.button64.connect('clicked', self.new_tag)
                self.button65.connect('clicked', self.undo_handler)
                self.button66.connect('clicked', self.scale)
                self.button67.connect('clicked', self.on_window8_delete_event)
                self.button68.connect('clicked', self.undo_handler)
                self.button69.connect('clicked', self.apply_img_border)
                self.button70.connect('clicked', self.apply_blur)
                self.button72.connect('clicked', self.undo_workflow)
                self.button74.connect('clicked', self.undo_handler)
                self.button75.connect('clicked', self.apply_reduce_noise)
                self.button76.connect('clicked', self.delete_tag)
                self.button77.connect('clicked', self.new_tag)
                self.toolbutton36.connect('clicked', self.save_workflow)
                self.button80.connect('clicked', self.undo_handler)
                self.button81.connect('clicked', self.apply_median_filter)
                self.button83.connect('clicked', self.undo_handler)
                self.button84.connect('clicked', self.apply_sepia_tone)
                self.button88.connect('clicked', self.clear_tags_database)
                self.button89.connect('clicked', self.undo_handler)
                self.button90.connect('clicked', self.grayscale)
                self.button91.connect('clicked', self.write_metadata)
                self.button92.connect('clicked', self.undo_handler)
                self.button93.connect('clicked', self.undo_workflow)
                self.button94.connect('clicked', self.undo_handler)
                self.button95.connect('clicked', self.apply_unsharp_mask)
                self.button97.connect('clicked', self.apply_crop)     
                self.button103.connect('clicked', self.apply_normalize)
                self.button110.connect('clicked', self.show_window15)
                self.button111.connect('clicked', self.undo_handler)
                self.button112.connect('clicked', self.apply_distort)
                self.button113.connect('clicked', self.reset_distort)
                self.button114.connect('clicked', self.undo_handler)
                self.button115.connect('clicked', self.undo_handler)
                self.button116.connect('clicked', self.apply_oil_paint)
                self.button117.connect('clicked', self.undo_handler)
                self.button118.connect('clicked', self.apply_despeckle)
                self.button119.connect('clicked', self.reset_curve)
                self.button121.connect('clicked', self.reset_curve)
                self.button122.connect('clicked', self.add_custom_tag)
                self.button123.connect('clicked', self.delete_template)
                self.button124.connect('clicked', self.clear_custom_tags)     
                self.button125.connect('clicked', self.stop_mail)       
                self.button126.connect('clicked', self.send_mail)       
                self.button127.connect('clicked', self.save_metadata_template)
                self.button128.connect('clicked', self.delete_template)
                self.button129.connect('clicked', self.save_metadata_template)
                self.button130.connect('clicked', self.add_custom_tag)
                self.button131.connect('clicked', self.clear_custom_tags)
                self.button132.connect('clicked', self.write_metadata)
                self.button133.connect('clicked', self.remove_from_custom_model2)
                self.button136.connect('clicked', self.log_window)
                self.button138.connect('clicked', self.burn)
                self.button139.connect('clicked', self.log_window)
                self.button140.connect('clicked', self.reset_all_curves)
                self.button141.connect('clicked', self.reset_gamma_curve)
                self.button142.connect('clicked', self.undo_handler)
                self.button143.connect('clicked', self.undo_handler)
                self.button144.connect('clicked', self.apply_spread)
                self.button145.connect('clicked', self.undo_handler)
                self.button146.connect('clicked', self.apply_charcoal)
                self.button147.connect('clicked', self.undo_handler)
                self.button148.connect('clicked', self.apply_sketch)
                self.button149.connect('clicked', self.undo_handler)
                self.button150.connect('clicked', self.apply_posterize)
                self.button151.connect('clicked', self.undo_handler)
                self.button152.connect('clicked', self.apply_negate)
                self.button153.connect('clicked', self.on_window14_delete_event)
                self.button154.connect('clicked', self.add_WB_row)
                self.button155.connect('clicked', self.remove_WB_row)
                self.button156.connect('clicked', self.save_WB_presets)
                self.button158.connect('clicked', self.on_window15_delete_event)
                self.button159.connect('clicked', self.delete_WB_presets)
                self.button162.connect('clicked', self.close_color_manager)
                self.button174.connect('clicked', self.show_tagmanager_info)
                self.menuitem7.connect('activate', self.open_with_other_)
                self.menuitem14.connect('activate', self.copy_and_move)
                self.menuitem15.connect('activate', self.copy_and_move)
                self.menuitem16.connect('activate', self.delete_files)
                self.menuitem17.connect('activate', self.rename_image)
                self.menuitem18.connect('activate', self.copy_and_move)
                self.menuitem19.connect('activate', self.copy_and_move)
                self.menuitem20.connect('activate', self.delete_files)
                self.menuitem21.connect('activate', self.rename_image)
                self.menuitem26.connect('activate', self.copy_and_move)
                self.menuitem27.connect('activate', self.copy_and_move)
                self.menuitem28.connect('activate', self.rename_image)
                self.menuitem12.connect('activate', self.show_window9)
                self.menuitem29.connect('activate', self.show_shortcuts)
                self.menuitem32.connect('activate', self.mail_client)
                self.menuitem33.connect('activate', self.show_window12)
                self.checkbutton17.connect('toggled', self.undo_workflow)
                self.checkbutton18.connect('toggled', self.on_checkbutton18_toggled)
                self.checkbutton19.connect('toggled', self.on_checkbutton19_toggled)
                self.checkbutton20.connect('toggled', self.on_checkbutton20_toggled)
                self.checkbutton21.connect('toggled', self.on_checkbutton21_toggled)
                self.checkbutton24.connect('toggled', self.on_checkbutton24_toggled)
                self.checkbutton28.connect('toggled', self.on_checkbutton28_toggled)
                self.checkbutton34.connect('toggled', self.on_checkbutton24_toggled)
                self.checkbutton35.connect('toggled', self.on_checkbutton35_toggled)
                self.checkbutton360.connect('toggled', self.on_checkbutton360_toggled)
                self.treeview1.connect('button_press_event', self.show_menu6)
                self.scrolledwindow2.connect('scroll_event', self.scroll_eventbox1)
                self.window3.connect('button_press_event', self.on_delete_window3)
                self.window3.connect('scroll_event', self.scroll_fullphoto)
                self.window4.connect('button_press_event', self.on_delete_event)
                self.window4.connect('scroll_event', self.on_delete_event)
                self.scrolledwindow3.connect('scroll_event', self.scroll_scrolledwindow3)
                self.window1.connect('key_press_event', self.on_key_press_event)
                self.window2.connect('key_press_event', self.on_key_press_event)
                self.window3.connect('key_press_event', self.on_key_press_event)
                self.window4.connect('key_press_event', self.on_key_press_event)
                self.window8.connect('key_press_event', self.on_key_press_event)
                self.window11.connect('key_press_event', self.on_key_press_event)
                self.window12.connect('key_press_event', self.on_key_press_event)
                self.window15.connect('key_press_event', self.on_key_press_event)
                self.window7.connect('delete_event', self.on_delete_event)
                self.window8.connect('delete_event', self.on_window8_delete_event)
                self.window9.connect('delete_event', self.on_window9_delete_event)      
                self.window11.connect('delete_event', self.stop_mail)       
                self.window12.connect('delete_event', self.on_window12_delete_event)
                self.window13.connect('delete_event', self.log_window)
                self.window14.connect('delete_event', self.on_window14_delete_event)
                self.window15.connect('delete_event', self.on_window15_delete_event)
                self.window16.connect('delete_event', self.close_color_manager)
                self.window17.connect('delete_event', self.show_tagmanager_info)
                self.button137.connect('clicked', self.on_window12_delete_event)
                self.window7.connect('window_state_event', self.on_window7_state_event)
                self.window16.connect('window_state_event', self.on_window16_state_event)
                self.imagemenuitem6.connect('activate', self.open_on_window2)
                self.radiobutton10.connect('toggled', self.on_radiobutton10_toggled)
                self.viewport1.connect('leave_notify_event', self.on_viewport1_leave_notify_event)
                self.viewport1.connect('enter_notify_event', self.on_viewport1_enter_notify_event)
                self.menuitem31.connect('activate', self.print_dialog)
                self.menuitem34.connect('activate', self.print_dialog)
                self.toolbutton13.connect('clicked', self.print_dialog)
                self.eventbox8.connect('unmap_event', self.on_eventbox_unmapped)
                self.eventbox8.connect('unmap_event', self.on_eventbox_unmapped)
                self.eventbox9.connect('unmap_event', self.on_eventbox_unmapped)
                self.eventbox10.connect('unmap_event', self.on_eventbox_unmapped)
                self.eventbox11.connect('unmap_event', self.on_eventbox_unmapped)
                self.eventbox12.connect('unmap_event', self.on_eventbox_unmapped)
                self.eventbox13.connect('unmap_event', self.on_eventbox_unmapped)
                self.eventbox14.connect('unmap_event', self.on_eventbox_unmapped)
                self.eventbox15.connect('unmap_event', self.on_eventbox_unmapped)
                self.eventbox16.connect('unmap_event', self.on_eventbox_unmapped)
                self.eventbox17.connect('unmap_event', self.on_eventbox_unmapped)
                self.eventbox18.connect('unmap_event', self.on_eventbox_unmapped)
                self.eventbox19.connect('unmap_event', self.on_eventbox_unmapped)
                self.eventbox20.connect('unmap_event', self.on_eventbox_unmapped)
                self.eventbox21.connect('unmap_event', self.on_eventbox_unmapped)
                self.eventbox22.connect('unmap_event', self.on_eventbox_unmapped)
                self.eventbox23.connect('unmap_event', self.on_eventbox_unmapped)
                self.eventbox24.connect('unmap_event', self.on_eventbox_unmapped)
                self.eventbox25.connect('unmap_event', self.on_eventbox_unmapped)
                self.eventbox26.connect('unmap_event', self.on_eventbox_unmapped)
                self.eventbox27.connect('unmap_event', self.on_eventbox_unmapped)
                self.eventbox28.connect('unmap_event', self.on_eventbox_unmapped)
                self.eventbox29.connect('unmap_event', self.on_eventbox_unmapped)
                self.eventbox30.connect('unmap_event', self.on_eventbox_unmapped)
                self.eventbox31.connect('unmap_event', self.on_eventbox_unmapped)
                self.eventbox32.connect('unmap_event', self.on_eventbox_unmapped)
                self.eventbox33.connect('unmap_event', self.on_eventbox_unmapped)
                self.eventbox34.connect('unmap_event', self.on_eventbox_unmapped)
                self.eventbox35.connect('unmap_event', self.on_eventbox_unmapped)
                self.eventbox36.connect('unmap_event', self.on_eventbox_unmapped)
                self.eventbox37.connect('unmap_event', self.on_eventbox_unmapped)
                self.eventbox38.connect('unmap_event', self.on_eventbox_unmapped)
                self.eventbox42.connect('unmap_event', self.on_eventbox_unmapped)
                self.eventbox43.connect('unmap_event', self.on_eventbox_unmapped)
                self.menuitem1.connect('activate', self.on_radiobutton7_toggled)
                self.colorselection1.connect('color_changed', self.apply_tint)
                self.toolbutton10.connect('clicked', self.sx_rotation, 0)
                self.toolbutton11.connect('clicked', self.dx_rotation, 0)
                self.id1 = self.toolbutton16.connect("clicked", self.reduce_gallery)
                self.id2 = self.toolbutton17.connect("clicked", self.enlarge_gallery)
                self.id3 = self.treeview1.connect("row_activated", self.show_album)
                self.id4 = self.open1.connect("activate", self.Open)
                self.id5 = self.open_folder1.connect("activate", self.show_filechooserdialog1)
                self.id6 = self.open_album1.connect("activate", self.open_album)
                self.id7 = self.treeview2.connect("row_activated", self.get_folder)
                self.id8 = self.toolbutton1.connect("clicked", self.show_filechooserdialog1)
                self.id9 = self.toolbutton14.connect("clicked", self.add_album,None)
                self.id10 = self.toolbutton15.connect("clicked", self.delete_album)
                self.id11 = self.toolbutton18.connect("clicked", self.select_all)
                self.id12 = self.toolbutton19.connect("clicked", self.unselect)
                self.id13 = self.toolbutton20.connect("clicked", self.inverted_selection)
                self.id14 = self.new1.connect('activate', self.add_album, None)
                self.id15 = self.delete2.connect('activate', self.delete_album)
                self.id16 = self.add_selection1.connect('activate', self.pre_add_selection)
                self.id18 = self.new2.connect('activate', self.add_album, None)
                self.id19 = self.toolbutton21.connect("clicked", self.slideshow, 1, 2)
                self.id20 = self.filechooserbutton4.connect("current_folder_changed", self.on_filechooserbutton4_current_folder_changed)
                self.id21 = self.filechooserbutton5.connect("current_folder_changed", self.on_filechooserbutton5_current_folder_changed)
                self.id22 = self.hpaned2.connect('notify', self.autoresize)
                self.id23 = self.toolbutton26.connect('clicked', self.set_panel, 'hide')
                self.id24 = self.toolbutton27.connect('clicked', self.set_panel, 'show')
                self.id26 = self.toolbutton29.connect('clicked', self.hide_pane, True)
                self.id27 = self.vpaned2.connect('notify', self.autoresize2)
                self.id28 = self.hpaned1.connect('notify', self.autoresize2)
                self.id29 = self.eventbox1.connect("button_press_event", self.eventbox1_handler)
                self.id30 = self.toolbutton30.connect('clicked', self.fullscreen_mode)
                self.id31 = self.toolbutton31.connect('clicked', self.first_b_photo)
                self.id32 = self.toolbutton32.connect('clicked', self.previous_b_photo)
                self.id33 = self.toolbutton33.connect('clicked', self.next_b_photo)
                self.id34 = self.toolbutton34.connect('clicked', self.last_b_photo)
                self.id35 = self.toolbutton6.connect('clicked', self.zoom_in_photo, 0)
                self.id36 = self.toolbutton7.connect('clicked', self.zoom_out_photo, 0)
                self.id37 = self.toolbutton8.connect('clicked', self.zoom_fit_photo, 0)
                self.id38 = self.toolbutton9.connect('clicked', self.zoom_1_photo, 0)
                self.id39 = self.toolbutton28.connect('clicked', self.hide_pane)
                self.id40 = self.toolbutton35.connect('clicked', self.close_window2)
                self.id42 = self.combobox5.connect('changed' , self.first_to_load_raw_preview)
                self.id43 = self.spinbutton4.connect('button-release-event', self.first_to_load_raw_preview)
                self.id44 = self.spinbutton5.connect('button-release-event', self.first_to_load_raw_preview)
                self.id45 = self.spinbutton6.connect('button-release-event', self.first_to_load_raw_preview)
                self.id46 = self.spinbutton7.connect('button-release-event', self.first_to_load_raw_preview)
                self.id47 = self.spinbutton8.connect('button-release-event', self.first_to_load_raw_preview)
                self.id48 = self.spinbutton9.connect('button-release-event', self.first_to_load_raw_preview)
                self.id49 = self.spinbutton1.connect('button-release-event', self.first_to_load_raw_preview)
                self.id65 = self.spinbutton11.connect('button-release-event', self.first_to_load_raw_preview)
                self.id79 = self.spinbutton35.connect('button-release-event', self.first_to_load_raw_preview)
                self.id50 = self.combobox1.connect('changed', lambda w: self.get_channel())
                self.id51 = self.hscale3.connect('button_release_event', self.first_to_load_raw_preview)
                self.id52 = self.radiobutton11.connect('toggled', self.first_to_load_raw_preview)
                self.id53 = self.checkbutton8.connect('toggled', self.first_to_load_raw_preview)
                self.id54 = self.checkbutton9.connect('toggled', self.first_to_load_raw_preview)
                self.id55 = self.checkbutton10.connect('toggled', self.first_to_load_raw_preview)
                self.id56 = self.checkbutton11.connect('toggled', self.first_to_load_raw_preview)
                self.id57 = self.checkbutton12.connect('toggled', self.first_to_load_raw_preview)
                self.id58 = self.checkbutton13.connect('toggled', self.first_to_load_raw_preview)
                self.id59 = self.checkbutton14.connect('toggled', self.first_to_load_raw_preview)
                self.id60 = self.combobox2.connect('changed', self.first_to_load_raw_preview)
                self.id61 = self.combobox3.connect('changed', self.first_to_load_raw_preview)
                self.id62 = self.combobox4.connect('changed', self.first_to_load_raw_preview)
                self.id63 = self.togglebutton1.connect('toggled', self.on_togglebutton1_toggled)
                self.id64 = self.togglebutton2.connect('toggled', self.on_togglebutton1_toggled)
                self.id66 = self.hscale15.connect('button_release_event', self.first_to_load_raw_preview)
                self.id67 = self.checkbutton16.connect('toggled', self.first_to_load_raw_preview)
                self.id68 = self.button53.connect('clicked', self.transform, 'flop')
                self.id69 = self.button56.connect('clicked', self.transform, 'flip')
                self.id70 = self.button59.connect('clicked', self.transform, 'rotate 90')
                self.id71 = self.button62.connect('clicked', self.transform, 'rotate -90')
                self.id72 = self.spinbutton19.connect('value-changed', self.on_spinbutton19_value_changed)
                self.id73 = self.spinbutton23.connect('value-changed', self.on_spinbutton23_value_changed)
                self.id74 = self.spinbutton24.connect('value-changed', self.on_spinbutton24_value_changed)
                self.id75 = self.spinbutton26.connect('value-changed', self.on_spinbutton26_value_changed)
                self.id77 = self.spinbutton30.connect('value-changed', self.on_spinbutton30_value_changed)
                self.id78 = self.spinbutton38.connect('value-changed', self.on_spinbutton38_value_changed)
                self.id80 = self.checkbutton26.connect('toggled', self.on_channel_mixer_changed, None)
                self.id76 = self.checkbutton23.connect('toggled', self.first_to_load_raw_preview)
                self.id82 = self.spinbutton39.connect('button-release-event', self.first_to_load_raw_preview)
                self.id83 = self.hscale25.connect('button_release_event', self.first_to_load_raw_preview)
                self.id84 = self.hscale28.connect('button_release_event', self.first_to_load_raw_preview)
                self.id85 = self.hscale34.connect('button_release_event', self.first_to_load_raw_preview)
                self.id86 = self.spinbutton42.connect('button-release-event', self.first_to_load_raw_preview)
                self.id87 = self.spinbutton45.connect('button-release-event', self.first_to_load_raw_preview)
                self.id88 = self.spinbutton55.connect('value-changed', self.on_spinbutton_value_changed, self.hscale38)
                self.id89 = self.spinbutton57.connect('value-changed', self.on_spinbutton_value_changed, self.hscale39)
                self.id90 = self.spinbutton58.connect('value-changed', self.on_spinbutton_value_changed, self.hscale40)
                self.id91 = self.spinbutton59.connect('value-changed', self.on_spinbutton_value_changed, self.hscale41)
                self.id92 = self.togglebutton3.connect('toggled', self.on_color_picker_toggled)
                self.id93 = self.togglebutton4.connect('toggled', self.on_color_picker_toggled)
                self.id94 = self.togglebutton5.connect('toggled', self.on_color_picker_toggled)
                self.id95 = self.combobox30.connect('changed', self.on_combobox30_changed)
                self.id96 = self.checkbutton1.connect('toggled', self.first_to_load_raw_preview)
                self.id97 = self.togglebutton6.connect('toggled', self.on_togglebutton6_toggled)
                self.id98 = self.hscale52.connect('button_release_event', self.first_to_load_raw_preview)
                self.id99 = self.hscale53.connect('button_release_event', self.first_to_load_raw_preview)
                self.id100 = self.checkbutton41.connect('toggled', self.update_color_manager)
                self.id101 = self.combobox33.connect('changed', self.update_color_manager)
                self.id102 = self.combobox34.connect('changed', self.update_color_manager)
                self.id103 = self.combobox35.connect('changed', self.update_color_manager)
                self.id104 = self.combobox36.connect('changed', self.update_color_manager)
                self.id105 = self.combobox37.connect('changed', self.update_color_manager)
                self.id106 = self.combobox38.connect('changed', self.update_color_manager)
                self.id107 = self.notebook2.connect("switch_page", self.on_notebook2_switched)
                self.id108 = self.combobox39.connect('changed' , self.first_to_load_raw_preview)
                self.hpaned1.handler_block(self.id28)
                self.window1.connect('delete-event', self.destroy)
                self.window2.connect('delete-event', self.on_delete_event)
                self.window5.connect('delete-event', self.on_delete_event)
                self.window6.connect('delete-event', self.on_delete_event)
                self.filechooserbutton1.connect("current_folder_changed", lambda w: self.entry4.set_text(self.filechooserbutton1.get_current_folder()))
                self.filechooserbutton2.connect("current_folder_changed", lambda w: self.entry5.set_text(self.filechooserbutton2.get_current_folder()))
                self.filechooserdialog2.connect("current_folder_changed", lambda w: self.entry8.set_text(self.filechooserdialog2.get_current_folder()))
                self.filechooserdialog3.connect("current_folder_changed", lambda w: self.entry6.set_text(self.filechooserdialog3.get_current_folder()))
                self.imagemenuitem2.connect('activate', self.set_panel, 'hide')
                self.imagemenuitem3.connect('activate', self.set_panel, 'show')
                self.toolbutton40.connect('clicked', self.zoom_fit_photo, 1)
                self.toolbutton41.connect('clicked', self.zoom_1_photo, 1)
                self.toolbutton42.connect('clicked', self.sx_rotation, 1)
                self.toolbutton43.connect('clicked', self.dx_rotation, 1)
                self.viewport1_id1 = self.viewport1.connect('motion_notify_event', self.drag_image, self.scrolledwindow3)
                self.viewport1_id2 = self.viewport1.connect('button_press_event', self.on_scrolledwindow_release_event)
                self.viewport1_id3 = self.viewport1.connect('button_release_event', self.on_scrolledwindow_release_event)
                self.button18.connect('clicked', self.reset_black_level, True)
                self.button19.connect('clicked', self.reset_white_level, True)
                self.button30.connect('clicked', self.reset_gamma, True)
                self.button32.connect('clicked', self.reset_brightness, True)
                self.button31.connect('clicked', self.reset_saturation, True)
                self.button46.connect('clicked', self.reset_midtones_contrast, True)
                self.button87.connect('clicked', self.reset_hue, True)
                self.button39.connect('clicked', self.reset_color_balance, True)
                self.button43.connect('clicked', self.reset_contrast, True)
                self.button60.connect('clicked', self.reset_mixer)
                self.button86.connect('clicked', lambda w: gobject.idle_add(self.synchronize_tag_database))
                self.button99.connect('clicked', self.reset_red_mixer, True)
                self.button100.connect('clicked', self.reset_green_mixer, True)
                self.button101.connect('clicked', self.reset_blue_mixer, True)
                self.button157.connect('clicked', lambda w: self.preset_model.clear())
                self.expander2.connect('activate', self.on_expander_activate, True)
                self.expander3.connect('activate', self.on_expander_activate, True)
                self.expander4.connect('activate', self.on_expander_activate, True)
                self.expander5.connect('activate', self.on_expander_activate, True)
                self.expander6.connect('activate', self.on_expander_activate, True)
                self.expander7.connect('activate', self.on_expander_activate, True)
                self.expander8.connect('activate', self.on_expander_activate, True)
                self.expander9.connect('activate', self.on_expander_activate, True)
                self.expander10.connect('activate', self.on_expander_activate, True)
                self.expander11.connect('activate', self.on_expander_activate, True)
                self.expander12.connect('activate', self.on_expander_activate, True)
                self.expander13.connect('activate', self.on_expander_activate, True)
                self.expander14.connect('activate', self.on_expander_activate, True)
                self.expander15.connect('activate', self.on_expander_activate, True)
                self.expander16.connect('activate', self.on_expander_activate, True)
                self.expander17.connect('activate', self.on_expander_activate, True)
                self.expander18.connect('activate', self.on_expander_activate, True)
                self.expander19.connect('activate', self.on_expander_activate, True)
                self.expander20.connect('activate', self.on_expander_activate, True)
                self.expander21.connect('activate', self.on_expander_activate, True)
                self.expander23.connect('activate', self.on_expander_activate, True)
                self.expander24.connect('activate', self.on_expander_activate, True)
                self.expander25.connect('activate', self.on_expander_activate, True)
                self.expander26.connect('activate', self.on_expander_activate, True)
                self.expander27.connect('activate', self.on_expander_activate, True)
                self.expander29.connect('activate', self.on_expander_activate, True)
                self.expander30.connect('activate', self.on_expander_activate, True)
                self.expander31.connect('activate', self.on_expander_activate, True)
                self.expander32.connect('activate', self.on_expander_activate, True)
                self.expander33.connect('activate', self.on_expander_activate, True)
                self.expander34.connect('activate', self.on_expander_activate, True)
                self.expander35.connect('activate', self.on_expander_activate, True)
                self.expander36.connect('activate', self.on_expander_activate, True)
                self.expander37.connect('activate', self.on_expander_activate, True)
                self.eventbox41.connect('enter_notify_event', self.show_overexposed_pixels, 'under')
                self.eventbox2.connect('enter_notify_event', self.show_overexposed_pixels, 'over')
                self.eventbox41.connect('leave_notify_event', self.on_exposure_buttons_left)
                self.eventbox2.connect('leave_notify_event', self.on_exposure_buttons_left)
                self.button71.connect('clicked', self.apply_tags_callback, 'add')
                self.button73.connect('clicked', self.apply_tags_callback, 'remove')
                self.button109.connect('clicked', self.apply_tags_callback, 'clear')
                self.window1.connect('drag_data_received', self.on_drag_data_received)
                self.window1.drag_dest_set( gtk.DEST_DEFAULT_MOTION | gtk.DEST_DEFAULT_HIGHLIGHT | gtk.DEST_DEFAULT_DROP,
                                                [ ( 'text/uri-list', 0, 80 ) ], gtk.gdk.ACTION_COPY | gtk.gdk.ACTION_MOVE)
                self.curve1.add_events(gtk.gdk.ALL_EVENTS_MASK)
                self.curve2.add_events(gtk.gdk.ALL_EVENTS_MASK)
                self.curve3.add_events(gtk.gdk.ALL_EVENTS_MASK)
                self.curve4.add_events(gtk.gdk.ALL_EVENTS_MASK)
                gobject.signal_new("curve1_signal", self.curve1, gobject.SIGNAL_RUN_LAST, gobject.TYPE_BOOLEAN, (gobject.TYPE_INT,))
                gobject.signal_new("curve2_signal", self.curve2, gobject.SIGNAL_RUN_LAST, gobject.TYPE_BOOLEAN, (gobject.TYPE_INT,))
                gobject.signal_new("curve3_signal", self.curve3, gobject.SIGNAL_RUN_LAST, gobject.TYPE_BOOLEAN, (gobject.TYPE_INT,))
                gobject.signal_new("curve4_signal", self.curve4, gobject.SIGNAL_RUN_LAST, gobject.TYPE_BOOLEAN, (gobject.TYPE_INT,))
                self.curve1.connect("curve1_signal", self.apply_gamma_curve, 0)
                self.curve2.connect("curve2_signal", self.apply_gamma_curve, 1)
                self.curve3.connect("curve3_signal", self.apply_gamma_curve, 2)
                self.curve4.connect("curve4_signal", self.apply_gamma_curve, 3)
                self.curve1.connect('enter_notify_event', self.start_curve_daemon, 'curve1_signal')
                self.curve1.connect('leave_notify_event', self.stop_curve_daemon)
                self.curve2.connect('enter_notify_event', self.start_curve_daemon, 'curve2_signal')
                self.curve2.connect('leave_notify_event', self.stop_curve_daemon)
                self.curve3.connect('enter_notify_event', self.start_curve_daemon, 'curve3_signal')
                self.curve3.connect('leave_notify_event', self.stop_curve_daemon)
                self.curve4.connect('enter_notify_event', self.start_curve_daemon, 'curve4_signal')
                self.curve4.connect('leave_notify_event', self.stop_curve_daemon)
                self.hscale3.connect('change_value', self.on_hscale_change_value, self.spinbutton11)
                self.hscale4.connect('change_value', self.on_hscale_change_value, self.spinbutton10)
                self.hscale5.connect('change_value', self.on_hscale_change_value, self.spinbutton12)
                self.hscale6.connect('change_value', self.on_hscale_change_value, self.spinbutton13)
                self.hscale7.connect('change_value', self.on_hscale_change_value, self.spinbutton14)
                self.hscale8.connect('change_value', self.on_hscale_change_value, self.spinbutton15)
                self.hscale9.connect('change_value', self.on_hscale_change_value, self.spinbutton16)
                self.hscale10.connect('change_value', self.on_hscale_change_value, self.spinbutton17)
                self.hscale11.connect('change_value', self.on_hscale_change_value, self.spinbutton18)
                self.hscale12.connect('change_value', self.on_hscale_change_value, self.spinbutton20)
                self.hscale13.connect('change_value', self.on_hscale_change_value, self.spinbutton21)
                self.hscale14.connect('change_value', self.on_hscale_change_value, self.spinbutton22)
                self.hscale15.connect('change_value', self.on_hscale_change_value, self.spinbutton1)
                self.hscale16.connect('change_value', self.on_hscale_change_value, self.spinbutton25)
                self.hscale17.connect('change_value', self.on_hscale_change_value, self.spinbutton29)
                self.hscale18.connect('change_value', self.on_hscale_change_value, self.spinbutton37)
                self.hscale19.connect('change_value', self.on_hscale_change_value, self.spinbutton31)
                self.hscale20.connect('change_value', self.on_hscale_change_value, self.spinbutton33)
                self.hscale21.connect('change_value', self.on_hscale_change_value, self.spinbutton32)
                self.hscale22.connect('change_value', self.on_hscale_change_value, self.spinbutton34)
                self.hscale24.connect('change_value', self.on_hscale_change_value, self.spinbutton36)
                self.hscale25.connect('change_value', self.on_hscale_change_value, self.spinbutton39)
                self.hscale26.connect('change_value', self.on_hscale_change_value, self.spinbutton41)
                self.hscale27.connect('change_value', self.on_hscale_change_value, self.spinbutton40)
                self.hscale28.connect('change_value', self.on_hscale_change_value, self.spinbutton45)
                self.hscale29.connect('change_value', self.on_hscale_change_value, self.spinbutton43)
                self.hscale30.connect('change_value', self.on_hscale_change_value, self.spinbutton44)
                self.hscale31.connect('change_value', self.on_hscale_change_value, self.spinbutton48)
                self.hscale32.connect('change_value', self.on_hscale_change_value, self.spinbutton47)
                self.hscale33.connect('change_value', self.on_hscale_change_value, self.spinbutton46)
                self.hscale34.connect('change_value', self.on_hscale_change_value, self.spinbutton42)
                self.hscale35.connect('change_value', self.on_hscale_change_value, self.spinbutton49, self.spinbutton52)
                self.hscale36.connect('change_value', self.on_hscale_change_value, self.spinbutton50, self.spinbutton54)
                self.hscale37.connect('change_value', self.on_hscale_change_value, self.spinbutton56)
                self.hscale38.connect('change_value', self.on_hscale_change_value, self.spinbutton55)
                self.hscale39.connect('change_value', self.on_hscale_change_value, self.spinbutton57)
                self.hscale40.connect('change_value', self.on_hscale_change_value, self.spinbutton58)
                self.hscale41.connect('change_value', self.on_hscale_change_value, self.spinbutton59)
                self.hscale42.connect('change_value', self.on_hscale_change_value, self.spinbutton64)
                self.hscale43.connect('change_value', self.on_hscale_change_value, self.spinbutton66)
                self.hscale44.connect('change_value', self.on_hscale_change_value, self.spinbutton65)
                self.hscale45.connect('change_value', self.on_hscale_change_value, self.spinbutton68)
                self.hscale46.connect('change_value', self.on_hscale_change_value, self.spinbutton67)
                self.hscale47.connect('change_value', self.on_hscale_change_value, self.spinbutton69)
                self.hscale48.connect('change_value', self.on_hscale_change_value, self.spinbutton70)
                self.hscale49.connect('change_value', self.on_hscale_change_value, self.spinbutton71)
                self.hscale50.connect('change_value', self.on_hscale_change_value, self.spinbutton72)
                self.hscale51.connect('change_value', self.on_hscale_change_value, self.spinbutton73)
                self.hscale52.connect('change_value', self.on_hscale_change_value, self.spinbutton9)
                self.hscale53.connect('change_value', self.on_hscale_change_value, self.spinbutton35)
                self.spinbutton1.connect('value_changed', self.on_spinbutton_value_changed, self.hscale15)
                self.spinbutton9.connect('value_changed', self.on_spinbutton_value_changed, self.hscale52)
                self.spinbutton10.connect('value_changed', self.on_spinbutton_value_changed, self.hscale4)
                self.spinbutton11.connect('value_changed', self.on_spinbutton_value_changed, self.hscale3)
                self.spinbutton12.connect('value_changed', self.on_spinbutton_value_changed, self.hscale5)
                self.spinbutton13.connect('value_changed', self.on_spinbutton_value_changed, self.hscale6)
                self.spinbutton14.connect('value_changed', self.on_spinbutton_value_changed, self.hscale7)
                self.spinbutton15.connect('value_changed', self.on_spinbutton_value_changed, self.hscale8)
                self.spinbutton16.connect('value_changed', self.on_spinbutton_value_changed, self.hscale9)
                self.spinbutton17.connect('value_changed', self.on_spinbutton_value_changed, self.hscale10)
                self.spinbutton18.connect('value_changed', self.on_spinbutton_value_changed, self.hscale11)
                self.spinbutton20.connect('value_changed', self.on_spinbutton_value_changed, self.hscale12)
                self.spinbutton21.connect('value_changed', self.on_spinbutton_value_changed, self.hscale13)
                self.spinbutton22.connect('value_changed', self.on_spinbutton_value_changed, self.hscale14)
                self.spinbutton25.connect('value_changed', self.on_spinbutton_value_changed, self.hscale16)
                self.spinbutton29.connect('value_changed', self.on_spinbutton_value_changed, self.hscale17)
                self.spinbutton31.connect('value_changed', self.on_spinbutton_value_changed, self.hscale19)
                self.spinbutton32.connect('value_changed', self.on_spinbutton_value_changed, self.hscale21)
                self.spinbutton33.connect('value_changed', self.on_spinbutton_value_changed, self.hscale20)
                self.spinbutton34.connect('value_changed', self.on_spinbutton_value_changed, self.hscale22)
                self.spinbutton35.connect('value_changed', self.on_spinbutton_value_changed, self.hscale53)
                self.spinbutton36.connect('value_changed', self.on_spinbutton_value_changed, self.hscale24)
                self.spinbutton37.connect('value_changed', self.on_spinbutton_value_changed, self.hscale18)
                self.spinbutton39.connect('value_changed', self.on_spinbutton_value_changed, self.hscale25)
                self.spinbutton40.connect('value_changed', self.on_spinbutton_value_changed, self.hscale27)
                self.spinbutton41.connect('value_changed', self.on_spinbutton_value_changed, self.hscale26)
                self.spinbutton42.connect('value_changed', self.on_spinbutton_value_changed, self.hscale34)
                self.spinbutton43.connect('value_changed', self.on_spinbutton_value_changed, self.hscale29)
                self.spinbutton44.connect('value_changed', self.on_spinbutton_value_changed, self.hscale30)
                self.spinbutton45.connect('value_changed', self.on_spinbutton_value_changed, self.hscale28)
                self.spinbutton46.connect('value_changed', self.on_spinbutton_value_changed, self.hscale33)
                self.spinbutton47.connect('value_changed', self.on_spinbutton_value_changed, self.hscale32)
                self.spinbutton48.connect('value_changed', self.on_spinbutton_value_changed, self.hscale31)
                self.spinbutton56.connect('value_changed', self.on_spinbutton_value_changed, self.hscale37)
                self.spinbutton64.connect('value_changed', self.on_spinbutton_value_changed, self.hscale42)
                self.spinbutton65.connect('value_changed', self.on_spinbutton_value_changed, self.hscale44)
                self.spinbutton66.connect('value_changed', self.on_spinbutton_value_changed, self.hscale43)
                self.spinbutton67.connect('value_changed', self.on_spinbutton_value_changed, self.hscale46)
                self.spinbutton68.connect('value_changed', self.on_spinbutton_value_changed, self.hscale45)
                self.spinbutton69.connect('value_changed', self.on_spinbutton_value_changed, self.hscale47)
                self.spinbutton70.connect('value_changed', self.on_spinbutton_value_changed, self.hscale48)
                self.spinbutton71.connect('value_changed', self.on_spinbutton_value_changed, self.hscale49)
                self.spinbutton72.connect('value_changed', self.on_spinbutton_value_changed, self.hscale50)
                self.spinbutton73.connect('value_changed', self.on_spinbutton_value_changed, self.hscale51)
                self.eventbox44.connect('motion_notify_event', self.on_eventbox44_motion)
                self.eventbox44.connect('leave_notify_event', lambda x, y: self.label56.set_text('x:'))
                self.button163.connect('clicked', self.show_filechooserdialog4, 'input')
                self.button164.connect('clicked', self.show_filechooserdialog4, 'output')
                self.button165.connect('clicked', self.show_filechooserdialog4, 'display')
                self.button166.connect('clicked', self.show_profile_info, 'input')
                self.button167.connect('clicked', self.show_profile_info, 'output')
                self.button168.connect('clicked', self.show_profile_info, 'display')

        def show_profile_info(self, obj, data):
                try:
                        if data == 'input':
                                title = '%s profile info' % self.input_profile_name
                                m_name = self.input_profile_name[:]
                                if self.settings.has_key('in_profile_icc'):
                                        name = self.settings['in_profile_icc']
                                raw = self.input_profile
                        elif data == 'output':
                                title = '%s profile info' % self.output_profile_name
                                m_name = self.output_profile_name[:]
                                if self.settings.has_key('out_profile_icc'):
                                        name = self.settings['out_profile_icc']
                                raw = self.output_profile
                        elif data == 'display':
                                title = '%s profile info' % self.display_profile_name
                                m_name = self.display_profile_name[:]
                                if self.settings.has_key('display_profile_icc'):
                                        name = self.settings['display_profile_icc']
                                raw = self.display_profile
                        if m_name == 'sRGB':
                                profile = littlecms.Profile(colourSpace=littlecms.PT_RGB)
                        elif m_name == 'AdobeRGB 1998':
                                from ctypes import c_void_p
                                profile = littlecms.Profile(rawProfile=(self.AdobeRGB1998_profile.ctypes.data_as(c_void_p), self.AdobeRGB1998_profile.size))
                        else:
                                profile = littlecms.Profile(name)
                        self.dialog32.set_title(title)
                        self.read_profile_info(profile, raw)
                        self.dialog32.show()
                        response = self.dialog32.run()
                        if response == gtk.RESPONSE_CLOSE or response == gtk.RESPONSE_DELETE_EVENT:
                                self.dialog32.hide()
                                return True
                except:
                        gobject.idle_add(sys.stderr.write, traceback.format_exc())

        def read_profile_info(self, profile, raw):
                try:
                        tags = littlecms.read_tags.ReadTags(profile, raw)
                        self.textview5.get_buffer().set_text(tags.get_report())
                except:
                        self.textview5.get_buffer().set_text(_('Cannot read profile!'))
                
        def show_filechooserdialog4(self, obj, data):
                self.filechooserdialog4.show()
                response = self.filechooserdialog4.run()
                if response == gtk.RESPONSE_CANCEL or response == gtk.RESPONSE_DELETE_EVENT:
                        self.filechooserdialog4.hide()
                        return True
                elif response == gtk.RESPONSE_OK:
                        self.filechooserdialog4.hide()
                        if data == 'input':
                                self.on_input_profile_changed(self.filechooserdialog4)
                        elif data == 'output':
                                self.on_output_profile_changed(self.filechooserdialog4)
                        elif data == 'display':
                                self.on_display_profile_changed(self.filechooserdialog4)
                                
        def on_input_profile_changed(self, obj):
                self.settings['in_profile_icc'] = profile = obj.get_filename()
                for row in self.boxmodel33:
                        if row[0] == os.path.basename(profile):
                                self.combobox33.set_active_iter(row.iter)
                                break
                else:
                        if len(self.boxmodel33) == 3:
                                self.boxmodel33.clear()
                                self.boxmodel33.append(['sRGB'])
                                self.boxmodel33.append(['AdobeRGB 1998'])
                        self.boxmodel33.append([os.path.basename(profile)])
                        for row in self.boxmodel33:
                                if row[0] == os.path.basename(profile):
                                        self.combobox33.set_active_iter(row.iter)
                                        break

        def on_output_profile_changed(self, obj):
                self.settings['out_profile_icc'] = profile = obj.get_filename()
                for row in self.boxmodel34:
                        if row[0] == os.path.basename(profile):
                                self.combobox34.set_active_iter(row.iter)
                                break
                else:
                        if len(self.boxmodel34) == 3:
                                self.boxmodel34.clear()
                                self.boxmodel34.append(['sRGB'])
                                self.boxmodel34.append(['AdobeRGB 1998'])
                        self.boxmodel34.append([os.path.basename(profile)])
                        for row in self.boxmodel34:
                                if row[0] == os.path.basename(profile):
                                        self.combobox34.set_active_iter(row.iter)
                                        break

        def on_display_profile_changed(self, obj):
                self.settings['display_profile_icc'] = profile = obj.get_filename()
                for row in self.boxmodel35:
                        if row[0] == os.path.basename(profile):
                                self.combobox35.set_active_iter(row.iter)
                                break
                else:
                        if len(self.boxmodel35) == 2:
                                self.boxmodel35.clear()
                                self.boxmodel35.append(['sRGB'])
                        self.boxmodel35.append([os.path.basename(profile)])
                        for row in self.boxmodel35:
                                if row[0] == os.path.basename(profile):
                                        self.combobox35.set_active_iter(row.iter)
                                        break
	
        def on_window14_delete_event(self, obj, event=None):
                self.window14.hide()
                return True

        def on_window15_delete_event(self, obj, event=None):
                self.window15.hide()
                self.window2.set_modal(True)
                self.preset_model.clear()
                return True

        def show_window15(self, obj):
                self.window2.set_modal(False)
                title = 'White Balance Presets Editor'
                self.window15.set_title(title)
                text = self.combobox30.get_active_text()
                if text == 'No WB Presets':
                        pass
                elif text == 'Auto WB Presets':
                        image = self.get_real_path(self.list[self.indice])
                        self.load_auto_wb_presets(image)
                        if self.auto_wb_presets[image]:
                                self.window15.set_title(title + ' - ' + text)
                                for key , value in sorted(self.auto_wb_presets[image].iteritems()):
                                        self.preset_model.append([str(len(self.preset_model) + 1), key, value[0], value[1], value[2]])
                else:
                        self.window15.set_title(title + ' - ' + text)
                        dct = self.presets[text]
                        for key, value in sorted(dct.iteritems()):
                                self.preset_model.append([str(len(self.preset_model) + 1), key, value[0], value[1], value[2]])
                self.window15.show()

        def add_WB_row(self, obj):
                self.preset_model.append([str(len(self.preset_model) + 1), '','','',''])

        def remove_WB_row(self, obj=None):
                selection = self.treeview16.get_selection()
                model, iter_ = selection.get_selected()
                if iter_ :
                        model.remove(iter_)

        def on_combobox30_changed(self, obj):
                preset = obj.get_active_text()
                wb = self.combobox5.get_active_text()
                active = self.combobox5.get_active()
                if preset == 'No WB Presets' and len(self.boxmodel5) > 3:
                        self.combobox5.handler_block(self.id42)
                        self.boxmodel5.clear()
                        self.boxmodel5.append(['Camera WB'])
                        self.boxmodel5.append(['Auto WB'])
                        self.boxmodel5.append(['Manual WB'])
                        self.combobox5.handler_unblock(self.id42)
                        for row in self.boxmodel5:
                                if row[0] == wb:
                                        self.combobox5.handler_block(self.id42)
                                        self.combobox5.set_active_iter(row.iter)
                                        self.combobox5.handler_unblock(self.id42)
                                        break
                        else:
                                self.combobox5.set_active(0)
                elif preset == 'Auto WB Presets':
                        self.get_auto_wb_presets(self.get_real_path(self.list[self.indice]), True)
                else:
                        self.combobox5.handler_block(self.id42)
                        self.boxmodel5.clear()
                        self.boxmodel5.append(['Camera WB'])
                        self.boxmodel5.append(['Auto WB'])
                        self.boxmodel5.append(['Manual WB'])
                        for key in sorted(self.presets[preset].iterkeys()):
                                self.boxmodel5.append([key])
                        self.combobox5.handler_unblock(self.id42) 
                        for row in self.boxmodel5:
                                if row[0] == wb:
                                        if active < 3:
                                                self.combobox5.handler_block(self.id42)
                                        self.combobox5.set_active_iter(row.iter)
                                        if active < 3:
                                                self.combobox5.handler_unblock(self.id42)
                                        break
                        else:
                                self.combobox5.set_active(0)

        def on_combobox31_changed(self, obj):
                preset = obj.get_active_text()
                if preset:
                        self.window15.set_title('White Balance Presets Editor - ' + preset)
                        self.preset_model.clear()
                        dct = self.presets[preset]
                        for key, value in sorted(dct.iteritems()):
                                self.preset_model.append([str(len(self.preset_model) + 1), key, value[0], value[1], value[2]])

        def get_auto_wb_presets(self, image, data=None):
                if not self.auto_wb_presets.has_key(image):
                        self.load_auto_wb_presets(image)
                text = self.combobox5.get_active_text()
                active = self.combobox5.get_active()
                if data:
                        self.combobox5.handler_block(self.id42)
                self.boxmodel5.clear()
                self.boxmodel5.append(['Camera WB'])
                self.boxmodel5.append(['Auto WB'])
                self.boxmodel5.append(['Manual WB'])
                if data:
                        self.combobox5.handler_unblock(self.id42)
                if self.auto_wb_presets[image]:
                        for key in sorted(self.auto_wb_presets[image].iterkeys()):
                                self.boxmodel5.append([key])
                for row in self.boxmodel5:
                        if row[0] == text:
                                if data and active < 3:
                                        self.combobox5.handler_block(self.id42)
                                self.combobox5.set_active_iter(row.iter)
                                if data and active < 3:
                                        self.combobox5.handler_unblock(self.id42)
                                break
                else:
                        self.combobox5.set_active(0)

        def load_auto_wb_presets(self, image):
                self.auto_wb_presets[image] = {}
                cmd = ['exiftool', '-j', '-n', '-WB_RGGBLevelsAsShot', '-WB_RGGBLevelsMeasured', '-WB_RGGBLevelsDaylight',
                       '-WB_RGGBLevelsShade', '-WB_RGGBLevelsCloudy', '-WB_RGGBLevelsTungsten', '-WB_RGGBLevelsFluorescent',
                       '-WB_RGGBLevelsFlash', '-WB_RGGBLevelsKelvin', '-WB_RGGBLevelsPC1', '-WB_RGGBLevelsPC2', '-WB_RGGBLevelsPC3', image]
                p = Popen(cmd, stdin=PIPE, stdout=PIPE, stderr=PIPE)
                output = p.stdout.read()
                p.stdout.close()
                p.stderr.close()
                p.stdin.close()
                if output:
                        dct = json.loads(output)[0]
                        if len(dct) > 1:
                                for key in dct:
                                        if key != 'SourceFile':
                                                wb_name = key.split('WB_RGGBLevels')[1]
                                                mult = map(float, dct[key].split(' '))
                                                self.auto_wb_presets[image][wb_name] = [str(mult[0] / mult[1]), str(mult[1] / mult[1]), str(mult[3] / mult[1])]
                        else:
                                self.auto_wb_presets[image] = None
                                              
        def save_WB_presets(self, obj):
                self.entry1.grab_focus()
                self.label3.set_text(_('Insert a name'))
                self.dialog1.set_title(_('Save WB Presets'))
                self.dialog1.show()
                response = self.dialog1.run()
                if response == gtk.RESPONSE_CANCEL or response == gtk.RESPONSE_DELETE_EVENT:
                        self.dialog1.hide()
                        return True
                elif response == gtk.RESPONSE_OK:
                        name = self.entry1.get_text().strip()
                        if name and len(self.preset_model):
                                self.dialog1.hide()
                                dic = {}
                                for row in self.preset_model:
                                        if row[1]:
                                                if not row[2]:
                                                        row[2] = '1.0'
                                                if not row[3]:
                                                        row[3] = '1.0'
                                                if not row[4]:
                                                        row[4] = '1.0'
                                                dic[row[1]] = [row[2], row[3], row[4]]
                                        else:
                                                gtk.gdk.beep()
                                                dialog25 = self.messagedialog(_("Attention!"), _('WB Name missing!'), None, gtk.BUTTONS_CLOSE, None,
                                                                              gtk.DIALOG_MODAL, gtk.MESSAGE_WARNING, None, (330,180))
                                                dialog25.set_keep_above(True)
                                                response = dialog25.run()
                                                if response == gtk.RESPONSE_CLOSE or response == gtk.RESPONSE_DELETE_EVENT:
                                                        dialog25.destroy()
                                                return True
                                self.presets[name] = dic
                                self.settings['presets'] = self.presets
                                self.store_settings_db()
                                for row in self.boxmodel30:
                                        if row[0] == name:
                                                break
                                else:
                                        self.boxmodel30.append([name])
                                        self.boxmodel31.append([name])
                        else:
                                gtk.gdk.beep()
                                self.save_WB_presets(obj)

        def delete_WB_presets(self, obj):
                self.dialog7.show()
                response = self.dialog7.run()
                if response == gtk.RESPONSE_CANCEL or response == gtk.RESPONSE_DELETE_EVENT:
                        self.dialog7.hide()
                elif response == gtk.RESPONSE_OK:
                        self.dialog7.hide()
                        name = self.combobox32.get_active_text()
                        if name:
                                if name == self.combobox30.get_active_text():
                                        self.combobox30.set_active(0)
                                for row in self.boxmodel30:
                                        if row[0] == name:
                                                self.boxmodel30.remove(row.iter)
                                for row in self.boxmodel31:
                                        if row[0] == name:
                                                self.boxmodel31.remove(row.iter)
                                                del self.presets[name]
                                                self.settings['presets'] = self.presets
                                                self.store_settings_db()
                                                break
                                        
        def show_error_console(self, obj, event=None):
                if self.window14.get_property('visible'):
                        self.window14.present()
                else:
                        self.window14.show()

# BURNER METHODS ===================================================================================
        def show_window12(self, obj):
                self.textview4.get_buffer().set_text('')
                if self.radiobutton7.get_active():
                        self.selected = [self.list[self.indice]]
                if self.selected:
                        self.label243.set_text('')
                        self.cdmodel.clear()
                        self.progressbar7.set_fraction(0)
                        self.window12.show_all()
                        self.window12.window.set_cursor(self.watch)
                        self.progressbar7.set_text(_('loading previews...'))
                        while gtk.events_pending():
                                gtk.main_iteration(False)
                        self.cd_size = 0
                        fraction = 0.99 / len(self.selected)
                        for item in self.selected:
                                if not self.window12.get_property('visible'):
                                        break
                                if self.radiobutton7.get_active():
                                        pixbuf = self.scale_thumbnail(self.b_img[self.list.index(item)].get_pixbuf().copy())
                                else:
                                        pixbuf = self.scale_thumbnail(self.image[self.list.index(item)].get_pixbuf().copy())
                                self.cdmodel.append([True, pixbuf, self.get_real_path(item)])
                                self.cd_size += os.path.getsize(self.get_real_path(item))
                                self.progressbar7.set_fraction(self.progressbar7.get_fraction() + fraction)
                                while gtk.events_pending():
                                        gtk.main_iteration(False)
                        self.window12.window.set_cursor(None)
                        self.progressbar7.set_text('')
                        self.progressbar7.set_fraction(0)
                        self.progressbar7.hide()
                        self.label243.set_text(_('Total size: %f Mb') % (self.cd_size / 1000000.0))
                        if self.radiobutton6.get_active():
                                self.unselect()
                        self.selected = []
                else:
                        dialog9 = self.messagedialog(_("Info!"), _("No file selected!"), None, gtk.BUTTONS_CLOSE, self.window1, gtk.DIALOG_MODAL, gtk.MESSAGE_INFO, None, (230,120))
                        dialog9.set_keep_above(True)
                        gtk.gdk.beep()
                        response = dialog9.run()
                        if response == gtk.RESPONSE_CLOSE or response == gtk.RESPONSE_DELETE_EVENT:
                                dialog9.destroy()

        def on_cdmodel_toggled(self, cell, path, model):
                model[path][0] = not model[path][0]
                while gtk.events_pending():
                        gtk.main_iteration(False)
                if model[path][0]:
                        self.cd_size += os.path.getsize(model[path][2])
                else:
                        self.cd_size -= os.path.getsize(model[path][2])
                self.label243.set_text(_('Total size: %f Mb') % (self.cd_size / 1000000.0))

        def on_window12_delete_event(self, obj, event=None):
                self.window12.hide()
                return True
                        
        def log_window(self, obj, event=None):
                if self.window13.get_property('visible'):
                        self.window13.hide()
                else:
                        self.window13.move(0,0)
                        self.window13.show()
                return True

        def show_tagmanager_info(self, obj, event=None):
                if self.window17.get_property('visible'):
                        self.window17.hide()
                else:
                        self.window17.move(0,0)
                        self.textview6.get_buffer().set_text('')
                        f = open(os.path.join(gtkrg_datadir, 'tagmanagerinfo.txt'))
                        text = f.read()
                        f.close()
                        self.textview6.get_buffer().set_text(text)
                        self.window17.show()
                return True

        def scanbus(self):
                self.button138.set_sensitive(True)
                p = Popen('cdrecord -scanbus', shell=True, stdin=PIPE, stdout=PIPE, stderr=STDOUT, env={'LANG':'en_Us'})
                for line in p.stdout.readlines():
                        if 'Removable CD-ROM' in line:
                                self.boxmodel24.append([line.strip()])
                if not len(self.boxmodel24):
                        self.boxmodel24.append([_("No device found. Burn disabled!")])
                        self.button138.set_sensitive(False)
                self.combobox24.set_active(0)
                p.stdout.close()
                p.stdin.close()

        def burn(self, obj):
                if len(self.cdmodel):
                        self.button137.set_sensitive(False)
                        blank = self.checkbutton37.get_active()
                        multi = self.checkbutton38.get_active()
                        device = self.combobox24.get_active_text()
                        if device:
                                self.progressbar7.show()
                                thread = threading.Thread(target=self.start_cdrecord, args=(device[:5], blank, multi))
                                thread.setDaemon(True)
                                thread.start()
                                thread1 = threading.Thread(target=self.pulse_progressbar7, args=(thread,))
                                thread1.start()
                        else:
                                self.show_dialog15(_('Device not found!'))
                        
        def pulse_progressbar7(self, thread):
                while thread.isAlive() and not self.cdrecord_error:
                        gobject.idle_add(self.progressbar7.pulse)
##                        while gtk.events_pending():
##                                gtk.main_iteration(False)
                        time.sleep(0.5)
                gobject.idle_add(self.progressbar7.set_fraction, 0)

        def start_cdrecord(self, device, blank, multi):
                list_ = [row[2] for row in self.cdmodel if row[0]]
                if not list_:
                        gobject.idle_add(self.button137.set_sensitive, True)
                        return True
                buffer = self.textview4.get_buffer()
                buffer.set_text('')
                if not hasattr(self, 'isodir'):
                        self.isodir = mkdtemp(prefix="gtkrg-iso")
                gobject.idle_add(self.progressbar7.set_text, _('creating disk image...'))
                for item in list_:
                        shutil.copy2(item, self.isodir)
                if multi:
                        p = Popen(['cdrecord', 'dev=' + device, '-msinfo'], stdin=PIPE, stdout=PIPE, stderr=STDOUT)
                        msinfo = p.communicate()[0]
                        p.stdin.close()
                        p.stdout.close()
                        cmd = 'mkisofs -R -J -o "%s" -C "%s" -M "%s" "%s"' % (os.path.join(self.temp, 'cd_image.iso'), msinfo, device, self.isodir)
                else:
                        cmd = 'mkisofs -R -J -o "%s" "%s"' % (os.path.join(self.temp, 'cd_image.iso'), self.isodir)
                p = Popen(cmd, stdin=PIPE, stdout=PIPE, stderr=STDOUT, shell=True)
                line = ''
                while p.poll() is None:
                        char = p.stdout.read(1)
                        line += char
                        if char not in ['\n', '\r']:
                                continue
                        gobject.idle_add(buffer.insert_at_cursor, unistr(line))
                        gobject.idle_add(self.textview4.scroll_to_mark, buffer.get_insert(), 0.1)
                        line = ''
                        while gtk.events_pending():
                                gtk.main_iteration(False)
                p.stdin.close()
                p.stdout.close()                        
                cmd = ['cdrecord' , '-v', '-eject', 'dev=' + device]
                if blank:
                        cmd.append('blank=fast')
                if multi:
                        cmd.append('-multi')
                cmd += ['-data', os.path.join(self.temp, 'cd_image.iso')]
                try:
                        gobject.idle_add(self.progressbar7.set_text, _('starting cdrecord...'))
                        p = Popen(cmd, stdin=PIPE, stdout=PIPE, stderr=STDOUT, env={'LANG':'en_Us'})
                        line = ''
                        while p.poll() is None:
                                char = p.stdout.read(1)
                                line += char
                                if char not in ['\n', '\r']:
                                        continue
                                if 'Cannot load media' in line:
                                        gobject.idle_add(self.progressbar7.set_text, _('Cannot load media!'))
                                        self.cdrecord_error = True
                                if 'Could not write Lead-in' in line:
                                        gobject.idle_add(self.progressbar7.set_text, unistr(line[:-1]))
                                        self.cdrecord_error = True
                                if 'hit <CR>' in line:
                                        self.cdrecord_error = True
                                        break
                                if line.startswith('Blanking'):
                                        gobject.idle_add(self.progressbar7.set_text, unistr(line[:-1]))
                                elif line.startswith('Writing'):
                                        gobject.idle_add(self.progressbar7.set_text, unistr(line[:-1]))
                                elif line.startswith('Fixating'):
                                        gobject.idle_add(self.progressbar7.set_text, unistr(line[:-1]))
                                elif line.startswith('Start new track at sector:'):
                                        gobject.idle_add(self.progressbar7.set_text, unistr(line[:-1]))
                                elif line.startswith('Track') and 'written' in line:
                                        gobject.idle_add(self.progressbar7.set_text, unistr(line.split('written')[0] + 'written'))
                                gobject.idle_add(buffer.insert_at_cursor, unistr(line))
                                gobject.idle_add(self.textview4.scroll_to_mark, buffer.get_insert(), 0.1)
                                line = ''
                                while gtk.events_pending():
                                        gtk.main_iteration(False)
                        p.stdin.close()
                        p.stdout.close()
                except:
                        gobject.idle_add(sys.stderr.write, traceback.format_exc())
                if os.path.isfile(os.path.join(self.temp, 'cd_image.iso')):
                        os.remove(os.path.join(self.temp, 'cd_image.iso'))
                if os.path.isdir(self.isodir):
                        shutil.rmtree(self.isodir, ignore_errors=True)
                del self.isodir
                gobject.idle_add(self.button137.set_sensitive, True)
                if self.cdrecord_error:
                        gobject.idle_add(self.show_dialog15, _('Cdrecord Error!\nRead debugging output for more info!'))
                else:
                        gobject.idle_add(self.progressbar7.set_text, _('Disk successfully burned!'))
                self.cdrecord_error = False
# END OF BURNER METHODS =========================================================================================

# SMTP CLIENT ===================================================================================================

        def scale_thumbnail(self, pixbuf):
                width = pixbuf.get_width()
                height = pixbuf.get_height()
                ratio = float(width) / height
                if ratio > 1:
                        pixbuf = pixbuf.scale_simple(50, int(50 / ratio), gtk.gdk.INTERP_BILINEAR)
                else:
                        pixbuf = pixbuf.scale_simple(int(50 * ratio), 50, gtk.gdk.INTERP_BILINEAR)
                return pixbuf
        
        def mail_client(self, obj):
                if self.radiobutton7.get_active():
                        self.selected = [self.list[self.indice]]
                if self.selected:
                        self.label239.set_text('')
                        self.window11.show()
                        while gtk.events_pending():
                                gtk.main_iteration(False)
                        self.mail_size = 0
                        fraction = 0.98 / len(self.selected)
                        self.progressbar6.set_fraction(0)
                        self.progressbar6.set_text(_('loading previews...'))
                        self.progressbar6.show()
                        for item in self.selected:
                                if not self.window11.get_property('visible'):
                                        break
                                if self.radiobutton7.get_active():
                                        pixbuf = self.scale_thumbnail(self.b_img[self.list.index(item)].get_pixbuf().copy())
                                else:
                                        pixbuf = self.scale_thumbnail(self.image[self.list.index(item)].get_pixbuf().copy())
                                self.attachment_model.append([pixbuf, self.get_real_path(item)])
                                self.mail_size += os.path.getsize(self.get_real_path(item))
                                self.progressbar6.set_fraction(self.progressbar6.get_fraction() + fraction)
                                while gtk.events_pending():
                                        gtk.main_iteration(False)
                        self.progressbar6.hide()
                        self.progressbar6.set_fraction(0)
                        self.label239.set_text(_('Total size: %f Mb') % (self.mail_size / 1000000.0))
                else:
                        dialog9 = self.messagedialog(_("Info!"), _("No file selected!"), None, gtk.BUTTONS_CLOSE, self.window1, gtk.DIALOG_MODAL, gtk.MESSAGE_INFO, None, (230,120))
                        dialog9.set_keep_above(True)
                        gtk.gdk.beep()
                        response = dialog9.run()
                        if response == gtk.RESPONSE_CLOSE or response == gtk.RESPONSE_DELETE_EVENT:
                                dialog9.destroy()

        def stop_mail(self, *obj):
                self.window11.hide()
                while gtk.events_pending():
                        gtk.main_iteration(False)
                self.entry30.set_text('')
                self.textview3.get_buffer().set_text('')
                self.progressbar6.hide()
                self.attachment_model.clear()
                self.settings['server'] = self.entry24.get_text()
                self.settings['port'] = self.entry25.get_text()
                self.settings['user'] = self.entry26.get_text()
                self.settings['password'] = self.entry27.get_text()
                self.settings['from_addr'] = self.entry28.get_text()
                self.settings['rcpt_to'] = self.entry29.get_text()
                self.settings['authenticate'] = self.checkbutton35.get_active()
                self.store_settings_db()
                self.unselect()
                return True

        def on_checkbutton35_toggled(self, obj):
                self.entry26.set_sensitive(obj.get_active())
                self.entry27.set_sensitive(obj.get_active())

        def send_mail(self, obj):
                host = self.entry24.get_text().strip()
                if not host:
                        self.expander28.set_expanded(True)
                        self.entry24.grab_focus()
                        self.show_dialog15(_('Server missing!'))
                        return True
                user = self.entry26.get_text().strip()
                if not user and self.checkbutton35.get_active():
                        self.expander28.set_expanded(True)
                        self.entry26.grab_focus()
                        self.show_dialog15(_('User missing!'))
                        return True
                if not user:
                        user = 1
                passwd = self.entry27.get_text().strip()
                if not passwd and self.checkbutton35.get_active():
                        self.expander28.set_expanded(True)
                        self.entry27.grab_focus()
                        self.show_dialog15(_('Password missing!'))
                        return True
                if not passwd:
                        passwd = 1
                port = self.entry25.get_text().strip()
                if not port:
                        port = '25'
                from_addr = self.entry28.get_text().strip()
                if not from_addr:
                        self.entry28.grab_focus()
                        self.show_dialog15(_('Sender address missing!'))
                        return True
                rcpt_to = self.entry29.get_text().strip()
                if not rcpt_to:
                        self.entry29.grab_focus()
                        self.show_dialog15(_('Destination address missing!'))
                        return True
                subject = self.entry30.get_text().strip()
                if not subject:
                        self.entry30.grab_focus()
                        self.show_dialog15(_('Subject missing!'))
                        return True
                thread = threading.Thread(target=self.smtp_connect, args=(host, user, passwd, from_addr, rcpt_to, subject, port))
                thread.start()

        def smtp_connect(self, host, user, passwd, from_addr, rcpt_to, subject, port):
                def pulse_pb(pb, event):
                        while not event.isSet():
                                gobject.idle_add(pb.pulse)
                                time.sleep(0.1)
                gobject.idle_add(self.progressbar6.set_fraction, 0)
                gobject.idle_add(self.progressbar6.show)
                gobject.idle_add(self.progressbar6.set_text, _('connecting...'))
                while gtk.events_pending():
                        gtk.main_iteration(False)
                thread0 = threading.Thread(target=pulse_pb, args=(self.progressbar6, self.mail_event))
                thread0.start()
                try:
                        from gtkrg_smtp import Smtp
                        smtp = Smtp(host, user, passwd, self.checkbutton35.get_active(), self.show_dialog15, int(port))
                except:
                        gobject.idle_add(self.progressbar6.set_text, _('connection failed!'))
                        self.mail_event.set()
                        thread0.join()
                        self.mail_event.clear()
                        gobject.idle_add(self.progressbar6.set_fraction, 0)
                        gobject.idle_add(self.progressbar6.set_text, '')
                        gobject.idle_add(self.progressbar6.hide)
                        return True
                self.mail_event.set()
                thread0.join()
                self.mail_event.clear()
                smtp.from_addr(from_addr)
                smtp.rcpt_to(rcpt_to)
                smtp.subject(subject)
                textbuffer = self.textview3.get_buffer()
                start_, end_ = textbuffer.get_bounds()
                message = textbuffer.get_text(start_, end_)
                smtp.message(message or subject)
                for row in self.attachment_model:
                        smtp.attach(row[1])
                gobject.idle_add(self.progressbar6.set_text, _('sending...'))
                while gtk.events_pending():
                        gtk.main_iteration(False)
                thread = threading.Thread(target=self.deliver, args=(smtp,))
                thread.start()
                thread2 = threading.Thread(target=self.pulse_progressbar, args=(thread,))
                thread2.start()
                
        def deliver(self, smtp):
                self.smtp = smtp
                mail_delivered = self.smtp.send()
                if mail_delivered:
                        self.mail_event.set()
                self.smtp.close()

        def pulse_progressbar(self, thread):
                while thread.isAlive():
                        gobject.idle_add(self.progressbar6.pulse)
                        while gtk.events_pending():
                                gtk.main_iteration(False)
                        time.sleep(0.5)
                gobject.idle_add(self.progressbar6.set_fraction, 1)
                if self.mail_event.isSet():
                        gobject.idle_add(self.progressbar6.set_text, _('Mail successfully sent!'))
                        self.mail_event.clear()
                else:
                        gobject.idle_add(self.progressbar6.set_fraction, 0)
                        gobject.idle_add(self.progressbar6.set_text, '')
                        gobject.idle_add(self.progressbar6.hide)
# END OF SMTP CLIENT ===================================================================================================
                
        def show_dialog15(self, text):
                dialog15 = self.messagedialog(_("Error!"), text, None, gtk.BUTTONS_CLOSE, self.window8, 0, gtk.MESSAGE_ERROR, None, (320,140))
                dialog15.set_keep_above(True)
                gtk.gdk.beep()
                response = dialog15.run()
                if response == gtk.RESPONSE_CLOSE or response == gtk.RESPONSE_DELETE_EVENT:
                        dialog15.destroy()

# PRINT METHODS =========================================================================================================

        def print_dialog(self, obj):
                self.print_operation = gtk.PrintOperation()
                self.print_operation.set_unit(gtk.UNIT_PIXEL)
                if self.print_settings: 
                        self.print_operation.set_print_settings(self.print_settings)
                else:
                        self.print_settings = gtk.PrintSettings()
                if not self.page_setup:
                        self.page_setup = gtk.PageSetup()
                        self.page_setup.set_orientation(gtk.PAGE_ORIENTATION_PORTRAIT)
                        paper_size = gtk.PaperSize(gtk.PAPER_NAME_A4)
                        self.page_setup.set_paper_size_and_default_margins(paper_size)
                self.print_operation.set_default_page_setup(self.page_setup)
                self.print_operation.set_n_pages(1)
                self.print_operation.set_show_progress(True)
                self.print_operation.set_custom_tab_label('Settings')
                if not os.path.isfile('/usr/bin/evince') and not os.path.isfile('/usr/local/bin/evince'):
                        self.print_operation.connect('preview', self.print_preview)
                self.print_operation.connect('create_custom_widget', self.create_custom_widget)
                self.print_operation.connect('begin_print', self.begin_print)
                self.draw_page_id = self.print_operation.connect('draw_page', self.draw_page)
                res = self.print_operation.run(gtk.PRINT_OPERATION_ACTION_PRINT_DIALOG, None)
                if res == gtk.PRINT_OPERATION_RESULT_APPLY:
                        self.print_settings = self.print_operation.get_print_settings()

        def print_preview(self, operation, preview, context, parent):
                self.window2.set_modal(False)
                def destroy_win(*args):
                        self.window2.set_modal(True)
                        args[-1].destroy()
                print_settings = operation.get_print_settings()
                scale = print_settings.get_scale() / 100.0
                win = gtk.Window()
                win.set_title('Preview')
                area = gtk.DrawingArea()
                button = gtk.Button('Close')
                vbox = gtk.VBox()
                vbox.pack_start(area)
                bbox = gtk.HButtonBox()
                bbox.pack_start(button)
                vbox.pack_start(bbox)
                win.add(vbox)
                win.set_icon_from_file(self.get_icon_path('gtkrawgallery.png'))
                win.connect('destroy', destroy_win, win)
                button.connect('clicked', destroy_win, win)
                area.connect('expose_event', self.on_preview_exposed, scale)
                area.set_size_request(int(self.page_setup.get_paper_width(gtk.UNIT_MM)), int(self.page_setup.get_paper_height(gtk.UNIT_MM)))
                win.set_resizable(False)
                win.show_all()
        
        def on_preview_exposed(self, area, event, scale):
                self.print_operation.handler_block(self.draw_page_id)
                while gtk.events_pending():
                        gtk.main_iteration(False)
                cr = area.window.cairo_create()
                cr.rectangle(0,0, self.page_setup.get_paper_width(gtk.UNIT_MM), self.page_setup.get_paper_height(gtk.UNIT_MM))
                cr.stroke_preserve()
                cr.set_source_rgb(1,1,1)
                cr.fill()
                w, h = int(self.page_setup.get_page_width(gtk.UNIT_MM) * scale), int(self.page_setup.get_page_height(gtk.UNIT_MM) * scale)
                if self.list[self.indice].lower().endswith(self.raw_formats):
                        if self.window2.get_property('visible') and self.raw_preview:
                                pixbuf = self.image28.get_pixbuf()
                        else:
                                options = self.dcraw_options2()
                                self.extract_fullsize_image(self.get_real_path(self.list[self.indice]), options, True)
                                self.full_process.join()
                                pixbuf = gtk.gdk.pixbuf_new_from_file(os.path.join(self.temp, 'fullsize_image.tiff'))
                else:
                        if self.window2.get_property('visible'):
                                pixbuf = self.image28.get_pixbuf()
                        else:
                                pixbuf = self.swin2_image.get_pixbuf()
                if pixbuf:
                        pixbuf = pixbuf.copy()
                else:
                        return True
                width, height = pixbuf.get_width(), pixbuf.get_height()
                ratio = float(width) / height
                rotated = False
                if not rotated:
                        if ratio > 1:
                                width, height = w, int(w / ratio)
                        else:
                                width, height = int(h * ratio), h
                else:
                        width, height = h, int(h / ratio)
                pixbuf = pixbuf.scale_simple(width, height, gtk.gdk.INTERP_BILINEAR)
                if self.checkbutton41.get_active() and not self.window2.get_property('visible'):
                        pixbuf = self.profile_pixbuf(pixbuf)
                cr.set_source_pixbuf(pixbuf, int(self.page_setup.get_left_margin(gtk.UNIT_MM)), int(self.page_setup.get_top_margin(gtk.UNIT_MM)))
                cr.paint()
                self.print_operation.handler_unblock(self.draw_page_id)
                        
        def create_custom_widget(self, operation):
                tree = gtk.Builder()
                tree.add_from_file(os.path.join(gtkrg_datadir, 'print.ui'))
                self.hbox97 = tree.get_object('hbox97')
                self.spinbutton60 = tree.get_object('spinbutton60')
                self.spinbutton61 = tree.get_object('spinbutton61')
                self.spinbutton62 = tree.get_object('spinbutton62')
                self.spinbutton63 = tree.get_object('spinbutton63')
                self.button120 = tree.get_object('button120')
                self.checkbutton36 = tree.get_object('checkbutton36')
                self.label208 = tree.get_object('label208')
                self.label218 = tree.get_object('label218')
                self.label219 = tree.get_object('label219')
                self.combobox17 = tree.get_object('combobox17')
                cell = gtk.CellRendererText()
                boxmodel17 = gtk.ListStore(str)
                boxmodel17.append(['millimeters'])
                boxmodel17.append(['inches'])
                self.combobox17.pack_start(cell, True)
                self.combobox17.add_attribute(cell, 'text', 0)
                self.combobox17.set_model(boxmodel17)
                self.combobox17.set_active(0)
                self.combobox17.connect('changed', self.on_combobox17_changed)
                self.spinbutton60.connect('button_release_event', self.set_margins, self.page_setup.set_left_margin)
                self.spinbutton61.connect('button_release_event', self.set_margins, self.page_setup.set_right_margin)
                self.spinbutton62.connect('button_release_event', self.set_margins, self.page_setup.set_top_margin)
                self.spinbutton63.connect('button_release_event', self.set_margins, self.page_setup.set_bottom_margin)
                self.button120.connect('clicked', self.setup_page)
                self.checkbutton36.connect('toggled', self.on_checkbutton36_toggled)
                self.spinbutton60.set_value(self.page_setup.get_left_margin(gtk.UNIT_MM))
                self.spinbutton61.set_value(self.page_setup.get_right_margin(gtk.UNIT_MM))
                self.spinbutton62.set_value(self.page_setup.get_top_margin(gtk.UNIT_MM))
                self.spinbutton63.set_value(self.page_setup.get_bottom_margin(gtk.UNIT_MM))
                size = self.page_setup.get_paper_size()
                orientation = self.page_setup.get_orientation()
                if orientation == gtk.PAGE_ORIENTATION_PORTRAIT:
                        orientation = 'Portrait'
                elif orientation == gtk.PAGE_ORIENTATION_REVERSE_PORTRAIT:
                        orientation = 'Reverse portrait'
                elif orientation == gtk.PAGE_ORIENTATION_REVERSE_LANDSCAPE:
                        orientation = 'Reverse landscape'
                elif orientation == gtk.PAGE_ORIENTATION_LANDSCAPE:
                        orientation = 'Landscape'
                self.label208.set_text(_('Orientation: %s') % orientation)
                self.label218.set_text(_('Paper: %s') % size.get_name().upper())
                self.label219.set_text(_('Size: %.1f x %.1f millimeters') % (self.page_setup.get_paper_width(gtk.UNIT_MM), \
                                                self.page_setup.get_paper_height(gtk.UNIT_MM)))
                if all([self.spinbutton60.get_value() == 0, self.spinbutton61.get_value() == 0, self.spinbutton62.get_value() == 0, self.spinbutton63.get_value() == 0]):
                        self.checkbutton36.set_active(True)
                self.hbox97.unparent()
                return self.hbox97
                
        def on_checkbutton36_toggled(self, obj):
                if self.checkbutton36.get_active():
                        self.page_setup.set_left_margin(0, gtk.UNIT_MM)
                        self.page_setup.set_right_margin(0, gtk.UNIT_MM)
                        self.page_setup.set_top_margin(0, gtk.UNIT_MM)
                        self.page_setup.set_bottom_margin(0, gtk.UNIT_MM)
                else:
                        self.page_setup.set_paper_size_and_default_margins(self.page_setup.get_paper_size())
                self.on_combobox17_changed(self.combobox17)

        def on_radiobutton7_toggled(self, obj):
                if self.radiobutton7.get_active():
                        if os.path.splitext(self.list[self.indice])[1].lower() in self.formats:
                                self.menuitem34.set_sensitive(True)
                        else:
                                self.menuitem34.set_sensitive(False)
                else:
                        self.menuitem34.set_sensitive(False)

        def begin_print(self, operation, context):
                if self.combobox17.get_active():
                        unit = gtk.UNIT_INCH
                else:
                        unit = gtk.UNIT_MM
                self.print_settings = operation.get_print_settings()
                if self.checkbutton36.get_active():
                        self.page_setup.set_left_margin(0, unit)
                        self.page_setup.set_right_margin(0, unit)
                        self.page_setup.set_top_margin(0, unit)
                        self.page_setup.set_bottom_margin(0, unit)
                else:
                        self.page_setup.set_left_margin(self.spinbutton60.get_value(), unit)
                        self.page_setup.set_right_margin(self.spinbutton61.get_value(), unit)
                        self.page_setup.set_top_margin(self.spinbutton62.get_value(), unit)
                        self.page_setup.set_bottom_margin(self.spinbutton63.get_value(), unit)
                self.print_settings.set_quality(gtk.PRINT_QUALITY_HIGH)
                operation.set_default_page_setup(self.page_setup)
                operation.set_print_settings(self.print_settings)

        def set_margins(self, obj, event, fun):
                if self.combobox17.get_active():
                        unit = gtk.UNIT_INCH
                else:
                        unit = gtk.UNIT_MM
                fun(obj.get_value(), unit)

        def get_print_pixbuf(self, filename):
                self.progressbar2.set_text(_('Processing Image...'))
                self.progressbar2.set_fraction(0)
                self.window5.show()
                while gtk.events_pending():
                        gtk.main_iteration(False)
                if self.list[self.indice].lower().endswith(self.raw_formats):
                        if self.window2.get_property('visible'):
                                self.load_raw_preview(self.rect.width, self.rect.height)
                        else:
                                options = self.dcraw_options2()
                                self.extract_fullsize_image(self.get_real_path(self.list[self.indice]), options)
                if self.history:
                        wand = self.process_fullsize_image(True)
                else:
                        if self.list[self.indice].lower().endswith(self.formats):
                                wand = MagickImage(self.get_real_path(self.list[self.indice]))
                        else:
                                if hasattr(self, 'full_process') and self.full_process.is_alive():
                                        self.full_process.join()
                                wand = MagickImage(os.path.join(self.temp, 'fullsize_image.tiff'))
                if self.checkbutton41.get_active():
                        self.load_embedded_profile(wand)
                        wand = self.profile_image(wand)
                width, height = wand.size
                ratio = float(width) / height
                rotated = False
                w = int(self.page_setup.get_page_width(gtk.UNIT_PIXEL))
                h = int(self.page_setup.get_page_height(gtk.UNIT_PIXEL))
                if not rotated:
                        if ratio > 1:
                                width, height = w, int(w / ratio)
                        else:
                                width, height = int(h * ratio), h
                else:
                        width, height = h, int(h / ratio)
                wand.scale(width, height)
                self.window5.hide()
                self.progressbar2.set_text('')
                return self.wand2pixbuf(wand)
                 
        def draw_page(self, operation, context, page_nr):
                cr = context.get_cairo_context()
                with warnings.catch_warnings():
                        warnings.simplefilter("ignore")
                        pixbuf = self.get_print_pixbuf(self.list[self.indice])
                cr.set_source_pixbuf(pixbuf, 0, 0)
                print 'DRAW PAGE'
                cr.paint()
                
        def setup_page(self, obj):
                if self.combobox17.get_active():
                        unit = gtk.UNIT_INCH
                else:
                        unit = gtk.UNIT_MM
                self.page_setup = gtk.print_run_page_setup_dialog(None, self.page_setup, self.print_settings)
                self.print_operation.set_default_page_setup(self.page_setup)
                orientation = self.page_setup.get_orientation()
                if orientation == gtk.PAGE_ORIENTATION_PORTRAIT:
                        orientation = 'Portrait'
                elif orientation == gtk.PAGE_ORIENTATION_REVERSE_PORTRAIT:
                        orientation = 'Reverse portrait'
                elif orientation == gtk.PAGE_ORIENTATION_REVERSE_LANDSCAPE:
                        orientation = 'Reverse landscape'
                elif orientation == gtk.PAGE_ORIENTATION_LANDSCAPE:
                        orientation = 'Landscape'
                self.label208.set_text(_('Orientation: %s') % orientation)
                size = self.page_setup.get_paper_size()
                self.label218.set_text(_('Paper: %s') % size.get_name().upper())
                self.label219.set_text(_('Size: %.3f x %.3f %s') % (self.page_setup.get_paper_width(unit), \
                                        self.page_setup.get_paper_height(unit), self.combobox17.get_active_text()))
                self.on_combobox17_changed(self.combobox17)

        def on_combobox17_changed(self, obj):
                if obj.get_active():
                        unit = gtk.UNIT_INCH
                        step = 0.01
                else:
                        unit = gtk.UNIT_MM
                        step = 1.0
                self.spinbutton60.set_increments(step,0)
                self.spinbutton61.set_increments(step,0)
                self.spinbutton62.set_increments(step,0)
                self.spinbutton63.set_increments(step,0)
                self.label219.set_text(_('Size: %.3f x %.3f %s') % (self.page_setup.get_paper_width(unit), \
                                       self.page_setup.get_paper_height(unit), self.combobox17.get_active_text()))
                self.spinbutton60.set_value(self.page_setup.get_left_margin(unit))
                self.spinbutton61.set_value(self.page_setup.get_right_margin(unit))
                self.spinbutton62.set_value(self.page_setup.get_top_margin(unit))
                self.spinbutton63.set_value(self.page_setup.get_bottom_margin(unit))
#END OF PRINT METHODS ==========================================================================================

        def make_aboutdialog(self):
                def launch_browser(dialog, link):
                        try:
                                import webbrowser
                                webbrowser.open(link)
                        except:
                                pass
                self.aboutdialog = gtk.AboutDialog()
                self.aboutdialog.set_name('GTKRawGallery')
                p = open(os.path.join(gtkrg_datadir, 'COPYING'))
                license_ = p.read()
                p.close()
                self.aboutdialog.set_version(__VERSION__)
                self.aboutdialog.set_copyright('Copyright ' + __COPYRIGHT__)
                self.aboutdialog.set_license(license_)
                self.aboutdialog.set_comments('A workflow oriented photo manager\nfor digital camera raw image development.')
                self.aboutdialog.set_authors(['Daniele Isca    email: <bit123@sourceforge.net>'])
                gtk.about_dialog_set_url_hook(launch_browser)
                self.aboutdialog.set_website('http://sourceforge.net/projects/gtkrawgallery')
                self.aboutdialog.set_website_label('Visit the Project Page')
                self.aboutdialog.set_logo(gtk.gdk.pixbuf_new_from_file_at_size(self.get_icon_path('splash.png'), 120, 90))
                self.aboutdialog.set_size_request(390,300)
                self.aboutdialog.set_position(gtk.WIN_POS_CENTER)

        def on_togglebutton6_toggled(self, obj):
                if obj.get_active():
                        self.image92.set_from_stock(gtk.STOCK_CONNECT, gtk.ICON_SIZE_BUTTON)
                        obj.set_tooltip_text(_('Disconnect GUI'))
                        self.connect_dcraw_gui()
                        gobject.idle_add(self.load_raw_preview, self.rect.width, self.rect.height)
                else:
                        self.image92.set_from_stock(gtk.STOCK_DISCONNECT, gtk.ICON_SIZE_BUTTON)
                        obj.set_tooltip_text(_('Connect GUI'))
                        self.disconnect_dcraw_gui()
                self.image92.show()

        def disconnect_dcraw_gui(self):
                self.combobox5.handler_block(self.id42)
                self.spinbutton4.handler_block(self.id43)
                self.spinbutton5.handler_block(self.id44)
                self.spinbutton6.handler_block(self.id45)
                self.spinbutton7.handler_block(self.id46)
                self.spinbutton8.handler_block(self.id47)
                self.spinbutton9.handler_block(self.id48)
                self.spinbutton1.handler_block(self.id49)
                self.spinbutton11.handler_block(self.id65)
                self.spinbutton35.handler_block(self.id79)
                self.hscale3.handler_block(self.id51)
                self.radiobutton11.handler_block(self.id52)
                self.checkbutton8.handler_block(self.id53)
                self.checkbutton9.handler_block(self.id54)
                self.checkbutton10.handler_block(self.id55)
                self.checkbutton11.handler_block(self.id56)
                self.checkbutton12.handler_block(self.id57)
                self.checkbutton13.handler_block(self.id58)
                self.checkbutton14.handler_block(self.id59)
                self.combobox2.handler_block(self.id60)
                self.combobox3.handler_block(self.id61)
                self.combobox4.handler_block(self.id62)
                self.combobox39.handler_block(self.id108)
                self.hscale15.handler_block(self.id66)
                self.checkbutton16.handler_block(self.id67)
                self.checkbutton23.handler_block(self.id76)
                self.spinbutton39.handler_block(self.id82)
                self.hscale25.handler_block(self.id83)
                self.hscale28.handler_block(self.id84)
                self.hscale34.handler_block(self.id85)
                self.spinbutton42.handler_block(self.id86)
                self.spinbutton45.handler_block(self.id87)
                self.checkbutton1.handler_block(self.id96)
                self.hscale52.handler_block(self.id98)
                self.hscale53.handler_block(self.id99)
                
        def connect_dcraw_gui(self):
                self.combobox5.handler_unblock(self.id42)
                self.spinbutton4.handler_unblock(self.id43)
                self.spinbutton5.handler_unblock(self.id44)
                self.spinbutton6.handler_unblock(self.id45)
                self.spinbutton7.handler_unblock(self.id46)
                self.spinbutton8.handler_unblock(self.id47)
                self.spinbutton9.handler_unblock(self.id48)
                self.spinbutton1.handler_unblock(self.id49)
                self.spinbutton11.handler_unblock(self.id65)
                self.spinbutton35.handler_unblock(self.id79)
                self.hscale3.handler_unblock(self.id51)
                self.radiobutton11.handler_unblock(self.id52)
                self.checkbutton8.handler_unblock(self.id53)
                self.checkbutton9.handler_unblock(self.id54)
                self.checkbutton10.handler_unblock(self.id55)
                self.checkbutton11.handler_unblock(self.id56)
                self.checkbutton12.handler_unblock(self.id57)
                self.checkbutton13.handler_unblock(self.id58)
                self.checkbutton14.handler_unblock(self.id59)
                self.combobox2.handler_unblock(self.id60)
                self.combobox3.handler_unblock(self.id61)
                self.combobox4.handler_unblock(self.id62)
                self.combobox39.handler_unblock(self.id108)
                self.hscale15.handler_unblock(self.id66)
                self.checkbutton16.handler_unblock(self.id67)
                self.checkbutton23.handler_unblock(self.id76)
                self.spinbutton39.handler_unblock(self.id82)
                self.hscale25.handler_unblock(self.id83)
                self.hscale28.handler_unblock(self.id84)
                self.hscale34.handler_unblock(self.id85)
                self.spinbutton42.handler_unblock(self.id86)
                self.spinbutton45.handler_unblock(self.id87)
                self.checkbutton1.handler_unblock(self.id96)
                self.hscale52.handler_unblock(self.id98)
                self.hscale53.handler_unblock(self.id99)
                
        def on_eventbox44_motion(self, obj, event):
                x = event.window.get_pointer()[0]
                if x > 256:
                        x = 256
                self.label56.set_text('x: %d (%d)' % (x, x * 256))
                
        def on_monitor_event(self, monitor, file1, file2, event):
                if not file1.get_basename().lower().endswith(self.formats + self.raw_formats):
                        if event == gio.FILE_MONITOR_EVENT_CREATED:
                                if not file1.get_basename().startswith('magick'):
                                        gobject.idle_add(self.refresh_directory_tree)      
                        elif event == gio.FILE_MONITOR_EVENT_DELETED:
                                path = file1.get_path()
                                if self.monitors.has_key(path):
                                        self.monitors[path].cancel()
                                        del self.monitors[path]
                                if not file1.get_basename().startswith('magick'):
                                        gobject.idle_add(self.refresh_directory_tree)
                                               
        def get_file_path_from_dnd_dropped_uri(self, uri):      # get the path to file
                from urllib import url2pathname
                path = ""
                if uri.startswith('file://'):           # nautilus, rox
                        path = uri[7:]                  # 7 is len('file://')
                elif uri.startswith('file:\\\\\\'):     # windows
                        path = uri[8:]                  # 8 is len('file:///')
                elif uri.startswith('file:'):           # xffm
                        path = uri[5:]                  # 5 is len('file:')
                path = url2pathname(path)       # escape special chars
                path = path.strip('\r\n\x00')           # remove \r\n and NULL
                return path

        def on_drag_data_received(self, widget, context, x, y, selection, target_type, timestamp):
                if target_type == 80:
                        uri = selection.data.strip('\r\n\x00')
                        uri_splitted = uri.split()                                      # we may have more than one file dropped
                        path = self.get_file_path_from_dnd_dropped_uri(uri_splitted[0]) # but only the first one is gotten
                        if os.path.isfile(path) and path.lower().endswith(self.formats + self.raw_formats):
                                self.open_with_gtkrawgallery(path)
                        elif os.path.isdir(path):
                                self.open_last_folder(path, None)
                                
        def get_icon_path(self, filename):
                if os.path.exists(os.path.join(gtkrg_datadir, 'icons', filename)):
                        icon_path = os.path.join(gtkrg_datadir, 'icons', filename)
                if os.path.exists(os.path.join('/usr/share/pixmaps', filename)):
                        icon_path = os.path.join('/usr/share/pixmaps', filename)
                elif os.path.exists(os.path.join('/usr/local/share/pixmaps', filename)):
                        icon_path = os.path.join('/usr/local/share/pixmaps', filename)
                return icon_path
                
        def show_shortcuts(self, obj):
                self.dialog28.show()
                response = self.dialog28.run()
                if response == gtk.RESPONSE_CLOSE or response == gtk.RESPONSE_DELETE_EVENT:
                        self.dialog28.hide()

################################ IMAGE TOOLS METHODS #####################################################################

        def on_color_spinbutton_released(self, obj, event):   
                if obj == self.spinbutton10:
                        threading.Thread(target=self.first_to_change_colors, args=(self.on_level_changed, self.hscale4, None)).start()
                elif obj == self.spinbutton12:
                        threading.Thread(target=self.first_to_change_colors, args=(self.on_level_changed, self.hscale5, None)).start()
                elif obj == self.spinbutton13:
                        threading.Thread(target=self.first_to_change_colors, args=(self.on_level_changed, self.hscale6, None)).start()
                elif obj == self.spinbutton14:
                        threading.Thread(target=self.first_to_change_colors, args=(self.on_brightness_and_saturation_changed, self.hscale7, event)).start()
                elif obj == self.spinbutton15:
                        threading.Thread(target=self.first_to_change_colors, args=(self.on_brightness_and_saturation_changed, self.hscale8, event)).start()
                elif obj == self.spinbutton16:
                        threading.Thread(target=self.first_to_change_colors, args=(self.color_balance, self.hscale9, event)).start()
                elif obj == self.spinbutton17:
                        threading.Thread(target=self.first_to_change_colors, args=(self.color_balance, self.hscale10, event)).start()
                elif obj == self.spinbutton18:
                        threading.Thread(target=self.first_to_change_colors, args=(self.color_balance, self.hscale11, event)).start()
                elif obj == self.spinbutton20:
                        threading.Thread(target=self.first_to_change_colors, args=(self.on_contrast_changed, self.hscale12, event)).start()
                elif obj == self.spinbutton21:
                        threading.Thread(target=self.first_to_change_colors, args=(self.on_midtones_contrast_changed, self.hscale13, event)).start()
                elif obj == self.spinbutton22:
                        threading.Thread(target=self.first_to_change_colors, args=(self.on_midtones_contrast_changed, self.hscale14, event)).start()
                elif obj == self.spinbutton27 or obj == self.spinbutton28:
                        threading.Thread(target=self.first_to_change_colors, args=(self.fine_rotation, obj, event)).start()
                elif obj == self.spinbutton37:
                        threading.Thread(target=self.first_to_change_colors, args=(self.on_brightness_and_saturation_changed, self.hscale18, event)).start()
                elif obj == self.spinbutton48:
                        threading.Thread(target=self.first_to_change_colors, args=(self.on_channel_mixer_changed, self.hscale31, None)).start()
                elif obj == self.spinbutton47:
                        threading.Thread(target=self.first_to_change_colors, args=(self.on_channel_mixer_changed, self.hscale32, None)).start()
                elif obj == self.spinbutton46:
                        threading.Thread(target=self.first_to_change_colors, args=(self.on_channel_mixer_changed, self.hscale33, None)).start()
                elif obj == self.spinbutton55:
                        threading.Thread(target=self.first_to_change_colors, args=(self.apply_curve, self.hscale38, None)).start()
                elif obj == self.spinbutton57:
                        threading.Thread(target=self.first_to_change_colors, args=(self.apply_curve, self.hscale39, None)).start()
                elif obj == self.spinbutton58:
                        threading.Thread(target=self.first_to_change_colors, args=(self.apply_curve, self.hscale40, None)).start()
                elif obj == self.spinbutton59:
                        threading.Thread(target=self.first_to_change_colors, args=(self.apply_curve, self.hscale41, None)).start()
                elif obj == self.spinbutton72:
                        threading.Thread(target=self.first_to_change_colors, args=(self.apply_colortemp, self.spinbutton72, None)).start()
                elif obj == self.spinbutton73:
                        threading.Thread(target=self.first_to_change_colors, args=(self.apply_colortemp, self.spinbutton73, None)).start()
                        
        def first_to_change_colors(self, fun, obj, event):
                self.pila.append(0)
                time.sleep(0.3)
                if len(self.pila) > 1:
                        self.pila.pop()
                else:
                        gobject.idle_add(fun, obj, event)     
                        self.pila = []

        def on_hscale_scrolled(self, obj, event):
                return True

        def on_spinbutton_scrolled(self, obj, event):
                return True

        def on_hscale_change_value(self, obj, event, value, spinbutton1, spinbutton2=None):
                spinbutton1.set_value(value)
                if spinbutton2:
                        spinbutton2.set_value(value)
                
        def on_spinbutton_value_changed(self, obj, hscale):
                hscale.get_adjustment().value = obj.get_value()
                hscale.queue_draw()
                
        def on_spinbutton30_value_changed(self, obj):
                self.on_drawing_area_expose_event(self.drawing_area, None, True)
                
        def on_spinbutton49_value_changed(self, obj):
                self.hscale35.get_adjustment().value = obj.get_value()
                self.hscale35.queue_draw()
                self.spinbutton52.set_value(obj.get_value())
                
        def on_spinbutton50_value_changed(self, obj):
                self.hscale36.get_adjustment().value = obj.get_value()
                self.hscale36.queue_draw()
                self.spinbutton54.set_value(obj.get_value())
        
        def on_spinbutton52_value_changed(self, obj):
                self.hscale35.get_adjustment().value = obj.get_value()
                self.hscale35.queue_draw()
                self.spinbutton49.set_value(obj.get_value())
                
        def on_spinbutton54_value_changed(self, obj):
                self.hscale36.get_adjustment().value = obj.get_value()
                self.hscale36.queue_draw()
                self.spinbutton50.set_value(obj.get_value())
        
        def on_spinbutton38_value_changed(self, obj):
                self.on_drawing_area_expose_event(self.drawing_area, None, True)
                
        def on_spinbutton19_value_changed(self, obj):
                self.spinbutton23.set_value(obj.get_value() / self.ratio)

        def on_spinbutton23_value_changed(self, obj):
                self.spinbutton19.set_value(obj.get_value() * self.ratio)

        def on_spinbutton24_value_changed(self, obj):
                self.spinbutton26.set_value(obj.get_value())

        def on_spinbutton26_value_changed(self, obj):
                self.spinbutton24.set_value(obj.get_value())

        def on_checkbutton18_toggled(self, *obj):
                if self.checkbutton18.get_active():
                        self.spinbutton19.handler_unblock(self.id72)
                        self.spinbutton23.handler_unblock(self.id73)
                else:
                        self.spinbutton19.handler_block(self.id72)
                        self.spinbutton23.handler_block(self.id73)

        def on_checkbutton19_toggled(self, *obj):
                if self.checkbutton19.get_active():
                        self.spinbutton24.handler_unblock(self.id74)
                        self.spinbutton26.handler_unblock(self.id75)
                else:
                        self.spinbutton24.handler_block(self.id74)
                        self.spinbutton26.handler_block(self.id75)

        def on_checkbutton20_toggled(self, *obj):
                if not self.checkbutton20.get_active() and not self.workflow_copy:
                        dialog25 = self.messagedialog(_("Attention!"), _('No workflow to save.\nPlease, make a new one!'), None, gtk.BUTTONS_CLOSE, None,
                                                        gtk.DIALOG_MODAL, gtk.MESSAGE_WARNING, None, (330,180))
                        dialog25.set_keep_above(True)
                        response = dialog25.run()
                        if response == gtk.RESPONSE_CLOSE or response == gtk.RESPONSE_DELETE_EVENT:
                                dialog25.destroy()
                        
        def on_checkbutton21_toggled(self, *obj):
                if self.checkbutton21.get_active(): 
                        self.combobox12.set_sensitive(False)
                        self.spinbutton30.handler_unblock(self.id77)
                        self.spinbutton38.handler_unblock(self.id78)
                        self.on_drawing_area_expose_event(self.drawing_area, None, True)
                else:                               
                        self.combobox12.set_sensitive(True)
                        self.spinbutton30.handler_block(self.id77)
                        self.spinbutton38.handler_block(self.id78)
                        self.on_drawing_area_expose_event(self.drawing_area, None, True)

        def on_checkbutton24_toggled(self, obj):
                if obj.get_active():
                        self.draw_grid(obj)
                else:
                        if self.checkbutton41.get_active():
                                self.image28.set_from_pixbuf(self.profile_pixbuf(self.pblist[self.pbindex]))
                        else:
                                self.image28.set_from_pixbuf(self.pblist[self.pbindex])
                        self.image28.show()

        def show_overexposed_pixels(self, obj, event, data):
                self.overexposed = True
                unexposed = self.image28.get_pixbuf()
                wand = self.pixbuf2wand(unexposed)
                if data == 'over':
                        wand.opaque_paint(Color('white'), Color('black'), 0)
                else:
                        wand.opaque_paint(Color('black'), Color('white'), 0)
                pixbuf = self.wand2pixbuf(wand, True)
                while self.overexposed:
                        if not self.overexposed:
                                break
                        self.image28.set_from_pixbuf(pixbuf)
                        while gtk.events_pending():
                                gtk.main_iteration(False)
                        time.sleep(0.5)
                        self.image28.set_from_pixbuf(unexposed)
                        while gtk.events_pending():
                                gtk.main_iteration(False)
                        time.sleep(0.5)

        def on_exposure_buttons_left(self, obj, event):
                self.overexposed = False
        
        def on_combobox10_changed(self, obj):
                if self.combobox10.get_active() == 0:
                        self.hscale23.set_sensitive(True)
                else:
                        self.hscale23.set_sensitive(False)

        def on_combobox6_changed(self, obj):
                adj4 = self.hscale4.get_adjustment()
                adj5 = self.hscale5.get_adjustment()
                adj6 = self.hscale6.get_adjustment()
                channel = self.combobox6.get_active()
                adj4.value, adj6.value, adj5.value = self.RGB_levels[channel][0], self.RGB_levels[channel][1], self.RGB_levels[channel][2]
                self.spinbutton10.set_value(adj4.value)
                self.spinbutton12.set_value(adj5.value)
                self.spinbutton13.set_value(adj6.value) 
                
        def on_combobox8_changed(self, obj):
                if obj.get_active() == 2:
                        self.hscale49.set_sensitive(True)
                        self.spinbutton71.set_sensitive(True)
                        self.spinbutton32.set_sensitive(False)
                        self.spinbutton33.set_sensitive(False)
                        self.hscale20.set_sensitive(False)
                        self.hscale21.set_sensitive(False)
                elif obj.get_active() == 3:
                        self.hscale49.set_sensitive(False)
                        self.spinbutton71.set_sensitive(False)
                        self.spinbutton32.set_sensitive(True)
                        self.spinbutton33.set_sensitive(True)
                        self.hscale20.set_sensitive(True)
                        self.hscale21.set_sensitive(True)
                elif obj.get_active() == 4:
                        self.hscale49.set_sensitive(True)
                        self.spinbutton71.set_sensitive(True)
                        self.spinbutton32.set_sensitive(True)
                        self.spinbutton33.set_sensitive(True)
                        self.hscale20.set_sensitive(True)
                        self.hscale21.set_sensitive(True)
                else:
                        self.hscale49.set_sensitive(False)
                        self.spinbutton71.set_sensitive(False)
                        self.spinbutton32.set_sensitive(True)
                        self.spinbutton33.set_sensitive(True)
                        self.hscale20.set_sensitive(True)
                        self.hscale21.set_sensitive(True)
                                     
        def on_combobox13_changed(self, event):
                channel = self.combobox13.get_active()
                adj31 = self.hscale31.get_adjustment()
                adj32 = self.hscale32.get_adjustment()
                adj33 = self.hscale33.get_adjustment()
                adj31.value, adj32.value, adj33.value = self.mixer_levels[channel][0], self.mixer_levels[channel][1], self.mixer_levels[channel][2]
                self.spinbutton48.set_value(adj31.value)
                self.spinbutton47.set_value(adj32.value)
                self.spinbutton46.set_value(adj33.value)
                
        def reset_black_level(self, obj, data):
                self.hscale4.get_adjustment().set_value(0)
                self.spinbutton10.set_value(0)
                channel = self.combobox6.get_active()
                self.RGB_levels[channel][0] = 0
                if data:
                        self.wand = MagickImage(self.wand_list[self.wand_index].get_image())
                        self.on_level_changed(self.hscale4, None)
                self.image_was_changed = False

        def reset_white_level(self, obj, data):
                self.hscale5.get_adjustment().set_value(100)
                self.spinbutton12.set_value(100)
                channel = self.combobox6.get_active()
                self.RGB_levels[channel][2] = 100
                if data:
                        self.wand = MagickImage(self.wand_list[self.wand_index].get_image())
                        self.on_level_changed(self.hscale5, None)
                self.image_was_changed = False

        def reset_gamma(self, obj, data):
                self.hscale6.get_adjustment().set_value(1.00)
                self.spinbutton13.set_value(1.00)
                channel = self.combobox6.get_active()
                self.RGB_levels[channel][1] = 1.00
                if data:
                        self.wand = MagickImage(self.wand_list[self.wand_index].get_image())
                        self.on_level_changed(self.hscale6, None)
                self.image_was_changed = False
                
        def reset_levels(self, data, show=True):
                self.reset_black_level(self.button18, data)
                self.reset_white_level(self.button19, data)
                self.reset_gamma(self.button30, data)
                self.image_was_changed = False
                if show:
                        if self.expander17.get_expanded():
                                self.set_cairo_pixbuf()
                        elif self.expander8.get_expanded():
                                self.draw_grid(self.checkbutton24)
                        elif self.expander19.get_expanded():
                                self.draw_grid(self.checkbutton34)
                        elif self.window2_closed:
                                self.image28.clear()
                        else:
                                if self.checkbutton41.get_active():
                                        self.image28.set_from_pixbuf(self.profile_pixbuf(self.pblist[self.pbindex]))
                                else:
                                        self.image28.set_from_pixbuf(self.pblist[self.pbindex])
                                self.image28.show()

        def reset_brightness(self, obj, data):
                self.hscale7.get_adjustment().set_value(0)
                self.spinbutton14.set_value(0)
                self.bright_sat[0] = 0
                while gtk.events_pending():
                        gtk.main_iteration(False)
                if data:
                        self.wand = MagickImage(self.wand_list[self.wand_index].get_image())
                        self.on_brightness_and_saturation_changed(self.hscale7, None)
                self.image_was_changed = False

        def reset_saturation(self, obj, data):
                self.hscale8.get_adjustment().set_value(0)
                self.spinbutton15.set_value(0)
                self.bright_sat[1] = 0
                while gtk.events_pending():
                        gtk.main_iteration(False)
                if data:
                        self.wand = MagickImage(self.wand_list[self.wand_index].get_image())
                        self.on_brightness_and_saturation_changed(self.hscale8, None)
                self.image_was_changed = False

        def reset_hue(self, obj, data):
                self.hscale18.get_adjustment().set_value(0)
                self.spinbutton37.set_value(0)
                self.bright_sat[2] = 0
                while gtk.events_pending():
                        gtk.main_iteration(False)
                if data:
                        self.wand = MagickImage(self.wand_list[self.wand_index].get_image())
                        self.on_brightness_and_saturation_changed(self.hscale18, None)
                self.image_was_changed = False
                
        def reset_brightness_and_saturation(self, data, show=True):
                self.reset_saturation(self.button31, data)
                self.reset_brightness(self.button32, data)
                self.reset_hue(self.button87, data)
                if show:
                        if self.expander17.get_expanded():
                                self.set_cairo_pixbuf()
                        elif self.expander8.get_expanded():
                                self.draw_grid(self.checkbutton24)
                        elif self.expander19.get_expanded():
                                self.draw_grid(self.checkbutton34)
                        else:
                                if self.checkbutton41.get_active():
                                        self.image28.set_from_pixbuf(self.profile_pixbuf(self.pblist[self.pbindex]))
                                else:
                                        self.image28.set_from_pixbuf(self.pblist[self.pbindex])
                                self.image28.show()

        def reset_color_balance(self, obj, data, show=True):
                self.hscale9.get_adjustment().set_value(0)
                self.hscale10.get_adjustment().set_value(0)
                self.hscale11.get_adjustment().set_value(0)
                self.spinbutton16.set_value(0)
                self.spinbutton17.set_value(0)
                self.spinbutton18.set_value(0)
                self.c_balance= [0, 0, 0]
                while gtk.events_pending():
                        gtk.main_iteration(False)
                if data:
                        self.wand = MagickImage(self.wand_list[self.wand_index].get_image())
                        self.color_balance(self.hscale9, None)
                self.image_was_changed = False
                if show:
                        if self.expander17.get_expanded():
                                self.set_cairo_pixbuf()
                        elif self.expander8.get_expanded():
                                self.draw_grid(self.checkbutton24)
                        elif self.expander19.get_expanded():
                                self.draw_grid(self.checkbutton34)
                        else:
                                if self.checkbutton41.get_active():
                                        self.image28.set_from_pixbuf(self.profile_pixbuf(self.pblist[self.pbindex]))
                                else:
                                        self.image28.set_from_pixbuf(self.pblist[self.pbindex])
                                self.image28.show()
                
        def reset_contrast(self, obj, data, show=True):
                self.window2.window.set_cursor(self.watch)
                while gtk.events_pending():
                        gtk.main_iteration(False)
                self.hscale12.get_adjustment().set_value(0)
                self.spinbutton20.set_value(0)
                while gtk.events_pending():
                        gtk.main_iteration(False)
                if data:
                        self.wand = MagickImage(self.wand_list[self.wand_index].get_image())
                        self.pblist[self.pbindex] = self.wand2pixbuf(self.wand, True)
                self.image_was_changed = False
                if show:
                        if self.expander17.get_expanded():
                                self.set_cairo_pixbuf()
                        elif self.expander8.get_expanded():
                                self.draw_grid(self.checkbutton24)
                        elif self.expander19.get_expanded():
                                self.draw_grid(self.checkbutton34)
                        else:
                                if self.checkbutton41.get_active():
                                        self.image28.set_from_pixbuf(self.profile_pixbuf(self.pblist[self.pbindex]))
                                else:
                                        self.image28.set_from_pixbuf(self.pblist[self.pbindex])
                                self.image28.show()
                        self.get_channel()
                self.window2.window.set_cursor(None)
                
        def reset_midtones_contrast(self, obj, data, show=True):
                self.window2.window.set_cursor(self.watch)
                while gtk.events_pending():
                        gtk.main_iteration(False)
                self.hscale13.get_adjustment().set_value(0)
                self.hscale14.get_adjustment().set_value(50)
                self.spinbutton21.set_value(0)
                self.spinbutton22.set_value(50)
                while gtk.events_pending():
                        gtk.main_iteration(False)
                if data:
                        self.wand = MagickImage(self.wand_list[self.wand_index].get_image())
                        self.pblist[self.pbindex] = self.wand2pixbuf(self.wand, True)
                self.image_was_changed = False
                if show:
                        if self.expander17.get_expanded():
                                self.set_cairo_pixbuf()
                        elif self.expander8.get_expanded():
                                self.draw_grid(self.checkbutton24)
                        elif self.expander19.get_expanded():
                                self.draw_grid(self.checkbutton34)
                        else:
                                if self.checkbutton41.get_active():
                                        self.image28.set_from_pixbuf(self.profile_pixbuf(self.pblist[self.pbindex]))
                                else:
                                        self.image28.set_from_pixbuf(self.pblist[self.pbindex])
                                self.image28.show()
                        self.get_channel()
                self.window2.window.set_cursor(None)

        def reset_unsharp_mask(self):
                self.spinbutton40.set_value(1.0)
                self.spinbutton41.set_value(0)
                self.spinbutton43.set_value(1.0)
                self.spinbutton44.set_value(0.05)
                while gtk.events_pending():
                        gtk.main_iteration(False)

        def reset_distort(self, *obj):
                self.spinbutton51.set_value(0)
                self.spinbutton52.set_value(0)
                self.spinbutton53.set_value(0)
                self.spinbutton54.set_value(1.0)
                while gtk.events_pending():
                        gtk.main_iteration(False)
                
        def reset_expander(self, data=True):
                for item in self.expander_list:
                        if item.get_expanded(): 
                                if item == self.expander2:
                                        self.reset_levels(False, False)
                                elif item == self.expander3:
                                        self.reset_brightness_and_saturation(False, False)
                                elif item == self.expander4:
                                        self.reset_color_balance(self.button39, False, False)
                                elif item == self.expander5:
                                        self.reset_contrast(self.button43, False, False)
                                elif item == self.expander6:
                                        self.reset_midtones_contrast(self.button46, False, False)
                                elif item == self.expander7:
                                        self.undo_handler(self.button65, False)
                                elif item == self.expander8:
                                        self.undo_handler(self.button54, False)
                                elif item == self.expander9:
                                        self.undo_handler(self.button72, False)
                                elif item == self.expander10:
                                        self.undo_handler(self.button57, False)
                                elif item == self.expander11:
                                        self.undo_handler(self.button68, False)
                                elif item == self.expander12:
                                        self.undo_handler(self.button61, False)
                                elif item == self.expander13:
                                        self.undo_handler(self.button74, False)
                                elif item == self.expander14:
                                        self.undo_handler(self.button80, False)
                                elif item == self.expander15:
                                        self.undo_handler(self.button83, False)
                                elif item == self.expander16:
                                        self.undo_handler(self.button89, False)
                                elif item == self.expander17:
                                        self.undo_handler(self.button93, False)
                                elif item == self.expander18:
                                        self.undo_handler(self.button94, False)
                                elif item == self.expander19:
                                        self.undo_handler(self.button111, False)
                                elif item == self.expander20:
                                        self.reset_mixer(False, False)
                                elif item == self.expander21:
                                        self.undo_handler(self.button92, False)
                                elif item == self.expander23:
                                        self.undo_handler(self.button114, False)
                                elif item == self.expander24:
                                        self.undo_handler(self.button115, False)
                                elif item == self.expander25:
                                        self.undo_handler(self.button117, False)
                                elif item == self.expander26:
                                        self.undo_handler(self.button121, False)
                                elif item == self.expander27:
                                        self.undo_handler(self.button119, False)
                                elif item == self.expander29:
                                        self.undo_handler(self.button140, False)
                                elif item == self.expander30:
                                        self.undo_handler(self.button142, False)
                                elif item == self.expander31:
                                        self.undo_handler(self.button143, False)
                                elif item == self.expander32:
                                        self.undo_handler(self.button145, False)
                                elif item == self.expander33:
                                        self.undo_handler(self.button147, False)
                                elif item == self.expander34:
                                        self.undo_handler(self.button149, False)
                                elif item == self.expander35:
                                        self.undo_handler(self.button151, False)
                                elif item == self.expander36:
                                        self.undo_handler(self.button33, False)
                                elif item == self.expander37:
                                        self.undo_handler(self.button34, False)
                if data:
                        if self.list[self.indice].lower().endswith(self.formats):
                                self.pblist[self.previous_index] = self.original_pixbuf

        def show_dialog21(self, data=None):
                if not self.checkbutton15.get_active():
                        self.dialog21.show()
                        response = self.dialog21.run()
                        if response == gtk.RESPONSE_CANCEL or response == gtk.RESPONSE_DELETE_EVENT:
                                self.dialog21.hide()
                                while gtk.events_pending():
                                        gtk.main_iteration(False)
                                return True
                        elif response == gtk.RESPONSE_OK:
                                self.dialog21.hide()
                                if data:
                                        self.window2.hide()
                                while gtk.events_pending():
                                        gtk.main_iteration(False)
                                self.reset_workflow()
                                return False
                else:
                        if data:
                                self.window2.hide()
                        self.reset_workflow()
                        return False

        def on_toolbutton47_clicked(self, obj):
                if self.workflow_started and self.wand_index > 0:
                        self.save_changes()
                        self.wand_index = 1
                        self.border_count = 0
                        self.crop_count = 0
                        self.undo_workflow(None)
                        

        def reset_workflow(self, data=True):
                self.reset_expander(data)
                self.workflow_started = False
                self.history= []
                self.wand = None
                self.wand_list= []
                self.wand_index = 0
                self.magick_size = 0
                self.size_history= [0] * 100
                self.set_toolbuttons_23_39_sensitivity()
                
        def set_filenames(self, data=True):
                format = os.path.splitext(self.list[self.indice])[1]
                if not self.workflow_started:
                        if not self.checkbutton17.get_active():
                                  #      try:
                                                self.pblist[self.pbindex].save(os.path.join(self.temp, 'filetemp.tiff'), 'tiff')
                                                self.fname = 'filetemp.tiff'
                                   #     except:
                                    #            self.pblist[self.pbindex].save(os.path.join(self.temp, self.fname), 'jpeg', {"quality" : "100"})
                                    #            self.fname = 'filetemp.jpeg'
                if format.lower() in self.formats:
                        if not self.checkbutton17.get_active():
                                self.infile =  os.path.join(self.temp, self.fname)
                        else:
                                self.infile = self.list[self.indice]
                else:
                        self.infile = os.path.join(self.temp, self.fname)
                if not self.workflow_started:
                        self.wand = MagickImage(self.infile)
                        self.wand_list += [ MagickImage(self.wand.get_image()) ]
                else:
                        self.wand = MagickImage(self.wand_list[self.wand_index].get_image())
                return  format
                
        def show_image1(self, scaled=True):
                if self.wand:
                        self.pblist[self.pbindex] = self.wand2pixbuf(self.wand, scaled)
                if self.expander8.get_expanded():
                        self.draw_grid(self.checkbutton24)
                elif self.expander19.get_expanded():
                        self.draw_grid(self.checkbutton34)
                else:
                        if self.checkbutton41.get_active():
                                self.image28.set_from_pixbuf(self.profile_pixbuf(self.pblist[self.pbindex]))
                        else:
                                self.image28.set_from_pixbuf(self.pblist[self.pbindex])
                        self.image28.show()
                while gtk.events_pending():
                        gtk.main_iteration(False)       

        def on_level_changed(self, obj, event):
                self.window2.window.set_cursor(self.watch)
                while gtk.events_pending():
                        gtk.main_iteration(False)
                value4 = self.hscale4.get_adjustment().value
                value5 = self.hscale5.get_adjustment().value
                value6 = self.hscale6.get_adjustment().value
                channel = self.combobox6.get_active()
                if [value4, value6, value5] == self.RGB_levels[channel] and event: 
                        self.window2.window.set_cursor(None)
                        return False
                format = self.set_filenames()
                self.RGB_levels[channel] = [value4, value6, value5]
                for item in self.level_hist:
                        if item[0] == channel:
                                item[1] = self.RGB_levels[channel]
                                break
                else:   
                        self.level_hist.append( [channel, self.RGB_levels[channel] ] )
                self.adjust_levels()
                if not self.workflow_started:
                        self.original_pixbuf = self.pblist[self.pbindex].copy()
                self.show_image1()
                self.get_channel()
                self.window2.window.set_cursor(None)
                self.image_was_changed = True 
                self.workflow_started = True

        def adjust_levels(self):
                for item in self.level_hist:
                        if item[1] == [0.0, 1.0, 100.0]:
                                del self.level_hist[self.level_hist.index(item)]
                for item in self.level_hist:
                        if item[0] == 0:
                                channel = api.AllChannels
                        elif item[0] == 1:
                                channel = api.RedChannel
                        elif item[0] == 2:
                                channel = api.GreenChannel
                        elif item[0] == 3:
                                channel = api.BlueChannel
                        self.wand.adjust_levels(item[1][0] * Q_RANGE / 100, item[1][1], item[1][2] * Q_RANGE / 100, channel)
                
        def apply_normalize(self, obj):
                self.window2.window.set_cursor(self.watch)
                while gtk.events_pending():
                        gtk.main_iteration(False)
                format = self.set_filenames()
                self.wand.normalize()
                if not self.workflow_started:
                        self.original_pixbuf = self.pblist[self.pbindex].copy()
                self.show_image1()
                self.get_channel()
                self.image_was_changed = True
                self.workflow_started = True
                self.button92.set_sensitive(True)
                self.button103.set_sensitive(False)
                self.window2.window.set_cursor(None)

        def on_channel_mixer_changed(self, obj, event):
                self.window2.window.set_cursor(self.watch)
                while gtk.events_pending():
                        gtk.main_iteration(False)
                value31 = self.hscale31.get_adjustment().value
                value32 = self.hscale32.get_adjustment().value
                value33 = self.hscale33.get_adjustment().value
                channel = self.combobox13.get_active()
                if [value31, value32, value33] == self.mixer_levels[channel] and event: 
                        self.window2.window.set_cursor(None)
                        return False
                format = self.set_filenames()
                self.mixer_levels[channel] = [value31, value32, value33]
                self.mix_channels()
                if not self.workflow_started:
                        self.original_pixbuf = self.pblist[self.pbindex].copy()
                self.show_image1()
                self.get_channel()
                self.window2.window.set_cursor(None)
                self.image_was_changed = True 
                self.workflow_started = True

        def mix_channels(self):
                red = self.mixer_levels[0]
                green = self.mixer_levels[1]
                blue = self.mixer_levels[2]
                self.matrix=(red[0] / 100, red[1] / 100, red[2] / 100, green[0] / 100, green[1] / 100, green[2] / 100,\
                blue[0] / 100, blue[1] / 100, blue[2] / 100)
                self.wand.recolor(3, self.matrix)
                if self.checkbutton26.get_active():
                        self.wand._set_colorspace(api.GRAYColorspace)

        def reset_mixer(self, data, show=True):
                self.combobox13.set_active(0)
                self.hscale31.get_adjustment().set_value(100)
                self.spinbutton48.set_value(100)
                self.hscale32.get_adjustment().set_value(0)
                self.spinbutton47.set_value(0)
                self.hscale33.get_adjustment().set_value(0)
                self.spinbutton46.set_value(0)
                self.mixer_levels = {0 : [100, 0, 0], 1 : [0, 100, 0], 2 : [0, 0, 100]}
                self.checkbutton26.set_active(False)
                self.on_channel_mixer_changed(None, None)
                self.image_was_changed = False
                if show:
                        if self.expander17.get_expanded():
                                self.set_cairo_pixbuf()
                        elif self.expander8.get_expanded():
                                self.draw_grid(self.checkbutton24)
                        elif self.expander19.get_expanded():
                                self.draw_grid(self.checkbutton34)
                        else:
                                if self.checkbutton41.get_active():
                                        self.image28.set_from_pixbuf(self.profile_pixbuf(self.pblist[self.pbindex]))
                                else:
                                        self.image28.set_from_pixbuf(self.pblist[self.pbindex])
                                self.image28.show()

        def reset_red_mixer(self, obj, data):
                time.sleep(0.1)
                channel = self.combobox13.get_active()
                if channel == 0:
                        value = 100
                else:
                        value = 0
                self.hscale31.get_adjustment().set_value(value)
                self.spinbutton48.set_value(value)
                self.mixer_levels[channel][0] = value
                if data:
                        self.wand = MagickImage(self.wand_list[self.wand_index].get_image())
                        self.on_channel_mixer_changed(self.hscale31, None)
                self.image_was_changed = False

        def reset_green_mixer(self, obj, data):
                time.sleep(0.1)
                channel = self.combobox13.get_active()
                if channel == 1:
                        value = 100
                else:
                        value = 0
                self.hscale32.get_adjustment().set_value(value)
                self.spinbutton47.set_value(value)
                self.mixer_levels[channel][1] = value
                if data:
                        self.wand = MagickImage(self.wand_list[self.wand_index].get_image())
                        self.on_channel_mixer_changed(self.hscale32, None)
                self.image_was_changed = False

        def reset_blue_mixer(self, obj, data):
                time.sleep(0.1)
                channel = self.combobox13.get_active()
                if channel == 2:
                        value = 100
                else:
                        value = 0
                self.hscale33.get_adjustment().set_value(value)
                self.spinbutton46.set_value(value)
                self.mixer_levels[channel][2] = value
                if data:
                        self.wand = MagickImage(self.wand_list[self.wand_index].get_image())
                        self.on_channel_mixer_changed(self.hscale33, None)
                self.image_was_changed = False

        def save_mixer(self):
                self.update_wand_list()
                while gtk.events_pending():
                        gtk.main_iteration(False)
                self.reset_red_mixer(self.button99, False)
                self.reset_green_mixer(self.button100, False)
                self.reset_blue_mixer(self.button101, False)
                self.mixer_levels = {0 :[100, 0, 0], 1 :[0, 100, 0], 2 :[0, 0, 100]}
                self.combobox13.set_active(0)
                if self.checkbutton26.get_active():
                        grayscale = True
                else:
                        grayscale = False
                self.history.append(['recolor', self.matrix[:], grayscale])
                self.image_was_changed = False
                self.checkbutton26.handler_block(self.id80)
                self.checkbutton26.set_active(False)
                self.checkbutton26.handler_unblock(self.id80)
                if self.expander8.get_expanded(): 
                        self.draw_grid(self.checkbutton24)

        def on_brightness_and_saturation_changed(self, obj, event):
                value7 = self.hscale7.get_adjustment().value
                value8 = self.hscale8.get_adjustment().value
                value18 = self.hscale18.get_adjustment().value
                if self.bright_sat == [value7, value8, value18] and event:
                        return False
                self.bright_sat = [value7, value8, value18]
                self.window2.window.set_cursor(self.watch)
                while gtk.events_pending():
                        gtk.main_iteration(False)
                format = self.set_filenames()
                self.wand.modulate(100.0 + value7, 100.0 + value8, 100.0 + value18)
                if not self.workflow_started:
                        self.original_pixbuf = self.pblist[self.pbindex].copy()
                self.show_image1()
                self.get_channel()
                self.window2.window.set_cursor(None)
                self.image_was_changed = True  
                self.workflow_started = True

        def color_balance(self, obj, event):
                adj9 = self.hscale9.get_adjustment()
                adj10 = self.hscale10.get_adjustment()
                adj11 = self.hscale11.get_adjustment()
                value9, value10, value11 = adj9.value, adj10.value, adj11.value
                if not event:
                        adj9.value, adj10.value, adj11.value = 0, 0, 0
                        value9, value10, value11 = 0, 0, 0
                if self.c_balance == [value9, value10, value11] and event:
                        return False
                self.c_balance = [value9, value10, value11]
                self.window2.window.set_cursor(self.watch)
                while gtk.events_pending():
                        gtk.main_iteration(False)
                format = self.set_filenames()
                if value9 != 0:
                        self.wand.evaluate('Multiply', 1 + value9 / 100, api.RedChannel)
                if value10 != 0:
                        self.wand.evaluate('Multiply', 1 + value10 / 100, api.GreenChannel)
                if value11 != 0:
                        self.wand.evaluate('Multiply', 1 + value11 / 100, api.BlueChannel)
                if not self.workflow_started:
                        self.original_pixbuf = self.pblist[self.pbindex].copy()
                self.show_image1()
                self.get_channel()
                self.window2.window.set_cursor(None)
                self.image_was_changed = True 
                self.workflow_started = True
                
        def on_contrast_changed(self, obj, event):
                adj12 = self.hscale12.get_adjustment()
                value12 = adj12.value
                if not event:
                        value12 = adj12.value = 0
                if self.contrast == value12 and event:
                        return False
                self.contrast = value12
                self.window2.window.set_cursor(self.watch)
                while gtk.events_pending():
                        gtk.main_iteration(False)
                format = self.set_filenames()
                self.wand.adjust_levels(value12 * Q_RANGE / 100, 1.0, Q_RANGE - value12 * Q_RANGE / 100)
                if not self.workflow_started:
                        self.original_pixbuf = self.pblist[self.pbindex].copy()
                self.show_image1()
                self.get_channel()
                self.window2.window.set_cursor(None)
                self.image_was_changed = True
                self.workflow_started = True

        def profile_image(self, wand):
                if self.embedded_profile_behaviour == 0 and self.embedded_profile != None:
                        wand.set_image_profile("icc", self.embedded_profile)
                else:
                        wand.set_image_profile("icc", self.input_profile)
                if not self.checkbutton42.get_active():
                        wand.set_rendering_intent(self.display_rendering_intent)
                        wand.profile_image("icc", self.display_profile)
                else:
                        wand.set_rendering_intent(self.out_rendering_intent)
                        wand.profile_image("icc", self.output_profile)
                        if self.display_profile_name != self.output_profile_name:
                                wand.set_rendering_intent(self.display_rendering_intent)
                                wand.profile_image("icc", self.display_profile)
                return wand
                
        def wand2pixbuf(self, wand_, scale=None, rotated=False):
                wand = MagickImage(wand_.get_image())
                width, height = wand.size
                if scale:
                        ratio = float(width) / height
                        if not rotated:
                                if ratio > 1:
                                        width, height = self.rect.width, int(self.rect.width / ratio)
                                else:
                                        if float(self.rect.width) / self.rect.height < ratio:
                                                width, height = self.rect.width, int(self.rect.width / ratio)
                                        else:
                                                width, height = int(self.rect.height * ratio), self.rect.height
                        else:
                                width, height = self.rect.height, int(self.rect.height / ratio)
                        wand.scale(width, height)
                type_ = wand.get_type()
                if 'Matte' in type_:
                        map_ = "RGBA"
                        IS_RGBA = True
                else:
                        map_ = "RGB"
                        IS_RGBA = False
                array_ = numpy.empty(width * height * len(map_), dtype=numpy.uint8)
                array_ = wand.get_pixels(0, 0, width, height, map_, array_)
                array_ = array_.reshape(height, width, len(map_))
                if pygtk_has_numpy_support:     # Faster way, requires pygtk to be compiled with numpy support!
                        pixbuf = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB, IS_RGBA, 8, width, height)
                        with warnings.catch_warnings():
                                warnings.simplefilter("ignore")
                                pixbuf.pixel_array[:] = array_
                else: 
                        pil_img = Image.fromarray(array_, map_)
                        pixbuf = gtk.gdk.pixbuf_new_from_data(pil_img.tostring(), gtk.gdk.COLORSPACE_RGB, IS_RGBA,\
                                        8, pil_img.size[0], pil_img.size[1], (IS_RGBA and 4 or 3) * pil_img.size[0])
                return pixbuf
                
        def on_midtones_contrast_changed(self, obj, event):
                adj13 = self.hscale13.get_adjustment()
                adj14 = self.hscale14.get_adjustment()
                value13, value14 = adj13.value, adj14.value
                if not event:
                        adj13.value, adj14.value = 0, 50
                        value13, value14 = 0, 50
                if self.mid_contrast == [value13, value14] and event:
                        return False
                self.mid_contrast = [value13, value14]
                self.window2.window.set_cursor(self.watch)
                while gtk.events_pending():
                        gtk.main_iteration(False)
                format = self.set_filenames()
                if value13 < 0: 
                        sharpen = False
                        value13 = abs(value13 - 0.1)
                else: 
                        sharpen = True
                        value13 = value13 + 0.1
                self.sigmoidal_contrast_args = (sharpen, value13, value14 * Q_RANGE / 100)
                self.wand.sigmoidal_contrast(*self.sigmoidal_contrast_args)
                if not self.workflow_started:
                        self.original_pixbuf = self.pblist[self.pbindex].copy()
                self.show_image1()
                self.get_channel()
                self.window2.window.set_cursor(None)
                self.image_was_changed = True
                self.workflow_started = True
                
        def save_levels(self):
                if self.RGB_levels[0] == [0,1.0,100] and self.RGB_levels[1] == [0,1.0,100] and self.RGB_levels[2] == [0,1.0,100]\
                and self.RGB_levels[3] == [0, 1.0, 100]:
                        self.expander2.set_expanded(False)
                        return True
                self.update_wand_list()
                while gtk.events_pending():
                        gtk.main_iteration(False)
                self.RGB_levels = {0 : [0,1.0,100], 1 : [0, 1.0, 100], 2 : [0, 1.0, 100], 3 : [0, 1.0, 100]}
                self.combobox6.set_active(0)
                channel_type = {0 : api.AllChannels, 1 : api.RedChannel, 2 : api.GreenChannel, 3 : api.BlueChannel}
                for item in self.level_hist:
                        self.history.append(['adjust_levels', (item[1][0] * Q_RANGE / 100, item[1][1], item[1][2] * Q_RANGE / 100, channel_type[item[0]])])
                self.level_hist = []
                self.reset_black_level(self.button18, False)
                self.reset_white_level(self.button19, False)
                self.reset_gamma(self.button30, False)
                self.image_was_changed = False
                if self.expander8.get_expanded():
                        self.draw_grid(self.checkbutton24)
                   
        def save_brightness_and_saturation(self):
                if self.bright_sat == [0, 0, 0]:
                        self.expander3.set_expanded(False)
                        return True
                self.update_wand_list()
                self.history.append(['modulate', (100 + self.bright_sat[0], 100 + self.bright_sat[1], 100 + self.bright_sat[2])])
                while gtk.events_pending():
                        gtk.main_iteration(False)
                self.reset_brightness(self.button32, False)
                self.reset_saturation(self.button31, False)
                self.reset_hue(self.button87, False)
                self.bright_sat = [0, 0, 0]
                self.image_was_changed = False
                if self.expander8.get_expanded():
                        self.draw_grid(self.checkbutton24)
                
        def save_color_balance(self):
                if self.c_balance == [0, 0, 0]:
                        self.expander4.set_expanded(False)
                        return True
                self.update_wand_list()
                self.history.append([['evaluate', ('Multiply', 1 + self.c_balance[0] / 100, api.RedChannel)],\
                ['evaluate', ('Multiply', 1 + self.c_balance[1] / 100, api.GreenChannel)],\
                ['evaluate', ('Multiply', 1 + self.c_balance[2] / 100, api.BlueChannel)]])
                while gtk.events_pending():
                        gtk.main_iteration(False)
                self.reset_color_balance(self.button39, False)
                self.c_balance = [0, 0, 0]
                self.image_was_changed = False
                if self.expander8.get_expanded():
                        self.draw_grid(self.checkbutton24)
                
        def save_contrast(self):
                if self.contrast == 0:
                        self.expander5.set_expanded(False)
                        return True
                self.update_wand_list()
                self.history.append(['adjust_levels', (self.contrast * Q_RANGE / 100, 1.0, Q_RANGE - self.contrast * Q_RANGE / 100)])
                while gtk.events_pending():
                        gtk.main_iteration(False)
                self.reset_contrast(self.button43, False)
                self.image_was_changed = False
                if self.expander8.get_expanded():
                        self.draw_grid(self.checkbutton24)
                
        def save_midtones_contrast(self):
                adj13_value = self.hscale13.get_adjustment().value
                adj14_value = self.hscale14.get_adjustment().value
                if adj13_value == 0 and adj14_value == 50:
                        self.expander6.set_expanded(False)
                        return True
                self.update_wand_list()
                self.history.append(['sigmoidal_contrast', self.sigmoidal_contrast_args][:])
                self.reset_midtones_contrast(self.button46, False)
                self.image_was_changed = False
                if self.expander8.get_expanded():
                        self.draw_grid(self.checkbutton24)

        def profile_workflow(self, history):
                if self.embedded_profile_behaviour == 0 and self.embedded_profile != None:
                        pass
                else:
                        if self.input_profile_name not in ['sRGB', 'AdobeRGB 1998']:
                                input_profile_name = os.path.join(self.profile_dir, os.path.basename(self.input_profile_name))
                        else:
                                input_profile_name = self.input_profile_name[:]
                        history.append(['set_image_profile', ("icc", input_profile_name)])
                        if os.path.isfile(self.input_profile_name) and not os.path.isfile(os.path.join(self.profile_dir, os.path.basename(self.input_profile_name))):
                                shutil.copy(self.input_profile_name, self.profile_dir)                                
                history.append(['set_rendering_intent', (self.out_rendering_intent,)])
                if self.output_profile_name not in ['sRGB', 'AdobeRGB 1998']:
                        output_profile_name = os.path.join(self.profile_dir, os.path.basename(self.output_profile_name))
                else:
                        output_profile_name = self.output_profile_name[:]
                history.append(['profile_image', ("icc", output_profile_name)])
                if os.path.isfile(self.output_profile_name) and not os.path.isfile(os.path.join(self.profile_dir, os.path.basename(self.output_profile_name))):
                        shutil.copy(self.output_profile_name, self.profile_dir)
                return history

        def save_workflow(self, obj):
                self.save_changes()
                expanded = False
                for item in self.expander_list:
                        if item.get_expanded():
                                expanded = True
                                break
                if expanded:
                        self.on_expander_activate(item, False, True)
                if self.history:
                        self.workflow_copy = self.history[:self.wand_index]
                        if self.checkbutton41.get_active():
                                self.workflow_copy = self.profile_workflow(self.workflow_copy)
                        if not self.checkbutton22.get_active():
                                self.label135.set_text(_('The workflow was saved for next conversion!'))
                                self.dialog24.show()
                                response = self.dialog24.run()
                                if response == gtk.RESPONSE_CLOSE or response == gtk.RESPONSE_DELETE_EVENT:
                                        self.dialog24.hide()
                else:
                        gtk.gdk.beep()
                        dialog25 = self.messagedialog(_("Attention!"), _('No workflow to save.\nPlease, make a new one!'), None, gtk.BUTTONS_CLOSE, None,
                                                        gtk.DIALOG_MODAL, gtk.MESSAGE_WARNING, None, (330,180))
                        dialog25.set_keep_above(True)
                        response = dialog25.run()
                        if response == gtk.RESPONSE_CLOSE or response == gtk.RESPONSE_DELETE_EVENT:
                                dialog25.destroy()

        def set_scale_size(self):
                if self.size_history[self.wand_index]:
                        size = self.size_history[self.wand_index]
                        size = [int(size[0]), int(size[1])]
                else:
                        size = self.image_size.split('x')
                        size = [int(size[0]), int(size[1])]
                        if self.pblist[self.pbindex].get_width() > self.pblist[self.pbindex].get_height():
                                size = [max(size), min(size)]
                        else:
                                size = [min(size), max(size)]
                self.entry11.set_text(str(size[0]))
                self.entry12.set_text(str(size[1]))
                self.spinbutton19.handler_block(self.id72)
                self.spinbutton23.handler_block(self.id73)
                self.spinbutton19.get_adjustment().upper = 99999
                self.spinbutton23.get_adjustment().upper = 99999
                self.spinbutton19.set_value(size[0])
                self.spinbutton23.set_value(size[1])
                self.spinbutton19.get_adjustment().upper = size[0]
                self.spinbutton23.get_adjustment().upper = size[1]
                self.spinbutton19.handler_unblock(self.id72)
                self.spinbutton23.handler_unblock(self.id73)
                self.ratio = float(size[0]) / float(size[1])
                
        def save_changes(self):
                if not self.image_was_changed:
                        return True
                for item in self.expander_list:
                        if item.get_expanded():
                                if item == self.expander2:
                                        self.save_levels()
                                elif item == self.expander3:
                                        self.save_brightness_and_saturation()
                                elif item == self.expander4:
                                        self.save_color_balance()
                                elif item == self.expander5:
                                        self.save_contrast()
                                elif item == self.expander6:
                                        self.save_midtones_contrast()
                                elif item == self.expander7:
                                        self.save_scale() 
                                elif item == self.expander8:
                                        self.save_fine_rotation()     
                                elif item == self.expander9:
                                        self.save_img_border()
                                elif item == self.expander10:
                                        self.save_sharpen()  
                                elif item == self.expander11:
                                        self.save_blur()
                                elif item == self.expander12:
                                        self.save_add_noise()
                                elif item == self.expander13:
                                        self.save_reduce_noise()
                                elif item == self.expander14:
                                        self.save_median() 
                                elif item == self.expander15:
                                        self.save_sepia_tone()
                                elif item == self.expander16:
                                        self.save_grayscale()
                                elif item == self.expander17:
                                        self.save_crop()
                                elif item == self.expander18:
                                        self.save_unsharp_mask()
                                elif item == self.expander19:
                                        self.save_distort()
                                elif item == self.expander20:
                                        self.save_mixer()
                                elif item == self.expander21:
                                        self.save_normalize()
                                elif item == self.expander24:
                                        self.save_oil_paint()
                                elif item == self.expander25:
                                        self.save_despeckle()
                                elif item == self.expander26 or item == self.expander27:
                                        self.save_curve()
                                elif item == self.expander29:
                                        self.save_gamma_curve()
                                elif item == self.expander30:
                                        self.save_tint()
                                elif item == self.expander31:
                                        self.save_spread()
                                elif item == self.expander32:
                                        self.save_charcoal()
                                elif item == self.expander33:
                                        self.save_sketch()
                                elif item == self.expander34:
                                        self.save_posterize()
                                elif item == self.expander35:
                                        self.save_negate()
                                elif item == self.expander36:
                                        self.save_white_balance()
                                elif item == self.expander37:
                                        self.save_colortemp()

        def undo_workflow(self, obj):
                if self.workflow_started and self.wand_index > 0:
                        self.window2.window.set_cursor(self.watch)
                        while gtk.events_pending():
                                gtk.main_iteration(False)
                        self.save_changes()
                        self.wand_index -= 1
                        image_copy = self.wand_list[self.wand_index].get_image()
                        self.wand = MagickImage(image_copy)
                        self.pblist[self.pbindex] = self.wand2pixbuf(self.wand, True)
                        if self.expander17.get_expanded():
                                self.set_cairo_pixbuf()
                                self.x, self.y = self.x1, self.y1
                        else:
                                if self.checkbutton41.get_active():
                                        self.image28.set_from_pixbuf(self.profile_pixbuf(self.pblist[self.pbindex]))
                                else:
                                        self.image28.set_from_pixbuf(self.pblist[self.pbindex])
                                self.image28.show()
                        if self.expander7.get_expanded():
                                self.set_scale_size()        
                        self.set_toolbuttons_23_39_sensitivity()
                        if self.expander17.get_expanded():
                                self.set_zoom_sensitivity(False)
                        if self.expander23.get_expanded():
                                self.button114.set_sensitive(False)
                        if self.border_count > 0:
                                self.border_count -= 1
                        if not self.border_count:
                                self.button72.set_sensitive(False)
                        if self.crop_count > 0:
                                self.crop_count -= 1
                        if not self.crop_count:
                                self.button93.set_sensitive(False)
                        self.window2.window.set_cursor(None)

        def redo_workflow(self, obj):
                if self.workflow_started and self.wand_index < len(self.wand_list) - 1:
                        self.window2.window.set_cursor(self.watch)
                        while gtk.events_pending():
                                gtk.main_iteration(False)
                        if self.image_was_changed:
                                self.save_changes()
                                self.window2.window.set_cursor(None)
                                return True
                        self.wand_index += 1
                        image_copy = self.wand_list[self.wand_index].get_image()
                        self.wand = MagickImage(image_copy)
                        self.pblist[self.pbindex] = self.wand2pixbuf(self.wand, True)
                        if self.expander17.get_expanded():
                                self.set_cairo_pixbuf()
                                self.x, self.y = self.x1, self.y1
                        else:
                                if self.checkbutton41.get_active():
                                        self.image28.set_from_pixbuf(self.profile_pixbuf(self.pblist[self.pbindex]))
                                else:
                                        self.image28.set_from_pixbuf(self.pblist[self.pbindex])
                                self.image28.show()
                        if self.expander7.get_expanded():
                                self.set_scale_size()   
                        self.set_toolbuttons_23_39_sensitivity()
                        if self.expander17.get_expanded():
                                self.set_zoom_sensitivity(False)
                        self.window2.window.set_cursor(None)
                        
        def set_toolbuttons_23_39_sensitivity(self):
                index = self.wand_index
                length = len(self.wand_list) - 1
                if length in [-1, 0]:
                        self.toolbutton23.set_sensitive(False)
                        self.toolbutton39.set_sensitive(False)
                elif length == 1:
                        if self.wand_index == 0:
                                self.toolbutton23.set_sensitive(False)
                                self.toolbutton39.set_sensitive(True)
                        else:
                                self.toolbutton23.set_sensitive(True)
                                self.toolbutton39.set_sensitive(False)
                else:
                        if 0 < index < length: 
                                self.toolbutton23.set_sensitive(True)
                                self.toolbutton39.set_sensitive(True)
                        elif 0 <= index < length:
                                self.toolbutton23.set_sensitive(False)
                                self.toolbutton39.set_sensitive(True)
                        elif 0 < index <= length:
                                self.toolbutton23.set_sensitive(True)
                                self.toolbutton39.set_sensitive(False)
                
        def on_expander_activate(self, obj, data, close=None):
                if obj.get_expanded() and self.image_was_changed:
                        if not data:
                                obj.set_expanded(data)
                                while gtk.events_pending():
                                        gtk.main_iteration(False)
                        if obj == self.expander36:
                                self.window2.window.set_cursor(None)
                elif obj.get_expanded() and not self.image_was_changed:
                        if obj == self.expander17:
                                self.destroy_drawingarea()
                                self.show_image1()
                        elif obj == self.expander8:
                                if self.checkbutton41.get_active():
                                        self.image28.set_from_pixbuf(self.profile_pixbuf(self.pblist[self.pbindex]))
                                else:
                                        self.image28.set_from_pixbuf(self.pblist[self.pbindex])
                                self.image28.show()
                        elif obj == self.expander19 and self.checkbutton34.get_active():
                                if self.checkbutton41.get_active():
                                        self.image28.set_from_pixbuf(self.profile_pixbuf(self.pblist[self.pbindex]))
                                else:
                                        self.image28.set_from_pixbuf(self.pblist[self.pbindex])
                                self.image28.show()
                        elif obj == self.expander36:
                                self.window2.window.set_cursor(None)
                        elif close:
                                obj.set_expanded(False)
                        self.crop_count = 0
                        self.border_count = 0
                elif not obj.get_expanded():
                        if obj == self.expander7:               
                                self.set_scale_size()
                                self.button65.set_sensitive(False)
                        elif obj == self.expander8:
                                self.button54.set_sensitive(False)
                                self.spinbutton27.set_value(0)
                                self.spinbutton28.set_value(0.0)
                                self.draw_grid(self.checkbutton24)
                        elif obj == self.expander9:
                                self.button72.set_sensitive(False)
                        elif obj == self.expander10:
                                self.button57.set_sensitive(False)
                                self.button58.set_sensitive(True)
                                self.spinbutton25.set_value(1.0)
                                self.spinbutton29.set_value(0)
                        elif obj == self.expander11:
                                self.button68.set_sensitive(False)
                                self.button70.set_sensitive(True)
                                self.spinbutton33.set_value(1)
                                self.spinbutton32.set_value(0)
                                self.spinbutton71.set_value(0)
                                self.combobox8.set_active(0)
                        elif obj == self.expander12:
                                self.button61.set_sensitive(False)
                                self.button63.set_sensitive(True)
                        elif obj == self.expander13:
                                self.button74.set_sensitive(False)
                                self.button75.set_sensitive(True)
                                self.spinbutton34.set_value(0)
                        elif obj == self.expander14:
                                self.button80.set_sensitive(False)
                                self.button81.set_sensitive(True)
                                self.spinbutton36.set_value(0)
                        elif obj == self.expander15:
                                self.button83.set_sensitive(False)
                                self.button84.set_sensitive(True)
                                self.spinbutton31.set_value(80)
                        elif obj == self.expander16:
                                self.button89.set_sensitive(False)
                                self.button90.set_sensitive(True)
                        elif obj == self.expander17:
                                self.button93.set_sensitive(False)
                                self.button97.set_sensitive(True)
                                self.make_drawingarea()
                        elif obj == self.expander18:
                                self.button94.set_sensitive(False)
                                self.button95.set_sensitive(True)
                                self.reset_unsharp_mask()
                        elif obj == self.expander19:
                                self.button111.set_sensitive(False)
                                self.button112.set_sensitive(True)
                                self.reset_distort()
                                self.draw_grid(self.checkbutton34)
                        elif obj == self.expander21:
                                self.button92.set_sensitive(False)
                                self.button103.set_sensitive(True)
                        elif obj == self.expander23:
                                self.button114.set_sensitive(False)
                        elif obj == self.expander24:
                                self.button115.set_sensitive(False)
                                self.button116.set_sensitive(True)
                                self.spinbutton56.set_value(0)
                        elif obj == self.expander25:
                                self.button117.set_sensitive(False)
                                self.button118.set_sensitive(True)
                        elif obj == self.expander29:
                                self.curve1.reset()
                                self.curve2.reset()
                                self.curve3.reset()
                                self.curve4.reset()
                                self.combobox25.set_active(0)
                        elif obj == self.expander30:
                                self.button142.set_sensitive(False)
                        elif obj == self.expander31:
                                self.button143.set_sensitive(False)
                                self.button144.set_sensitive(True)
                                self.spinbutton64.set_value(0)
                        elif obj == self.expander32:
                                self.button145.set_sensitive(False)
                                self.button146.set_sensitive(True)
                                self.spinbutton65.set_value(1.0)
                                self.spinbutton66.set_value(0)
                        elif obj == self.expander33:
                                self.button147.set_sensitive(False)
                                self.button148.set_sensitive(True)
                                self.spinbutton67.set_value(10.0)
                                self.spinbutton68.set_value(0)
                                self.spinbutton69.set_value(45)
                        elif obj == self.expander34:
                                self.button149.set_sensitive(False)
                                self.button150.set_sensitive(True)
                                self.spinbutton70.set_value(4)
                                self.checkbutton40.set_active(False)
                        elif obj == self.expander35:
                                self.button151.set_sensitive(False)
                                self.button152.set_sensitive(True)
                                self.combobox26.set_active(0)
                                self.checkbutton39.set_active(False)
                        elif obj == self.expander36:
                                self.make_color_picker_cursor()
                        elif obj == self.expander37:
                                self.spinbutton72.set_value(7500)
                                self.spinbutton73.set_value(1)
                        for item in self.expander_list:
                                if item.get_expanded() and item != obj:
                                        if self.image_was_changed: 
                                                self.on_expander_activate(item, False)
                                                if item == self.expander8 and self.checkbutton24.get_active():
                                                        if self.checkbutton41.get_active():
                                                                self.image28.set_from_pixbuf(self.profile_pixbuf(self.pblist[self.pbindex]))
                                                        else:
                                                                self.image28.set_from_pixbuf(self.pblist[self.pbindex])
                                                        self.image28.show()
                                        else: 
                                                item.set_expanded(False)
                                                if item == self.expander17:
                                                        self.destroy_drawingarea()
                                                        if self.checkbutton41.get_active():
                                                                self.image28.set_from_pixbuf(self.profile_pixbuf(self.pblist[self.pbindex]))
                                                        else:
                                                                self.image28.set_from_pixbuf(self.pblist[self.pbindex])
                                                        self.image28.show()
                                                        if obj == self.expander8:
                                                                self.draw_grid(self.checkbutton24)
                                                elif item == self.expander8 and self.checkbutton24.get_active():
                                                        if self.checkbutton41.get_active():
                                                                self.image28.set_from_pixbuf(self.profile_pixbuf(self.pblist[self.pbindex]))
                                                        else:
                                                                self.image28.set_from_pixbuf(self.pblist[self.pbindex])
                                                        self.image28.show()
                                                elif item == self.expander19 and self.checkbutton34.get_active():
                                                        if self.checkbutton41.get_active():
                                                                self.image28.set_from_pixbuf(self.profile_pixbuf(self.pblist[self.pbindex]))
                                                        else:
                                                                self.image28.set_from_pixbuf(self.pblist[self.pbindex])
                                                        self.image28.show()

        def undo_handler(self, obj, show=True):
                if self.image_was_changed or obj == self.button114:
                        self.window2.window.set_cursor(self.watch)
                        while gtk.events_pending():
                                gtk.main_iteration(False)
                        if obj == self.button33:
                                self.wb_hist[1] = []
                        elif obj == self.button34:
                                self.spinbutton72.set_value(7500)
                                self.spinbutton73.set_value(1)
                        elif obj == self.button65:
                                self.button65.set_sensitive(False)
                        elif obj == self.button72:
                                self.button72.set_sensitive(False)
                                self.button69.set_sensitive(True)
                        elif obj == self.button54:
                                self.button54.set_sensitive(False)
                                self.spinbutton27.set_value(0)
                                self.spinbutton28.set_value(0)
                                if not self.expander8.get_expanded():
                                        if self.checkbutton41.get_active():
                                                self.image28.set_from_pixbuf(self.profile_pixbuf(self.pblist[self.pbindex]))
                                        else:
                                                self.image28.set_from_pixbuf(self.pblist[self.pbindex])
                                        self.image28.show()
                        elif obj == self.button57:
                                self.button57.set_sensitive(False)
                                self.button58.set_sensitive(True)
                                self.spinbutton25.set_value(1)
                                self.spinbutton29.set_value(0)
                        elif obj == self.button68:
                                self.button68.set_sensitive(False)
                                self.button70.set_sensitive(True)
                                self.spinbutton32.set_value(0)
                                self.spinbutton33.set_value(0)
                        elif obj == self.button61:
                                self.button61.set_sensitive(False)
                                self.button63.set_sensitive(True)
                        elif obj == self.button74:
                                self.button74.set_sensitive(False)
                                self.button75.set_sensitive(True)
                                self.spinbutton34.set_value(0)
                        elif obj == self.button80:
                                self.button80.set_sensitive(False)
                                self.button81.set_sensitive(True)
                                self.spinbutton36.set_value(0)
                        elif obj == self.button83:
                                self.button83.set_sensitive(False)
                                self.button84.set_sensitive(True)
                                self.spinbutton31.set_value(80)
                        elif obj == self.button89:
                                self.button89.set_sensitive(False)
                                self.button90.set_sensitive(True)
                        elif obj == self.button92:
                                self.button92.set_sensitive(False)
                                self.button103.set_sensitive(True)
                        elif obj == self.button93:
                                self.button93.set_sensitive(False)
                                self.button97.set_sensitive(True)
                                if not self.expander17.get_expanded():
                                        self.destroy_drawingarea()
                        elif obj == self.button94:
                                self.button94.set_sensitive(False)
                                self.button95.set_sensitive(True)
                                self.reset_unsharp_mask()
                        elif obj == self.button111:
                                self.button111.set_sensitive(False)
                                self.button112.set_sensitive(True)
                                self.reset_distort()
                                self.image66.set_from_pixbuf(gtk.gdk.pixbuf_new_from_file(os.path.join(self.temp, "grid.png")))
                        elif obj == self.button114:
                                self.button114.set_sensitive(False)
                                self.wand_index -= 1
                                self.set_toolbuttons_23_39_sensitivity()
                        elif obj == self.button115:
                                self.button115.set_sensitive(False)
                                self.button116.set_sensitive(True)
                                self.spinbutton56.set_value(0)
                        elif obj == self.button117:
                                self.button117.set_sensitive(False)
                                self.button118.set_sensitive(True)
                        elif obj == self.button142:
                                self.button142.set_sensitive(False)
                        elif obj == self.button143:
                                self.button143.set_sensitive(False)
                                self.button144.set_sensitive(True)
                        elif obj == self.button145:
                                self.button145.set_sensitive(False)
                                self.button146.set_sensitive(True)
                        elif obj == self.button147:
                                self.button147.set_sensitive(False)
                                self.button148.set_sensitive(True)
                        elif obj == self.button149:
                                self.button149.set_sensitive(False)
                                self.button150.set_sensitive(True)
                        elif obj == self.button151:
                                self.button151.set_sensitive(False)
                                self.button152.set_sensitive(True)
                        if show:
                                self.wand = MagickImage(self.wand_list[self.wand_index].get_image())
                                self.pblist[self.pbindex] = self.wand2pixbuf(self.wand, True)
                                if self.expander17.get_expanded():
                                        self.set_cairo_pixbuf()
                                        self.x, self.y = self.x1, self.y1
                                elif self.expander8.get_expanded():
                                        self.draw_grid(self.checkbutton24)
                                elif self.expander19.get_expanded():
                                        self.draw_grid(self.checkbutton34)
                                else:
                                        if self.checkbutton41.get_active():
                                                self.image28.set_from_pixbuf(self.profile_pixbuf(self.pblist[self.pbindex]))
                                        else:
                                                self.image28.set_from_pixbuf(self.pblist[self.pbindex])
                                        gobject.idle_add(self.image28.show)
                                while gtk.events_pending():
                                        gtk.main_iteration(False)
                                self.get_channel()
                        self.image_was_changed = False
                        if self.expander36.get_expanded():
                                self.set_color_picker_cursor()
                        else:
                                self.window2.window.set_cursor(None)

        def block_viewport1_signals(self):
                self.viewport1.handler_block(self.viewport1_id1)
                self.viewport1.handler_block(self.viewport1_id2)
                self.viewport1.handler_block(self.viewport1_id3)

        def unblock_viewport1_signals(self):
                self.viewport1.handler_unblock(self.viewport1_id1)
                self.viewport1.handler_unblock(self.viewport1_id2)
                self.viewport1.handler_unblock(self.viewport1_id3)

        def set_zoom_sensitivity(self, data):
                self.toolbutton6.set_sensitive(data)
                self.toolbutton7.set_sensitive(data)
                self.toolbutton8.set_sensitive(data)
                self.toolbutton9.set_sensitive(data)
                self.toolbutton10.set_sensitive(data)
                self.toolbutton11.set_sensitive(data)
                self.button53.set_sensitive(data)
                self.button56.set_sensitive(data)
                self.button59.set_sensitive(data)
                self.button62.set_sensitive(data)

        def draw_grid(self, obj):
                if self.checkbutton41.get_active():
                        pixbuf = self.profile_pixbuf(self.pblist[self.pbindex])
                else:
                        pixbuf = self.pblist[self.pbindex]
                if obj.get_active():
                        pixmap, mask = pixbuf.render_pixmap_and_mask()
                        color = pixmap.get_colormap().alloc_color('red')
                        gc = pixmap.new_gc(foreground=color)
                        width, height = pixmap.get_size()
                        for x in xrange(0, width, 30):
                                pixmap.draw_line(gc, x, 0, x, height)
                        for y in xrange(0, height, 30):
                                pixmap.draw_line(gc, 0, y, width, y)
                        self.image28.set_from_pixmap(pixmap, mask)
                else:
                        self.image28.set_from_pixbuf(pixbuf)
                gobject.idle_add(self.image28.show)
        
        def make_drawingarea(self):
                self.set_zoom_sensitivity(False)
                self.block_viewport1_signals()
                self.zoom_fit_photo(None, False)
                self.image28.destroy()
                self.drawing_area = gtk.DrawingArea()
                self.viewport1.add(self.drawing_area)
                self.drawing_area.modify_bg(gtk.STATE_NORMAL, self.image_background_color)
                self.drawing_area.add_events(gtk.gdk.BUTTON_PRESS_MASK | gtk.gdk.BUTTON1_MOTION_MASK | gtk.gdk.POINTER_MOTION_HINT_MASK)
                self.drawing_area.connect('expose_event', self.on_drawing_area_expose_event, False)
                self.drawing_area.connect('size_allocate', self.on_drawing_area_size_allocate)
                self.drawing_area.connect('button_press_event', self.on_drawing_area_button_press_event)
                self.combobox12_id = self.combobox12.connect('changed', self.on_drawing_area_expose_event, self.drawing_area, True)     
                self.drawing_area.show()

        def on_drawing_area_size_allocate(self, obj, allocation):
                self.rect = self.viewport1.allocation
                width = self.pblist[self.pbindex].get_width()
                height = self.pblist[self.pbindex].get_height()
                self.x_offset, self.y_offset = int((self.rect.width - width) / 2.0), int((self.rect.height - height) / 2.0)
                self.drawing_area.set_size_request(width, height)

        def on_drawing_area_expose_event(self, obj, event, data):
                if self._id and self.drawing_area.handler_is_connected(self._id):
                        self.drawing_area.disconnect(self._id)
                self._id = self.drawing_area.connect('motion_notify_event', self.on_drawing_area_motion_notify_event,\
                                                 self.pblist[self.pbindex].get_width(), self.pblist[self.pbindex].get_height())
                self._color = self.colorbutton7.get_color()
                ct = self.set_cairo_pixbuf()
                if data:
                       self.draw_rectangle_with_ratio(ct)
                
        def set_cairo_pixbuf(self):
                self.on_drawing_area_size_allocate(self.drawing_area, self.drawing_area.allocation)
                ct = self.drawing_area.window.cairo_create()
                if self.checkbutton41.get_active():
                        self.current_pixbuf = self.profile_pixbuf(self.pblist[self.pbindex])
                else:
                        self.current_pixbuf = self.pblist[self.pbindex]
                ct.set_source_pixbuf(self.current_pixbuf, self.x_offset, self.y_offset)
                ct.paint()
                return ct

        def draw_rectangle_with_ratio(self, ct):
                width = self.pblist[self.pbindex].get_width()
                height = self.pblist[self.pbindex].get_height()
                if not self.checkbutton21.get_active():
                        ratio = self.combobox12.get_active()
                        if ratio == 0:
                                self.crop_ratio = 1
                                h = w = min(width / 2, height / 2)
                        elif ratio == 1:
                                self.crop_ratio = 3 / 2.0
                                w = min(width / 2, height / 2)
                                h = w / self.crop_ratio
                        elif ratio == 2:
                                self.crop_ratio = 2 / 3.0
                                h = min(width / 2, height / 2)
                                w = h * self.crop_ratio
                        elif ratio == 3:
                                self.crop_ratio = 4 / 3.0
                                w = min(width / 2, height / 2)
                                h = w / self.crop_ratio
                        elif ratio == 4:
                                self.crop_ratio = 3 / 4.0
                                h = min(width / 2, height / 2)
                                w = h * self.crop_ratio
                        elif ratio == 5:
                                self.crop_ratio = 5 / 4.0
                                w = min(width / 2, height / 2)
                                h = w / self.crop_ratio 
                        elif ratio == 6:
                                self.crop_ratio = 4 / 5.0
                                h = min(width / 2, height / 2)
                                w = h * self.crop_ratio
                        elif ratio == 7:
                                self.crop_ratio = 7 / 5.0
                                w = min(width / 2, height / 2)
                                h = w / self.crop_ratio 
                        elif ratio == 8:
                                self.crop_ratio = 5 / 7.0
                                h = min(width / 2, height / 2)
                                w = h * self.crop_ratio
                        elif ratio == 9:
                                self.crop_ratio = 3 / 1.0
                                w = min(width / 2, height / 2)
                                h = w / self.crop_ratio 
                        elif ratio == 10:
                                self.crop_ratio = 1 / 3.0
                                h = min(width / 2, height / 2)
                                w = h * self.crop_ratio
                        elif ratio == 11:
                                self.crop_ratio = 0
                                w = width / 2
                                h = height / 2
                else:
                        self.crop_ratio = self.spinbutton30.get_value() / self.spinbutton38.get_value()
                        if self.crop_ratio > 1:
                                w = min(width / 2, height / 2)
                                h = w / self.crop_ratio
                        else:
                                h = min(width / 2, height / 2)
                                w = h * self.crop_ratio
                self.x,  self.y = int(self.rect.width / 2 - w / 2), int(self.rect.height / 2 - h / 2)
                self.x1, self.y1 = self.x + int(w), self.y + int(h) 
                ct.set_source_color(self._color)
                ct.rectangle(self.x, self.y, w, h)
                ct.stroke()
                
        def on_drawing_area_button_press_event(self, obj, event):
                if event.button == 1:
                        width = self.pblist[self.pbindex].get_width()
                        height = self.pblist[self.pbindex].get_height()
                        self.rect = self.viewport1.allocation
                        self.x_offset, self.y_offset = int((self.rect.width - width) / 2.0), int((self.rect.height - height) / 2.0)
                        x0, x1, y0, y1 = (self.rect.width - width) / 2.0, self.rect.width - (self.rect.width - width) / 2.0,\
                        (self.rect.height - height) / 2.0, self.rect.height - (self.rect.height - height) / 2.0
                        self.x, self.y = self.x1, self.y1 = event.window.get_pointer()[:2]
                        ct = self.set_cairo_pixbuf()
                        if x0 < self.x < x1 and y0 < self.y < y1 :
                                ct.set_source_color(self._color)
                                ct.move_to(x0, event.y)
                                ct.line_to(x1, event.y)
                                ct.move_to(event.x, y0)
                                ct.line_to(event.x, y1)
                                ct.stroke()
                        
        def on_drawing_area_motion_notify_event(self, obj, event, width, height):
                if event.x > self.x and event.y > self.y:
                        if self.crop_ratio:
                                if (event.x - self.x) / (event.y - self.y) > self.crop_ratio:
                                        event.x = self.crop_ratio * (event.y - self.y) + self.x
                                else:
                                        event.y = (1 / self.crop_ratio) * (event.x - self.x) + self.y
                        ct = obj.window.cairo_create()
                        ct.rectangle(self.x_offset, self.y_offset, width, height)
                        ct.clip()
                        ct.set_source_pixbuf(self.current_pixbuf, self.x_offset, self.y_offset)
                        ct.paint()
                        ct.set_source_color(self._color)
                        ct.rectangle(self.x, self.y, event.x - self.x, event.y - self.y)
                        ct.stroke()
                        self.x1, self.y1 = event.x, event.y

        def destroy_drawingarea(self):
                self.drawing_area.destroy()
                self.image28 = gtk.Image()
                self.viewport1.add(self.image28)
                self.unblock_viewport1_signals()
                self.viewport1.modify_bg(gtk.STATE_NORMAL, self.image_background_color)
                self.set_zoom_sensitivity(True)
                self.combobox12.disconnect(self.combobox12_id)

        def scale(self, obj):
                self.window2.window.set_cursor(self.watch)
                while gtk.events_pending():
                        gtk.main_iteration(False)
                width = self.spinbutton19.get_value()
                height = self.spinbutton23.get_value()
                if width and height:
                        if int(self.entry11.get_text()) == width and int(self.entry12.get_text()) == height:
                                pass
                        else:
                                format = self.set_filenames()
                                self.scale_args = long(width), long(height)
                                if self.checkbutton25.get_active():
                                        self.wand.thumbnail(*self.scale_args)
                                else:
                                        self.wand.scale(*self.scale_args)
                                if not self.workflow_started:
                                        self.original_pixbuf = self.pblist[self.pbindex].copy()
                                self.show_image1(False)
                                self.get_channel()
                                self.image_was_changed = True
                                self.workflow_started = True
                                self.button65.set_sensitive(True)
                else:
                        gtk.gdk.beep()
                self.window2.window.set_cursor(None)

        def save_scale(self):
                self.button65.set_sensitive(False)
                self.image_was_changed = False
                self.update_wand_list()
                if self.checkbutton25.get_active():
                        self.history.append(['thumbnail', self.scale_args[:]])
                else:
                        self.history.append(['scale', self.scale_args[:]])
                if self.expander8.get_expanded():
                        self.draw_grid(self.checkbutton24)
                self.magick_size = [str(self.scale_args[0]), str(self.scale_args[1])]
                self.entry11.set_text(self.magick_size[0])
                self.entry12.set_text(self.magick_size[1])
                self.spinbutton19.get_adjustment().upper = int(self.magick_size[0])
                self.spinbutton23.get_adjustment().upper = int(self.magick_size[1])
                self.size_history[self.wand_index:] = [self.scale_args for i in xrange(self.wand_index, 100)]
                
        def transform(self, obj, data):
                self.save_changes()
                self.window2.window.set_cursor(self.watch)
                while gtk.events_pending():
                        gtk.main_iteration(False)
                format = self.set_filenames()
                if data == 'flip':
                        self.wand.flip()
                        self.history.append(['flip',()])
                elif data == 'flop':
                        self.wand.flop()
                        self.history.append(['flop',()])
                elif data == 'rotate 90':
                        self.wand.rotate(90.0, None)
                        self.history.append(['rotate', (90.0, None)])
                elif data == 'rotate -90':
                        self.wand.rotate(-90.0, None)
                        self.history.append(['rotate', (-90.0, None)])
                self.update_wand_list()
                if not self.workflow_started:
                        self.original_pixbuf = self.pblist[self.pbindex].copy()
                self.show_image1()
                self.get_channel()
                self.workflow_started = True
                self.window2.window.set_cursor(None)
                        
        def apply_img_border(self, obj):
                self.window2.window.set_cursor(self.watch)
                while gtk.events_pending():
                        gtk.main_iteration(False)
                format = self.set_filenames()
                self.bordercolor = self.colorbutton6.get_color().to_string()
                f=1
                size = self.size_history[self.wand_index]
                if not size:
                        size = self.image_size.split('x')
                        size = [float(size[0]), float(size[1])]
                f = max(self.wand.size) / float(max(size))
                self.IM_border_args = (self.bordercolor, long(self.spinbutton24.get_value()), long(self.spinbutton26.get_value()))
                self.wand.border(Color(self.bordercolor), long(self.IM_border_args[1] * f), long(self.IM_border_args[2] * f))
                if not self.workflow_started:
                        self.original_pixbuf = self.pblist[self.pbindex].copy()
                self.show_image1()
                self.get_channel()
                self.image_was_changed = True
                self.workflow_started = True
                self.button72.set_sensitive(True)
                self.save_img_border()
                self.border_count +=1
                self.window2.window.set_cursor(None)

        def save_img_border(self, obj=None):
                self.button69.set_sensitive(True)
                self.update_wand_list()
                self.image_was_changed = False
                self.history.append(['border', self.IM_border_args[:]])
                if self.expander8.get_expanded():
                        self.draw_grid(self.checkbutton24)

        def fine_rotation(self, obj, event):
                self.window2.window.set_cursor(self.watch)
                while gtk.events_pending():
                        gtk.main_iteration(False)
                format = self.set_filenames()
                self.fine_degree = self.spinbutton27.get_value() + self.spinbutton28.get_value()
                self.wand.rotate(self.fine_degree, None)
                if not self.workflow_started:
                        self.original_pixbuf = self.pblist[self.pbindex].copy()
                self.show_image1()
                self.get_channel()
                self.image_was_changed = True
                self.workflow_started = True
                self.button54.set_sensitive(True)
                self.window2.window.set_cursor(None)

        def save_fine_rotation(self):
                self.button54.set_sensitive(False)
                self.spinbutton27.set_value(0)
                self.spinbutton28.set_value(0)
                self.update_wand_list()
                self.image_was_changed = False
                self.history.append(['rotate', (self.fine_degree, None)])

        def apply_crop(self,obj):
                if (self.x, self.y) == (self.x1, self.y1):
                        gtk.gdk.beep()
                        dialog25 = self.messagedialog(_("Attention!"), _('You must select a rectangle first!'), None, gtk.BUTTONS_CLOSE, None,
                                                        gtk.DIALOG_MODAL, gtk.MESSAGE_WARNING, None, (330,180))
                        dialog25.set_keep_above(True)
                        response = dialog25.run()
                        if response == gtk.RESPONSE_CLOSE or response == gtk.RESPONSE_DELETE_EVENT:
                                dialog25.destroy()
                        return True
                self.window2.window.set_cursor(self.watch)
                while gtk.events_pending():
                        gtk.main_iteration(False)
                format = self.set_filenames()
                i_size = self.wand._get_size()
                width = float(self.x1 - self.x) / self.pblist[self.pbindex].get_width()
                height = float(self.y1 - self.y) / self.pblist[self.pbindex].get_height()
                w_offset = float(self.x - self.x_offset) / self.pblist[self.pbindex].get_width()
                h_offset = float(self.y - self.y_offset) / self.pblist[self.pbindex].get_height()
                w = width * i_size[0]
                h = height * i_size[1]
                x_offset = w_offset * i_size[0]
                y_offset = h_offset * i_size[1]
                crop_args = (long(w), long(h)), (long(x_offset), long(y_offset))
                repage_args = (long(w), long(h)), (0, 0)
                self.crop_args = (width, height), (w_offset, h_offset)
                self.repage_args = (width, height), (0, 0)
                self.wand.crop(*crop_args)
                self.wand.set_page(*repage_args)
                if not self.workflow_started:
                        self.original_pixbuf = self.pblist[self.pbindex].copy()
                self.pblist[self.pbindex] = self.wand2pixbuf(self.wand, True)
                self.set_cairo_pixbuf()
                self.get_channel()
                self.image_was_changed = True
                self.workflow_started = True
                self.button93.set_sensitive(True)
                self.save_crop()
                self.crop_count += 1
                self.window2.window.set_cursor(None)
                
        def save_crop(self, obj=None):
                self.button97.set_sensitive(True)
                self.update_wand_list()
                size = self.size_history[self.wand_index]
                if not size:
                        size = self.image_size.split('x')
                        size = [int(size[0]), int(size[1])]
                crop_size = [int(self.crop_args[0][0] * size[0]), int(self.crop_args[0][1] * size[1])]
                self.size_history[self.wand_index:] = [ crop_size for i in xrange(self.wand_index, 100)]
                self.image_was_changed = False
                self.history.append([['_get_size', None], ['crop', self.crop_args[:]], ['set_page', self.repage_args[:]]])
                self.x, self.y = self.x1, self.y1
                
        def apply_effect(self, fun, args, reprocess=None):
                if reprocess:
                        size = self.wand.size
                        if self.history:
                                self.process_fullsize_image()
                        else:
                                if self.list[self.indice].lower().endswith(self.formats):
                                        self.wand = MagickImage(self.get_real_path(self.list[self.indice]))
                                else:
                                        if hasattr(self, 'full_process') and self.full_process.is_alive():
                                                self.full_process.join()
                                        self.wand = MagickImage(os.path.join(self.temp, 'fullsize_image.tiff'))
                        fun(self.wand, *args)
                        if self.wand.size != size:
                                self.wand = self.scale_wand(self.wand)
                else:
                        fun(self.wand, *args)

        def process_fullsize_image(self, data=False):
                if self.list[self.indice].lower().endswith(self.formats):
                        if not self.checkbutton17.get_active():
                                tupla_ = (self.list[self.indice], 0)
                                if data:
                                        wand = self.process(history=self.history, tupla=tupla_)
                                        return wand
                                self.wand = self.process(history=self.history, tupla=tupla_)
                else:
                        if not self.checkbutton17.get_active() or self.checkbutton16.get_active():
                                if hasattr(self, 'full_process') and self.full_process.is_alive():
                                        self.full_process.join()
                                tupla_ = (os.path.join(self.temp, 'fullsize_image.tiff'), 0)
                                if data:
                                        wand = self.process(history=self.history, tupla=tupla_)
                                        return wand
                                self.wand = self.process(history=self.history, tupla=tupla_)
                                
        def apply_sharpen(self, obj):
                self.button58.set_sensitive(False)
                self.window2.window.set_cursor(self.watch)
                self.progressbar2.set_text(_('Sharpening...'))
                self.window5.show()
                while gtk.events_pending():
                        gtk.main_iteration(False)
                format = self.set_filenames()
                sigma = self.hscale16.get_adjustment().value
                radius = float(self.hscale17.get_adjustment().value)
                fun = MagickImage.sharpen
                self.sharpen_args = (radius, sigma)
                thread = threading.Thread(target=self.apply_effect, args=(fun, self.sharpen_args, True))
                thread.start()
                self.pulse_progressbar2(thread)
                if not self.workflow_started:
                        self.original_pixbuf = self.pblist[self.pbindex].copy()
                self.show_image1()
                self.get_channel()
                self.image_was_changed = True
                self.workflow_started = True
                self.button57.set_sensitive(True)
                self.window2.window.set_cursor(None)
                self.window5.hide()
                self.progressbar2.set_fraction(0)

        def save_sharpen(self):
                self.button57.set_sensitive(False)
                self.button58.set_sensitive(True)
                self.spinbutton25.set_value(1)
                self.spinbutton29.set_value(0)
                self.update_wand_list()
                self.image_was_changed = False
                self.history.append(['sharpen', self.sharpen_args[:]])
                if self.expander8.get_expanded():
                        self.draw_grid(self.checkbutton24)

        def apply_unsharp_mask(self, obj):
                self.button95.set_sensitive(False)
                self.window2.window.set_cursor(self.watch)
                self.progressbar2.set_text(_('Unsharp Mask...'))
                self.window5.show()
                while gtk.events_pending():
                        gtk.main_iteration(False)
                format = self.set_filenames()
                sigma = self.hscale27.get_adjustment().value
                radius = float(self.hscale26.get_adjustment().value)
                amount = self.hscale29.get_adjustment().value
                threshold = self.hscale30.get_adjustment().value
                fun = MagickImage.unsharp_mask
                self.unsharp_args = (radius, sigma, amount, threshold)
                thread = threading.Thread(target=self.apply_effect, args=(fun, self.unsharp_args, True))
                thread.start()
                self.pulse_progressbar2(thread)
                if not self.workflow_started:
                        self.original_pixbuf = self.pblist[self.pbindex].copy()
                self.show_image1()
                self.get_channel()
                self.image_was_changed = True
                self.workflow_started = True
                self.button94.set_sensitive(True)
                self.window2.window.set_cursor(None)
                self.window5.hide()
                self.progressbar2.set_fraction(0)

        def save_unsharp_mask(self):
                self.button94.set_sensitive(False)
                self.button95.set_sensitive(True)
                self.reset_unsharp_mask()
                self.update_wand_list()
                self.image_was_changed = False
                self.history.append(['unsharp_mask', self.unsharp_args[:]])
                if self.expander8.get_expanded():
                        self.draw_grid(self.checkbutton24)
        
        def apply_blur(self, obj):
                self.button70.set_sensitive(False)
                self.window2.window.set_cursor(self.watch)
                self.progressbar2.set_text(_('Blurring...'))
                self.window5.show()
                while gtk.events_pending():
                        gtk.main_iteration(False)
                format = self.set_filenames()
                sigma = self.hscale20.get_adjustment().value
                radius = self.hscale21.get_adjustment().value
                angle = self.hscale49.get_adjustment().value
                mode = self.combobox8.get_active()
                if mode == 0:
                        self.blur_type = 'blur'
                        fun = MagickImage.blur
                        self.blur_args = (radius, sigma)
                elif mode == 1:
                        self.blur_type = 'gaussian_blur'
                        fun = MagickImage.gaussian_blur
                        self.blur_args = (radius, sigma)
                elif mode == 2:
                        self.blur_type = 'radial_blur'
                        fun = MagickImage.radial_blur
                        self.blur_args = (angle,)
                elif mode == 3:
                        self.blur_type = 'adaptive_blur'
                        fun = MagickImage.adaptive_blur
                        self.blur_args = (radius, sigma)
                elif mode == 4:
                        self.blur_type = 'motion_blur'
                        fun = MagickImage.motion_blur
                        self.blur_args = (radius, sigma, angle)
                thread = threading.Thread(target=self.apply_effect, args=(fun, self.blur_args, True))
                thread.start()
                self.pulse_progressbar2(thread)
                if not self.workflow_started:
                        self.original_pixbuf = self.pblist[self.pbindex].copy()
                self.show_image1()
                self.get_channel()
                self.image_was_changed = True
                self.workflow_started = True
                self.button68.set_sensitive(True)
                self.window2.window.set_cursor(None)
                self.window5.hide()
                self.progressbar2.set_fraction(0)
                
        def save_blur(self):
                self.button68.set_sensitive(False)
                self.button70.set_sensitive(True)
                self.spinbutton33.set_value(1)
                self.spinbutton32.set_value(0)
                self.spinbutton71.set_value(0)
                self.update_wand_list()
                self.image_was_changed = False
                self.history.append([self.blur_type, self.blur_args[:]])
                if self.expander8.get_expanded():
                        self.draw_grid(self.checkbutton24)

        def apply_charcoal(self, obj):
                self.button146.set_sensitive(False)
                self.window2.window.set_cursor(self.watch)
                self.progressbar2.set_text(_('Charcoal...'))
                self.window5.show()
                while gtk.events_pending():
                        gtk.main_iteration(False)
                format = self.set_filenames()
                sigma = self.hscale44.get_adjustment().value
                radius = self.hscale43.get_adjustment().value
                fun = MagickImage.charcoal
                self.charcoal_args=(radius, sigma)
                thread = threading.Thread(target=self.apply_effect, args=(fun, self.charcoal_args, True))
                thread.start()
                self.pulse_progressbar2(thread)
                if not self.workflow_started:
                        self.original_pixbuf = self.pblist[self.pbindex].copy()
                self.show_image1()
                self.get_channel()
                self.image_was_changed = True
                self.workflow_started = True
                self.button145.set_sensitive(True)
                self.window2.window.set_cursor(None)
                self.window5.hide()
                self.progressbar2.set_fraction(0)
                
        def save_charcoal(self):
                self.button145.set_sensitive(False)
                self.button146.set_sensitive(True)
                self.update_wand_list()
                self.image_was_changed = False
                self.history.append(['charcoal', self.charcoal_args[:]])
                self.spinbutton65.set_value(1)
                self.spinbutton66.set_value(0)
                if self.expander8.get_expanded():
                        self.draw_grid(self.checkbutton24)

        def apply_sketch(self, obj):
                self.button148.set_sensitive(False)
                self.window2.window.set_cursor(self.watch)
                self.progressbar2.set_text(_('Sketch...'))
                self.window5.show()
                while gtk.events_pending():
                        gtk.main_iteration(False)
                format = self.set_filenames()
                sigma = self.hscale46.get_adjustment().value
                radius = self.hscale45.get_adjustment().value
                angle = self.hscale47.get_adjustment().value
                fun = MagickImage.sketch
                self.sketch_args=(radius, sigma, angle)
                thread = threading.Thread(target=self.apply_effect, args=(fun, self.sketch_args, True))
                thread.start()
                self.pulse_progressbar2(thread)
                if not self.workflow_started:
                        self.original_pixbuf = self.pblist[self.pbindex].copy()
                self.show_image1()
                self.get_channel()
                self.image_was_changed = True
                self.workflow_started = True
                self.button147.set_sensitive(True)
                self.window2.window.set_cursor(None)
                self.window5.hide()
                self.progressbar2.set_fraction(0)
                
        def save_sketch(self):
                self.button147.set_sensitive(False)
                self.button148.set_sensitive(True)
                self.update_wand_list()
                self.image_was_changed = False
                self.history.append(['sketch', self.sketch_args[:]])
                if self.expander8.get_expanded():
                        self.draw_grid(self.checkbutton24)
                self.spinbutton67.set_value(10.0)
                self.spinbutton68.set_value(0)
                self.spinbutton69.set_value(45)

        def apply_posterize(self, obj):
                self.button150.set_sensitive(False)
                self.window2.window.set_cursor(self.watch)
                self.progressbar2.set_text(_('Posterize...'))
                self.window5.show()
                while gtk.events_pending():
                        gtk.main_iteration(False)
                format = self.set_filenames()
                levels = self.hscale48.get_adjustment().value
                fun = MagickImage.posterize
                self.posterize_args = (int(levels), self.checkbutton40.get_active())
                thread = threading.Thread(target=self.apply_effect, args=(fun, self.posterize_args, True))
                thread.start()
                self.pulse_progressbar2(thread)
                if not self.workflow_started:
                        self.original_pixbuf = self.pblist[self.pbindex].copy()
                self.show_image1()
                self.get_channel()
                self.image_was_changed = True
                self.workflow_started = True
                self.button149.set_sensitive(True)
                self.window2.window.set_cursor(None)
                self.window5.hide()
                self.progressbar2.set_fraction(0)
                
        def save_posterize(self):
                self.button149.set_sensitive(False)
                self.button150.set_sensitive(True)
                self.update_wand_list()
                self.image_was_changed = False
                self.history.append(['posterize', self.posterize_args[:]])
                if self.expander8.get_expanded():
                        self.draw_grid(self.checkbutton24)
                self.spinbutton70.set_value(4)
                self.checkbutton40.set_active(False)

        def apply_negate(self, obj):
                self.button152.set_sensitive(False)
                self.window2.window.set_cursor(self.watch)
                self.progressbar2.set_text(_('Negate...'))
                self.window5.show()
                while gtk.events_pending():
                        gtk.main_iteration(False)
                format = self.set_filenames()
                fun = MagickImage.negate
                channel = {0 : api.AllChannels, 1 : api.RedChannel, 2 : api.GreenChannel, 3 : api.BlueChannel}
                self.negate_args=(self.checkbutton39.get_active(), channel[self.combobox26.get_active()])
                thread = threading.Thread(target=self.apply_effect, args=(fun, self.negate_args))
                thread.start()
                self.pulse_progressbar2(thread)
                if not self.workflow_started:
                        self.original_pixbuf = self.pblist[self.pbindex].copy()
                self.show_image1()
                self.get_channel()
                self.image_was_changed = True
                self.workflow_started = True
                self.button151.set_sensitive(True)
                self.window2.window.set_cursor(None)
                self.window5.hide()
                self.progressbar2.set_fraction(0)
                
        def save_negate(self):
                self.button151.set_sensitive(False)
                self.button152.set_sensitive(True)
                self.update_wand_list()
                self.image_was_changed = False
                self.history.append(['negate', self.negate_args[:]])
                if self.expander8.get_expanded():
                        self.draw_grid(self.checkbutton24)
                self.combobox26.set_active(0)
                self.checkbutton39.set_active(0)
                
        def apply_noise(self, obj):
                self.button63.set_sensitive(False)
                self.window2.window.set_cursor(self.watch)
                self.progressbar2.set_text(_('Noising...'))
                self.window5.show()
                while gtk.events_pending():
                        gtk.main_iteration(False)
                format = self.set_filenames()
                noise_type = self.combobox11.get_active() + 1
                fun = MagickImage.add_noise
                self.add_noise_args = (noise_type,)
                thread = threading.Thread(target=self.apply_effect, args=(fun, self.add_noise_args, True))
                thread.start()
                self.pulse_progressbar2(thread)
                if not self.workflow_started:
                        self.original_pixbuf = self.pblist[self.pbindex].copy()
                self.show_image1()
                self.get_channel()
                self.image_was_changed = True
                self.workflow_started = True
                self.button61.set_sensitive(True)
                self.window2.window.set_cursor(None)
                self.window5.hide()
                self.progressbar2.set_fraction(0)
                
        def save_add_noise(self):
                self.button61.set_sensitive(False)
                self.button63.set_sensitive(True)
                self.update_wand_list()
                self.image_was_changed = False
                self.history.append(['add_noise', self.add_noise_args[:]])
                if self.expander8.get_expanded():
                        self.draw_grid(self.checkbutton24)
        
        def apply_reduce_noise(self, obj):
                self.button75.set_sensitive(False)
                self.window2.window.set_cursor(self.watch)
                self.progressbar2.set_text(_('Denoising...'))
                self.window5.show()
                while gtk.events_pending():
                        gtk.main_iteration(False)
                format = self.set_filenames()
                radius = self.hscale22.get_adjustment().value
                fun = MagickImage.reduce_noise
                self.reduce_noise_args = (radius,)
                thread = threading.Thread(target=self.apply_effect, args=(fun, self.reduce_noise_args, True))
                thread.start()
                self.pulse_progressbar2(thread)
                if not self.workflow_started:
                        self.original_pixbuf = self.pblist[self.pbindex].copy()
                self.show_image1()
                self.get_channel()
                self.image_was_changed = True
                self.workflow_started = True
                self.button74.set_sensitive(True)
                self.window2.window.set_cursor(None)
                self.window5.hide()
                self.progressbar2.set_fraction(0)
                
        def save_reduce_noise(self):
                self.button74.set_sensitive(False)
                self.button75.set_sensitive(True)
                self.spinbutton34.set_value(0)
                self.update_wand_list()
                self.image_was_changed = False
                self.history.append(['reduce_noise', self.reduce_noise_args[:]])
                if self.expander8.get_expanded():
                        self.draw_grid(self.checkbutton24)
        
        def apply_median_filter(self, obj):
                self.button81.set_sensitive(False)
                self.window2.window.set_cursor(self.watch)
                self.progressbar2.set_text(_('Median Filter...'))
                self.window5.show()
                while gtk.events_pending():
                        gtk.main_iteration(False)
                format = self.set_filenames()
                radius = self.hscale24.get_adjustment().value
                fun = MagickImage.median_filter
                self.median_args = (radius,)
                thread = threading.Thread(target=self.apply_effect, args=(fun, self.median_args, True))
                thread.start()
                self.pulse_progressbar2(thread)
                if not self.workflow_started:
                        self.original_pixbuf = self.pblist[self.pbindex].copy()
                self.show_image1()
                self.get_channel()
                self.image_was_changed = True
                self.workflow_started = True
                self.button80.set_sensitive(True)
                self.window2.window.set_cursor(None)
                self.window5.hide()
                self.progressbar2.set_fraction(0)
        
        def save_median(self):
                self.button80.set_sensitive(False)
                self.button81.set_sensitive(True)
                self.spinbutton36.set_value(0)
                self.update_wand_list()
                self.image_was_changed = False
                self.history.append(['median_filter', self.median_args[:]])
                if self.expander8.get_expanded():
                        self.draw_grid(self.checkbutton24)
                        
        def apply_sepia_tone(self, obj):
                self.button84.set_sensitive(False)
                self.window2.window.set_cursor(self.watch)
                self.progressbar2.set_text(_('Sepia Tone...'))
                self.window5.show()
                while gtk.events_pending():
                        gtk.main_iteration(False)
                format = self.set_filenames()
                threshold = self.hscale19.get_adjustment().value * Q_RANGE / 100
                fun = MagickImage.sepia_tone
                self.sepia_tone_args = (threshold,)
                thread = threading.Thread(target=self.apply_effect, args=(fun, self.sepia_tone_args))
                thread.start()
                self.pulse_progressbar2(thread)
                if not self.workflow_started:
                        self.original_pixbuf = self.pblist[self.pbindex].copy()
                self.show_image1()
                self.get_channel()
                self.image_was_changed = True
                self.workflow_started = True
                self.button83.set_sensitive(True)
                self.window2.window.set_cursor(None)
                self.window5.hide()
                self.progressbar2.set_fraction(0)
        
        def save_sepia_tone(self):
                self.button83.set_sensitive(False)
                self.button84.set_sensitive(True)
                self.spinbutton31.set_value(80)
                self.update_wand_list()
                self.image_was_changed = False
                self.history.append(['sepia_tone', self.sepia_tone_args[:]])
                if self.expander8.get_expanded():
                        self.draw_grid(self.checkbutton24)
                                        
        def grayscale(self, obj):
                self.button90.set_sensitive(False)
                self.window2.window.set_cursor(self.watch)
                while gtk.events_pending():
                        gtk.main_iteration(False)
                format = self.set_filenames()
                self.wand._set_colorspace(api.GRAYColorspace)
                if not self.workflow_started:
                        self.original_pixbuf = self.pblist[self.pbindex].copy()
                self.show_image1()
                self.get_channel()
                self.image_was_changed = True
                self.workflow_started = True
                self.button89.set_sensitive(True)
                self.window2.window.set_cursor(None)
        
        def save_grayscale(self):
                self.button89.set_sensitive(False)
                self.button90.set_sensitive(True)
                self.update_wand_list()
                self.image_was_changed = False
                self.history.append(['_set_colorspace', (api.GRAYColorspace,)])
                if self.expander8.get_expanded():
                        self.draw_grid(self.checkbutton24)
                
        def save_normalize(self):
                self.button92.set_sensitive(False)
                self.button103.set_sensitive(True)
                self.update_wand_list()
                self.image_was_changed = False
                self.history.append(['normalize', ()])
                if self.expander8.get_expanded():
                        self.draw_grid(self.checkbutton24)

        def apply_distort(self, obj):
                self.window2.window.set_cursor(self.watch)
                while gtk.events_pending():
                        gtk.main_iteration(False)
                format = self.set_filenames()
                a = self.spinbutton51.get_adjustment().value
                b = self.spinbutton52.get_adjustment().value
                c = self.spinbutton53.get_adjustment().value
                d = self.spinbutton54.get_adjustment().value
                self.distort_args = (api.BarrelDistortion, (a,b,c,d))
                self.wand.distort(api.BarrelDistortion, (a,b,c,d), True)
                self.draw_distort_grid(api.BarrelDistortion, (a,b,c,d))
                if not self.workflow_started:
                        self.original_pixbuf = self.pblist[self.pbindex].copy()
                self.show_image1()
                self.get_channel()
                self.image_was_changed = True
                self.workflow_started = True
                self.button111.set_sensitive(True)
                self.window2.window.set_cursor(None)

        def save_distort(self):
                self.button111.set_sensitive(False)
                self.update_wand_list()
                self.image_was_changed = False
                self.history.append(['distort', self.distort_args[:]])
                if self.expander8.get_expanded():
                        self.draw_grid(self.checkbutton24)
                self.image66.set_from_pixbuf(gtk.gdk.pixbuf_new_from_file(os.path.join(self.temp, "grid.png")))
                
        def draw_distort_grid(self, DistortImageMethod, args):
                grid_wand = MagickImage(os.path.join(self.temp, "grid.png")) 
                grid_wand.distort(DistortImageMethod, args, True)
                self.image66.set_from_pixbuf(self.wand2pixbuf(grid_wand))

        def make_cursor(self):
                width = self.radius * 2
                surface = cairo.ImageSurface(cairo.FORMAT_ARGB32, width, width)
                cr = cairo.Context(surface)
                cr.set_source_rgb(0, 0, 1)
                cr.set_line_width(1)
                cr.arc(width/2.0, width/2.0, width/2.0 - 2, 0.0, 2 * math.pi)
                cr.stroke()
                data = surface.get_data()  
                stride = surface.get_stride()
                pixbuf = gtk.gdk.pixbuf_new_from_data(data, gtk.gdk.COLORSPACE_RGB, True, 8, width, width, stride)
                return gtk.gdk.Cursor(self.display, pixbuf, int(width/2.0), int(width/2.0))
                
        def on_viewport1_enter_notify_event(self, obj, event):
                if self.expander23.get_expanded() and self.notebook2.get_current_page() == 4:
                        self.scrolledwindow3.window.set_cursor(self.make_cursor())      
                        
        def on_viewport1_leave_notify_event(self, obj, event):
                if self.expander23.get_expanded():
                        self.scrolledwindow3.window.set_cursor(None)
                        
        def remove_red_eye(self, pointer):
                if not self.lock.locked():
                        with self.lock:
                                self.window2.window.set_cursor(self.watch)
                                while gtk.events_pending():
                                        gtk.main_iteration(False)
                                x, y = pointer
                                rect_width, rect_height = self.viewport1.get_hadjustment().upper, self.viewport1.get_vadjustment().upper
                                width = self.pblist[self.pbindex].get_width()
                                height = self.pblist[self.pbindex].get_height()
                                x_offset, y_offset = (rect_width - width) / 2.0, (rect_height - height) / 2.0
                                x0, x1, y0, y1 = x_offset, rect_width - x_offset, y_offset, rect_height - y_offset
                                if x0 < x < x1 and y0 < y < y1 :
                                        format = self.set_filenames()
                                        i_size = self.wand._get_size()
                                        self.red_eye_args = (x - x0) / width, (y - y0) / height, (x - x0) / width, (y - y0 - self.radius + 2) / height
                                        self.wand = self.apply_red_eye_remotion(i_size, self.wand, self.red_eye_args)
                                        if not self.workflow_started:
                                                self.original_pixbuf = self.pblist[self.pbindex].copy()
                                        self.show_image1()
                                        self.get_channel()
                                        self.image_was_changed = True
                                        self.workflow_started = True
                                        self.save_red_eye_remotion()
                                        self.scrolledwindow3.window.set_cursor(self.make_cursor())

        def apply_red_eye_remotion(self, i_size, wand_, red_eye_args):
                mask = MagickImage()
                clone = MagickImage(wand_.get_image())
                clone.recolor(3, (0.1, 0.6, 0.3, 0, 1, 0 ,0, 0, 1))
                mask.new_image(i_size[0], i_size[1], Color('white'))
                d_wand = drawing.Drawingwand()
                d_wand.set_fill_color(Color('black'))
                d_wand.draw_circle(red_eye_args[0] * i_size[0], red_eye_args[1] * i_size[1], red_eye_args[2] * i_size[0], red_eye_args[3] * i_size[1])
                mask.draw(d_wand)
                mask.scale(i_size[0], i_size[1]) 
                mask.set_alpha_channel(api.DeactivateAlphaChannel)
                wand_.set_clip_mask(mask)
                wand_.composite(clone,(0,0))
                mask.opaque_paint(Color('white'), Color('black'), 0)
                wand_.set_clip_mask(mask)
                return wand_
                        
        def save_red_eye_remotion(self):
                if self.history and self.history[-1][0] == 'red_eye' and self.wand_index == len(self.wand_list) - 1:
                        self.history[-1][1].append((self.red_eye_args))
                        self.wand_list[-1] = MagickImage(self.wand.get_image())
                else:
                        self.update_wand_list()
                        self.history.append(['red_eye', [(self.red_eye_args[:])]])
                        gobject.idle_add(self.set_toolbuttons_23_39_sensitivity)
                self.button114.set_sensitive(True)
                self.image_was_changed = False
                if self.expander8.get_expanded():
                        self.draw_grid(self.checkbutton24)

        def apply_oil_paint(self, obj):
                self.button116.set_sensitive(False)
                self.window2.window.set_cursor(self.watch)
                self.progressbar2.set_text(_('Oil Paint...'))
                self.window5.show()
                while gtk.events_pending():
                        gtk.main_iteration(False)
                format = self.set_filenames()
                radius = self.hscale37.get_adjustment().value
                fun = MagickImage.oil_paint
                self.oil_paint_args = (radius,)
                thread = threading.Thread(target=self.apply_effect, args=(fun, self.oil_paint_args, True))
                thread.start()
                self.pulse_progressbar2(thread)
                if not self.workflow_started:
                        self.original_pixbuf = self.pblist[self.pbindex].copy()
                self.show_image1()
                self.get_channel()
                self.image_was_changed = True
                self.workflow_started = True
                self.button115.set_sensitive(True)
                self.window2.window.set_cursor(None)
                self.window5.hide()
                self.progressbar2.set_fraction(0)

        def save_oil_paint(self):
                self.button115.set_sensitive(False)
                self.button116.set_sensitive(True)
                self.spinbutton56.set_value(0)
                self.update_wand_list()
                self.image_was_changed = False
                self.history.append(['oil_paint', self.oil_paint_args[:]])
                if self.expander8.get_expanded():
                        self.draw_grid(self.checkbutton24)

        def apply_spread(self, obj):
                self.button144.set_sensitive(False)
                self.window2.window.set_cursor(self.watch)
                self.progressbar2.set_text(_('Spread...'))
                self.window5.show()
                while gtk.events_pending():
                        gtk.main_iteration(False)
                format = self.set_filenames()
                radius = self.hscale42.get_adjustment().value
                fun = MagickImage.spread
                self.spread_args = (radius,)
                thread = threading.Thread(target=self.apply_effect, args=(fun, self.spread_args, True))
                thread.start()
                self.pulse_progressbar2(thread)
                if not self.workflow_started:
                        self.original_pixbuf = self.pblist[self.pbindex].copy()
                self.show_image1()
                self.get_channel()
                self.image_was_changed = True
                self.workflow_started = True
                self.button143.set_sensitive(True)
                self.window2.window.set_cursor(None)
                self.window5.hide()
                self.progressbar2.set_fraction(0)

        def save_spread(self):
                self.button143.set_sensitive(False)
                self.button144.set_sensitive(True)
                self.spinbutton64.set_value(0)
                self.update_wand_list()
                self.image_was_changed = False
                self.history.append(['spread', self.spread_args[:]])
                if self.expander8.get_expanded():
                        self.draw_grid(self.checkbutton24)
                
        def apply_despeckle(self, obj):
                self.button118.set_sensitive(False)
                self.window2.window.set_cursor(self.watch)
                self.progressbar2.set_text(_('Despeckling...'))
                self.window5.show()
                while gtk.events_pending():
                        gtk.main_iteration(False)
                format = self.set_filenames()
                fun = MagickImage.despeckle
                thread = threading.Thread(target=self.apply_effect, args=(fun,(), True))
                thread.start()
                self.pulse_progressbar2(thread)
                if not self.workflow_started:
                        self.original_pixbuf = self.pblist[self.pbindex].copy()
                self.show_image1()
                self.get_channel()
                self.image_was_changed = True
                self.workflow_started = True
                self.button117.set_sensitive(True)
                self.window2.window.set_cursor(None)
                self.window5.hide()
                self.progressbar2.set_fraction(0)

        def save_despeckle(self):
                self.button117.set_sensitive(False)
                self.button118.set_sensitive(True)
                self.update_wand_list()
                self.image_was_changed = False
                self.history.append(['despeckle', ()])
                if self.expander8.get_expanded():
                        self.draw_grid(self.checkbutton24)
                
        def apply_curve(self, obj, event):
                self.window2.window.set_cursor(self.watch)
                while gtk.events_pending():
                        gtk.main_iteration(False)
                format = self.set_filenames()
                h_radius = self.spinbutton55.get_value()
                s_radius = self.spinbutton58.get_value()
                htx = hty = self.spinbutton57.get_value() - 0.1
                stx = sty = self.spinbutton59.get_value() + 0.1
                if h_radius < 0:
                        h_axis = 0.7
                else:
                        h_axis = 0.3
                x0 = (1.0 - htx) * h_axis + htx
                y0 = x0 + h_radius
                if s_radius < 0:
                        s_axis = 0.7
                else:
                        s_axis = 0.3
                x1 = stx * s_axis
                y1 = x1 + s_radius
                if h_radius and not s_radius:
                        x = [0, 0.05] + numpy.arange(0.1, htx, 0.1).tolist() + [htx, x0, 1]
                        y = [0, 0.05] + numpy.arange(0.1, hty, 0.1).tolist() + [hty, y0, 1]
                elif s_radius and not h_radius:
                        x = [0, x1, stx] + numpy.arange(stx, 0.95, 0.1).tolist() + [0.95, 1]
                        y = [0, y1, sty] + numpy.arange(sty, 0.95, 0.1).tolist() + [0.95, 1]
                else:
                        x = numpy.arange(0, 1.1, 0.05).tolist()
                        y = numpy.arange(0, 1.1, 0.05).tolist()
                coeffs = numpy.polyfit(x, y, deg=len(x)-1) 
                self.coeffs = (coeffs,)
                self.wand.function(coeffs)
                if not self.workflow_started:
                        self.original_pixbuf = self.pblist[self.pbindex].copy()
                self.show_image1()
                self.get_channel()
                self.image_was_changed = True
                self.workflow_started = True
                self.window2.window.set_cursor(None)

        def reset_curve(self, obj, undo=True):
                self.spinbutton55.handler_block(self.id88)
                self.spinbutton57.handler_block(self.id89)
                self.spinbutton58.handler_block(self.id90)
                self.spinbutton59.handler_block(self.id91)
                self.spinbutton55.set_value(0)
                self.spinbutton57.set_value(0.50)
                self.spinbutton58.set_value(0)
                self.spinbutton59.set_value(0.50)
                self.on_spinbutton_value_changed(self.spinbutton55, self.hscale38)
                self.on_spinbutton_value_changed(self.spinbutton57, self.hscale39)
                self.on_spinbutton_value_changed(self.spinbutton58, self.hscale40)
                self.on_spinbutton_value_changed(self.spinbutton59, self.hscale41)
                self.spinbutton55.handler_unblock(self.id88)
                self.spinbutton57.handler_unblock(self.id89)
                self.spinbutton58.handler_unblock(self.id90)
                self.spinbutton59.handler_unblock(self.id91)
                while gtk.events_pending():
                        gtk.main_iteration(False)
                if undo:
                        self.undo_handler(obj)

        def save_curve(self):
                self.update_wand_list()
                self.image_was_changed = False
                self.history.append(['function', self.coeffs[:]])
                self.reset_curve(None, False)
                if self.expander8.get_expanded():
                        self.draw_grid(self.checkbutton24)

        def on_combobox25_changed(self, obj):
                self.notebook6.set_current_page(self.combobox25.get_active())
                gobject.idle_add(self.combobox1.set_active, self.combobox25.get_active())
                
        def start_curve_daemon(self, obj, event, signal):
                self.stop_thread2.clear()
                threading.Thread(target=self.curve_daemon, args=(obj, signal)).start()

        def stop_curve_daemon(self, obj, event):
                self.stop_thread2.set()
                        
        def curve_daemon(self, curve, signal):
                count = 0
                while 1:
                        if self.stop_thread2.isSet():
                                break
                        event = None
                        with gtk.gdk.lock:
                                event = gtk.get_current_event()
                        if event and event.type == gtk.gdk.BUTTON_RELEASE:
                                count += 1
                                if count == 1:
                                        gobject.idle_add(curve.emit, signal, 0)
                        else:
                                count = 0
                        time.sleep(0.001)
                                
        def apply_gamma_curve(self, curve, event, channel):
                if not self.lock.locked():
                        with self.lock:
                                self.window2.window.set_cursor(self.watch)
                                while gtk.events_pending():
                                        gtk.main_iteration(False)
                                format = self.set_filenames()
                                x = numpy.arange(0, 1.025, 0.025)
                                y = curve.get_vector(41)
                                coeffs = numpy.polyfit(x, y, deg=len(x)-1)
                                self.vector[channel] = coeffs
                                if self.vector[0] != ():       
                                        self.wand.function(self.vector[0], api.AllChannels)
                                if self.vector[1] != ():       
                                        self.wand.function(self.vector[1], api.RedChannel)
                                if self.vector[2] != ():       
                                        self.wand.function(self.vector[2], api.GreenChannel)
                                if self.vector[3] != ():       
                                        self.wand.function(self.vector[3], api.BlueChannel)
                                if not self.workflow_started:
                                        self.original_pixbuf = self.pblist[self.pbindex].copy() 
                                if self.wand:
                                        self.pblist[self.pbindex] = self.wand2pixbuf(self.wand, True)
                                if self.checkbutton41.get_active():
                                        self.image28.set_from_pixbuf(self.profile_pixbuf(self.pblist[self.pbindex]))
                                else:
                                        self.image28.set_from_pixbuf(self.pblist[self.pbindex])
                                self.image28.show()
                                gobject.idle_add(self.get_channel)
                                self.workflow_started = True
                                self.image_was_changed = True
                                self.window2.window.set_cursor(None)

        def reset_gamma_curve(self, obj):
                if not self.lock.locked():
                        curve = None
                        with self.lock:
                                channel = self.combobox25.get_active()
                                if channel == 0:
                                        curve = self.curve1
                                elif channel == 1:
                                        curve = self.curve2
                                elif channel == 2:
                                        curve = self.curve3
                                elif channel == 3:
                                        curve = self.curve4
                                curve.reset()
                                self.vector[channel] = ()
                        if curve:
                                self.apply_gamma_curve(curve, None, channel) 

        def reset_all_curves(self, obj):
                if not self.lock.locked():
                        with self.lock:
                                self.curve1.reset()
                                self.curve2.reset()
                                self.curve3.reset()
                                self.curve4.reset()
                                while gtk.events_pending():
                                        gtk.main_iteration(False)
                                self.vector = {0 : (), 1 : (), 2 : (), 3 : ()}
                                self.undo_handler(obj)
                
        def save_gamma_curve(self):
                self.update_wand_list()
                self.image_was_changed = False
                curve_hist = ['curves', []]
                if self.vector[0] != ():
                        curve_hist[1].append(['function', (self.vector[0][:], api.AllChannels)])
                if self.vector[1] != ():
                        curve_hist[1].append(['function', (self.vector[1][:], api.RedChannel)])
                if self.vector[2] != ():
                        curve_hist[1].append(['function', (self.vector[2][:], api.GreenChannel)])
                if self.vector[3] != ():
                        curve_hist[1].append(['function', (self.vector[3][:], api.BlueChannel)])
                if curve_hist[1]:
                        self.history.append(curve_hist)
                if self.expander29.get_expanded():
                        self.curve1.reset()
                        self.curve2.reset()
                        self.curve3.reset()
                        self.curve4.reset()
                if self.expander8.get_expanded():
                        self.draw_grid(self.checkbutton24)
                self.vector = {0 : (), 1 : (), 2 : (), 3 : ()}
                
        def on_eventbox_unmapped(self, obj, event):
                if self.image_was_changed:
                        if obj == self.eventbox8:
                                self.save_levels()
                        elif obj == self.eventbox9:
                                self.save_curve()
                        elif obj == self.eventbox10:
                                self.save_curve()
                        elif obj == self.eventbox11:
                                self.save_brightness_and_saturation()
                        elif obj == self.eventbox12:
                                self.save_color_balance()
                        elif obj == self.eventbox13:
                                self.save_contrast()
                        elif obj == self.eventbox14:
                                self.save_midtones_contrast()
                        elif obj == self.eventbox15:
                                self.save_mixer()
                        elif obj == self.eventbox16:
                                self.save_normalize()
                        elif obj == self.eventbox17:
                                self.save_scale()
                        elif obj == self.eventbox18:
                                self.save_fine_rotation()
                                if self.checkbutton41.get_active():
                                        self.image28.set_from_pixbuf(self.profile_pixbuf(self.pblist[self.pbindex]))
                                else:
                                        self.image28.set_from_pixbuf(self.pblist[self.pbindex])
                                self.image28.show()
                        elif obj == self.eventbox19:
                                self.save_img_border()
                        elif obj == self.eventbox20:
                                self.destroy_drawingarea()
                                if self.checkbutton41.get_active():
                                        self.image28.set_from_pixbuf(self.profile_pixbuf(self.pblist[self.pbindex]))
                                else:
                                        self.image28.set_from_pixbuf(self.pblist[self.pbindex])
                                self.image28.show()
                                self.save_crop()
                        elif obj == self.eventbox21:
                                self.save_distort()
                                if self.checkbutton34.get_active():
                                        if self.checkbutton41.get_active():
                                                self.image28.set_from_pixbuf(self.profile_pixbuf(self.pblist[self.pbindex]))
                                        else:
                                                self.image28.set_from_pixbuf(self.pblist[self.pbindex])
                                        self.image28.show()
                        elif obj == self.eventbox22:
                                self.save_sharpen()
                        elif obj == self.eventbox23:
                                self.save_unsharp_mask()
                        elif obj == self.eventbox24:
                                self.save_blur()
                        elif obj == self.eventbox25:
                                self.save_reduce_noise()
                        elif obj == self.eventbox26:
                                self.save_despeckle()
                        elif obj == self.eventbox27:
                                self.save_add_noise()
                        elif obj == self.eventbox28:
                                self.save_median()
                        elif obj == self.eventbox29:
                                self.save_sepia_tone()
                        elif obj == self.eventbox30:
                                self.save_grayscale()
                        elif obj == self.eventbox31:
                                self.save_oil_paint()
                        elif obj == self.eventbox32:
                                self.save_gamma_curve()
                                if self.combobox1.get_active() != 0:
                                        self.combobox1.set_active(0)
                        elif obj == self.eventbox33:
                                self.save_spread()
                        elif obj == self.eventbox34:
                                self.save_tint()
                        elif obj == self.eventbox35:
                                self.save_charcoal()
                        elif obj == self.eventbox36:
                                self.save_sketch()
                        elif obj == self.eventbox37:
                                self.save_posterize()
                        elif obj == self.eventbox38:
                                self.save_negate()
                        elif obj == self.eventbox42:
                                self.save_white_balance()
                        elif obj == self.eventbox43:
                                self.save_colortemp()
                                
        def apply_tint(self, obj):
                if not self.lock.locked():
                        threading.Thread(target=self.make_tint, args=()).start()
                        
        def make_tint(self):
                with self.lock:
                        gobject.idle_add(self.window2.window.set_cursor, self.watch)
                        while gtk.events_pending():
                                gtk.main_iteration(False)
                        format = self.set_filenames()
                        self.tint_color = self.colorselection1.get_current_color().to_string()
                        self.wand = self._tint(self.wand, self.tint_color)
                        if not self.workflow_started:
                                self.original_pixbuf = self.pblist[self.pbindex].copy()
                        gobject.idle_add(self.show_image1)
                        gobject.idle_add(self.get_channel)
                        self.image_was_changed = True
                        self.workflow_started = True
                        gobject.idle_add(self.window2.window.set_cursor, None)
                        gobject.idle_add(self.button142.set_sensitive, True)
                        time.sleep(0.2)

        def _tint(self, wand, tint_color):
                pixelwand = Color(tint_color)
                pixelwand.set_opacity(0.0)
                wand.tint(Color(tint_color), pixelwand)
                return wand
                
        def save_tint(self):
                self.update_wand_list()
                self.image_was_changed = False
                self.button142.set_sensitive(False)
                self.history.append(['tint', (self.tint_color,)])
                if self.expander8.get_expanded():
                        self.draw_grid(self.checkbutton24)

        def on_color_picker_toggled(self, obj):
                if obj.get_active():
                        self.togglebutton3.handler_block(self.id92)
                        self.togglebutton4.handler_block(self.id93)
                        self.togglebutton5.handler_block(self.id94)
                        for item in [self.togglebutton3, self.togglebutton4, self.togglebutton5]:
                                if item != obj:
                                        item.set_active(False)
                        self.togglebutton3.handler_unblock(self.id92)
                        self.togglebutton4.handler_unblock(self.id93)
                        self.togglebutton5.handler_unblock(self.id94)
                self.set_color_picker_cursor()

        def set_color_picker_cursor(self):
                if self.expander36.get_expanded():
                        self.make_color_picker_cursor()

        def make_color_picker_cursor(self):
                cursor = None
                if self.togglebutton3.get_active():
                        pixbuf = gtk.gdk.pixbuf_new_from_file(self.get_icon_path('stock-color-picker-black-18.png'))
                        cursor = gtk.gdk.Cursor(self.display, pixbuf, 2, 15)
                elif self.togglebutton4.get_active():
                        pixbuf = gtk.gdk.pixbuf_new_from_file(self.get_icon_path('stock-color-picker-gray-18.png'))
                        cursor = gtk.gdk.Cursor(self.display, pixbuf, 2, 15)
                elif self.togglebutton5.get_active():
                        pixbuf = gtk.gdk.pixbuf_new_from_file(self.get_icon_path('stock-color-picker-white-18.png'))
                        cursor = gtk.gdk.Cursor(self.display, pixbuf, 2, 15)
                self.window2.window.set_cursor(cursor)
                self.color_picker_cursor = cursor
                
        def apply_white_balance(self, pointer):
                if not self.lock.locked():
                        with self.lock:
                                x, y = pointer
                                rect_width, rect_height = self.viewport1.get_hadjustment().upper, self.viewport1.get_vadjustment().upper
                                width = self.pblist[self.pbindex].get_width()
                                height = self.pblist[self.pbindex].get_height()
                                x_offset, y_offset = (rect_width - width) / 2.0, (rect_height - height) / 2.0
                                x0, x1, y0, y1 = x_offset, rect_width - x_offset, y_offset, rect_height - y_offset
                                if x0 < x < x1 and y0 < y < y1 :
                                        self.black_balance = False
                                        self.window2.window.set_cursor(self.watch)
                                        while gtk.events_pending():
                                                gtk.main_iteration(False)
                                        format = self.set_filenames()
                                        assert(self.pblist[self.pbindex].get_colorspace() == gtk.gdk.COLORSPACE_RGB)
                                        if not pygtk_has_numpy_support:
                                                dimensions = self.pblist[self.pbindex].get_width(), self.pblist[self.pbindex].get_height()
                                                stride = self.pblist[self.pbindex].get_rowstride()
                                                pixels = self.pblist[self.pbindex].get_pixels()
                                                mode = self.pblist[self.pbindex].get_has_alpha() and "RGBA" or "RGB"
                                                im = Image.frombuffer(mode, dimensions, pixels, "raw", mode, stride, 1)
                                                pixels = im.load()
                                                ured, ugreen, ublue = pixels[x - x0, y - y0][0:3]
                                        else:
                                                with warnings.catch_warnings():
                                                        warnings.simplefilter("ignore")
                                                        pixels = self.pblist[self.pbindex].get_pixels_array()
                                                ured, ugreen, ublue = pixels[y-y0][x-x0][0:3]
                                        if self.togglebutton5.get_active():
                                                rred = 255
                                                rgreen = 255
                                                rblue = 255
                                        elif self.togglebutton4.get_active():
                                                rred = 128
                                                rgreen = 128
                                                rblue = 128
                                        if ured == 0:
                                                red_frac = rred
                                        else:
                                                red_frac = float(rred) / (ured)
                                        if ugreen == 0:
                                                green_frac = rgreen
                                        else:
                                                green_frac = float(rgreen) / (ugreen)
                                        if ublue == 0:
                                                blue_frac = rblue
                                        else:
                                                blue_frac = float(rblue) / (ublue)
                                        self.matrix = (red_frac, 0, 0, 0, green_frac, 0, 0, 0, blue_frac)
                                        self.wand.recolor(3, self.matrix)
                                        self.wb_hist[1].append(['recolor', (3, self.matrix)])
                                        if not self.workflow_started:
                                                self.original_pixbuf = self.pblist[self.pbindex].copy()
                                        self.show_image1()
                                        self.get_channel()
                                        self.image_was_changed = True
                                        self.workflow_started = True
                                        self.set_color_picker_cursor()

        def apply_black_balance(self, pointer):
                if not self.lock.locked():
                        with self.lock:
                                x, y = pointer
                                rect_width, rect_height = self.viewport1.get_hadjustment().upper, self.viewport1.get_vadjustment().upper
                                width = self.pblist[self.pbindex].get_width()
                                height = self.pblist[self.pbindex].get_height()
                                x_offset, y_offset = (rect_width - width) / 2.0, (rect_height - height) / 2.0
                                x0, x1, y0, y1 = x_offset, rect_width - x_offset, y_offset, rect_height - y_offset
                                if x0 < x < x1 and y0 < y < y1 :
                                        self.black_balance = True
                                        self.window2.window.set_cursor(self.watch)
                                        while gtk.events_pending():
                                                gtk.main_iteration(False)
                                        format = self.set_filenames()
                                        assert(self.pblist[self.pbindex].get_colorspace() == gtk.gdk.COLORSPACE_RGB)
                                        if not pygtk_has_numpy_support:
                                                dimensions = self.pblist[self.pbindex].get_width(), self.pblist[self.pbindex].get_height()
                                                stride = self.pblist[self.pbindex].get_rowstride()
                                                pixels = self.pblist[self.pbindex].get_pixels()
                                                mode = self.pblist[self.pbindex].get_has_alpha() and "RGBA" or "RGB"
                                                im = Image.frombuffer(mode, dimensions, pixels, "raw", mode, stride, 1)
                                                pixels = im.load()
                                                ured, ugreen, ublue = pixels[x - x0, y - y0][0:3]
                                        else:
                                                with warnings.catch_warnings():
                                                        warnings.simplefilter("ignore")
                                                        pixels = self.pblist[self.pbindex].get_pixels_array()
                                                ured, ugreen, ublue = pixels[y-y0][x-x0][0:3]
                                        self.black_balance_args = float(ured) * Q_RANGE / 255, float(ugreen) * Q_RANGE / 255, float(ublue) *Q_RANGE / 255
                                        self.wand.evaluate('Subtract', self.black_balance_args[0], api.RedChannel)
                                        self.wand.evaluate('Subtract', self.black_balance_args[1], api.GreenChannel)
                                        self.wand.evaluate('Subtract', self.black_balance_args[2], api.BlueChannel)
                                        if not self.workflow_started:
                                                self.original_pixbuf = self.pblist[self.pbindex].copy()
                                        self.show_image1()
                                        self.get_channel()
                                        self.image_was_changed = True
                                        self.workflow_started = True
                                        self.set_color_picker_cursor()
                                        self.wb_hist[1].append(['evaluate', ('Subtract', self.black_balance_args[0], api.RedChannel)])
                                        self.wb_hist[1].append(['evaluate', ('Subtract', self.black_balance_args[1], api.GreenChannel)])
                                        self.wb_hist[1].append(['evaluate', ('Subtract', self.black_balance_args[2], api.BlueChannel)])
                                        
        def save_white_balance(self):
                self.update_wand_list()
                self.image_was_changed = False
                self.history.append(self.wb_hist[:])
                if self.expander8.get_expanded():
                        self.draw_grid(self.checkbutton24)
                self.wb_hist[1] = []

        def apply_colortemp(self, spin, event=None):
                if not self.lock.locked():
                        with self.lock:
                                self.window2.window.set_cursor(self.watch)
                                while gtk.events_pending():
                                        gtk.main_iteration(False)
                                format = self.set_filenames()
                                temperature = round(self.spinbutton72.get_value() / 100) * 100
                                r, g, b = self.kelvin2RGB[temperature]
                                tint = self.spinbutton73.get_value()
                                self.matrix = (r, 0, 0, 0, g * tint, 0, 0, 0, b)
                                self.wand.recolor(3, self.matrix)
                                if not self.workflow_started:
                                        self.original_pixbuf = self.pblist[self.pbindex].copy()
                                self.show_image1()
                                self.get_channel()
                                self.image_was_changed = True
                                self.workflow_started = True
                                self.window2.window.set_cursor(None)

        def save_colortemp(self):
                self.update_wand_list()
                self.image_was_changed = False                       
                self.history.append(['recolor', self.matrix[:], False])       
                if self.expander8.get_expanded():
                        self.draw_grid(self.checkbutton24)
                self.spinbutton72.set_value(7500)
                self.spinbutton73.set_value(1)
                                        
        def update_wand_list(self):
                self.wand_index += 1
                if self.wand_index == len(self.wand_list):
                        self.wand_list += [ MagickImage(self.wand.get_image()) ]
                else:
                        self.wand_list[self.wand_index] = MagickImage(self.wand.get_image())
                        del self.wand_list[self.wand_index + 1:]
                        del self.history[self.wand_index - 1:]
                        self.size_history[self.wand_index:] = [self.size_history[self.wand_index] for i in xrange(self.wand_index, 100)]
                self.set_toolbuttons_23_39_sensitivity()
                
        def get_real_path(self, fname):
                if self.ark not in fname:
                        if fname.lower().endswith(self.raw_formats):
                                if self.rawpath.has_key(fname):
                                        filename = self.rawpath[fname] 
                                else:
                                        filename = fname
                        else:
                                filename = fname
                else:
                        if fname.lower().endswith(self.raw_formats):
                                filename = fname.replace(homepath + 'thumb', '').replace('.thumb.jpg', '')
                        else:
                                filename = fname
                return filename
                        
        def show_batch_queue(self, obj):
                if self.window7.get_property('visible'): 
                        if self.window7_was_iconified:
                                self.window7.present()
                        else:
                                self.on_delete_event(self.window7, None)
                else:
                        self.window7.set_title('Batch Manager')
                        self.window7.show()

        def on_renderer_toggle_toggled(self, cell, path, model):
                if model[path][4] in [None, 'not found!']:
                        model[path][2] = not model[path][2]
                else:
                        gtk.gdk.beep()

        def cell_data_function(self, column, cell, model, _iter, data=None):
                value = model.get_value(_iter, 4)
                if value == None:
                        color = 'white'
                elif value == 'processing...':
                        color = 'yellow'
                elif value == 'not found!':
                        color = 'red'
                else:
                        color = 'green'
                cell.set_property('cell_background', color)

        def add_to_batch_queue(self, obj):              # Adds item to batch manager
                expanded = False
                for item in self.expander_list:
                        if item.get_expanded():
                                expanded = True
                                break
                if expanded:
                        self.on_expander_activate(item, False)
                filename = self.get_real_path(self.list[self.indice])
                if not self.batch_queue.has_key(filename):
                        pixbuf = self.pblist[self.pbindex].copy()
                        width, height = float(pixbuf.get_width()), float(pixbuf.get_height())
                        if width > height:
                                pixbuf = pixbuf.scale_simple(int(50.0 * (width / height)), 50, gtk.gdk.INTERP_BILINEAR)
                        else:
                                pixbuf = pixbuf.scale_simple(int(50.0 * (width / height)), 50, gtk.gdk.INTERP_BILINEAR)
                        pixbuf.save(os.path.join(self.batch_thumbs, os.path.basename(filename) + '.jpg'), "jpeg", {"quality" : "100"})
                        self.batch_iter = self.batchmodel.append([len(self.batchmodel) + 1, pixbuf, True, filename, None] )
                        if filename.lower().endswith(self.raw_formats):
                                dcraw_cmd = self.make_dcraw_command()
                        else:
                                dcraw_cmd = (None, None, None)
                        ###########################################       
                        if not self.checkbutton22.get_active():
                                self.label135.set_text(_('The file was added to the batch queue!'))
                                gtk.gdk.beep()
                                self.dialog24.show()
                                response = self.dialog24.run()
                                if response == gtk.RESPONSE_CLOSE or response == gtk.RESPONSE_DELETE_EVENT:
                                        self.dialog24.hide()
                        if self.checkbutton41.get_active():
                                history = self.profile_workflow(self.history[:self.wand_index])
                        else:
                                history = self.history[:self.wand_index]
                        self.batch_queue[filename] = [history, dcraw_cmd]
                        self.store_batch_queue()
                else:
                        gtk.gdk.beep()
                        dialog25 = self.messagedialog(_("Attention!"), _('Cannot add twin items!'), None, gtk.BUTTONS_CLOSE, None,
                                                        gtk.DIALOG_MODAL, gtk.MESSAGE_WARNING, None, (330,180))
                        dialog25.set_keep_above(True)
                        response = dialog25.run()
                        if response == gtk.RESPONSE_CLOSE or response == gtk.RESPONSE_DELETE_EVENT:
                                dialog25.destroy()
                                
        def show_dialog31(self, obj):
                check = False
                for row in self.batchmodel:
                        if row[2]:
                                check = True
                                break
                if check:
                        if self.checkbutton29.get_active():
                                self.remove_from_batch_queue(obj)
                        else:
                                self.dialog31.show()
                                response = self.dialog31.run()
                                if response == gtk.RESPONSE_CANCEL or response == gtk.RESPONSE_DELETE_EVENT:
                                        self.dialog31.hide()
                                elif response == gtk.RESPONSE_OK:
                                        self.dialog31.hide()
                                        self.remove_from_batch_queue(obj)       

        def remove_from_batch_queue(self, obj):         # Removes all checked items from batch manager
                filename = None
                for row in self.batchmodel:
                        if row[2]:
                                del self.batch_queue[row[3]]
                                filename = row[3]
                                self.batchmodel.remove(row.iter)
                                threading.Thread(target=os.remove, args=(os.path.join(self.batch_thumbs, os.path.basename(filename) + '.jpg'),)).start()
                i=0
                for row in self.batchmodel:
                        row[0] = i + 1
                        i += 1
                self.store_batch_queue()
                
        def on_button36_clicked(self, obj):             # Selects/Unselects all items on batch manager
                if len(self.batchmodel):
                        if obj.get_label() == 'Select All':
                                obj.set_label('Unselect All')
                                value = True
                        else:
                                obj.set_label('Select All')
                                value = False
                        for row in self.batchmodel:
                                row[2] = value
                  
        def on_button42_clicked(self, obj):             # Closes/Stops batch processor
                if obj.get_label() == 'gtk-close':
                        self.on_delete_event(self.window7, None)
                        obj.set_tooltip_text('Close Batch Manager')
                else:
                        self.stop_batch_process = True
                        gobject.idle_add(self.batchmodel.set_value, self.batch_iter, 4, 'stopping...')
                        gobject.idle_add(self.button42.set_label, 'gtk-close')
                        gobject.idle_add(self.button42.set_property, 'image', gtk.image_new_from_stock(gtk.STOCK_CLOSE, gtk.ICON_SIZE_BUTTON))
                                
        def on_button45_clicked(self, obj):             # Starts the batch processor
                if len(self.batchmodel):
                        obj.set_sensitive(False)
                        self.button36.set_sensitive(False)
                        self.button40.set_sensitive(False)
                        self.button45.set_sensitive(False)
                        self.combobox10.set_sensitive(False)
                        self.filechooserbutton7.set_sensitive(False)
                        self.hscale23.set_sensitive(False)
                        self.button42.set_label('gtk-stop')
                        self.button42.set_property('image', gtk.image_new_from_stock(gtk.STOCK_STOP, gtk.ICON_SIZE_BUTTON))
                        self.button42.set_tooltip_text('Stop Batch Processor')
                        threading.Thread(target=self.start_batch_processor, args=()).start()

        def get_process_options(self, infile, extension, dest):
                history, dcraw_cmd = self.batch_queue[infile]
                fname = os.path.basename(infile).split('.')[0]
                tempfile = os.path.join(self.thumbdir, '_tempfile')
                if infile.lower().endswith(self.raw_formats):
                        raw = True
                else:
                        raw = False
                if extension == 0:
                        outfile = os.path.join(dest, fname + '.jpg')
                        depth = None
                        quality = self.hscale23.get_adjustment().value
                        if raw:
                                tupla = (infile, tempfile)
                                data = True
                        else:
                                tupla = (infile, 0)
                                data = False
                elif extension == 1:
                        outfile = os.path.join(dest, fname + '.png')
                        depth = None
                        quality = None
                        if raw:
                                tupla = (infile, tempfile)
                                data = True
                        else:
                                tupla = (infile, 0)
                                data = False
                elif extension == 2:
                        outfile = os.path.join(dest, fname + '.tiff')
                        depth = 8
                        quality = None
                        if raw:
                                tupla = (infile, tempfile)
                                data = True
                        else:
                                tupla = (infile, 0)
                                data = False
                elif extension == 3:
                        outfile = os.path.join(dest, fname + '.tiff')
                        depth = 16
                        quality = None
                        if raw:
                                tupla = (infile, tempfile)
                                data = True
                        else:
                                tupla = (infile, 0)
                                data = False
                elif extension == 4:
                        outfile = os.path.join(dest, fname + '.tiff')
                        depth = QUANTUM
                        quality = None
                        if raw:
                                tupla = (infile, tempfile)
                                data = True
                        else:
                                tupla = (infile, 0)
                                data = False
                elif extension == 5:
                        outfile = os.path.join(dest, fname + '.bmp')
                        depth = None
                        quality = None
                        if raw:
                                tupla = (infile, tempfile)
                                data = True
                        else:
                                tupla = (infile, 0)
                                data = False
                return history, dcraw_cmd, tupla, outfile, data, depth, quality, True

        def start_batch_processor(self):
                self.stop_batch_process = None
                dest = self.filechooserbutton7.get_current_folder()
                extension = self.combobox10.get_active()
                self.batch_iter = self.batchmodel.get_iter_first()
                while gtk.events_pending():
                        gtk.main_iteration(False)
                while self.batch_iter:
                        if self.batchmodel.get_value(self.batch_iter, 2):
                                infile = self.batchmodel.get_value(self.batch_iter, 3)
                                if not os.path.isfile(infile):
                                        gobject.idle_add(self.batchmodel.set_value, self.batch_iter, 4, _('not found!'))
                                        gobject.idle_add(self.batchmodel.set_value, self.batch_iter, 2, False)
                                        self.batch_iter = self.batchmodel.iter_next(self.batch_iter)
                                        continue
                                run = self.batchmodel.get_value(self.batch_iter, 0)
                                selection = [row[2] for row in self.batchmodel if row[2] == True]
                                gobject.idle_add(self.window7.set_title, _('Batch Manager - processing file %s - remaining %s') % (run, max(0, len(selection)-1)))
                                gobject.idle_add(self.batchmodel.set_value, self.batch_iter, 4, _('processing...'))
                                process_options = self.get_process_options(infile, extension, dest)
                                self.process(*process_options)
                                gobject.idle_add(self.batchmodel.set_value, self.batch_iter, 4, _('completed'))
                                gobject.idle_add(self.batchmodel.set_value, self.batch_iter, 2, False)
                                self.batch_iter = self.batchmodel.iter_next(self.batch_iter)
                                if self.batch_iter:
                                        gobject.idle_add(self.treeview4.scroll_to_cell, self.batchmodel.get_path(self.batch_iter))
                                if self.stop_batch_process:
                                        self.batch_iter = None
                        else:
                                self.batch_iter = self.batchmodel.iter_next(self.batch_iter)
                        while gtk.events_pending():
                                gtk.main_iteration(False)
                gobject.idle_add(self.window7.set_title, _('Batch Manager - Queue completed!'))
                gobject.idle_add(self.button36.set_sensitive, True)
                gobject.idle_add(self.button40.set_sensitive, True)
                gobject.idle_add(self.button45.set_sensitive, True)
                gobject.idle_add(self.combobox10.set_sensitive, True)
                gobject.idle_add(self.filechooserbutton7.set_sensitive, True)
                gobject.idle_add(self.hscale23.set_sensitive, True)
                gobject.idle_add(self.button42.set_label, 'gtk-close')
                gobject.idle_add(self.button42.set_property, 'image', gtk.image_new_from_stock(gtk.STOCK_CLOSE, gtk.ICON_SIZE_BUTTON))
                gobject.idle_add(self.button42.set_tooltip_text, 'Close Batch Manager')
                self.stop_batch_process = None
                if self.checkbutton27.get_active():
                        for row in self.batchmodel:
                                del self.batch_queue[row[3]]
                        gobject.idle_add(self.batchmodel.clear)
                else:
                        selected = False
                        for row in self.batchmodel:
                                if row[4] != 'not found!':
                                        gobject.idle_add(self.batchmodel.set_value, row.iter, 4, None)
                                if row[2]:
                                        selected = True
                        if not selected:
                                gobject.idle_add(self.button36.set_label, 'Select All')

########## END OF IMAGE TOOLS METHODS ##########################################################################

        def on_combobox18_changed(self, obj):
                if obj == self.combobox18:
                        boxmodel = self.boxmodel21
                        combobox = self.combobox21
                else:
                        boxmodel = self.boxmodel22
                        combobox = self.combobox22
                if self.window8.get_property('visible'):
                        self.window8.window.set_cursor(self.watch)
                if self.window2.get_property('visible'):
                        self.window2.window.set_cursor(self.watch)
                while gtk.events_pending():
                        gtk.main_iteration(False)
                p = Popen([exiftool, '-listw', self.tag_group[obj.get_active()] + ':all'], stdin=PIPE, stdout=PIPE, stderr=STDOUT, universal_newlines=True)
                output = p.stdout.read()
                p.stdout.close()
                p.stdin.close()
                l = output.split('\n')[1:-1]
                tags = []
                boxmodel.clear()
                for item in l:
                        tags += item.strip().split(' ')
                for item in tags:
                        boxmodel.append([item])
                combobox.set_model(boxmodel)
                combobox.set_active(0)
                if self.window8.get_property('visible'):
                        self.window8.window.set_cursor(None)
                if self.window2.get_property('visible'):
                        self.window2.window.set_cursor(None)

        def on_combobox20_changed(self, obj):
                if obj == self.combobox20:
                        model = self.custom_model2
                else:
                        model = self.custom_model
                template = obj.get_active_text()
                model.clear()
                if template not in ['None', None]:
                        for item in self.templates[template]:
                                model.append(item)
                
        def add_custom_tag(self, obj):
                if obj == self.button122:
                        groupbox = self.combobox18
                        model = self.custom_model
                        tagbox = self.combobox21
                else:
                        groupbox = self.combobox19
                        model = self.custom_model2
                        tagbox = self.combobox22
                group = self.tag_group[groupbox.get_active()][1:]
                tag = tagbox.get_active_text()
                found = False
                for row in model:
                        if row[0] == group and row[1] == tag:
                                found = True
                                break
                if not found:
                        model.append([group, tag, ''])
                        
        def remove_from_custom_model2(self, obj):
                selection = self.treeview14.get_selection()
                model, iter_ = selection.get_selected()
                if iter_:
                        model.remove(iter_)
                        
        def on_custom_model_edited(self, cell, path, new_text, model, column):
                if model == self.preset_model and column > 1:
                        if not new_text.isdigit():
                                gtk.gdk.beep()
                                return True
                model.set_value(model.get_iter(path), column, new_text)

        def clear_custom_tags(self, obj):
                if obj == self.button124:
                        self.custom_model.clear()
                else:
                        self.custom_model2.clear()

        def save_metadata_template(self, obj):
                if obj == self.button127:
                        model = self.custom_model
                else:
                        model = self.custom_model2
                if len(model):
                        name = self.save_template()
                        if name:
                                tags = []
                                for row in model:
                                        tags.append([row[0], row[1], row[2]])
                                if not self.templates.has_key(name):
                                        self.boxmodel14.append([name])
                                self.templates[name] = tags
                                self.store_templates_db()
                else:
                        dialog25 = self.messagedialog(_("Attention!"), _('Add preferred tags to list first!\nDouble click the value row to edit.'),
                                                      None, gtk.BUTTONS_CLOSE, None, gtk.DIALOG_MODAL, gtk.MESSAGE_WARNING, None, (330,180))
                        dialog25.set_keep_above(True)
                        response = dialog25.run()
                        if response == gtk.RESPONSE_CLOSE or response == gtk.RESPONSE_DELETE_EVENT:
                                dialog25.destroy()

        def save_template(self):
                self.entry1.grab_focus()
                self.label3.set_text(_('Template name'))
                self.dialog1.set_title(_('Save metadata template'))
                self.dialog1.show()
                response = self.dialog1.run()
                if response == gtk.RESPONSE_CANCEL or response == gtk.RESPONSE_DELETE_EVENT:
                        self.dialog1.hide()
                        return None
                elif response == gtk.RESPONSE_OK:
                        name = self.entry1.get_text().strip()
                        if name and name not in ['None', 'Template1']:
                                self.dialog1.hide()
                                return name
                        else:
                                gtk.gdk.beep()
                                self.save_template()
                                
        def delete_template(self, obj):
                self.dialog33.show()
                response = self.dialog33.run()
                if response == gtk.RESPONSE_CANCEL or response == gtk.RESPONSE_DELETE_EVENT:
                        self.dialog33.hide()
                elif response == gtk.RESPONSE_OK:
                        self.dialog33.hide()
                        template = self.combobox23.get_active_text()
                        if template not in ['None', 'Template1']:
                                for row in self.boxmodel14:
                                        if row[0] == template:
                                                self.boxmodel14.remove(row.iter)
                                                del self.templates[template]
                                                self.combobox14.set_active(0)
                                                self.combobox20.set_active(0)
                                                self.combobox23.set_active(0)
                                self.store_templates_db()
                        else:
                                self.show_dialog15(_('Operation not permitted!'))
                                
        def show_tag_manager(self, obj):
                if self.radiobutton7.get_active():
                        self.selected = [self.list[self.indice]]
                if self.selected:
                        for row in self.tagmodel:
                                if row[0]:
                                        row[0] = False
                        self.window8.show()
                        return True
                else:
                        dialog9 = self.messagedialog(_("Info!"), _("No file selected!"), None, gtk.BUTTONS_CLOSE, self.window1, gtk.DIALOG_MODAL, gtk.MESSAGE_INFO, None, (230,120))
                        dialog9.set_keep_above(True)
                        gtk.gdk.beep()
                        response = dialog9.run()
                        if response == gtk.RESPONSE_CLOSE or response == gtk.RESPONSE_DELETE_EVENT:
                                dialog9.destroy()
                
        def on_tag_toggle_toggled(self, cell, path, model):
                model[path][0] = not model[path][0]
                if model == self.tagmodel2:
                        while gtk.events_pending():
                                gtk.main_iteration(False)
                        if model[path][0]:
                                self.process_metadata(self.list[self.indice], [model[path][1]], 'add', None, True)
                        else:
                                self.process_metadata(self.list[self.indice], [model[path][1]], 'remove')
                                
        def write_metadata(self, obj):                  # writes metadata when write metadata button is clicked
                self.window2.window.set_cursor(self.watch)
                while gtk.events_pending():
                        gtk.main_iteration(False)
                filename = self.get_real_path(self.list[self.indice])
                if obj == self.button91:
                        taglist = [row[1] for row in self.tagmodel2 if row[0]]
                        cmd = [exiftool, '-overwrite_original', '-Keywords=' + ", ".join(taglist)]
                else:
                        if len(self.custom_model2):
                                cmd = [exiftool, '-overwrite_original']
                                for row in self.custom_model2:
                                        cmd.append('-' + row[0] + ':' + row[1] + '=' + row[2])
                        else:
                                gtk.gdk.beep()
                                self.window2.window.set_cursor(None)
                                return True     
                cmd.append(filename)
                p = Popen(cmd, stdin=PIPE, stdout=PIPE, stderr=STDOUT, env={'LANG':'en_Us'})
                output = p.communicate()[0]
                p.stdin.close()
                p.stdout.close()
                self.window2.window.set_cursor(None)
                if '0 image files updated' in output:
                        self.exiftool_warning(filename, output)
                else:
                        dialog27 = self.messagedialog(_("Info!"), _("Metadata was written!"), None, gtk.BUTTONS_CLOSE,
                                                      self.window2, gtk.DIALOG_MODAL, gtk.MESSAGE_INFO, None, (300,140))
                        dialog27.set_keep_above(True)
                        gtk.gdk.beep()
                        response = dialog27.run()
                        if response == gtk.RESPONSE_CLOSE or response == gtk.RESPONSE_DELETE_EVENT:
                                dialog27.destroy()

        def clear_tags_database(self, obj):
                text = _('Are you sure to clear\nthe tags database?')
                dialog8 = self.messagedialog(_("Attention!"), text, None, gtk.BUTTONS_OK_CANCEL, self.dialog12, gtk.DIALOG_MODAL, gtk.MESSAGE_WARNING, None, (285,180))
                dialog8.set_keep_above(True)
                response = dialog8.run()
                if response in [gtk.RESPONSE_CANCEL , gtk.RESPONSE_DELETE_EVENT]:
                        dialog8.destroy()
                elif response == gtk.RESPONSE_OK:
                        dialog8.destroy()
                        while gtk.events_pending():
                                gtk.main_iteration(False)
                        Sqlite.clear_database()
                        
        def on_checkbutton28_toggled(self, *obj):
                if self.checkbutton28.get_active():
                        self.button91.set_sensitive(False)
                else:
                        self.button91.set_sensitive(True)
                
        def process_metadata(self, item, tag_list, data, adv_tags=None, toggled=False):
                filename = self.get_real_path(item)
                if self.temp in item:
                        item = filename[:]
                keywords = Sqlite.get_keywords(item)
                if data == 'add':
                        diff = set(tag_list) - set(keywords)
                        if diff:
                                tags = keywords + list(diff)
                                for tag in diff:
                                        if Sqlite.has_filename(os.path.basename(item), tag):
                                                tags.remove(tag)
                                                if toggled:
                                                        for row in self.tagmodel2:
                                                                if row[1] == tag:
                                                                        row[0] = not row[0]
                                                                        break
                                                gtk.gdk.beep()
                                                text29 = _('Attention, a file named %s \nwith the tag: "%s" already exists. The tag will not be stored!') % (os.path.basename(item), tag)
                                                dialog29 = self.messagedialog(_("Warning!"), text29, None, gtk.BUTTONS_CLOSE, None,
                                                                              gtk.DIALOG_MODAL, gtk.MESSAGE_WARNING, None, (500,230))
                                                dialog29.set_keep_above(true)
                                                response = dialog29.run()
                                                if response == gtk.RESPONSE_CLOSE or response == gtk.RESPONSE_DELETE_EVENT:
                                                        dialog29.destroy()
                                if keywords:
                                        Sqlite.update_keywords(item, tags)
                                else:
                                        if tags:
                                                Sqlite.insert_row(item, tags, os.path.dirname(filename))
                        if self.checkbutton28.get_active() or adv_tags:
                                self.exiftool_add(tag_list, filename)
                elif data == 'remove':
                        intersection = set(keywords) & set(tag_list)
                        if intersection:
                                tags = list(set(keywords) - intersection)
                                if tags:
                                        Sqlite.update_keywords(item, tags)
                                else:
                                        Sqlite.remove_row(item)
                        if self.checkbutton28.get_active():
                                self.exiftool_remove(tag_list, filename)
                elif data == 'clear':
                        if Sqlite.get_keywords(item):
                                Sqlite.remove_row(item)
                        if self.checkbutton28.get_active():
                                cmd = [exiftool, '-overwrite_original', '-Keywords=', filename]
                                p = Popen(cmd, stdin=PIPE, stdout=PIPE, stderr=STDOUT)
                                output = p.communicate()[0]
                                p.stdin.close()
                                p.stdout.close()
                                
        def get_advanced_metadata(self):
                if len(self.custom_model):
                        args = []
                        keywords = []
                        for row in self.custom_model:
                                if row[0] == 'IPTC' and row[1] == 'Keywords':
                                        keywords = [s.strip() for s in row[2].split(',')]
                                else:
                                        args.append('-' + row[0] + ':' + row[1] + '=' + row[2])
                        return args, keywords
                return None, None

        def exiftool_add(self, tags, filename):
                adv_tags, adv_keywords = self.get_advanced_metadata()
                list_ = [exiftool]
                if adv_keywords:
                        tags += list(set(adv_keywords) - set(tags))
                if tags:
                        cmd = [exiftool, '-j', '-n', '-Keywords', filename]
                        p = Popen(cmd, stdin=PIPE, stdout=PIPE, stderr=PIPE)
                        output = p.communicate()[0]
                        p.stdin.close()
                        p.stdout.close()
                        p.stderr.close()
                        try:
                                dct = json.loads(output)[0]
                                error = False
                        except:
                                dct = json.loads(utf8conv(output))[0]
                                error = True
                        if dct.has_key('Keywords'):
                                if not error:
                                        if isinstance(dct['Keywords'], (unicode, str)):
                                                taglist = [s.strip().encode('utf8') for s in dct['Keywords'].split(',')]
                                        elif isinstance(dct['Keywords'], list):
                                                taglist = [s.strip().encode('utf8') for s in dct['Keywords']]
                                else:
                                        if isinstance(dct['Keywords'], str):
                                                taglist = [s.strip().encode(encoding) for s in dct['Keywords'].split(',')]
                                        elif isinstance(dct['Keywords'], list):
                                                taglist = [s.strip().encode(encoding) for s in dct['Keywords']]
                                diff = set(tags) - set(taglist)
                                if diff:
                                        keys = taglist + list(diff)
                                else:
                                        keys = None
                        else:
                                keys = tags
                        if keys:
                                list_ += ['-overwrite_original', '-Keywords=' + ", ".join(keys)]
                if adv_tags:
                        list_ += adv_tags
                if len(list_) > 1:
                        list_.append(filename)
                p = Popen(list_, stdin=PIPE, stdout=PIPE, stderr=STDOUT, env={'LANG':'en_Us'})
                output = p.communicate()[0]
                p.stdin.close()
                p.stdout.close()
                if '0 image files updated' in output:
                        self.exiftool_warning(filename, output)

        def exiftool_remove(self, tags, filename):
                cmd = [exiftool, '-j', '-n', '-Keywords', filename]
                p = Popen(cmd, stdin=PIPE, stdout=PIPE, stderr=PIPE)
                output = p.communicate()[0]
                p.stdin.close()
                p.stdout.close()
                p.stderr.close()
                try:
                        dct = json.loads(output)[0]
                        error = False
                except:
                        dct = json.loads(utf8conv(output))[0]
                        error = True
                if dct.has_key('Keywords'):
                        if not error:
                                if isinstance(dct['Keywords'], (unicode, str)):
                                        taglist = [s.strip().encode('utf8') for s in dct['Keywords'].split(',')]
                                elif isinstance(dct['Keywords'], list):
                                        taglist = [s.strip().encode('utf8') for s in dct['Keywords']]
                        else:
                                if isinstance(dct['Keywords'], str):
                                        taglist = [s.strip().encode(encoding) for s in dct['Keywords'].split(',')]
                                elif isinstance(dct['Keywords'], list):
                                        taglist = [s.strip().encode(encoding) for s in dct['Keywords']]
                        intersection = set(tags) & set(taglist)
                        if intersection:
                                keys = list(set(taglist) - intersection)
                                list_ = [exiftool, '-overwrite_original', '-Keywords=' + ", ".join(keys), filename]
                                p = Popen(list_, stdin=PIPE, stdout=PIPE, stderr=STDOUT, env={'LANG':'en_Us'})
                                output = p.communicate()[0]
                                p.stdin.close()
                                p.stdout.close()
                                if '0 image files updated' in output:
                                        self.exiftool_warning(filename, output)
                
        def exiftool_warning(self, filename, output):
                if not self.checkbutton32.get_active():
                        self.label174.set_text(_('Exiftool cannot write %s\n%s') % (os.path.basename(filename), output))
                        self.dialog30.show()
                        response = self.dialog30.run()
                        if response == gtk.RESPONSE_CLOSE or response == gtk.RESPONSE_DELETE_EVENT:
                                self.dialog30.hide()
                                while gtk.events_pending():
                                        gtk.main_iteration(False)
                        
        def apply_tags_callback(self, obj, data):
                if self.selected or self.radiobutton7.get_active():
                        if obj == self.button109:
                                if self.list:
                                        self.window5.set_decorated(True)
                                        self.window5.set_resizable(True)
                                        self.progressbar2.set_fraction(0)
                                        self.window5.show()
                                        self.apply_tags(data, [], [])
                        else:
                                tag_list = []
                                adv_tags, adv_keywords = self.get_advanced_metadata()
                                if adv_keywords:
                                        tag_list += adv_keywords
                                for row in self.tagmodel:
                                        if row[0]:
                                                if row[1] not in tag_list:
                                                        tag_list += [row[1]]
                                                row[0] = False
                                if (tag_list or adv_tags) and self.list:
                                        self.window5.set_decorated(True)
                                        self.window5.set_resizable(True)
                                        self.progressbar2.set_fraction(0)
                                        self.window5.show()
                                        self.apply_tags(data, tag_list, adv_tags)
                else:
                        dialog9 = self.messagedialog(_("Info!"), _("No file selected!"), None, gtk.BUTTONS_CLOSE, self.window1, gtk.DIALOG_MODAL, gtk.MESSAGE_INFO, None, (230,120))
                        dialog9.set_keep_above(True)
                        gtk.gdk.beep()
                        response = dialog9.run()
                        if response == gtk.RESPONSE_CLOSE or response == gtk.RESPONSE_DELETE_EVENT:
                                dialog9.destroy()
                                
        def apply_tags(self, data, tag_list, adv_tags):        # updates metadata of selected images with checked tags
                keys = tag_list[:]
                selected = self.selected[:]
                self.progressbar2.set_text(_('Updating Metadata...'))
                fraction = 0.99 / len(selected)
                for item in selected:
                        self.process_metadata(item, keys, data, adv_tags)           
                        self.progressbar2.set_fraction(self.progressbar2.get_fraction() + fraction)
                        while gtk.events_pending():
                                gtk.main_iteration(False)
                        if len(selected) == 1:
                                time.sleep(0.1)
                        keys = tag_list[:]
                self.window5.hide()
                self.window5.set_decorated(False)
                self.window5.set_resizable(False)
                self.progressbar2.set_fraction(0)

        def delete_tag(self, obj):      # removes selection from tag list
                if obj == None:
                        treeview1 = self.treeview5
                        treeview2 = self.treeview6
                elif obj == self.button76:
                        treeview1 = self.treeview6
                        treeview2 = self.treeview5
                selection = treeview1.get_selection()
                model, iter_ = selection.get_selected()
                if iter_:
                        path = model.get_path(iter_)
                        tag = model.get_value(iter_,1)
                        toggle = model.get_value(iter_,0)
                        if model == self.tagmodel2 and toggle:
                                gtk.gdk.beep()
                        else:
                                model.remove(iter_)
                                for row in self.tagmodel3:
                                        if row[0] == tag:
                                                self.tagmodel3.remove(row.iter)
                                for row in treeview2.get_model():
                                        if row[1] == tag:
                                                treeview2.get_model().remove(row.iter) 
                                del self.settings['tags'][self.settings['tags'].index(tag)]
                                self.store_settings_db()

        def new_tag(self, obj):         # adds new item to tag list
                self.dialog19.show()
                response = self.dialog19.run()
                if response == gtk.RESPONSE_CANCEL or response == gtk.RESPONSE_DELETE_EVENT:
                        self.dialog19.hide()
                elif response == gtk.RESPONSE_OK:
                        tag = self.entry13.get_text().strip()
                        self.dialog19.hide()
                        self.entry13.set_text('')
                        if tag and tag not in self.settings['tags']:
                                self.treeview5.get_model().append([False, tag])
                                self.treeview6.get_model().append([False, tag])
                                self.tagmodel3.append([tag])
                                self.settings['tags'].append(tag)
                                self.store_settings_db()
                
        def show_tags(self):
                for row in self.tagmodel2:
                        row[0] = False
                if self.temp in self.list[self.indice]:
                        tag_list = Sqlite.get_keywords(self.get_real_path(self.list[self.indice]))
                else:
                        tag_list = Sqlite.get_keywords(self.list[self.indice])
                if tag_list:
                        for row in self.tagmodel2:
                                if row[1] in tag_list:
                                        self.tag_toggle2.handler_block(self.id81)
                                        row[0] = True
                                        self.tag_toggle2.handler_unblock(self.id81)
                        for item in tag_list:
                                if item not in self.settings['tags']:
                                        self.settings['tags'].append(item)
                                        self.store_settings_db()
                                        self.tag_toggle2.handler_block(self.id81)
                                        self.treeview5.get_model().append([False, item])
                                        self.treeview6.get_model().append([True, item])
                                        self.tagmodel3.append([item])
                                        self.tag_toggle2.handler_unblock(self.id81)

        def load_last_path(self):
                if self.checkbutton3.get_active():
                        if self.settings.has_key('switch') and self.settings['switch']:
                                if self.settings['switch'] == 'cartella':
                                        try:
                                                if os.path.isdir(self.settings['last_folder_path']):
                                                        self.open_last_folder(self.settings['last_folder_path'])
                                        except:
                                                gobject.idle_add(sys.stderr.write, traceback.format_exc())
                                                print "warning: cannot open last visited folder"
                                elif self.settings['switch'] == 'album':
                                        try:
                                                self.open_last_album(self.settings['last_album_path'])
                                        except:
                                                print "warning: cannot open last visited album"

        def make_background_pixbuf(self):
                width, height = 800, 600
                surface = cairo.ImageSurface(cairo.FORMAT_ARGB32, width, height)
                cr = cairo.Context(surface)
                cr.rectangle(0, 0, width, height)
                cr.fill()
                data = surface.get_data()  
                stride = surface.get_stride()
                pixbuf = gtk.gdk.pixbuf_new_from_data(data, gtk.gdk.COLORSPACE_RGB, True, 8, width, height, stride)
                self.background = pixbuf.scale_simple(self.screen0.get_width(), self.screen0.get_height(), gtk.gdk.INTERP_BILINEAR)
                self.thumb_background = pixbuf.scale_simple(262, 176, gtk.gdk.INTERP_BILINEAR)

        def make_distort_grid(self):
                width, height = 100, 100
                surface = cairo.ImageSurface(cairo.FORMAT_ARGB32, width, height)
                cr = cairo.Context(surface)
                cr.set_source_rgb(1, 1, 1)
                cr.rectangle(0, 0, width, height)
                cr.fill()
                cr.set_source_rgb(0, 0, 0)
                cr.set_line_width(1)
                for item in range(0, 100, 10):
                        cr.move_to(item + 0.5, 0)
                        cr.line_to(item + 0.5, 100)
                for item in range(0, 100, 10):
                        cr.move_to(0, item + 0.5)
                        cr.line_to(100, item + 0.5)
                cr.stroke()
                data = surface.get_data()  
                stride = surface.get_stride()
                pixbuf = gtk.gdk.pixbuf_new_from_data(data, gtk.gdk.COLORSPACE_RGB, True, 8, width, height, stride)
                self.image66.set_from_pixbuf(pixbuf)
                pixbuf.save(os.path.join(self.temp, "grid.png"), "png")

        def add_patterns(self):
                if any([os.path.isfile('/usr/bin/sox'), os.path.isfile('/usr/local/bin/sox'), os.path.isfile('sox.exe')]):
                        formats = ['8svx', 'aif', 'aifc', 'aiff', 'aiffc', 'al', 'amb', 'au', 'avr', 'cdda', 'cdr', 'cvs', 'cvsd', 'cvu',
                        'dat', 'dvms', 'f32', 'f4', 'f64','f8', 'flac', 'fssd', 'gsm', 'hcom', 'htk', 'ima', 'ircam', 'la', 'lpc', 'lpc10',
                        'lu', 'maud', 'mp2', 'mp3', 'nist', 'ogg', 'prc', 'raw', 's1', 's16','s2', 's24', 's3', 's32', 's4', 's8', 'sb',
                        'sf', 'sl', 'smp', 'snd', 'sndr', 'sndt', 'sou', 'sox', 'sph', 'sw', 'txw', 'u1', 'u16', 'u2', 'u24', 'u3', 'u32',
                        'u4', 'u8', 'ub', 'ul', 'uw', 'vms', 'voc', 'vorbis', 'vox', 'wav', 'wavpcm', 'wv', 'wve', 'xa', 'm3u', 'pls']
                        for item in formats:
                                self.filter0.add_pattern("*." + item)
                        self.filechooserbutton3.add_filter(self.filter0)
                else:
                        self.checkbutton6.set_sensitive(False)
                        self.filechooserbutton3.set_sensitive(False)
                for item in self.formats + self.raw_formats:
                        self.filter1.add_pattern("*" + item)
                        self.filter1.add_pattern("*" + item.upper())
                self.filechooserdialog1.add_filter(self.filter1)
                self.filter2.add_pattern("*.icc")
                self.filter2.add_pattern("*.icm")
                self.filechooserbutton4.add_filter(self.filter2)
                self.filechooserbutton5.add_filter(self.filter2)
                self.filechooserdialog4.add_filter(self.filter2)

        def autoresize(self, obj, param):
                if param.name == 'position':
                        self.rect = self.viewport1.allocation
                        format = os.path.splitext(self.list[self.indice])[1]    
                        pixbuf = None
                        if format.lower() in self.raw_formats:
                                if self.workflow_started :
                                        preview = None
                                else:                   
                                        if self.raw_preview:
                                                preview = os.path.join(self.temp, self.fname)
                                        else:
                                                preview = self.list[self.indice][:-len(format)]
                                if preview and self.raw_preview:
                                        self.pblist[self.pbindex] = gtk.gdk.pixbuf_new_from_file_at_size(preview, self.rect.width, self.rect.height)
                                elif preview and not self.raw_preview:
                                        if self.rotated.has_key(self.indice) and self.checkbutton2.get_active():
                                                self.pblist[self.pbindex] = gtk.gdk.pixbuf_new_from_file_at_size(preview, \
                                                self.rect.height, self.rect.width).rotate_simple(self.rotated[self.indice])
                                        else:
                                                self.pblist[self.pbindex] = gtk.gdk.pixbuf_new_from_file_at_size(preview, self.rect.width, self.rect.height)
                                else:
                                        self.pblist[self.pbindex] = self.wand2pixbuf(self.wand, True)
                        else:
                                if self.workflow_started:
                                        if self.rotated.has_key(self.indice) and self.checkbutton2.get_active():
                                                self.pblist[self.pbindex] = self.wand2pixbuf(self.wand, True, True)
                                        else:
                                                self.pblist[self.pbindex] = self.wand2pixbuf(self.wand, True)
                                else:
                                        if self.rotated.has_key(self.indice) and self.checkbutton2.get_active() and \
                                                                                self.notebook2.get_current_page() == 0:
                                                self.pblist[self.pbindex] = gtk.gdk.pixbuf_new_from_file_at_size(self.list[self.indice],\
                                                self.rect.height, self.rect.width).rotate_simple(self.rotated[self.indice])
                                        else:
                                                self.pblist[self.pbindex] = gtk.gdk.pixbuf_new_from_file_at_size(self.list[self.indice],\
                                                                                self.rect.width, self.rect.height)
                        self.pblist[self.pbindex] = self.check_icons(format, self.pblist[self.pbindex])
                        if self.expander8.get_expanded():
                                self.draw_grid(self.checkbutton24)
                        else:
                                if self.checkbutton41.get_active():
                                        self.image28.set_from_pixbuf(self.profile_pixbuf(self.pblist[self.pbindex]))
                                else:
                                        self.image28.set_from_pixbuf(self.pblist[self.pbindex])
                        self.window2.set_sensitive(True)
                        self.window2.show_all()
                        while gtk.events_pending():
                                gtk.main_iteration(False)
                        
        def autoresize2(self, obj, param, force=None):
                if not self.n:
                        return True
                if self.radiobutton6.get_active():
                        return True
                if force or param.name == 'position':
                        self.rect2 = self.scrolledwindow2.allocation
                        format = os.path.splitext(self.list[self.indice])[1]    
                        if format.lower() in self.raw_formats:
                                if self.rotated.has_key(self.indice) and self.checkbutton2.get_active():
                                        self.b_pixlist[self.pixindex] = gtk.gdk.pixbuf_new_from_file_at_size(self.list[self.indice]\
                                        [:-len(format)], self.rect2.height - self.swin2_label.allocation.height, self.rect2.width)
                                        self.b_pixlist[self.pixindex] = self.b_pixlist[self.pixindex].rotate_simple(self.rotated[self.indice])
                                else:   
                                        self.b_pixlist[self.pixindex] = gtk.gdk.pixbuf_new_from_file_at_size(self.list[self.indice]\
                                        [:-len(format)], self.rect2.width, self.rect2.height - self.swin2_label.allocation.height)
                        else:
                                if self.rotated.has_key(self.indice) and self.checkbutton2.get_active():
                                        self.b_pixlist[self.pixindex] = gtk.gdk.pixbuf_new_from_file_at_size(self.list[self.indice],\
                                        self.rect2.height - self.swin2_label.allocation.height, self.rect2.width)
                                        self.b_pixlist[self.pixindex] = self.b_pixlist[self.pixindex].rotate_simple(self.rotated[self.indice])
                                else:
                                        self.b_pixlist[self.pixindex] = gtk.gdk.pixbuf_new_from_file_at_size(self.list[self.indice],\
                                        self.rect2.width, self.rect2.height - self.swin2_label.allocation.height)
                        self.b_pixlist[self.pixindex] = self.check_icons(format, self.b_pixlist[self.pixindex])
                        self.swin2_image.set_from_pixbuf(self.b_pixlist[self.pixindex])
                        self.scrolledwindow2.show_all()
                        while gtk.events_pending():
                                gtk.main_iteration(False)
                        
        def on_key_press_event(self, obj, event):
                keyname = gtk.gdk.keyval_name(event.keyval)
                if obj == self.window3:
                        if keyname == 'Up':
                                self.previous_fullphoto()
                        elif keyname == 'Down':
                                self.next_fullphoto()
                        elif keyname in ['Return', 'space']:
                                self.on_delete_window3(self.window3, event)
                elif obj == self.window4:
                        self.on_delete_event(self.window4, event)
                elif obj == self.window1 and self.list:
                        if keyname == 'Delete':
                                if self.radiobutton6.get_active() and not self.selected:
                                        return True
                                self.delete_files(None)
                elif obj == self.window2:
                        if keyname == 'Delete' and self.list:
                                self.delete_image_from_window2(None)
                elif obj == self.window8 and keyname == 'Delete':
                        if self.notebook3.get_current_page() == 1:
                                selection = self.treeview8.get_selection()
                                model, iter_ = selection.get_selected()
                                if iter_ :
                                        model.remove(iter_)
                        elif self.notebook3.get_current_page() == 0:
                                self.delete_tag(None)
                elif obj == self.window11 and len(self.attachment_model):
                        if keyname == 'Delete':
                                selection = self.treeview9.get_selection()
                                model, iter_ = selection.get_selected()
                                if iter_:
                                        filename = model.get_value(iter_, 1)
                                        model.remove(iter_)
                                        while gtk.events_pending():
                                                gtk.main_iteration(False)
                                        self.mail_size -= os.path.getsize(filename)
                                        self.label239.set_text(_('Total size: %f Mb') % (self.mail_size / 1000000.0))
                elif obj == self.window12 and len(self.cdmodel):
                        if keyname == 'Delete':
                                selection = self.treeview15.get_selection()
                                model, iter_ = selection.get_selected()
                                if iter_:
                                        filename = model.get_value(iter_, 2)
                                        model.remove(iter_)
                                        while gtk.events_pending():
                                                gtk.main_iteration(False)
                                        self.cd_size -= os.path.getsize(filename)
                                        self.label243.set_text(_('Total size: %f Mb') % (self.cd_size / 1000000.0))
                elif obj == self.window15 and len(self.preset_model):
                        if keyname == 'Delete':
                                self.remove_WB_row()
                                                                          
        def scroll_bottom_panel(self, obj, event):
                adj = self.scrolledwindow6.get_hadjustment()
                if event.direction == gtk.gdk.SCROLL_DOWN:
                        if adj.value < adj.upper - adj.page_size:
                                adj.set_value(adj.value + adj.step_increment)
                        elif adj.value >= adj.upper - adj.page_size:
                                adj.set_value(adj.upper - adj.page_size)
                elif event.direction == gtk.gdk.SCROLL_UP:
                        adj.set_value(adj.value - adj.step_increment)
                        
        def toolbuttons(self, data1, data2):
                        self.toolbutton31.set_sensitive(data1)
                        self.toolbutton32.set_sensitive(data1)
                        self.toolbutton33.set_sensitive(data1)
                        self.toolbutton34.set_sensitive(data1)
                        self.toolbutton18.set_sensitive(data2)
                        self.toolbutton19.set_sensitive(data2)
                        self.toolbutton20.set_sensitive(data2)
                        self.toolbutton40.set_sensitive(data1)
                        self.toolbutton41.set_sensitive(data1)
                        self.toolbutton42.set_sensitive(data1)
                        self.toolbutton43.set_sensitive(data1)

        def store_dcraw_settings(self):
                if self.radiobutton10.get_active():
                        self.settings['depth'] = 8
                else:
                        self.settings['depth'] = 16
                self.settings['-f'] = self.checkbutton8.get_active()
                self.settings['output_colorspace'] = self.combobox4.get_active()
                self.settings['camera_profile'] = self.checkbutton9.get_active()
                self.settings['camera_ICC'] = self.filechooserbutton4.get_filename()
                self.settings['output_profile'] = self.checkbutton10.get_active()
                self.settings['output_ICC'] = self.filechooserbutton5.get_filename()
                self.settings['+M'] = self.checkbutton11.get_active()
                self.settings['dark_frame'] = self.checkbutton13.get_active()
                self.settings['dark_filename'] = self.filechooserbutton6.get_filename()
                self.settings['-j'] = self.checkbutton14.get_active()
                self.settings['-h'] = self.checkbutton16.get_active()
                self.settings['-D'] = self.checkbutton1.get_active()
                self.store_settings_db()

        def show_menu6(self, obj, event):
                if event.button == 3:
                        x = int(event.x)
                        y = int(event.y)
                        path = self.treeview1.get_path_at_pos(x, y)
                        if path:
                                self.path = path[0]
                                self.menu11.popup(None, None, None, event.button, event.time)

        def about_gtkrawgallery(self, obj):
                self.aboutdialog.show()
                response = self.aboutdialog.run()
                if response in [-4, -6]:
                        self.aboutdialog.hide()
                        
        def on_window7_state_event(self, obj, event):
                self.window7_was_iconified = event.new_window_state & gtk.gdk.WINDOW_STATE_ICONIFIED

        def on_window16_state_event(self, obj, event):
                self.window16_was_iconified = event.new_window_state & gtk.gdk.WINDOW_STATE_ICONIFIED
                        
        def window_state_event(self, obj, event):
                if event.new_window_state & gtk.gdk.WINDOW_STATE_MAXIMIZED:
                        if obj == self.window2:
                                if self.hide_exif_panel:
                                        self.frame1.hide_all()
                                        while gtk.events_pending():
                                                gtk.main_iteration(False)
                                        self.zoom_fit_photo(None, 0)            
                                else:
                                        self.frame1.show_all()
                        elif obj == self.window1 and self.radiobutton7.get_active():
                                self.vpaned2.set_position(self.settings['window1_size'][1] - self.height + 30)
                                        
        def get_options(self, event=None):                      # Saves settings
                self.dialog12.show()
                while gtk.events_pending():
                        gtk.main_iteration(False)
                self.cache_control()
                response = self.dialog12.run()
                if response == gtk.RESPONSE_CANCEL or response == gtk.RESPONSE_DELETE_EVENT:
                        self.dialog12.hide()
                        while gtk.events_pending():
                                gtk.main_iteration(False)
                        self.settings['adj_value'] = self.oldadj
                        self.adj.set_value(self.settings['adj_value'])
                        self.pause = int(self.settings['adj_value'])
                        if self.settings['radiobutton8']:
                                self.radiobutton8.set_active(True)
                                self.width = 100
                                self.height = 67
                        if self.settings['radiobutton1']:
                                self.radiobutton1.set_active(True)
                                self.width = 150
                                self.height = 100
                        if self.settings['radiobutton2']:
                                self.radiobutton2.set_active(True)
                                self.width = 201
                                self.height = 134
                        if self.settings['radiobutton3']:
                                self.radiobutton3.set_active(True)
                                self.width = 260
                                self.height = 174
                        self.radiobutton6.set_active(self.settings['radiobutton6'])
                        self.checkbutton2.set_active(self.settings['checkbutton2'])
                        self.checkbutton4.set_active(self.settings['checkbutton4'])
                        self.checkbutton5.set_active(self.settings['checkbutton5'])
                        self.checkbutton6.set_active(self.settings['checkbutton6'])
                        self.checkbutton7.set_active(self.settings['checkbutton7'])
                        self.checkbutton3.set_active(self.settings['checkbutton3'])
                        self.checkbutton17.set_active(self.settings['checkbutton17'])
                        self.checkbutton28.set_active(self.settings['checkbutton28'])
                        self.checkbutton33.set_active(self.settings['checkbutton33'])
                        self.checkbutton360.set_active(self.settings['checkbutton360'])
                        self.ark = self.settings['ark']
                        self.filechooserbutton1.set_current_folder(self.ark)
                        self.entry4.set_text(self.ark)
                        self.colorbutton1.set_color(self.gallery_color)
                        self.colorbutton2.set_color(self.label_color)
                        self.colorbutton3.set_color(self.treeview_color)
                        self.colorbutton4.set_color(self.toolbar_color)
                        self.colorbutton5.set_color(self.selection_color)
                        self.colorbutton8.set_color(self.image_background_color)
                        self.border_color = self.settings['border_color']
                        if self.border_color == 0xffffffff:
                                self.combobox27.set_active(0)
                        elif self.border_color == 0x00000000:
                                self.combobox27.set_active(2)
                        else:
                                self.combobox27.set_active(1)
                        self.spinbutton2.set_value(self.settings['frames'])
                        self.spinbutton3.set_value(self.settings['delay'])
                        if self.settings['sample']:
                                self.filechooserbutton3.set_filename(self.settings['sample'])
                        self.entry9.set_text(str(self.cache_size))
                elif response == -7:
                        dialog20 = self.messagedialog(_("Attention!"), _("Restore default settings?"), None, gtk.BUTTONS_OK_CANCEL, self.dialog12,
                                                      gtk.DIALOG_MODAL, gtk.MESSAGE_WARNING, None, (375,200))
                        response20 = dialog20.run()
                        if response20 == gtk.RESPONSE_CANCEL or response20 == gtk.RESPONSE_DELETE_EVENT:
                                dialog20.destroy()
                                self.get_options()
                        elif response20 == gtk.RESPONSE_OK:
                                dialog20.destroy()
                                self.default_settings()
                                self.get_options()
                elif response == gtk.RESPONSE_OK:
                        self.dialog12.hide()
                        while gtk.events_pending():
                                gtk.main_iteration(False)
                        self.cache_control_thread = threading.Thread(target=self.cache_control, args=())
                        self.cache_control_thread.start()
                        self.delay = self.settings['delay'] = self.spinbutton3.get_value()
                        self.sample = self.settings['sample'] = self.filechooserbutton3.get_filename()
                        self.gallery_color = self.colorbutton1.get_color()
                        self.label_color = self.colorbutton2.get_color()
                        self.toolbar_color = self.colorbutton4.get_color()
                        self.selection_color = self.colorbutton5.get_color()
                        self.treeview_color = self.colorbutton3.get_color()
                        self.image_background_color = self.colorbutton8.get_color()
                        self.settings['treeview_color'] = self.treeview_color.to_string()
                        self.settings['label_color'] = self.label_color.to_string()
                        self.settings['gallery_color'] = self.gallery_color.to_string()
                        self.settings['toolbar_color'] = self.toolbar_color.to_string()
                        self.settings['selection_color'] = self.selection_color.to_string()
                        self.settings['image_background_color'] = self.image_background_color.to_string()
                        self.window1.modify_bg(gtk.STATE_NORMAL, self.toolbar_color)
                        self.window2.modify_bg(gtk.STATE_NORMAL, self.toolbar_color)
                        self.toolbar1.modify_bg(gtk.STATE_NORMAL, self.toolbar_color)
                        self.toolbar2.modify_bg(gtk.STATE_NORMAL, self.toolbar_color)
                        self.treeview1.modify_base(gtk.STATE_NORMAL, self.treeview_color)
                        self.treeview2.modify_base(gtk.STATE_NORMAL, self.treeview_color)
                        self.treeview3.modify_base(gtk.STATE_NORMAL, self.treeview_color)
                        self.treeview6.modify_base(gtk.STATE_NORMAL, self.treeview_color)
                        self.treeview10.modify_base(gtk.STATE_NORMAL, self.treeview_color)
                        self.treeview11.modify_base(gtk.STATE_NORMAL, self.treeview_color)
                        self.treeview12.modify_base(gtk.STATE_NORMAL, self.treeview_color)
                        self.treeview13.modify_base(gtk.STATE_NORMAL, self.treeview_color)
                        self.treeview14.modify_base(gtk.STATE_NORMAL, self.treeview_color)
                        self.eventbox3.modify_bg(gtk.STATE_NORMAL, self.treeview_color)
                        self.eventbox5.modify_bg(gtk.STATE_NORMAL, self.treeview_color)
                        self.eventbox6.modify_bg(gtk.STATE_NORMAL, self.treeview_color)
                        self.viewport1.modify_bg(gtk.STATE_NORMAL, self.image_background_color)
                        self.viewport2.modify_bg(gtk.STATE_NORMAL, self.treeview_color)
                        self.eventbox7.modify_bg(gtk.STATE_NORMAL, self.treeview_color)
                        self.eventbox39.modify_bg(gtk.STATE_NORMAL, self.treeview_color)
                        self.eventbox40.modify_bg(gtk.STATE_NORMAL, self.treeview_color)
                        self.progressbar3.modify_bg(gtk.STATE_NORMAL, self.toolbar_color)
                        self.progressbar8.modify_bg(gtk.STATE_NORMAL, self.toolbar_color)
                        self.button5.modify_bg(gtk.STATE_NORMAL, self.toolbar_color)
                        if self.radiobutton6.get_active():
                                self.eventbox1.modify_bg(gtk.STATE_NORMAL, self.gallery_color)
                                self.eb.modify_bg(gtk.STATE_NORMAL, self.gallery_color)
                                if self.scrolledwindow2.get_child():
                                        for i in xrange(len(self.label)):
                                                self.event[i].modify_bg(gtk.STATE_NORMAL, self.gallery_color)
                                                self.e[i].modify_bg(gtk.STATE_NORMAL, self.gallery_color)
                                                if type(self.label[i]) != int:
                                                        self.label[i].modify_fg(gtk.STATE_NORMAL, self.label_color)
                        else:
                                try:
                                        self.swin2_label.modify_fg(gtk.STATE_NORMAL, self.label_color)
                                        self.eventbox9.modify_bg(gtk.STATE_NORMAL, self.gallery_color)
                                        if self.scrolledwindow2.get_child():
                                                self.scrolledwindow2.get_child().modify_bg(gtk.STATE_NORMAL, self.image_background_color)
                                        for i in xrange(self.n):
                                                self.b_event[i].modify_bg(gtk.STATE_NORMAL, self.gallery_color)
                                        self.b_event[self.indice].modify_bg(gtk.STATE_NORMAL, self.selection_color)
                                except:
                                        pass
                        self.settings['adj_value'] = self.oldadj = self.adj.value
                        self.pause = self.adj.value
                        if self.checkbutton2.get_active():
                                if not self.settings['checkbutton2']: 
                                        self.delete_cached_thread = threading.Thread(target=self.delete_cached, args=())
                                        self.delete_cached_thread.start()
                                self.settings['checkbutton2'] = True
                        else:
                                self.settings['checkbutton2'] = False
                        self.settings['checkbutton3'] = self.checkbutton3.get_active()
                        if self.checkbutton4.get_active():
                                self.settings['checkbutton4'] = True
                                border_color = self.combobox27.get_active()
                                if border_color == 0:
                                        self.border_color = 0xffffffff
                                elif border_color == 2:
                                        self.border_color = 0x00000000
                                else:
                                        self.border_color = 0x858585ff
                                self.settings['border_color'] = self.border_color
                        else:
                                self.settings['checkbutton4'] = False
                        if self.checkbutton5.get_active():
                                self.settings['checkbutton5'] = True
                                self.frames = int(self.spinbutton2.get_value())
                                self.settings['frames'] = self.frames
                        else:
                                self.settings['checkbutton5'] = False
                        self.settings['checkbutton6'] = self.checkbutton6.get_active()
                        self.settings['checkbutton17'] = self.checkbutton17.get_active()
                        self.settings['checkbutton28'] = self.checkbutton28.get_active()
                        if self.checkbutton7.get_active():
                                self.settings['checkbutton7'] = True
                                try:
                                        self.cache_size = int(self.entry9.get_text())
                                except:
                                        gobject.idle_add(sys.stderr.write, traceback.format_exc())
                                        self.cache_size = 50
                        else:
                                self.settings['checkbutton7'] = False
                        if self.radiobutton8.get_active():
                                self.width = 100
                                self.height = 67
                                self.settings['radiobutton8'] = True
                        else:
                                self.settings['radiobutton8'] = False
                        if self.radiobutton1.get_active():
                                self.width = 150
                                self.height = 100
                                self.settings['radiobutton1'] = True
                        else:
                                self.settings['radiobutton1'] = False
                        if self.radiobutton2.get_active():
                                self.width = 201
                                self.height = 134
                                self.settings['radiobutton2'] = True
                        else:
                                self.settings['radiobutton2'] = False
                        if self.radiobutton3.get_active():
                                self.width = 260
                                self.height = 174
                                self.settings['radiobutton3'] = True
                        else:
                                self.settings['radiobutton3'] = False
                        self.settings['radiobutton6'] = self.radiobutton6.get_active()
                        self.settings['radiobutton7'] = not self.radiobutton6.get_active()
                        archives = self.filechooserbutton1.get_current_folder()
                        if archives != self.ark:
                                f = open(os.path.join(self.ark, 'albums.db'), 'w')
                                cPickle.dump(self.albums, f)
                                f.close()
                                self.ark = archives
                                self.settings['ark'] = archives
                                self.model.clear()
                                self.load_album_database()
                        if self.checkbutton33.get_active() != self.settings['checkbutton33']:
                                gobject.idle_add(self.refresh_directory_tree)
                        self.settings['checkbutton33'] = self.checkbutton33.get_active()
                        self.store_settings_db()
                self.unselect()

        def show_color_manager(self, obj):
                if self.window16.get_property('visible'):
                        if self.window16_was_iconified:
                                self.window16.present()
                        else:
                                self.close_color_manager(self.window16)
                else:
                        self.window2.set_modal(False)
                        self.window16.show()

        def close_color_manager(self, obj, event=None):
                self.window16.hide()
                while gtk.events_pending():
                        gtk.main_iteration(False)
                self.window2.set_modal(True)
                return True

        def update_color_manager(self, obj=None):      # Color Management settings
                colorspace = self.combobox4.get_active()
                self.input_profile_name = self.combobox33.get_active_text()
                if self.input_profile_name == 'sRGB':
                        self.input_profile = self.sRGB_profile
                elif self.input_profile_name == 'AdobeRGB 1998':
                        self.input_profile = self.AdobeRGB1998_profile
                else:
                        if os.path.isfile(self.settings['in_profile_icc']):
                                self.input_profile = numpy.fromfile(self.settings['in_profile_icc'], 'u1')
                                self.input_profile_name = self.settings['in_profile_icc']
                        else:
                                self.show_dialog15(_('Unknown file: %s') % self.settings['in_profile_icc'])
                                return True
                self.output_profile_name = self.combobox34.get_active_text()
                if self.output_profile_name == 'sRGB':
                        self.output_profile = self.sRGB_profile
                elif self.output_profile_name == 'AdobeRGB 1998':
                        self.output_profile = self.AdobeRGB1998_profile
                else:
                        if os.path.isfile(self.settings['out_profile_icc']):
                                self.output_profile = numpy.fromfile(self.settings['out_profile_icc'], 'u1')
                                self.output_profile_name = self.settings['out_profile_icc']
                        else:
                                self.show_dialog15(_('Unknown file: %s') % self.settings['out_profile_icc'])
                                return True
                self.out_rendering_intent = self.combobox36.get_active_text() or 'Perceptual'
                self.display_profile_name = self.combobox35.get_active_text()
                if self.display_profile_name == 'sRGB':
                        self.display_profile = self.sRGB_profile
                else:
                        if os.path.isfile(self.settings['display_profile_icc']):
                                self.display_profile = numpy.fromfile(self.settings['display_profile_icc'], 'u1')
                                self.display_profile_name = self.settings['display_profile_icc']
                        else:
                                self.show_dialog15(_('Unknown file: %s') % self.settings['display_profile_icc'])
                                return True
                self.display_rendering_intent = self.combobox37.get_active_text() or 'Perceptual'
                self.settings['embedded_profile'] = self.embedded_profile_behaviour = self.combobox38.get_active()
                if self.checkbutton41.get_active():
                        self.settings['cms'] = True
                        if colorspace == 1:
                                self.combobox4.handler_block(self.id62)
                                self.combobox4.set_active(0)    #if sRGB, sets a wide gamut colorspace; default is raw.
                                self.combobox4.handler_unblock(self.id62)
                else:
                        self.settings['cms'] = False
                        self.combobox4.handler_block(self.id62)
                        self.combobox4.set_active(1)
                        self.combobox4.handler_unblock(self.id62)
                if not self.settings['cms']:
                        self.checkbutton9.handler_block(self.id54)
                        self.checkbutton10.handler_block(self.id55)
                        self.checkbutton9.set_active(False)
                        self.checkbutton10.set_active(False)
                        self.checkbutton9.handler_unblock(self.id54)
                        self.checkbutton10.handler_unblock(self.id55)
                self.checkbutton9.set_sensitive(not self.settings['cms'])
                self.checkbutton10.set_sensitive(not self.settings['cms'])
                if self.window16.get_property('visible') and self.window2.get_property('visible'):
                        gobject.idle_add(self.refresh_image1)

        def load_embedded_profile(self, wand):
                if self.toolbutton12.get_property('sensitive'):
                        profile, lenght = wand.get_image_profile('icc')
                        if lenght:
                                self.embedded_profile = numpy.frombuffer(profile, 'u1')
                                if self.checkbutton41.get_active() and self.combobox38.get_active() == 2:
                                        from ctypes import c_void_p
                                        text = ''
                                        profile = littlecms.Profile(rawProfile=(self.embedded_profile.ctypes.data_as(c_void_p), lenght))
                                        if profile.productInfo:
                                                text = profile.productInfo
                                        self.label272.set_text(text)
                                        self.dialog34.show()
                                        response = self.dialog34.run()
                                        if response == -6:
                                                self.dialog34.hide()
                                                self.settings['embedded_profile'] = self.embedded_profile_behaviour = 0
                                        elif response == -5 or response == gtk.RESPONSE_DELETE_EVENT:
                                                self.dialog34.hide()
                                                self.settings['embedded_profile'] = self.embedded_profile_behaviour = 1
                                        while gtk.events_pending():
                                                gtk.main_iteration(False)
                                        if self.checkbutton43.get_active():
                                                self.checkbutton43.set_active(False)
                                                self.combobox38.handler_block(self.id106)
                                                self.combobox38.set_active(self.embedded_profile_behaviour)
                                                self.combobox38.handler_unblock(self.id106)     
                        else:
                                self.embedded_profile = None
                else:
                        self.embedded_profile = None

        def refresh_image1(self):
                self.pbindex = 1
                self.window16.window.set_cursor(self.watch)
                while gtk.events_pending():
                        gtk.main_iteration(False)
                format = os.path.splitext(self.list[self.indice])[1]
                if format.lower() in self.raw_formats:
                        if self.raw_preview:
                                rotate = False
                                filename = os.path.join(self.temp, self.fname)
                                rotate_simple = False
                        else:
                                rotate = True
                                filename = self.list[self.indice][:-len(format)]
                                rotate_simple = True
                        width, height = self.rect.height, self.rect.width
                        if not (self.rotated.has_key(self.indice) and self.checkbutton2.get_active()):
                                width, height = self.rect.width, self.rect.height
                                rotate = False
                                rotate_simple = False
                        if self.checkbutton41.get_active():
                                if not self.workflow_started:
                                        wand = MagickImage(filename)
                                        self.pblist[1] = self.wand2pixbuf(wand, True, rotate)
                                else:
                                        self.pblist[1] = self.wand2pixbuf(self.wand, True, rotate)
                        else:
                                if not self.workflow_started:
                                        self.pblist[1] = gtk.gdk.pixbuf_new_from_file_at_size(filename, width, height)
                                else:
                                        self.pblist[1] = self.wand2pixbuf(self.wand, True, rotate)
                        if rotate_simple:
                                self.pblist[1] = self.pblist[1].rotate_simple(self.rotated[self.indice])
                else:
                        if self.rotated.has_key(self.indice) and self.checkbutton2.get_active() and self.notebook2.get_current_page() == 0:
                                if self.checkbutton41.get_active():
                                        if not self.workflow_started:
                                                wand = MagickImage(self.list[self.indice])
                                                self.pblist[1] = self.wand2pixbuf(self.wand, True, True)
                                else:
                                        if not self.workflow_started:
                                                self.pblist[1] = gtk.gdk.pixbuf_new_from_file_at_size(self.list[self.indice],\
                                                self.rect.height, self.rect.width)
                                        else:
                                                self.pblist[1] = self.wand2pixbuf(self.wand, True, True)
                                self.pblist[1] = self.pblist[1].rotate_simple(self.rotated[self.indice])
                        else:
                                if self.checkbutton41.get_active():
                                        if not self.workflow_started:
                                                wand = MagickImage(self.list[self.indice])
                                                self.pblist[1] = self.wand2pixbuf(wand, True, False)
                                        else:
                                                self.pblist[1] = self.wand2pixbuf(self.wand, True)
                                else:
                                        if not self.workflow_started:
                                                self.pblist[1] = gtk.gdk.pixbuf_new_from_file_at_size(self.list[self.indice],\
                                                self.rect.width, self.rect.height)
                                        else:
                                                self.pblist[1] = self.wand2pixbuf(self.wand, True)
                self.pblist[1] = self.check_icons(format, self.pblist[1])
                if self.expander17.get_expanded():
                        self.set_cairo_pixbuf()
                        self.set_zoom_sensitivity(False)
                elif self.expander8.get_expanded():
                        self.draw_grid(self.checkbutton24)
                elif self.expander19.get_expanded():
                                self.draw_grid(self.checkbutton34)
                else:
                        if self.checkbutton41.get_active():
                                self.image28.set_from_pixbuf(self.profile_pixbuf(self.pblist[1]))
                        else:
                                self.image28.set_from_pixbuf(self.pblist[1])
                        self.image28.show()
                self.window16.window.set_cursor(None)
                while gtk.events_pending():
                        gtk.main_iteration(False)
                if self.notebook2.get_current_page() == 3:
                        self.get_channel()

        def scale_wand(self, wand_, rotated=False):
                wand = MagickImage(wand_.get_image())
                width, height = wand.size
                ratio = float(width) / height
                if not rotated:
                        if ratio > 1:
                                width, height = self.rect.width, int(self.rect.width / ratio)
                        else:
                                width, height = int(self.rect.height * ratio), self.rect.height
                else:
                        width, height = self.rect.height, int(self.rect.height / ratio)
                wand.scale(width, height)
                return wand
                
        def refresh_directory_tree(self):
                self.treeview2.freeze_child_notify()
                for col in self.treeview2.get_columns():
                        self.treeview2.remove_column(col)
                self.treestore.clear()
                if sys.platform.startswith('win'):
                        for item in ascii_uppercase:
                                if os.path.exists(item + ':' + homepath):
                                        iter_ = self.treestore.append(None, [self.rootpb, item + ':'])
                                        self.treestore.append(iter_)
                else:
                        iter_ = self.treestore.append(None, [self.rootpb, '/'])
                        self.treestore.append(iter_) 
                column0 = gtk.TreeViewColumn("Directory Tree")
                self.treeview2.append_column(column0)
                cellpb = gtk.CellRendererPixbuf()
                cell = gtk.CellRendererText()
                column0.pack_start(cellpb, False)
                column0.add_attribute(cellpb, 'pixbuf', 0)
                column0.pack_start(cell, True)
                column0.add_attribute(cell, 'text', 1)
                self.treeview2.thaw_child_notify()
                self.treeview2.set_model(self.treestore)
                self.expand_to_path(self.cartella)
                while gtk.events_pending():
                        gtk.main_iteration(False)

        def get_default_style(self):
                self.treeview_style = self.treeview1.rc_get_style().copy()
                self.window_style = self.eventbox1.rc_get_style().copy()
                self.statusbar_style = self.statusbar1.rc_get_style().copy()
                        
        def default_colors(self, obj=None):
                self.colorbutton1.set_color(self.window_style.bg[gtk.STATE_NORMAL])
                self.colorbutton2.set_color(gtk.gdk.color_parse('#000000'))
                self.colorbutton3.set_color(self.treeview_style.base[gtk.STATE_NORMAL])
                self.colorbutton4.set_color(self.statusbar_style.bg[gtk.STATE_NORMAL])
                self.colorbutton5.set_color(gtk.gdk.color_parse('#CD7300'))
                self.colorbutton8.set_color(gtk.gdk.color_parse('#262626'))
                
        def on_radiobutton10_toggled(self, obj):
                if dcraw_version < "8.94":
                        self.hscale28.set_sensitive(obj.get_active())
                        self.hscale34.set_sensitive(obj.get_active())
                        self.hscale3.set_sensitive(obj.get_active())
                        self.spinbutton11.set_sensitive(obj.get_active())
                        self.spinbutton42.set_sensitive(obj.get_active())
                        self.spinbutton45.set_sensitive(obj.get_active())
                        
        def dcraw_options2(self):
                options=[dcraw + ' -c -v']
                WB = self.combobox5.get_active_text()
                if self.checkbutton1.get_active():
                        options += ['-D']
                else:
                        if WB == 'Camera WB':
                                options += ['-w']
                        elif WB == 'Auto WB':
                                options += ['-a']
                        elif WB == 'Manual WB':
                                options += ['-r']
                                options += [str(self.spinbutton4.get_value())]
                                options += [str(self.spinbutton5.get_value())]
                                options += [str(self.spinbutton6.get_value())]
                                options += ['0']
                        else:
                                if self.combobox30.get_active() == 1:
                                        options += ['-r']
                                        options += self.auto_wb_presets[self.get_real_path(self.list[self.indice])][WB] + ['0']
                                else:
                                        options += ['-r']
                                        options += self.presets[self.combobox30.get_active_text()][WB] + ['0']
                if self.checkbutton11.get_active():
                        options += ['+M']
                else:
                        options += ['-M']
                options += ['-C']
                options += [str(1.0 + self.spinbutton7.get_value() / 1000.0)]
                options += [str(1.0 + self.spinbutton8.get_value() / 1000.0)]
                options += ['-b']
                adj = self.hscale3.get_adjustment()
                options += [str(adj.value)]
                options += ['-g']
                adj = self.hscale28.get_adjustment()
                options += [str(adj.value)]
                adj = self.hscale34.get_adjustment()
                options += [str(adj.value)]
                options += ['-n']
                options += [str(self.spinbutton1.get_value())]
                options += ['-m']
                options += [str(self.spinbutton39.get_value())]
                if self.checkbutton12.get_active():
                        options += ['-k']
                        options += [str(self.spinbutton9.get_value())]
                if self.checkbutton23.get_active():
                        options += ['-S']
                        options += [str(self.spinbutton35.get_value())]
                if self.checkbutton13.get_active() and self.filechooserbutton6.get_filename():
                        options += ['-K']
                        fname = self.filechooserbutton6.get_filename()
                        options += ['"%s"' % fname]
                rotation = self.combobox39.get_active()
                if rotation != 0:
                        options += ['-t']
                        if rotation == 1:
                                options += ['0']
                        elif rotation == 2:
                                options += ['3']
                        elif rotation == 3:
                                options += ['5']
                        elif rotation == 4:
                                options += ['7']
                options += ['-H']
                options += [str(self.combobox3.get_active())]
                options += ['-o']                       
                if self.checkbutton10.get_active() and self.filechooserbutton5.get_filename():
                        output_profile = self.filechooserbutton5.get_filename()
                        options += ['"%s"' % output_profile]
                else:
                        options += [str(self.combobox4.get_active())]
                if has_lcms:                       
                        if self.checkbutton9.get_active() and self.filechooserbutton4.get_filename():
                                camera_profile = self.filechooserbutton4.get_filename()
                                options += ['-p']
                                options += ['"%s"' % camera_profile]
                if self.checkbutton14.get_active():
                        options += ['-j']
                if self.checkbutton8.get_active():
                        options += ['-f']
                if self.checkbutton16.get_active():
                        options += ['-h']
                if self.radiobutton9.get_active():
                                options += ['-4']
                elif self.radiobutton11.get_active():
                                options += ['-6']
                options += ['-T']
                options += ['"%s"', '>', '"%s"']
                return options

        def set_default_dcraw_options(self, event):
                self.spinbutton1.handler_block(self.id49)
                self.spinbutton7.handler_block(self.id46)
                self.spinbutton8.handler_block(self.id47)
                self.spinbutton11.handler_block(self.id65)
                self.spinbutton39.handler_block(self.id82)
                self.spinbutton42.handler_block(self.id86)
                self.spinbutton45.handler_block(self.id87)
                self.radiobutton11.handler_block(self.id52)
                self.hscale3.handler_block(self.id51)
                self.hscale15.handler_block(self.id66)
                self.hscale25.handler_block(self.id83)
                self.hscale28.handler_block(self.id84)
                self.hscale34.handler_block(self.id85)
                self.checkbutton1.handler_block(self.id96)
                self.checkbutton8.handler_block(self.id53)
                self.checkbutton9.handler_block(self.id54)
                self.checkbutton10.handler_block(self.id55)
                self.checkbutton11.handler_block(self.id56)
                self.checkbutton12.handler_block(self.id57)
                self.checkbutton13.handler_block(self.id58)
                self.checkbutton14.handler_block(self.id59)
                self.checkbutton16.handler_block(self.id67)
                self.checkbutton23.handler_block(self.id76)
                self.combobox2.handler_block(self.id60)
                self.combobox3.handler_block(self.id61)
                self.combobox4.handler_block(self.id62)
                self.combobox5.handler_block(self.id42)
                self.combobox30.handler_block(self.id95)
                self.radiobutton11.set_active(True)
                self.combobox2.set_active(0)
                self.combobox3.set_active(0)
                self.combobox4.set_active(1)
                self.combobox5.set_active(0)
                self.combobox30.set_active(0)
                self.combobox39.handler_block(self.id108)
                self.combobox39.set_active(0)
                self.combobox39.handler_unblock(self.id108)
                self.spinbutton1.set_value(0)
                self.spinbutton7.set_value(0)
                self.spinbutton8.set_value(0)
                self.spinbutton39.set_value(0)
                self.checkbutton1.set_active(False)
                self.checkbutton8.set_active(False)
                self.checkbutton9.set_active(False)
                self.checkbutton10.set_active(False)
                self.checkbutton11.set_active(True)
                self.checkbutton12.set_active(False)
                self.checkbutton13.set_active(False)
                self.checkbutton14.set_active(False)
                self.checkbutton16.set_active(True)
                self.checkbutton23.set_active(False)
                adj3 = self.hscale3.get_adjustment()
                adj3.set_value(1.00)
                self.spinbutton11.set_value(adj3.value)
                adj25 = self.hscale25.get_adjustment()
                adj25.set_value(0)
                adj28 = self.hscale28.get_adjustment()
                adj28.set_value(2.222)
                self.spinbutton45.set_value(adj28.value)
                adj34 = self.hscale34.get_adjustment()
                adj34.set_value(4.5)
                self.spinbutton42.set_value(adj34.value)
                self.spinbutton1.handler_unblock(self.id49)
                self.spinbutton7.handler_unblock(self.id46)
                self.spinbutton8.handler_unblock(self.id47)
                self.spinbutton11.handler_unblock(self.id65)
                self.radiobutton11.handler_unblock(self.id52)
                self.spinbutton39.handler_unblock(self.id82)
                self.spinbutton42.handler_unblock(self.id86)
                self.spinbutton45.handler_unblock(self.id87)
                self.hscale3.handler_unblock(self.id51)
                self.hscale15.handler_unblock(self.id66)
                self.hscale25.handler_unblock(self.id83)
                self.hscale28.handler_unblock(self.id84)
                self.hscale34.handler_unblock(self.id85)
                self.checkbutton1.handler_unblock(self.id96)
                self.checkbutton8.handler_unblock(self.id53)
                self.checkbutton9.handler_unblock(self.id54)
                self.checkbutton10.handler_unblock(self.id55)
                self.checkbutton11.handler_unblock(self.id56)
                self.checkbutton12.handler_unblock(self.id57)
                self.checkbutton13.handler_unblock(self.id58)
                self.checkbutton14.handler_unblock(self.id59)
                self.checkbutton16.handler_unblock(self.id67)
                self.checkbutton23.handler_unblock(self.id76)
                self.combobox2.handler_unblock(self.id60)
                self.combobox3.handler_unblock(self.id61)
                self.combobox4.handler_unblock(self.id62)
                self.combobox5.handler_unblock(self.id42)
                self.combobox30.handler_unblock(self.id95)
                if self.togglebutton6.get_active():
                        gobject.idle_add(self.load_raw_preview, self.rect.width, self.rect.height)
                        
        def wb_changed(self, event):
                text = self.combobox5.get_active_text()
                if text:
                        if  'Manual WB' not in text:
                                self.spinbutton4.set_sensitive(False)
                                self.spinbutton5.set_sensitive(False)
                                self.spinbutton6.set_sensitive(False)
                        else:
                                self.spinbutton4.set_sensitive(True)
                                self.spinbutton5.set_sensitive(True)
                                self.spinbutton6.set_sensitive(True)
                
        def show_convert(self, obj=None):
                self.window6.move(0, 0)
                self.progressbar4.hide()
                self.window6.set_title(_('Convert Selection  (%d files)') % len(self.selected)) 
                self.window6.show()

        def multiple_conversion(self, event):
                dest = self.entry5.get_text()
                extension = self.combobox7.get_active()
                if extension == 0:
                        self.dialog16.show()
                        response16 = self.dialog16.run()
                        if response16 == gtk.RESPONSE_CANCEL or response16 == gtk.RESPONSE_DELETE_EVENT:
                                self.dialog16.hide()
                                self.show_convert()
                                return True
                        elif response16 == gtk.RESPONSE_OK:
                                self.dialog16.hide()
                                while gtk.events_pending():
                                        gtk.main_iteration(False)
                                self.settings['jpg_quality'] = self.jpg_quality.value
                                self.progressbar4.show()
                                threading.Thread(target=self.convert_as_jpg, args=(self.selected, self.jpg_quality.value, dest)).start()
                elif extension == 1:
                        self.progressbar4.show()
                        threading.Thread(target=self.convert_as_png, args=(self.selected, dest)).start()
                elif extension == 2:
                        self.progressbar4.show()
                        threading.Thread(target=self.convert_as_tiff, args=(self.selected, 8, dest)).start()
                elif extension == 3:
                        self.progressbar4.show()
                        threading.Thread(target=self.convert_as_tiff, args=(self.selected, 16, dest)).start() 
                elif extension == 4:
                        self.progressbar4.show()
                        threading.Thread(target=self.convert_as_tiff, args=(self.selected, QUANTUM, dest)).start()   
                elif extension == 5:
                        self.progressbar4.show()
                        threading.Thread(target=self.convert_as_bmp, args=(self.selected, dest)).start()

        def save_as_dialog(self, obj=None):
                if self.radiobutton6.get_active() and not self.window2.get_property('visible'):
                        if self.obj in self.event:
                                self.indice = self.event.index(self.obj)
                        elif  self.obj in self.e:
                                self.indice = self.e.index(self.obj)
                format = os.path.splitext(self.list[self.indice])[1]
                filename = self.get_real_path(self.list[self.indice])
                self.entry7.set_text(os.path.basename(filename)[:-len(format)])
                if self.window2.get_property('visible'):
                        self.filechooserdialog2.set_transient_for(self.window2)
                else:
                        self.filechooserdialog2.set_transient_for(self.window1)
                self.filechooserdialog2.move(0, 0)
                self.filechooserdialog2.set_resizable(True)
                self.filechooserdialog2.set_size_request(600,400)
                self.filechooserdialog2.show()
                while gtk.events_pending():
                        gtk.main_iteration(False)
                response = self.filechooserdialog2.run()
                if response == gtk.RESPONSE_CANCEL or response == gtk.RESPONSE_DELETE_EVENT:
                        self.filechooserdialog2.hide()
                elif response == gtk.RESPONSE_OK:
                        self.filechooserdialog2.hide()
                        while gtk.events_pending():
                                gtk.main_iteration(False)
                        if not self.expander17.get_expanded():
                                expanded = False
                                for item in self.expander_list:
                                        if item.get_expanded():
                                                expanded = True
                                                break
                                if expanded:
                                        self.on_expander_activate(item, False) 
                        fname = self.entry7.get_text()
                        dest = self.filechooserdialog2.get_current_folder()
                        extension = self.combobox9.get_active()
                        if extension == 0:
                                self.dialog16.show()
                                response16 = self.dialog16.run()
                                if response16 == gtk.RESPONSE_CANCEL or response16 == gtk.RESPONSE_DELETE_EVENT:
                                        self.dialog16.hide()
                                        self.save_as_dialog()
                                elif response16 == gtk.RESPONSE_OK:
                                        self.dialog16.hide()
                                        while gtk.events_pending():
                                                gtk.main_iteration(False)
                                        self.settings['jpg_quality'] = self.jpg_quality.value
                                        self.save_as_jpg(fname, filename, self.jpg_quality.value, dest)
                        elif extension == 1:
                                self.save_as_png(fname, filename, dest)        
                        elif extension == 2:
                                self.save_as_tiff(fname, filename, 8, dest)
                        elif extension == 3:
                                self.save_as_tiff(fname, filename, 16, dest)
                        elif extension == 4:
                                self.save_as_tiff(fname, filename, QUANTUM, dest)      
                        elif extension == 5:
                                self.save_as_bmp(fname, filename, dest)                 
                self.unselect()

        def conversion_error(self, item):
                text = _('Cannot convert %s') % os.path.split(item)[1]
                dialog15 = gtk.MessageDialog(self.window8, 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_CLOSE, text)
                gobject.idle_add(dialog15.set_size_request, 320,140)
                gobject.idle_add(dialog15.set_keep_above, True)
                gobject.idle_add(dialog15.show)
                gtk.gdk.beep()
                response = dialog15.run()
                if response == gtk.RESPONSE_CANCEL or response == gtk.RESPONSE_DELETE_EVENT:
                        gobject.idle_add(self.dialog15.destroy)

        def overwrite_dialog(self, dest, filename):
                self.overwrite = False
                filelist = os.listdir(dest)
                if filename in filelist:
                        text = _('A file named:\n"%s" already exists.\nDo you want to overwrite it?') % filename
                        dialog18 = self.messagedialog(_("Attention!"), text, None, gtk.BUTTONS_OK_CANCEL, self.window1, gtk.DIALOG_MODAL, gtk.MESSAGE_WARNING, None, (400,200))
                        dialog18.set_keep_above(True)
                        gtk.gdk.beep()
                        response = dialog18.run()
                        if response == gtk.RESPONSE_CANCEL or response == gtk.RESPONSE_DELETE_EVENT:
                                dialog18.destroy()
                                while gtk.events_pending():
                                        gtk.main_iteration(False)
                                return False
                        elif response == gtk.RESPONSE_OK:
                                dialog18.destroy()
                                self.overwrite = True
                                while gtk.events_pending():
                                        gtk.main_iteration(False)
                                return True
                else:
                        return True

        def make_dcraw_command(self):
                options = self.dcraw_options2()
                interpolation = self.combobox2.get_active_text()
                if '-h' in options:
                        if interpolation == 'AUTO':
                                del options[options.index('-h')]
                        elif interpolation == 'BILINEAR':
                                options[options.index('-h')] = '-q 0'
                        elif interpolation == 'VNG':
                                options[options.index('-h')] = '-q 1'
                        elif interpolation == 'PPG':
                                options[options.index('-h')] = '-q 2'
                        elif interpolation == 'AHG':
                                options[options.index('-h')] = '-q 3'
                return ' '.join(options)

        def pulse_progressbar2(self, thread):
                while thread.isAlive():
                        self.progressbar2.pulse()
                        while gtk.events_pending():
                                gtk.main_iteration(False)
                        time.sleep(0.5)
                
        def process(self, history, command=None, tupla=None, outfile=None, data=None, depth=None, quality=None, batch=None):
                if batch or self.window6.get_property('visible') and history:
                        index = len(history)
                        tail = []
                else:
                        index = self.wand_index
                        tail = []
                        if history and history[-1][0] == 'profile_image':
                                if len(history) >= 3 and history[-3][0] == 'set_image_profile':
                                        tail.append(history[-3])
                                tail.append(history[-2])
                                tail.append(history[-1])
                                
                if data:
                        call(command % tupla, shell=True)
                i_magick = MagickImage(tupla[-1] or tupla[-2])
                if history:
                        for item in history[:index] + tail:
                                try:
                                        if len(item) == 3 :
                                                if item[0][0] == '_get_size' :
                                                        try:
                                                                size = getattr(i_magick, item[0][0])()
                                                                args = (long(item[1][1][0][0] * size[0]), long(item[1][1][0][1] * size[1])),\
                                                                (long(item[1][1][1][0] * size[0]), long(item[1][1][1][1] * size[1]))
                                                                getattr(i_magick, item[1][0])(*args)
                                                                args = (long(item[2][1][0][0] * size[0]), long(item[2][1][0][1] * size[1])),(0,0)       
                                                                getattr(i_magick, item[2][0])(*args)
                                                        except:
                                                                gobject.idle_add(sys.stderr.write, traceback.format_exc())
                                                elif item[0] == 'recolor':
                                                        getattr(i_magick, item[0])(3, item[1])
                                                        if item[2]:
                                                                i_magick._set_colorspace(api.GRAYColorspace)
                                                else:
                                                        getattr(i_magick, item[0][0])(*item[0][1])
                                                        getattr(i_magick, item[1][0])(*item[1][1])
                                                        getattr(i_magick, item[2][0])(*item[2][1])
                                        elif item[0] == 'border':
                                                getattr(i_magick, item[0])(Color(item[1][0]), *item[1][1:])
                                        elif item[0] == 'distort':
                                                colorspace = i_magick._get_colorspace()
                                                i_magick._set_colorspace(api.LabColorspace)
                                                i_magick.distort(item[1][0], item[1][1], True)
                                                i_magick._set_colorspace(colorspace)
                                        elif item[0] == 'red_eye':
                                                size = i_magick._get_size()
                                                for tupla_ in item[1]:
                                                        i_magick = self.apply_red_eye_remotion(size, i_magick, tupla_)
                                        elif item[0] == 'tint':
                                                i_magick = self._tint(i_magick, *item[1])
                                        elif item[0] == 'white balance':
                                                for i in item[1]:
                                                        getattr(i_magick, i[0])(*i[1])
                                        elif item[0] == 'set_image_profile':
                                                i_magick = self.profile_wand(i_magick, item[0], item[1][1])
                                        elif item[0] == 'profile_image':
                                                i_magick = self.profile_wand(i_magick, item[0], item[1][1])
                                        elif item[0] == 'curves':
                                                for i in item[1]:
                                                        getattr(i_magick, i[0])(*i[1])
                                        else: 
                                                getattr(i_magick, item[0])(*item[1])
                                except:
                                        gobject.idle_add(sys.stderr.write, traceback.format_exc())
                                        continue
                if outfile:
                        format_ = os.path.splitext(outfile)[1][1:].upper()
                        i_magick.format = format_
                        if depth:
                                i_magick.set_depth(long(depth))
                        if quality:
                                i_magick._set_compression_quality(quality)
                        if batch or self.window6.get_property('visible'):
                                outfile = self.make_outname(outfile)
                        i_magick.save(outfile)
                        self.add_missed_tags(tupla[-2], outfile, format_)
                else:
                        return i_magick

        def profile_wand(self, wand, fun, profile_name):
                if profile_name == 'sRGB':
                        profile = self.sRGB_profile
                elif profile_name == 'AdobeRGB 1998':
                        profile = self.AdobeRGB1998_profile
                else:
                        try:
                                profile = numpy.fromfile(profile_name, 'u1')
                        except:
                                gobject.idle_add(sys.stderr.write, traceback.format_exc())
                                return wand
                getattr(wand, fun)('icc', profile)
                return wand

        def make_outname(self, outfile): 
                n = 0          
                while os.path.isfile(outfile):
                        root, format = os.path.splitext(os.path.basename(outfile))
                        if '_(' not in root and ')_' not in root:
                                outfile = os.path.join(os.path.dirname(outfile), root + '_(1)_' + format)
                                n = 1
                        else:
                                outfile = outfile.replace('_(' + str(n) + ')_', '_(' + str(n + 1) + ')_')
                                n += 1
                return outfile
                        
        def add_missed_tags(self, filename, outfile, format):
                if format == 'PNG':
                        cmd = '%s -P -overwrite_original -TagsFromFile "%s" -ExposureTime -FNumber -ISO \
                        -ExposureProgram -ExposureCompensation -Flash -FocalLength -Lens >xmp "%s"' % (exiftool, filename, outfile)
                        call(cmd, shell=True)
                else:
                        cmd = '%s -P -overwrite_original -TagsFromFile "%s" -ExposureTime -FNumber -ISO \
                        -ExposureProgram -ExposureCompensation -Flash -FocalLength -Lens "%s"' % (exiftool, filename, outfile)
                        call(cmd, shell=True)
                
        def save_as_jpg(self, fname, filename, quality, dest):
                if not self.overwrite_dialog(dest, fname + '.jpg'):
                        return True
                self.window1.window.set_cursor(self.watch)
                self.progressbar2.set_text('Saving...')
                self.window5.show()
                while gtk.events_pending():
                        gtk.main_iteration(False)
                if self.checkbutton41.get_active():
                        history = self.profile_workflow(self.history)
                else:
                        history = self.history[:]
                if filename.lower().endswith(self.raw_formats):
                        try:
                                tempfile = os.path.join(self.thumbdir, '_' + fname + '.tiff')
                                outfile = os.path.join(dest, fname + '.jpg')
                                command = self.make_dcraw_command()
                                tupla = (filename, tempfile)
                                thread = threading.Thread(target=self.process, args=(history, command, tupla, outfile, True, None, quality))
                                thread.setDaemon(True)
                                thread.start()
                                self.pulse_progressbar2(thread)
                                os.remove(tempfile)
                        except:
                                gobject.idle_add(sys.stderr.write, traceback.format_exc())
                                self.conversion_error(filename)
                else:
                        try:
                                outfile = os.path.join(dest, fname + '.jpg')
                                tupla = (filename, 0)
                                thread = threading.Thread(target=self.process, args=(history, None, tupla, outfile, False, None, quality))
                                thread.setDaemon(True)
                                thread.start()
                                self.pulse_progressbar2(thread)       
                        except:
                                gobject.idle_add(sys.stderr.write, traceback.format_exc())
                                self.conversion_error(filename)
                                if os.path.isfile(outfile):
                                        os.remove(outfile)
                self.window1.window.set_cursor(None)
                self.window5.hide()
                self.progressbar2.set_fraction(0)

        def save_as_png(self, fname, filename, dest):
                if not self.overwrite_dialog(dest, fname + '.png'):
                        return True
                self.window1.window.set_cursor(self.watch)
                self.progressbar2.set_text('Saving...')
                self.window5.show()
                while gtk.events_pending():
                        gtk.main_iteration(False)
                if self.checkbutton41.get_active():
                        history = self.profile_workflow(self.history)
                else:
                        history = self.history[:]
                if filename.lower().endswith(self.raw_formats):
                        try:
                                tempfile = os.path.join(self.thumbdir, '_' + fname + '.tiff')
                                outfile = os.path.join(dest, fname + '.png')
                                command = self.make_dcraw_command()
                                tupla = (filename, tempfile)
                                thread = threading.Thread(target=self.process, args=(history, command, tupla, outfile, True, None, None))
                                thread.setDaemon(True)
                                thread.start()
                                self.pulse_progressbar2(thread)
                                os.remove(tempfile)
                        except:
                                gobject.idle_add(sys.stderr.write, traceback.format_exc())
                                self.conversion_error(filename)
                else:
                        try:
                                outfile = os.path.join(dest, fname + '.png')
                                tupla = (filename, 0)
                                thread = threading.Thread(target=self.process, args=(history, None, tupla, outfile, False, None, None))
                                thread.setDaemon(True)
                                thread.start()
                                self.pulse_progressbar2(thread)
                        except:
                                gobject.idle_add(sys.stderr.write, traceback.format_exc())
                                self.conversion_error(filename)
                                if os.path.isfile(outfile):
                                        os.remove(outfile)
                self.window1.window.set_cursor(None)
                self.window5.hide()
                self.progressbar2.set_fraction(0)
                
        def on_filechooserbutton4_current_folder_changed(self,obj):
                if self.checkbutton9.get_active() and self.window2.get_property("visible"):
                        gobject.idle_add(self.load_raw_preview, self.rect.width, self.rect.height)      
                
        def on_filechooserbutton5_current_folder_changed(self,obj):
                if self.checkbutton10.get_active() and self.window2.get_property("visible"):
                        gobject.idle_add(self.load_raw_preview, self.rect.width, self.rect.height)      
                
        def first_to_load_raw_preview(self, *obj):
                if self.window2.get_property('visible') and self.notebook2.get_current_page() == 2:
                        if self.workflow_started:
                                if self.show_dialog21():
                                        return True             
                        if obj[0] == self.combobox5 and not self.combobox5.get_active() == 2:
                                gobject.idle_add(self.load_raw_preview, self.rect.width, self.rect.height)
                        elif obj[0] in [self.spinbutton1, self.spinbutton4, self.spinbutton5, self.spinbutton6, self.spinbutton7, self.spinbutton8, self.spinbutton11, self.spinbutton39, self.spinbutton42, self.spinbutton45]:
                                threading.Thread(target=self.on_multiplier_changed, args=()).start()
                        elif obj[0] in [self.spinbutton9, self.hscale52] and self.checkbutton12.get_active():
                                threading.Thread(target=self.on_multiplier_changed, args=()).start()
                        elif obj[0] in [self.spinbutton35, self.hscale53] and self.checkbutton23.get_active():
                                threading.Thread(target=self.on_multiplier_changed, args=()).start()
                        elif obj[0] in [self.combobox39, self.combobox3, self.combobox4, self.checkbutton8, self.checkbutton1, self.checkbutton11, self.checkbutton14, self.radiobutton11,\
                        self.checkbutton9, self.checkbutton10, self.checkbutton12, self.checkbutton13, self.checkbutton16, self.checkbutton23, self.hscale3, self.hscale15, self.hscale25, self.hscale28, self.hscale34]:
                                gobject.idle_add(self.load_raw_preview, self.rect.width, self.rect.height)

        def on_multiplier_changed(self):
                self.pila.append(0)
                time.sleep(0.3)
                if len(self.pila) > 1:
                        self.pila.pop()
                else:
                        gobject.idle_add(self.load_raw_preview, self.rect.width, self.rect.height)
                        self.pila = []
                                                                
        def load_raw_preview(self, width, height, pbindex=None):
                format = os.path.splitext(self.list[self.indice])[1]
                if format.lower() in self.raw_formats:
                        self.window2.window.set_cursor(self.watch)
                        self.make_surface(True)
                        self.draw_gradient(self.r, self.g, self.b)
                        filename = self.get_real_path(self.list[self.indice])
                        fname = os.path.basename(filename)[:-len(format)]
                        outfile = os.path.join(self.temp, self.fname)
                        if self.combobox30.get_active() == 1 and not self.auto_wb_presets.has_key(filename):
                                self.combobox5.handler_block(self.id42)
                                self.get_auto_wb_presets(filename)
                                self.combobox5.handler_unblock(self.id42)
                        options = self.dcraw_options2()
                        command = ' '.join(options)
                        p = Popen(command % (filename, outfile), shell=True, stdin=PIPE, stdout=PIPE, stderr=STDOUT, universal_newlines=True, env={'LANG':'en_Us'})
                        output = ''
                        while p.poll() is None:
                                line = p.stdout.readline()
                                self.progressbar8.set_text(line)
                                fraction = self.progressbar8.get_fraction()
                                if fraction <= 0.92:
                                        self.progressbar8.set_fraction(fraction + 0.07)
                                while gtk.events_pending():
                                        gtk.main_iteration(False)
                                output += line
                        self.progressbar8.set_fraction(1)
                        while gtk.events_pending():
                                gtk.main_iteration(False)
                        p.stdout.close()
                        p.stdin.close()
                        self.progressbar8.set_fraction(0)
                        try:
                                if 'multipliers' in output:
                                        outlist = output.split('and\nmultipliers ')
                                        levels = outlist[0].split('\n')[-1].split(', ')
                                        black_point = levels[0].split(' ')[-1]
                                        saturation = levels[1].split(' ')[-1]
                                        multipliers = outlist[1].split('\n')[0].split(' ')
                                else:
                                        black_point = 0
                                        saturation = 4095
                                        multipliers = [1, 1, 1]
                                self.spinbutton4.set_value(float(multipliers[0]))   
                                self.spinbutton5.set_value(float(multipliers[1]))   
                                self.spinbutton6.set_value(float(multipliers[2]))
                                self.spinbutton9.set_value(int(black_point)) 
                                self.spinbutton35.set_value(int(saturation))
                        except:
                                gobject.idle_add(sys.stderr.write, traceback.format_exc())
                        if pbindex:
                                self.pbindex = pbindex
                        wand = MagickImage(outfile)
                        self.load_embedded_profile(wand)
                        self.pblist[self.pbindex] = self.wand2pixbuf(wand, True)
                        if self.expander17.get_expanded():
                                self.set_cairo_pixbuf()
                                self.x, self.y = self.x1, self.y1
                                self.set_zoom_sensitivity(False)
                        elif self.expander8.get_expanded():
                                self.draw_grid(self.checkbutton24)  
                        elif self.expander19.get_expanded():
                                self.draw_grid(self.checkbutton34) 
                        else:
                                if self.checkbutton41.get_active():
                                        self.image28.set_from_pixbuf(self.profile_pixbuf(self.pblist[self.pbindex]))
                                else:
                                        self.image28.set_from_pixbuf(self.pblist[self.pbindex])
                                self.image28.show()
                        self.window2.window.set_cursor(None)
                        while gtk.events_pending():
                                gtk.main_iteration(False)
                        self.calculate_percentage()
                        self.add_dimensions(format, self._label)
                        self.raw_preview = True
                        if self.expander7.get_expanded():
                                self.set_scale_size()
                        if '-h' in options:
                                self.extract_fullsize_image(filename, options)
                self.get_channel()

        def extract_fullsize_image(self, filename, options, half=None):
                def extract_fullsize_image_(filename, fullsize_image, options):
                        p = Popen(' '.join(options) % (filename, fullsize_image), shell=True, stdin=PIPE, stdout=PIPE, stderr=STDOUT)
                        pid = p.stdout.read()
                        p.stdin.close()
                        p.stdout.close()
                if not half:
                        options.remove('-h')
                fullsize_image = os.path.join(self.temp, 'fullsize_image.tiff')
                self.full_process = threading.Thread(target=extract_fullsize_image_, args=(filename, fullsize_image, options))
                self.full_process.start()

        def pixbuf2wand(self, pixbuf):
                assert(pixbuf.get_colorspace() == gtk.gdk.COLORSPACE_RGB)
                pixbuf = pixbuf.copy()
                if not pygtk_has_numpy_support:
                        dimensions = pixbuf.get_width(), pixbuf.get_height()
                        stride = pixbuf.get_rowstride()
                        pixels = pixbuf.get_pixels()
                        mode = pixbuf.get_has_alpha() and "RGBA" or "RGB"
                        im = Image.frombuffer(mode, dimensions, pixels, "raw", mode, stride, 1)
                        array = numpy.asarray(im)
                        wand = MagickImage()
                        shape = array.shape
                        if shape[2] == 3:
                                map_ = 'RGB'
                                is_rgba = False
                        else:
                                map_ = 'RGBA'
                                is_rgba = True
                        width, height = im.size
                        wand.constitute_image(width, height, map_, array.ravel())
                else:
                        with warnings.catch_warnings():
                                warnings.simplefilter("ignore")
                                array = pixbuf.get_pixels_array()
                                pixbuf.pixel_array[:] = array #workaround
                        width, height = pixbuf.get_width(), pixbuf.get_height()
                        wand = MagickImage()
                        shape = array.shape
                        if shape[2] == 3:
                                map_ = 'RGB'
                        else:
                                map_ = 'RGBA'
                        wand.constitute_image(width, height, map_, array.ravel())
                return wand

        def profile_pixbuf(self, pixbuf):
                assert(pixbuf.get_colorspace() == gtk.gdk.COLORSPACE_RGB)
                pixbuf = pixbuf.copy()
                if not pygtk_has_numpy_support:
                        dimensions = pixbuf.get_width(), pixbuf.get_height()
                        stride = pixbuf.get_rowstride()
                        pixels = pixbuf.get_pixels()
                        mode = pixbuf.get_has_alpha() and "RGBA" or "RGB"
                        im = Image.frombuffer(mode, dimensions, pixels, "raw", mode, stride, 1)
                        array = numpy.asarray(im)
                        wand = MagickImage()
                        shape = array.shape
                        if shape[2] == 3:
                                map_ = 'RGB'
                                is_rgba = False
                        else:
                                map_ = 'RGBA'
                                is_rgba = True
                        width, height = im.size
                        wand.constitute_image(width, height, map_, array.ravel())
                        wand = self.profile_image(wand)
                        array_ = numpy.empty(width * height * len(map_), dtype=numpy.uint8)
                        array_ = wand.get_pixels(0, 0, width, height, map_, array_)
                        array_.shape = shape
                        image = Image.fromarray(array_, map_)
                        pixbuf = gtk.gdk.pixbuf_new_from_data(image.tostring(), gtk.gdk.COLORSPACE_RGB, is_rgba, 8, width, height, (is_rgba and 4 or 3) * width)
                else:
                        with warnings.catch_warnings():
                                warnings.simplefilter("ignore")
                                array = pixbuf.get_pixels_array()
                                pixbuf.pixel_array[:] = array #workaround
                        width, height = pixbuf.get_width(), pixbuf.get_height()
                        wand = MagickImage()
                        shape = array.shape
                        if shape[2] == 3:
                                map_ = 'RGB'
                        else:
                                map_ = 'RGBA'
                        wand.constitute_image(width, height, map_, array.ravel())
                        wand = self.profile_image(wand)
                        array_ = numpy.empty(width * height * len(map_), dtype=numpy.uint8)
                        array_ = wand.get_pixels(0, 0, width, height, map_, array_)
                        array_.shape = shape
                        with warnings.catch_warnings():
                                warnings.simplefilter("ignore")
                                pixbuf.pixel_array[:] = array_
                return pixbuf
                
        def save_as_tiff(self, fname, filename, depth, dest):
                if not self.overwrite_dialog(dest, fname + '.tiff'):
                        return True
                self.window1.window.set_cursor(self.watch)
                self.progressbar2.set_text(_('Saving...'))
                self.window5.show()
                while gtk.events_pending():
                        gtk.main_iteration(False)
                if self.checkbutton41.get_active():
                        history = self.profile_workflow(self.history)
                else:
                        history = self.history[:]
                if filename.lower().endswith(self.raw_formats):
                        try:
                                tempfile = os.path.join(self.thumbdir, '_' + fname + '.tiff')
                                outfile = os.path.join(dest, fname + '.tiff')
                                command = self.make_dcraw_command()
                                tupla = (filename, tempfile)
                                thread = threading.Thread(target=self.process, args=(history, command, tupla, outfile, True, depth, None))
                                thread.setDaemon(True)
                                thread.start()
                                self.pulse_progressbar2(thread)
                                os.remove(tempfile)
                        except:
                                gobject.idle_add(sys.stderr.write, traceback.format_exc())
                                self.conversion_error(filename)
                else:
                        try:
                                outfile = os.path.join(dest, fname + '.tiff')
                                tupla = (filename, 0)
                                thread = threading.Thread(target=self.process, args=(history, None, tupla, outfile, False, depth, None))
                                thread.setDaemon(True)
                                thread.start()
                                self.pulse_progressbar2(thread)
                        except:
                                gobject.idle_add(sys.stderr.write, traceback.format_exc())
                                self.conversion_error(filename)
                                if os.path.isfile(outfile):
                                        os.remove(outfile)
                self.window1.window.set_cursor(None)
                self.window5.hide()
                self.progressbar2.set_fraction(0)
                
        def save_as_bmp(self, fname, filename, dest):
                if not self.overwrite_dialog(dest, fname + '.bmp'):
                        return True
                self.window1.window.set_cursor(self.watch)
                self.progressbar2.set_text(_('Saving...'))
                self.window5.show()
                while gtk.events_pending():
                        gtk.main_iteration(False)
                if self.checkbutton41.get_active():
                        history = self.profile_workflow(self.history)
                else:
                        history = self.history[:]
                if filename.lower().endswith(self.raw_formats):
                        try:
                                tempfile = os.path.join(self.thumbdir, '_' + fname + '.tiff')
                                outfile = os.path.join(dest, fname + '.bmp')
                                command = self.make_dcraw_command()
                                tupla = (filename, tempfile)
                                thread = threading.Thread(target=self.process, args=(history, command, tupla, outfile, True, None, None))
                                thread.setDaemon(True)
                                thread.start()
                                self.pulse_progressbar2(thread)
                                os.remove(tempfile)
                        except:
                                gobject.idle_add(sys.stderr.write, traceback.format_exc())
                                self.conversion_error(filename)
                else:
                        try:
                                outfile = os.path.join(dest, fname + '.bmp')
                                tupla = (filename, 0)
                                thread = threading.Thread(target=self.process, args=(history, None, tupla, outfile, False, None, None))
                                thread.setDaemon(True)
                                thread.start()
                                self.pulse_progressbar2(thread)
                        except:
                                gobject.idle_add(sys.stderr.write, traceback.format_exc())
                                self.conversion_error(filename)
                                if os.path.isfile(outfile):
                                        os.remove(outfile)
                self.window1.window.set_cursor(None)
                self.window5.hide()
                self.progressbar2.set_fraction(0)
               
        def convert_as_jpg(self, selected, quality, dest):
                gobject.idle_add(self.win6_okbutton.set_sensitive, False)
                gobject.idle_add(self.checkbutton20.set_sensitive, False)
                gobject.idle_add(self.win6_cancelbutton.set_label, 'Stop')
                gobject.idle_add(self.win6_cancelbutton.set_property, 'image', gtk.image_new_from_stock(gtk.STOCK_STOP, gtk.ICON_SIZE_BUTTON))
                n = 0
                fraction = 0.99 / len(selected)
                command = self.make_dcraw_command()
                if self.checkbutton20.get_active() and self.workflow_copy:
                        workflow = self.workflow_copy
                else:
                        workflow = None
                for item in selected:
                        if self.stop_thread.isSet():
                                break
                        n += 1
                        gobject.idle_add(self.window6.set_title, _('Converting %s of %s files...') % (n, len(selected)))
                        format = os.path.splitext(item)[1]
                        if format.lower() in self.raw_formats:
                                filename = self.get_real_path(item)
                                try:
                                        fname = os.path.split(filename)[1][:-len(format)]
                                        tempfile = os.path.join(self.thumbdir, '_' + fname + '.tiff')
                                        outfile = os.path.join(dest, fname + '.jpg')
                                        tupla = (filename, tempfile)
                                        self.process(workflow, command, tupla, outfile, True, None, quality)
                                        os.remove(tempfile)
                                except:
                                        gobject.idle_add(sys.stderr.write, traceback.format_exc())
                                        self.conversion_error(item)
                        else:
                                try:
                                        fname = os.path.split(item)[1][:-len(format)]
                                        outfile = os.path.join(dest, fname + '.jpg')
                                        tupla = (item, 0)
                                        self.process(workflow, None, tupla, outfile, None, None, quality)
                                except:
                                        gobject.idle_add(sys.stderr.write, traceback.format_exc())
                                        self.conversion_error(item)
                        gobject.idle_add(self.progressbar4.set_fraction, self.progressbar4.get_fraction() + fraction)
                gobject.idle_add(self.window6.hide)
                self.unselect()
                gobject.idle_add(self.progressbar4.set_fraction, 0)
                gobject.idle_add(self.win6_okbutton.set_sensitive, True)
                gobject.idle_add(self.checkbutton20.set_sensitive, True)
                gobject.idle_add(self.win6_cancelbutton.set_label, 'Cancel')
                gobject.idle_add(self.win6_cancelbutton.set_property, 'image', gtk.image_new_from_stock(gtk.STOCK_CANCEL, gtk.ICON_SIZE_BUTTON))
                self.stop_thread.clear()
                
        def convert_as_png(self, selected, dest):
                gobject.idle_add(self.win6_okbutton.set_sensitive, False)
                gobject.idle_add(self.checkbutton20.set_sensitive, False)
                gobject.idle_add(self.win6_cancelbutton.set_label, 'Stop')
                gobject.idle_add(self.win6_cancelbutton.set_property, 'image', gtk.image_new_from_stock(gtk.STOCK_STOP, gtk.ICON_SIZE_BUTTON))
                n = 0
                fraction = 0.99 / len(selected)
                command = self.make_dcraw_command()
                if self.checkbutton20.get_active() and self.workflow_copy:
                        workflow = self.workflow_copy
                else:
                        workflow = None
                for item in selected:
                        if self.stop_thread.isSet():
                                break
                        n += 1
                        gobject.idle_add(self.window6.set_title, _('Converting %s of %s files...') % (n, len(selected)))
                        format = os.path.splitext(item)[1]
                        if format.lower() in self.raw_formats:
                                filename = self.get_real_path(item)
                                try:
                                        fname = os.path.split(filename)[1][:-len(format)]
                                        tempfile = os.path.join(self.thumbdir, '_' + fname + '.tiff')
                                        outfile = os.path.join(dest, fname + '.png')
                                        tupla=(filename, tempfile)
                                        self.process(workflow, command, tupla, outfile, True)
                                        os.remove(tempfile)
                                except:
                                        gobject.idle_add(sys.stderr.write, traceback.format_exc())
                                        self.conversion_error(item)
                        else:
                                try:
                                        fname = os.path.split(item)[1][:-len(format)]
                                        outfile = os.path.join(dest, fname + '.png')
                                        tupla = (item, 0)
                                        self.process(workflow, None, tupla, outfile)
                                except:
                                        gobject.idle_add(sys.stderr.write, traceback.format_exc())
                                        self.conversion_error(item)
                        gobject.idle_add(self.progressbar4.set_fraction, self.progressbar4.get_fraction() + fraction)
                gobject.idle_add(self.window6.hide)
                self.unselect()
                gobject.idle_add(self.progressbar4.set_fraction, 0)
                gobject.idle_add(self.win6_okbutton.set_sensitive, True)
                gobject.idle_add(self.checkbutton20.set_sensitive, True)
                gobject.idle_add(self.win6_cancelbutton.set_label, 'Cancel')
                gobject.idle_add(self.win6_cancelbutton.set_property, 'image', gtk.image_new_from_stock(gtk.STOCK_CANCEL, gtk.ICON_SIZE_BUTTON))
                self.stop_thread.clear()
                
        def convert_as_tiff(self, selected, depth, dest):
                gobject.idle_add(self.win6_okbutton.set_sensitive, False)
                gobject.idle_add(self.checkbutton20.set_sensitive, False)
                gobject.idle_add(self.win6_cancelbutton.set_label, 'Stop')
                gobject.idle_add(self.win6_cancelbutton.set_property, 'image', gtk.image_new_from_stock(gtk.STOCK_STOP, gtk.ICON_SIZE_BUTTON))
                n = 0
                fraction = 0.99 / len(selected)
                command = self.make_dcraw_command()
                if self.checkbutton20.get_active() and self.workflow_copy:
                        workflow = self.workflow_copy
                else:
                        workflow = None
                for item in selected:
                        if self.stop_thread.isSet():
                                break
                        n += 1
                        gobject.idle_add(self.window6.set_title, _('Converting %s of %s files...') % (n, len(selected)))
                        format = os.path.splitext(item)[1]
                        if format.lower() in self.raw_formats:
                                filename = self.get_real_path(item)
                                try:
                                        fname = os.path.split(filename)[1][:-len(format)]
                                        tempfile = os.path.join(self.thumbdir, '_' + fname + '.tiff')
                                        outfile = os.path.join(dest, fname + '.tiff')
                                        tupla=(filename, tempfile)
                                        self.process(workflow, command, tupla, outfile, True, depth)
                                        os.remove(tempfile)
                                except:
                                        gobject.idle_add(sys.stderr.write, traceback.format_exc())
                                        self.conversion_error(item)
                        else:
                                try:
                                        fname = os.path.split(item)[1][:-len(format)]
                                        outfile = os.path.join(dest, fname + '.tiff')
                                        tupla = (item, 0)
                                        self.process(workflow, None, tupla, outfile, None, depth)
                                except:
                                        gobject.idle_add(sys.stderr.write, traceback.format_exc())
                                        self.conversion_error(item)
                        gobject.idle_add(self.progressbar4.set_fraction, self.progressbar4.get_fraction() + fraction)
                gobject.idle_add(self.window6.hide)
                self.unselect()
                gobject.idle_add(self.progressbar4.set_fraction, 0)
                gobject.idle_add(self.win6_okbutton.set_sensitive, True)
                gobject.idle_add(self.checkbutton20.set_sensitive, True)
                gobject.idle_add(self.win6_cancelbutton.set_label, 'Cancel')
                gobject.idle_add(self.win6_cancelbutton.set_property, 'image', gtk.image_new_from_stock(gtk.STOCK_CANCEL, gtk.ICON_SIZE_BUTTON))
                self.stop_thread.clear()
                
        def convert_as_bmp(self, selected, dest):
                gobject.idle_add(self.win6_okbutton.set_sensitive, False)
                gobject.idle_add(self.checkbutton20.set_sensitive, False)
                gobject.idle_add(self.win6_cancelbutton.set_label, 'Stop')
                gobject.idle_add(self.win6_cancelbutton.set_property, 'image', gtk.image_new_from_stock(gtk.STOCK_STOP, gtk.ICON_SIZE_BUTTON))
                n = 0
                fraction = 0.99 / len(selected)
                command = self.make_dcraw_command()
                if self.checkbutton20.get_active() and self.workflow_copy:
                        workflow = self.workflow_copy
                else:
                        workflow = None
                for item in selected:
                        if self.stop_thread.isSet():
                                break
                        n += 1
                        gobject.idle_add(self.window6.set_title, _('Converting %s of %s files...') % (n, len(selected)))
                        format = os.path.splitext(item)[1]
                        if format.lower() in self.raw_formats:
                                filename = self.get_real_path(item)
                                try:
                                        fname = os.path.split(filename)[1][:-len(format)]
                                        tempfile = os.path.join(self.thumbdir, '_' + fname + '.tiff')
                                        outfile = os.path.join(dest, fname + '.bmp')
                                        tupla = (filename, tempfile)
                                        self.process(workflow, command, tupla, outfile, True)
                                        os.remove(tempfile)
                                except:
                                        gobject.idle_add(sys.stderr.write, traceback.format_exc())
                                        self.conversion_error(item)
                        else:
                                try:
                                        fname = os.path.split(item)[1][:-len(format)]
                                        outfile = os.path.join(dest, fname + '.bmp')
                                        tupla = (item, 0)
                                        self.process(workflow, None, tupla, outfile)
                                except:
                                        gobject.idle_add(sys.stderr.write, traceback.format_exc())
                                        self.conversion_error(item)
                        gobject.idle_add(self.progressbar4.set_fraction, self.progressbar4.get_fraction() + fraction)
                gobject.idle_add(self.window6.hide)
                self.unselect()
                gobject.idle_add(self.progressbar4.set_fraction, 0)
                gobject.idle_add(self.win6_okbutton.set_sensitive, True)
                gobject.idle_add(self.checkbutton20.set_sensitive, True)
                gobject.idle_add(self.win6_cancelbutton.set_label, 'Cancel')
                gobject.idle_add(self.win6_cancelbutton.set_property, 'image', gtk.image_new_from_stock(gtk.STOCK_CANCEL, gtk.ICON_SIZE_BUTTON))
                self.stop_thread.clear()
                   
        def statusbar_push(self, statusbar, context_id, text):
                statusbar.push(context_id, text)

        def statusbar_pop(self, statusbar, context_id):
                statusbar.pop(context_id)

        def set_statusbar2(self, image_size, filename):
                self.statusbar_pop(self.statusbar2, self.context_id2)
                text=' %s                          %.3f kb' % (image_size, os.path.getsize(filename) / 1000.0)
                self.statusbar_push(self.statusbar2, self.context_id2, text)

        def calculate_percentage(self):                 # Calculates zoom percentage
                if self.n:
                        try:
                                if self.window2.get_property('visible'):
                                        h = self.rect.height
                                        w = self.rect.width
                                        i = (h ** 2 + w ** 2) ** 0.5
                                        filename = self.get_real_path(self.list[self.indice])
                                        if filename.lower().endswith(self.raw_formats):
                                                self.image_size = self.get_dcraw_output_size(filename)
                                        else:
                                                list_ = [exiftool, '-j', '-n', '-ExifImageWidth', '-ExifImageLength', '-ExifImageHeight', '-ImageWidth',
                                                 '-ImageHeight', filename]
                                                p = Popen(list_, stdin=PIPE, stdout=PIPE, stderr=PIPE)
                                                output = p.communicate()[0]
                                                p.stdout.close()
                                                p.stdin.close()
                                                p.stderr.close()
                                                self.grep_image_size(json.loads(output))
                                        full_size = self.image_size.split('x')
                                        full_i = ((int(full_size[0]) ** 2 + int(full_size[1]) ** 2) ** 0.5)
                                        text = ''
                                        if full_i:
                                                text = str(int((i / full_i) * 100)) + '%'
                                        self.statusbar_pop(self.statusbar1, self.context_id1)
                                        self.statusbar_push(self.statusbar1, self.context_id1, text)
                                else:
                                        h = self.rect2.height
                                        w = self.rect2.width
                                        i=(h ** 2 + w ** 2) ** 0.5
                                        filename = self.get_real_path(self.list[self.indice])
                                        if filename.lower().endswith(self.raw_formats):
                                                self.image_size = self.get_dcraw_output_size(filename)
                                        else:
                                                list_ = [exiftool, '-j', '-n',  '-ExifImageWidth', '-ExifImageLength', '-ExifImageHeight', '-ImageWidth',
                                                 '-ImageHeight', filename]
                                                p = Popen(list_, stdin=PIPE, stdout=PIPE, stderr=PIPE)
                                                output = p.communicate()[0]
                                                p.stdout.close()
                                                p.stdin.close()
                                                p.stderr.close()
                                                self.grep_image_size(json.loads(output))
                                        full_size = self.image_size.split('x')
                                        full_i = (int(full_size[0]) ** 2 + int(full_size[1]) ** 2) ** 0.5
                                        text = ''
                                        if full_i:
                                                text = str(int((i / full_i) * 100)) + '%'
                                        self.statusbar_pop(self.statusbar4, self.context_id4)
                                        self.statusbar_push(self.statusbar4, self.context_id4, text)
                        except:
                                gobject.idle_add(sys.stderr.write, traceback.format_exc())

        def get_dcraw_output_size(self, filename):
                cmd = 'dcraw -i -v "%s"' % filename
                p = Popen(cmd, shell=True, stdin=PIPE, stdout=PIPE, stderr=STDOUT, universal_newlines=True, env={'LANG':'en_Us'})
                output = p.communicate()[0]
                p.stdin.close()
                p.stdout.close()
                if output:
                        list_ = output.split('\n')
                        for item in list_:
                                if 'Output size' in item:
                                        size = item.split(':')[1].strip().split(' x ')
                                        return size[0] + 'x' + size[1]                       
                return '0x0'
                                    
        def stop_gallery(self):
                if hasattr(self, 'gallery_thread') and self.gallery_thread.isAlive():
                        self.stop_thread3.set()
                        self.gallery_thread.join()
                                
        def get_folder(self, treeview, path, view_column):   # Make a gallery double clicking a folder in the directory tree
                self.stop_gallery()  
                self.vpaned2.handler_block(self.id27)
                iter0 = self.treestore.get_iter(path)
                n = 1
                percorso = self.treestore[path][1]
                while n < len(path):
                        iterparent = self.treestore.iter_parent(iter0)
                        pathparent = self.treestore.get_path(iterparent)
                        percorso = self.treestore[pathparent][1] + homepath + percorso
                        n += 1
                        del iter0
                        iter0 = iterparent.copy()
                        del iterparent
                if not sys.platform.startswith('win'):
                        percorso = percorso[1:]
                viewport = self.scrolledwindow2.get_child()
                if viewport:
                        viewport.destroy()
                self.cartella = percorso
                self.window1.set_title(self.version + self.cartella)
                while gtk.events_pending():
                        gtk.main_iteration(False)
                self.n = self.count_images()
                if self.n:
                        if self.radiobutton7.get_active():
                                self.make_bottom_panel(None, self.n)
                        else:
                                self.make_table()
                                self.attach_images()
                                self.make_thumbnails(self.width, self.height, self.n)
                        self.statusbar_pop(self.statusbar3, self.context_id3)
                        self.statusbar_push(self.statusbar3, self.context_id3,\
                        'There are %s images     Total size %f MB' % (self.n, self.total_size / 1000000.0))
                else:
                        self.statusbar_pop(self.statusbar3, self.context_id3)
                        self.statusbar_push(self.statusbar3, self.context_id3, 'There are %s images' % self.n)
                        if self.radiobutton7.get_active():
                                try:
                                        self.hbox1.destroy()
                                except:
                                        pass
                self.add_selection1.set_sensitive(True)
                self.selected=[]
                self.settings['last_folder_path'] = self.cartella[:]
                self.settings['switch'] = 'cartella'
                self.vpaned2.handler_unblock(self.id27)

        def expand_rows(self, treeview, iter_, path):
                self.treeview2.freeze_child_notify()
                percorso = self.treestore[path][1]
                folderpb = self.folderpb
                join = os.path.join
                isdir = os.path.isdir
                gio_files = []
                n=1
                iter0 = iter_.copy()
                while n < len(path):
                        iterparent = self.treestore.iter_parent(iter0)
                        pathparent = self.treestore.get_path(iterparent)
                        percorso = self.treestore[pathparent][1] + homepath + percorso
                        n += 1
                        del iter0
                        iter0 = iterparent.copy()
                        del iterparent
                if not sys.platform.startswith('win'):
                        percorso = percorso[1:]
                if not percorso:
                        percorso = '/'
                if sys.platform.startswith('win') and percorso[-1] == ':':
                        percorso = percorso + homepath
                store = treeview.get_model()
                child = store.iter_children(iter_)
                for item in sorted(os.listdir(percorso)):
                        if not self.checkbutton33.get_active() and item.startswith('.'):
                                continue
                        basedir = join(percorso, item)
                        if isdir(basedir):
                                iter = store.append(iter_, [folderpb, item])
                                if not self.monitors.has_key(basedir):
                                        gio_files += [basedir]
                                try:
                                        for dir_ in os.listdir(basedir):
                                                if isdir(join(basedir, dir_)):
                                                        store.append(iter)
                                                        break
                                except:
                                        pass
                if store.get_value(child, 1) == None:
                        store.remove(child)
                self.treeview2.thaw_child_notify()
                if gio and gio_files:
                        threading.Thread(target=self.start_directory_monitors, args=(gio_files,)).start()

        def start_directory_monitors(self, dirs):
                userpath_ = os.path.expanduser('~')
                for item in dirs:
                        if item.startswith(userpath_) and not self.monitors.has_key(item):
                                gfile = gio.File(item)
                                monitor = gfile.monitor_directory(gio.FILE_MONITOR_NONE, None)
                                monitor.connect('changed', self.on_monitor_event)
                                self.monitors[item] = monitor


        def collapse_rows(self, treeview, iter_, path):
                self.treeview2.freeze_child_notify()
                store = treeview.get_model()
                child = store.iter_children(iter_)
                while child:
                        store.remove(child)
                        child = store.iter_children(iter_)
                store.append(iter_)
                self.treeview2.thaw_child_notify()
                
        def show_filechooserdialog1(self, event):
                self.vpaned2.handler_block(self.id27)
                self.filechooserdialog1.show()
                if self.checkbutton33.get_active():
                        self.filechooserdialog1.set_show_hidden(True)
                else:
                        self.filechooserdialog1.set_show_hidden(False)
                response = self.filechooserdialog1.run()
                if response == gtk.RESPONSE_CANCEL or response == gtk.RESPONSE_DELETE_EVENT:
                        self.filechooserdialog1.hide()
                elif response == gtk.RESPONSE_OK:
                        self.stop_gallery()
                        viewport = self.scrolledwindow2.get_child()
                        if viewport:
                                viewport.destroy()
                        self.cartella = self.filechooserdialog1.get_current_folder()
                        self.window1.set_title(self.version + self.cartella)
                        self.filechooserdialog1.hide()
                        while gtk.events_pending():
                                gtk.main_iteration(False)
                        self.n = self.count_images()
                        gobject.idle_add(self.refresh_directory_tree)
                        if self.n:
                                if self.radiobutton7.get_active():
                                        self.make_bottom_panel(None, self.n)
                                else:
                                        self.make_table()
                                        self.attach_images()
                                        self.make_thumbnails(self.width, self.height, self.n)
                                self.statusbar_pop(self.statusbar3, self.context_id3)
                                self.statusbar_push(self.statusbar3, self.context_id3,\
                                'There are %s images     Total size %f MB' % (self.n, self.total_size / 1000000.0))
                        else:
                                self.statusbar_pop(self.statusbar3, self.context_id3)
                                self.statusbar_push(self.statusbar3, self.context_id3, 'There are %s images' % self.n)
                                if self.radiobutton7.get_active():
                                        try:
                                                self.hbox1.destroy()
                                        except:
                                                pass
                self.add_selection1.set_sensitive(True)
                self.selected = []
                self.settings['last_folder_path'] = self.cartella[:]
                self.settings['switch'] = 'cartella'
                self.vpaned2.handler_unblock(self.id27)
                
        def open_last_folder(self, cartella, fname=None):            # Opens last visited folder at start
                if not os.path.dirname(cartella):
                        cartella = os.path.join(os.getcwd(), cartella)
                while gtk.events_pending():
                        gtk.main_iteration(False)
                self.vpaned2.handler_block(self.id27)
                self.cartella = cartella
                self.window1.set_title(self.version + self.cartella)
                while gtk.events_pending():
                        gtk.main_iteration(False)
                self.n = self.count_images()
                viewport = self.scrolledwindow2.get_child()
                if viewport:
                        viewport.destroy()
                if self.n:
                        if self.radiobutton7.get_active():
                                self.make_bottom_panel(None, self.n, fname)
                        else:
                                self.make_table()
                                self.attach_images()
                                self.make_thumbnails(self.width, self.height, self.n)
                        self.statusbar_pop(self.statusbar3, self.context_id3)
                        self.statusbar_push(self.statusbar3, self.context_id3,\
                        'There are %s images     Total size %f MB '% (self.n, self.total_size / 1000000.0))
                else:
                        self.statusbar_pop(self.statusbar3, self.context_id3)
                        self.statusbar_push(self.statusbar3, self.context_id3, 'There are %s images' % self.n)
                self.add_selection1.set_sensitive(True)
                self.selected = []
                self.settings['last_folder_path'] = self.cartella[:]
                self.settings['switch'] = 'cartella'
                self.vpaned2.handler_unblock(self.id27)
                self.expand_to_path(cartella)
                
        def expand_to_path(self, folder):                # Autoexpands directory tree
                folder = os.path.normpath(folder)
                lista = folder.split(homepath)
                if not sys.platform.startswith('win'):
                        lista[0] = homepath
                if not self.checkbutton33.get_active():
                        while lista[-1][0] == '.':
                                del lista[-1]
                iter0 = self.treestore.get_iter_root()
                for item in lista:
                        value = ''
                        while value != item:
                                value = self.treestore.get_value(iter0, 1)
                                if value == item:
                                        break
                                iter0 = self.treestore.iter_next(iter0)
                        path0 = self.treestore.get_path(iter0)
                        self.treeview2.expand_to_path(path0)
                        iter0 = self.treestore.iter_nth_child(iter0, 0)
                column = self.treeview2.get_column(len(lista))
                self.treeview2.scroll_to_cell(path0, column, True, 0.5, 0.0)
                
        def album_control(self):                 # Controls for new files on album directory
                files = os.listdir(os.path.join(self.ark, self.album))
                filelist = []
                for item in files:
                        if item == '_preview_.png':
                                continue
                        format = os.path.splitext(item)[1]
                        if format.lower().endswith(self.raw_formats):
                                filelist += [os.path.join(self.ark, self.album, 'thumb', item[:-len(format)] + '.thumb.jpg' + format)]
                        elif format.lower().endswith(self.formats):
                                filelist += [os.path.join(self.ark, self.album, item)]
                        else:
                                continue
                if len(filelist) != len(self.albums[self.album]):
                        text = _('Attention! The %s archive has been modified!\nThe album will be updated!') % self.album
                        dialog14 = self.messagedialog(_("Warning!"), text, None, gtk.BUTTONS_OK_CANCEL, None, gtl.DIALOG_MODAL, gtk.MESSAGE_WARNING, None, (450,190))
                        dialog14.set_keep_above(True)
                        gtk.gdk.beep()
                        response = dialog14.run()#eliminare cancel button
                        if response == gtk.RESPONSE_CANCEL or response == gtk.RESPONSE_DELETE_EVENT or response == gtk.RESPONSE_OK:
                                dialog14.destroy()
                                self.albums[self.album] = []
                                self.albums[self.album] = filelist
                                self.store_albums_db()

        def open_with_gtkrawgallery(self, filename):
                self.radiobutton7.set_active(True)
                folder = os.path.dirname(filename)
                if not folder:
                        folder = os.getcwd()
                self.open_last_folder(folder, os.path.join(folder, os.path.basename(filename)))

        def open_last_album(self, album):                # Opens last visited album at start
                try:
                        self.expand_to_path(self.settings['last_folder_path'])
                except:
                        pass
                self.album = album
                self.album_control()
                self.make_album_gallery()
                self.add_selection1.set_sensitive(True)
                self.selected = []
                self.settings['last_album_path'] = self.album[:]
                self.settings['switch'] = 'album'                        
                                
        def show_album(self, treeview, path, column):      # Makes gallery double clicking the album name.
                self.stop_gallery()
                iter_ = self.model.get_iter(path)
                self.album = self.model.get_value(iter_,1)
                self.album_control()
                self.make_album_gallery()
                self.add_selection1.set_sensitive(True)
                self.selected = []
                self.settings['last_album_path'] = self.album[:]
                self.settings['switch'] = 'album'
                
        def make_album_gallery(self):
                self.vpaned2.handler_block(self.id27)
                self.window1.set_title(self.version + 'Albums/' + self.album)
                while gtk.events_pending():
                        gtk.main_iteration(False)
                self.n = len(self.albums[self.album])
                self.list = []
                self.total_size = 0
                get_size = os.path.getsize
                formats_ = self.formats
                for infile in self.albums[self.album]:
                        if infile.lower().endswith(formats_):
                                self.list += [infile]
                                self.total_size += get_size(infile)
                        else:
                                self.list += [infile]
                                f = infile.replace('thumb' + homepath, '').replace('.thumb.jpg', '')
                                self.total_size += get_size(f)
                viewport = self.scrolledwindow2.get_child()
                if viewport:
                        viewport.destroy()
                if self.n > 0:
                        if self.radiobutton7.get_active():
                                self.make_bottom_panel(None, self.n)
                        else:
                                self.make_table()
                                self.attach_images()
                                self.make_thumbnails(self.width, self.height, self.n)
                        self.statusbar_pop(self.statusbar3, self.context_id3)
                        self.statusbar_push(self.statusbar3, self.context_id3,\
                        _('There are %s images     Total size %f MB') % (self.n, self.total_size / 1000000.0))
                else:
                        if self.radiobutton7.get_active():
                                try:
                                        self.hbox1.destroy()
                                except:
                                        pass
                        self.statusbar_pop(self.statusbar3, self.context_id3)
                        self.statusbar_push(self.statusbar3, self.context_id3, _('There are 0 images'))
                self.vpaned2.handler_unblock(self.id27)
                
        def Open(self, event):
                self.stop_gallery()
                self.treeiter = self.model.get_iter(self.path)
                self.album = self.model.get_value(self.treeiter,1)
                self.album_control()
                self.make_album_gallery()
                self.add_selection1.set_sensitive(True)
                self.settings['last_album_path'] = self.album[:]
                self.settings['switch'] = 'album'

        def open_album(self, event):
                self.dialog3.set_title(_('Open Album'))
                self.label4.set_text(_('Choose the album to open'))
                self.dialog3.show()
                response = self.dialog3.run()
                if response == gtk.RESPONSE_CANCEL or response == gtk.RESPONSE_DELETE_EVENT:
                        self.dialog3.hide()
                elif response == gtk.RESPONSE_OK:
                        self.album = self.combobox28.get_active_text()
                        if not self.album:
                                self.dialog3.hide()
                        else:
                                self.dialog3.hide()
                                self.stop_gallery()
                                self.album_control()
                                self.make_album_gallery()
                self.add_selection1.set_sensitive(True)
                self.selected = []
                self.settings['last_album_path'] = self.album[:]
                self.settings['switch'] = 'album'
                
        def add_album(self, event, data):                       # Makes a new album 
                self.entry1.grab_focus()
                self.label3.set_text(_('Insert the new album name'))
                self.dialog1.set_title(_('New Album'))
                self.dialog1.show()
                response = self.dialog1.run()
                if response == gtk.RESPONSE_CANCEL or response == gtk.RESPONSE_DELETE_EVENT:
                        self.dialog1.hide()
                        self.entry1.set_text('')
                        if data == 'button':
                                self.add_selection()
                                return False
                elif response == gtk.RESPONSE_OK:
                        self.added_album = self.entry1.get_text().strip()
                        if not self.added_album:
                                self.dialog1.hide()
                                if data == 'button':
                                        self.add_selection()
                                        return False
                        elif self.albums.has_key(self.added_album):
                                gtk.gdk.beep()
                                dialog2 = self.messagedialog(_("Error!"), _("This album name already exists.\nChoose another name!"),
                                                             None, gtk.BUTTONS_CLOSE, self.window1, gtk.DIALOG_MODAL, gtk.MESSAGE_ERROR, None, (280,150))
                                dialog2.set_keep_above(True)
                                response = dialog2.run()
                                if response == gtk.RESPONSE_CLOSE or response == gtk.RESPONSE_DELETE_EVENT:
                                        dialog2.destroy()
                                        self.add_album(self, event)
                        else:
                                os.mkdir(os.path.join(self.ark, self.added_album))
                                os.mkdir(os.path.join(self.ark, self.added_album, 'thumb'))
                                self.albums[self.added_album] = []
                                self.model.append([gtk.gdk.pixbuf_new_from_file(self.get_icon_path('no_preview.png')), self.added_album])
                                self.boxmodel.append([self.added_album])
                                self.boxmodel3.append([self.added_album])
                                self.dialog1.hide()
                                self.entry1.set_text('')
                                self.store_albums_db()
                        return True
                
        def Delete(self, event):                                 #Deletes the album       
                self.treeiter = self.model.get_iter(self.path)
                album = self.model.get_value(self.treeiter, 1)
                text = _('Are you sure to delete\n %s ?') % album
                dialog10 = self.messagedialog(_('Delete Album'), text, None, gtk.BUTTONS_OK_CANCEL, None, gtk.DIALOG_MODAL, gtk.MESSAGE_WARNING, None, (320,160))
                dialog10.set_keep_above(True)
                response = dialog10.run()
                if response == gtk.RESPONSE_CANCEL or response == gtk.RESPONSE_DELETE_EVENT:
                        dialog10.destroy()
                elif response == gtk.RESPONSE_OK:
                        if album in self.window1.get_title():
                                        viewport = self.scrolledwindow2.get_child()
                                        if viewport:
                                                viewport.destroy()
                                                self.statusbar_pop(self.statusbar3, self.context_id3)
                                                self.statusbar_push(self.statusbar3, self.context_id3, _('There are 0 images'))
                                        if self.radiobutton7.get_active():
                                                try:
                                                        self.hbox1.destroy()
                                                except:
                                                        pass
                                                del self.b_event
                                                gc.collect()       
                        iter0 = self.model.get_iter_first()
                        value = ''
                        while value != album:
                                value = self.model.get_value(iter0, 1)
                                if value == album:
                                        break
                                iter0 = self.model.iter_next(iter0)
                        self.model.remove(iter0)
                        for item in [self.boxmodel, self.boxmodel3]:
                                iter1 = item.get_iter_root()
                                value = ''
                                while value != album:
                                        value = item.get_value(iter1, 0)
                                        if value == album:
                                                break
                                        iter1 = item.iter_next(iter1)
                                item.remove(iter1)
                        dialog10.destroy()
                        del self.albums[album]
                        shutil.rmtree(os.path.join(self.ark, album), ignore_errors = True)
                        self.n = 0
                        self.list = []
                        self.store_albums_db()
                                                
        def delete_album(self, event):                                   
                self.dialog3.set_title(_('Delete Album'))
                self.label4.set_text(_('Choose the album to delete!'))
                self.dialog3.show()
                response = self.dialog3.run()
                if response == gtk.RESPONSE_CANCEL or response == gtk.RESPONSE_DELETE_EVENT:
                        self.dialog3.hide()
                elif response == gtk.RESPONSE_OK:
                        album = self.combobox28.get_active_text()
                        if not album:
                                self.dialog3.hide()
                        else:
                                if album in self.window1.get_title():
                                        viewport = self.scrolledwindow2.get_child()
                                        if viewport:
                                                viewport.destroy()
                                                self.statusbar_pop(self.statusbar3, self.context_id3)
                                                self.statusbar_push(self.statusbar3, self.context_id3, _('There are 0 images'))
                                        if self.radiobutton7.get_active:
                                                try:
                                                        self.hbox1.destroy()
                                                except:
                                                        pass
                                                del self.b_event
                                                gc.collect()                 
                                iter0 = self.model.get_iter_first()
                                value = ''
                                while value != album:
                                        value = self.model.get_value(iter0, 1)
                                        if value == album:
                                                break
                                        iter0 = self.model.iter_next(iter0)
                                self.model.remove(iter0)
                                for item in [self.boxmodel, self.boxmodel3]:
                                        iter1 = item.get_iter_root()
                                        value = ''
                                        while value != album:
                                                value = item.get_value(iter1, 0)
                                                if value == album:
                                                        break
                                                iter1 = item.iter_next(iter1)
                                        item.remove(iter1)
                                self.dialog3.hide()
                                del self.albums[album]
                                shutil.rmtree(os.path.join(self.ark, album), ignore_errors = True)
                                self.n = 0
                                self.list = []
                                self.store_albums_db()
        
        def rename_album(self, obj=None):                
                self.treeiter = self.model.get_iter(self.path)
                album = self.model.get_value(self.treeiter, 1)
                self.label10.set_text(_('Insert a new name for\n %s') % album)
                self.entry2.grab_focus()
                self.dialog11.show()
                response = self.dialog11.run()
                if response == gtk.RESPONSE_CANCEL or response == gtk.RESPONSE_DELETE_EVENT:
                        self.dialog11.hide()
                elif response == gtk.RESPONSE_OK:
                        new_name = self.entry2.get_text().strip()
                        if not new_name:
                                self.dialog11.hide()
                        elif self.albums.has_key(new_name):
                                gtk.gdk.beep()
                                dialog2 = self.messagedialog("Error!", "This album name already exists.\nChoose another name!",
                                                             None, gtk.BUTTONS_CLOSE, self.window1, gtk.DIALOG_MODAL, gtk.MESSAGE_ERROR, None, (280,150))
                                dialog2.set_keep_above(True)
                                response = dialog2.run()
                                if response == gtk.RESPONSE_CLOSE or response == gtk.RESPONSE_DELETE_EVENT:
                                        dialog2.destroy()
                                        self.rename_album()
                        else:
                                self.dialog11.hide()
                                self.entry2.set_text('')
                                temp = self.albums[album][:]
                                os.rename(os.path.join(self.ark, album), os.path.join(self.ark, new_name))
                                del self.albums[album]
                                self.albums[new_name] = [item.replace(os.path.join(self.ark, album), os.path.join(self.ark, new_name)) for item in temp]
                                for item in [self.boxmodel, self.boxmodel3]:
                                        iter1 = item.get_iter_root()
                                        value = ''
                                        while value != album:
                                                value = item.get_value(iter1, 0)
                                                if value == album:
                                                        break
                                                iter1 = item.iter_next(iter1)
                                        item.set_value(iter1, 0, new_name)
                                self.model.set_value(self.treeiter, 1, new_name)
                                self.window1.set_title(self.version + 'Albums/' + new_name)
                                if self.settings.has_key('last_album_path') and self.settings['last_album_path'] == album:
                                        self.settings['last_album_path'] = new_name
                                self.store_albums_db()
                                tagged = Sqlite.get_tagged_from_dir(os.path.join(self.ark, album))
                                for item in tagged:
                                        filename = item[1][:]
                                        new_filename = item[1].replace(os.path.join(self.ark, album), os.path.join(self.ark, new_name))
                                        Sqlite.update_row((new_filename, os.path.join(self.ark, new_name), filename))
                                        

        def set_panel(self, event, data):         # Hides and shows left panel
                self.window1.window.set_cursor(self.watch)
                self.toolbuttons(False, True)
                while gtk.events_pending():
                        gtk.main_iteration(False)
                self.vpaned2.handler_block(self.id27)
                self.toolbutton16.handler_block(self.id1)
                self.toolbutton17.handler_block(self.id2)
                self.toolbutton14.handler_block(self.id9)
                self.toolbutton15.handler_block(self.id10)
                self.toolbutton18.handler_block(self.id11)
                self.toolbutton19.handler_block(self.id12)
                self.toolbutton20.handler_block(self.id13)
                self.toolbutton21.handler_block(self.id19)
                self.toolbutton26.handler_block(self.id23)
                self.toolbutton27.handler_block(self.id24)
                self.toolbutton28.handler_block(self.id39)
                self.toolbutton29.handler_block(self.id26)
                self.toolbutton30.handler_block(self.id30)
                self.hpaned1.handler_block(self.id28)
                self.new1.handler_block(self.id14)
                self.delete2.handler_block(self.id15)
                self.add_selection1.handler_block(self.id16)
                if self.scrolledwindow6 and self.radiobutton7.get_active():
                        images = [i.get_pixbuf().copy() for i in self.b_img[:self.n]]
                        self.scrolledwindow6.destroy()
                elif self.radiobutton6.get_active():
                        images = [i.get_pixbuf().copy() for i in self.image[:self.n]]
                if data == 'hide':
                        self.hpaned1.set_position(0)
                        self.close_panel = False
                elif data == 'show':
                        self.hpaned1.set_position(int(self.screen0.get_width() * 0.2))
                viewport = self.scrolledwindow2.get_child()
                if viewport:
                        viewport.destroy()
                        if self.n:
                                self.make_table()
                                self.attach_images(images)
                                for i in self.label[:self.n]:
                                        i.set_size_request(self.width, -1)
                                self.window1.show_all()
                                self.button5.hide()
                                while gtk.events_pending():
                                        gtk.main_iteration(False)
                self.vpaned2.handler_unblock(self.id27)
                self.radiobutton6.set_active(True)
                self.settings['radiobutton6'] = True
                self.settings['radiobutton7'] = False
                self.window1.window.set_cursor(None)
                self.toolbutton16.handler_unblock(self.id1)
                self.toolbutton17.handler_unblock(self.id2)
                self.toolbutton14.handler_unblock(self.id9)
                self.toolbutton15.handler_unblock(self.id10)
                self.toolbutton18.handler_unblock(self.id11)
                self.toolbutton19.handler_unblock(self.id12)
                self.toolbutton20.handler_unblock(self.id13)
                self.toolbutton21.handler_unblock(self.id19)
                self.toolbutton26.handler_unblock(self.id23)
                self.toolbutton27.handler_unblock(self.id24)
                self.toolbutton28.handler_unblock(self.id39)
                self.toolbutton29.handler_unblock(self.id26)
                self.toolbutton30.handler_unblock(self.id30)
                self.hpaned1.handler_unblock(self.id28)
                self.new1.handler_unblock(self.id14)
                self.delete2.handler_unblock(self.id15)
                self.add_selection1.handler_unblock(self.id16)
                self.store_settings_db()
                gc.collect()
                
        def hide_pane(self, obj, show=False):
                self.window1.window.set_cursor(self.watch)
                while gtk.events_pending():
                        gtk.main_iteration(False)
                if self.radiobutton6.get_active():
                        if obj == self.toolbutton28:
                                self.close_panel = True
                        else:
                                self.close_panel = False
                        self.vpaned2.handler_block(self.id27)   
                        self.hpaned1.set_position(0)
                        self.vpaned2.handler_unblock(self.id27)
                        images = [i.get_pixbuf().copy() for i in self.image[:self.n]]
                        self.make_bottom_panel(None, self.n, None, images)
                else:
                        self.vpaned2.handler_block(self.id27)
                        if show:
                                self.hpaned1.set_position(int(self.screen0.get_width() * 0.2))
                        else:
                                self.hpaned1.set_position(0)
                        self.autoresize2(None, None, True)
                        self.vpaned2.handler_unblock(self.id27)
                self.window1.window.set_cursor(None)
                gc.collect()
                                                                           
        def enlarge_gallery(self, event):
                if self.radiobutton7.get_active():
                        self.zoom_out_photo(self, 1)
                        return True
                if self.width == 260:
                        return True
                viewport = self.scrolledwindow2.get_child()
                if viewport:
                        viewport.destroy()
                        if self.n:
                                if self.width == 100:
                                        self.width = 150
                                        self.height = 100
                                elif self.width == 150:
                                        self.width = 201
                                        self.height = 134
                                elif self.width == 201:
                                        self.width = 260
                                        self.height = 174
                                self.make_table()
                                self.attach_images()
                                self.make_thumbnails(self.width, self.height, self.n)
                
        def reduce_gallery(self, event):
                if self.radiobutton7.get_active():
                        self.zoom_in_photo(self,1)
                        return True
                if self.width == 100:
                        return True
                viewport = self.scrolledwindow2.get_child()
                if viewport:
                        images = [i.get_pixbuf().copy() for i in self.image[:self.n]]
                        viewport.destroy()
                        if self.n:
                                self.window1.window.set_cursor(self.watch)
                                while gtk.events_pending():
                                        gtk.main_iteration(False)
                                if self.width == 260:
                                        self.width = 201
                                        self.height = 134
                                elif self.width == 201:
                                        self.width = 150
                                        self.height = 100 
                                elif self.width == 150:
                                        self.width = 100
                                        self.height = 67
                                images = map(self.scale_pixbuf, images)
                                for i in self.label[:self.n]:
                                        i.set_size_request(self.width, -1)
                                self.make_table()
                                self.attach_images(images)
                                self.window1.show_all()
                                self.button5.hide()
                                self.window1.window.set_cursor(None)
                                gc.collect()

        def on_checkbutton360_toggled(self, obj):
                if not obj.get_active():
                        self.height = int(self.width / 1.5)
                        self.settings['checkbutton360'] = False
                else:
                        self.settings['checkbutton360'] = True
                        
        def scale_pixbuf(self, pixbuf):
                width = pixbuf.get_width()
                height = pixbuf.get_height()
                if self.checkbutton4.get_active() and width > 10 and height > 10:
                        pixbuf = pixbuf.subpixbuf(2, 2, width - 4, height - 4)
                ratio = float(width) / height
                if ratio >= 1:
                        pixbuf = pixbuf.scale_simple(self.width, int(self.width / ratio), gtk.gdk.INTERP_BILINEAR)
                else:
                        pixbuf = pixbuf.scale_simple(int(self.height * ratio), self.height, gtk.gdk.INTERP_BILINEAR)
                if self.checkbutton4.get_active():
                        pixbuf = self.add_thumbnail_border(pixbuf, self.border_color)
                return pixbuf

        def select_all(self, obj=None):
                if self.scrolledwindow2.get_child():
                        for indice in xrange(self.n):
                                if (self.list[indice] not in self.selected) and (indice not in self.blocked_list):
                                        self.e[indice].modify_bg(gtk.STATE_NORMAL, self.selection_color)
                                        self.selected += [self.list[indice]]
                                               
        def unselect(self, obj=None):
                if self.selected:
                        for item in self.e:
                                gobject.idle_add(item.modify_bg, gtk.STATE_NORMAL, self.colorbutton1.get_color())
                        self.selected = []

        def inverted_selection(self, event):    
                if not self.selected: 
                        self.select_all()
                else:
                        selected = self.selected[:]
                        self.select_all()
                        for item in selected:
                                if item.lower().endswith(self.raw_formats):
                                        filename = os.path.split(item)[1].replace('.thumb.jpg', '')
                                else:  
                                        filename = os.path.split(item)[1]
                                for indice in xrange(self.n):
                                        label = self.e[indice].get_child()
                                        fname = label.get_text()
                                        if fname == filename:
                                                self.e[indice].modify_bg(gtk.STATE_NORMAL, self.colorbutton1.get_color())
                                                del self.selected[self.selected.index(item)]
                                                break
                                        else:   
                                                continue
        
        def fullscreen_mode(self, *obj):
                if not self.n: 
                        return True
                if not self.window2.get_property('visible'):
                        if self.radiobutton6.get_active(): 
                                self.indice = 0
                        indice = self.indice
                        self.old_indice = indice
                self.fullpbindex = 1
                self.window3.set_size_request(self.screen0.get_width(), self.screen0.get_height())
                self.window3.fullscreen()
                width, height = self.window3.get_size()
                format = os.path.splitext(self.list[self.indice])[1]
                if format.lower() in self.raw_formats:
                        if self.rotated.has_key(self.indice) and self.checkbutton2.get_active():
                                self.fullpblist[1] = gtk.gdk.pixbuf_new_from_file_at_size(self.list[self.indice][:-len(format)], height, width)
                                self.fullpblist[1] = self.fullpblist[1].rotate_simple(self.rotated[self.indice])        
                        else:
                                self.fullpblist[1] = gtk.gdk.pixbuf_new_from_file_at_size(self.list[self.indice][:-len(format)], width, height)
                else:
                        if self.rotated.has_key(self.indice) and self.checkbutton2.get_active():
                                self.fullpblist[1] = gtk.gdk.pixbuf_new_from_file_at_size(self.list[self.indice], height, width)
                                self.fullpblist[1] = self.fullpblist[1].rotate_simple(self.rotated[self.indice])        
                        else:
                                self.fullpblist[1] = gtk.gdk.pixbuf_new_from_file_at_size(self.list[self.indice], width, height)
                self.image1.set_from_pixbuf(self.fullpblist[1])
                self.window3.show_all()
                while gtk.events_pending():
                        gtk.main_iteration(False)
                gobject.idle_add(self.load_cache, width, height, 'fullscreen')

        def sox(self):
                p = Popen(["sox", "-q", self.sample, "-d", "repeat", "50"], stdin=PIPE, stdout=PIPE, stderr=STDOUT)
                self.pid = p.pid
                p.stdout.close()
                p.stdin.close()

        def slideshow(self, *data):
                if self.list:
                        if len(data) == 3:
                                if self.radiobutton6.get_active():
                                        self.indice = 0
                                self.loop = data[1]
                                indice = self.indice
                                self.old_indice = indice
                                if self.checkbutton6.get_active() and self.sample:
                                        self.pid = 0
                                        try:
                                                self.sox()
                                        except:
                                                gobject.idle_add(sys.stderr.write, traceback.format_exc())
                        if self.loop:
                                self.window4.set_size_request(self.screen0.get_width(), self.screen0.get_height())
                                self.window4.fullscreen()
                                width, height = self.window4.get_size()
                                if not self.pixbuf1 or not self.checkbutton5.get_active():
                                        format = os.path.splitext(self.list[self.indice])[1]
                                        if format.lower() in self.raw_formats:
                                                if not self.rotated.has_key(self.indice):
                                                        self.pixbuf1 = gtk.gdk.pixbuf_new_from_file_at_size(self.list[self.indice]\
                                                        [:-len(format)], width, height)
                                                else:
                                                        self.pixbuf1 = gtk.gdk.pixbuf_new_from_file_at_size(self.list[self.indice]\
                                                        [:-len(format)], height, width)
                                                        self.pixbuf1 = self.pixbuf1.rotate_simple(self.rotated[self.indice])
                                        else:
                                                if not self.rotated.has_key(self.indice):   
                                                        self.pixbuf1 = gtk.gdk.pixbuf_new_from_file_at_size(self.list[self.indice],\
                                                        width, height)
                                                else:   
                                                        self.pixbuf1 = gtk.gdk.pixbuf_new_from_file_at_size(self.list[self.indice],\
                                                        height, width)
                                                        self.pixbuf1 = self.pixbuf1.rotate_simple(self.rotated[self.indice])
                                        width1 = self.pixbuf1.get_width()
                                        height1 = self.pixbuf1.get_height()
                                        nero = self.background.copy()
                                        if self.pixbuf1.get_has_alpha():
                                                nero = nero.add_alpha(False, chr(255), chr(255), chr(255))
                                        self.pixbuf1.copy_area(0, 0, width1, height1, nero, (width - width1) / 2,(height - height1)/2)
                                        self.pixbuf1 = nero.copy()
                                self.image3.set_from_pixbuf(self.pixbuf1)
                                self.window4.show_all()
                                while gtk.events_pending():
                                        gtk.main_iteration(False)
                                t1 = time.time()
                                if self.indice == len(self.list) - 1:
                                        self.indice = -1
                                format = os.path.splitext(self.list[self.indice + 1])[1]
                                if format.lower() in self.raw_formats:
                                        if not self.rotated.has_key(self.indice + 1):
                                                pixbuf2 = gtk.gdk.pixbuf_new_from_file_at_size(self.list[self.indice + 1][:-len(format)],\
                                                width, height)
                                        else:
                                                pixbuf2 = gtk.gdk.pixbuf_new_from_file_at_size(self.list[self.indice + 1][:-len(format)],\
                                                height, width)
                                                pixbuf2 = pixbuf2.rotate_simple(self.rotated[self.indice+1])
                                else:   
                                        if not self.rotated.has_key(self.indice + 1):
                                                pixbuf2 = gtk.gdk.pixbuf_new_from_file_at_size(self.list[self.indice + 1], width, height)
                                        else:
                                                pixbuf2 = gtk.gdk.pixbuf_new_from_file_at_size(self.list[self.indice + 1], height, width)
                                                pixbuf2 = pixbuf2.rotate_simple(self.rotated[self.indice + 1])
                                delay = time.time() - t1
                                pause = self.pause
                                if delay > pause:
                                        delay = pause
                                for i in xrange(10):
                                        time.sleep((pause - delay) / 10.0)
##                                        while gtk.events_pending():
##                                                gtk.main_iteration(False)
                                        if self.loop == 0:
                                                self.pixbuf1 = 0
                                                return True 
                                if self.checkbutton5.get_active():
                                        self.pixbuf1 = self.fading_transition(self.pixbuf1, pixbuf2, width, height,\
                                        self.pixbuf1.get_width(), self.pixbuf1.get_height())
                                        if not self.pixbuf1:
                                                gc.collect()
                                                return True
                                self.image3.set_from_pixbuf(pixbuf2)
                                self.window4.show_all()
                                while gtk.events_pending():
                                        gtk.main_iteration(False)
                                self.indice += 1
                                if self.indice >= len(self.list) - 1:
                                        self.indice = -1
                                self.slideshow(None)
                        else:
                                self.pixbuf1 = 0

        def fading_transition(self, pixbuf1, pixbuf2, width, height, width1, height1):
                width2 = pixbuf2.get_width()
                height2 = pixbuf2.get_height()
                nero = self.background.copy()
                if pixbuf2.get_has_alpha():
                        nero = nero.add_alpha(False, chr(255), chr(255), chr(255))
                pixbuf2.copy_area(0, 0, width2, height2, nero, (width - width2) / 2, (height - height2) / 2)
                pixbuf2 = nero.copy()
                n = 255 / self.frames
                alpha = 255 - n
                for i in xrange(self.frames):
                        if self.loop == 0:
                                pixbuf1 = 0
                                return pixbuf1
                        transition = pixbuf2.copy()
                        pixbuf1.composite(transition, 0, 0, width1, height1, 0, 0, 1.0, 1.0, gtk.gdk.INTERP_BILINEAR, alpha)
                        self.image3.set_from_pixbuf(transition)
                        self.window4.show_all()
                        while gtk.events_pending():
                                gtk.main_iteration(False)
                        alpha -= n
                        pixbuf1 = transition.copy()
                        time.sleep(self.delay)
                return pixbuf2

        def set_image1(self, format, pbindex=None, previous_indice=None, cache_args=None):
                if self.workflow_started:
                        self.pbindex = self.previous_index
                        if self.show_dialog21():
                                if self.previous_index:
                                        self.indice = previous_indice
                                return True
                if self.gallery_thread.isAlive() and self.indice > self.i:
                        self.get_filename(self.list[self.indice], self.indice)
                if format.lower() in self.raw_formats:
                        self._label = os.path.split(self.list[self.indice])[1].replace('.thumb.jpg', '')
                        self.window2.set_title(os.path.join(self.window1.get_title(), self._label))
                        if self.notebook2.get_current_page() in [0, 1, 6]:
                                if self.rotated.has_key(self.indice) and self.checkbutton2.get_active():
                                        self.pblist[1] = gtk.gdk.pixbuf_new_from_file_at_size(self.list[self.indice][:-len(format)],\
                                        self.rect.height, self.rect.width).rotate_simple(self.rotated[self.indice])
                                else:
                                        self.pblist[1] = gtk.gdk.pixbuf_new_from_file_at_size(self.list[self.indice][:-len(format)],\
                                                        self.rect.width, self.rect.height)      
                        else:
                                gobject.idle_add(self.load_raw_preview, self.rect.width, self.rect.height, pbindex)
                                if cache_args:
                                        self.cache_pixbuf(cache_args[0], cache_args[1])
                                return True
                else:
                        self.window2.window.set_cursor(self.watch)
                        while gtk.events_pending():
                                gtk.main_iteration(False)
                        if pbindex:
                                self.pbindex = pbindex
                        self._label = os.path.split(self.list[self.indice])[1]
                        self.window2.set_title(os.path.join(self.window1.get_title(), self._label))
                        if self.rotated.has_key(self.indice) and self.checkbutton2.get_active() and self.notebook2.get_current_page() == 0:
                                if self.checkbutton41.get_active():
                                        try:
                                                wand = MagickImage(self.list[self.indice])
                                        except:
                                                wand = MagickImage(self.get_icon_path('gtk-missing-image.png'))
                                        self.load_embedded_profile(wand)
                                        self.pblist[1] = self.wand2pixbuf(wand, True, True)
                                else:
                                        self.pblist[1] = self.load_cache_pixbuf(self.list[self.indice], self.rect.height, self.rect.width)
                                self.pblist[1] = self.pblist[1].rotate_simple(self.rotated[self.indice])
                        else:
                                if self.checkbutton41.get_active():
                                        wand = MagickImage(self.list[self.indice])
                                        self.load_embedded_profile(wand)
                                        self.pblist[1] = self.wand2pixbuf(wand, True)
                                else:
                                        self.pblist[1] = self.load_cache_pixbuf(self.list[self.indice], self.rect.width, self.rect.height)
                self.pblist[1] = self.check_icons(format, self.pblist[1])
                if self.expander17.get_expanded():
                        self.set_cairo_pixbuf()
                        self.set_zoom_sensitivity(False)
                elif self.expander8.get_expanded():
                        self.draw_grid(self.checkbutton24)
                elif self.expander19.get_expanded():
                                self.draw_grid(self.checkbutton34)
                else:
                        if self.checkbutton41.get_active():
                                self.image28.set_from_pixbuf(self.profile_pixbuf(self.pblist[self.pbindex]))
                        else:
                                self.image28.set_from_pixbuf(self.pblist[self.pbindex])
                        self.image28.show()
                self.window2.window.set_cursor(None)
                while gtk.events_pending():
                        gtk.main_iteration(False)
                if self.notebook2.get_current_page() == 3:
                        self.get_channel()
                elif self.notebook2.get_current_page() == 1:
                        self.show_tags()
                self.calculate_percentage()
                self.raw_preview = False
                if cache_args:
                        self.cache_pixbuf(cache_args[0], cache_args[1])
                
        def set_image2(self, format):
                if format.lower() in self.raw_formats:
                        self._label = os.path.split(self.list[self.indice])[1].replace('.thumb.jpg', '')
                        if self.rotated.has_key(self.indice) and self.checkbutton2.get_active():
                                self.b_pixlist[1] = gtk.gdk.pixbuf_new_from_file_at_size(self.list[self.indice][:-len(format)],\
                                self.rect2.height - self.swin2_label.allocation.height, self.rect2.width).rotate_simple(self.rotated[self.indice])
                        else:
                                self.b_pixlist[1] = gtk.gdk.pixbuf_new_from_file_at_size(self.list[self.indice][:-len(format)],\
                                self.rect2.width, self.rect2.height - self.swin2_label.allocation.height)
                else:
                        self._label = os.path.split(self.list[self.indice])[1]
                        if self.rotated.has_key(self.indice) and self.checkbutton2.get_active():
                                self.b_pixlist[1] = self.load_cache_pixbuf(self.list[self.indice],\
                                self.rect2.height - self.swin2_label.allocation.height, self.rect2.width, self.rotated[self.indice])
                        else:
                                self.b_pixlist[1] = self.load_cache_pixbuf(self.list[self.indice],\
                                self.rect2.width, self.rect2.height - self.swin2_label.allocation.height)
                self.b_pixlist[1] = self.check_icons(format, self.b_pixlist[1])
                self.swin2_image.set_from_pixbuf(self.b_pixlist[1])
                self.swin2_label.set_text(self.label[self.indice].get_text())
                self.scroll_to_selection()
                while gtk.events_pending():
                        gtk.main_iteration(False)
                self.calculate_percentage()
               
        def set_pixbuf(self, format, pbindex, previous_indice, cache_args):
                bul = False
                if self.workflow_started:
                        self.pbindex = self.previous_index
                        if self.show_dialog21():
                                self.indice = previous_indice
                                return True
                        else: 
                                bul = True
                if self.gallery_thread.isAlive() and self.indice > self.i:
                        self.get_filename(self.list[self.indice], self.indice)  
                if format.lower() in self.raw_formats:
                        self.pbindex = pbindex
                        self._label = os.path.split(self.list[self.indice])[1].replace('.thumb.jpg', '')
                        self.window2.set_title(os.path.join(self.window1.get_title(), self._label))
                        if self.notebook2.get_current_page() in [2, 3, 4, 5]:
                                gobject.idle_add(self.load_raw_preview, self.rect.width, self.rect.height, pbindex)
                                if bul:
                                        while gtk.events_pending():
                                                gtk.main_iteration(False)
                                        self.pblist[self.previous_index] = self.original_pixbuf
                                self.cache_pixbuf(cache_args[0], cache_args[1])
                                return True
                else:
                        self.pbindex = pbindex
                        self._label = os.path.split(self.list[self.indice])[1]
                        self.window2.set_title(os.path.join(self.window1.get_title(), self._label))
                        if self.rotated.has_key(self.indice) and self.notebook2.get_current_page() != 0:
                                if self.checkbutton41.get_active():
                                        try:
                                                wand = MagickImage(self.list[self.indice])
                                        except:
                                                wand = MagickImage(self.get_icon_path('gtk-missing-image.png'))
                                        self.load_embedded_profile(wand)
                                        self.pblist[pbindex] = self.wand2pixbuf(wand, True, True)
                                else:
                                        self.pblist[pbindex] = self.load_cache_pixbuf(self.list[self.indice], self.rect.width, self.rect.height)
                        else:
                                if self.checkbutton41.get_active():
                                        try:
                                                wand = MagickImage(self.list[self.indice])
                                        except:
                                                wand = MagickImage(self.get_icon_path('gtk-missing-image.png'))
                                        self.load_embedded_profile(wand)
                self.pblist[pbindex] = self.check_icons(format, self.pblist[pbindex])
                if self.expander17.get_expanded():
                        self.set_cairo_pixbuf()
                        self.set_zoom_sensitivity(False)
                elif self.expander8.get_expanded():
                        self.draw_grid(self.checkbutton24)
                elif self.expander19.get_expanded():
                                self.draw_grid(self.checkbutton34)
                else:
                        if self.checkbutton41.get_active():
                                self.image28.set_from_pixbuf(self.profile_pixbuf(self.pblist[pbindex]))
                        else:
                                self.image28.set_from_pixbuf(self.pblist[self.pbindex])
                        self.image28.show()
                while gtk.events_pending(): 
                        gtk.main_iteration(False)       
                self.calculate_percentage()
                if self.notebook2.get_current_page() == 3: 
                        self.get_channel()
                elif self.notebook2.get_current_page() == 1:
                        self.show_tags()
                self.raw_preview = False
                if self.pblist[pbindex].get_height() > self.rect.height or self.pblist[pbindex].get_width() > self.rect.width: 
                        self.zoom_fit_photo(None, 0)
                self.cache_pixbuf(cache_args[0], cache_args[1])
                return True
                
        def set_b_pixbuf(self, format, pixindex):
                self.b_pixlist[pixindex] = self.check_icons(format, self.b_pixlist[pixindex])
                self.swin2_image.set_from_pixbuf(self.b_pixlist[pixindex])
                self.swin2_label.set_text(self.label[self.indice].get_text())
                self.scroll_to_selection()
                while gtk.events_pending():
                        gtk.main_iteration(False)
                if self.b_pixlist[pixindex].get_height() > self.rect2.height or self.b_pixlist[pixindex].get_width() > self.rect2.width:
                        self.zoom_fit_photo(None, 1)
                self.calculate_percentage()
                        
        def cache_pixbuf(self, pbindex, indice):          # Preloads pixbuf on detailed window
                if indice in xrange(self.n):
                        if pbindex > 2:
                                pbindex = 0
                        if pbindex < 0:
                                pbindex = 2
                        if self.gallery_thread.isAlive() and indice > self.i:
                                self.get_filename(self.list[indice], indice)
                        format = os.path.splitext(self.list[indice])[1]
                        if format.lower() in self.raw_formats:
                                filename = self.list[indice][:-len(format)]
                        else:
                                filename = self.list[indice]
                        if self.rotated.has_key(indice) and self.checkbutton2.get_active():
                                self.pblist[pbindex] = self.load_cache_pixbuf(filename, self.rect.height, self.rect.width, self.rotated[indice])
                        else:
                                self.pblist[pbindex] = self.load_cache_pixbuf(filename, self.rect.width, self.rect.height)              

        def cache_b_pixbuf(self, pixindex, indice):     # Preloads pixbuf on detailed window
                if indice in xrange(self.n):
                        if pixindex > 2:
                                pixindex = 0
                        if pixindex < 0:
                                pixindex = 2
                        format = os.path.splitext(self.list[indice])[1]
                        if format.lower() in self.raw_formats:
                                filename = self.list[indice][:-len(format)]
                        else:
                                filename = self.list[indice]
                        if self.rotated.has_key(indice) and self.checkbutton2.get_active():     
                                self.b_pixlist[pixindex] = self.load_cache_pixbuf(filename, \
                                self.rect2.height - self.swin2_label.allocation.height, self.rect2.width, self.rotated[indice])
                        else:
                                self.b_pixlist[pixindex] = self.load_cache_pixbuf(filename, \
                                                self.rect2.width, self.rect2.height - self.swin2_label.allocation.height)
                
        def cache_fullpixbuf(self, fullpbindex, indice):          # Preloads pixbuf on fullscreen mode
                if indice in xrange(self.n):
                        if fullpbindex > 2:
                                fullpbindex = 0
                        elif fullpbindex < 0:
                                fullpbindex = 2
                        format = os.path.splitext(self.list[indice])[1]
                        if format.lower() in self.raw_formats:
                                filename = self.list[indice][:-len(format)]
                        else:
                                filename = self.list[indice]
                        if self.rotated.has_key(indice) and self.checkbutton2.get_active():
                                self.fullpblist[fullpbindex] = self.load_cache_pixbuf(filename, \
                                self.screen0.get_height(), self.screen0.get_width(), self.rotated[indice])
                        else:   
                                self.fullpblist[fullpbindex] = self.load_cache_pixbuf(filename, self.screen0.get_width(), self.screen0.get_height())
                
        def add_dimensions(self, format, label):        # Adds image dimensions on the statusbar
                if format.lower() in self.raw_formats and self.ark not in self.list[self.indice]:
                        self.display_exif(self.rawpath[self.list[self.indice]], label)
                        if self.image_size:
                                self.set_statusbar2(self.image_size, self.rawpath[self.list[self.indice]])
                        else:
                                self.set_statusbar2('', self.list[self.indice][:-len(format)])
                elif format.lower() in self.raw_formats and self.ark in self.list[self.indice]:
                        filename = self.list[self.indice].replace(homepath + 'thumb', '').replace('.thumb.jpg', '')
                        self.display_exif(filename, label)
                        if self.image_size:
                                self.set_statusbar2(self.image_size, filename)
                        else:
                                self.set_statusbar2('', filename)
                else:
                        try:
                                self.display_exif(self.list[self.indice], label)
                                if self.image_size:
                                        self.set_statusbar2(self.image_size, self.list[self.indice])
                                else:
                                        self.set_statusbar2('', self.list[self.indice])
                        except:
                                self.exifmodel.clear()
                                
        def first_photo(self, event):
                if not self.lock.locked():
                        with self.lock:
                                self.previous_index = self.pbindex
                                indice = self.indice
                                self.pbindex = 1
                                self.indice = 0
                                format = os.path.splitext(self.list[self.indice])[1]
                                self.set_image1(format, 1,indice, (self.pbindex + 1, self.indice + 1))
                                self.add_dimensions(format, self._label)
                                self.count = 0
                                if self.expander7.get_expanded():
                                        self.set_scale_size()
                
        def first_b_photo(self, event):
                if not self.lock2.locked() and self.n > 0:
                        with self.lock2:
                                self.pixindex = 1
                                self.indice = 0
                                for item in self.b_event:
                                        item.modify_bg(gtk.STATE_NORMAL, self.gallery_color)
                                self.b_event[self.indice].modify_bg(gtk.STATE_NORMAL, self.selection_color)
                                while gtk.events_pending():
                                        gtk.main_iteration(False)
                                self.set_image2(os.path.splitext(self.list[self.indice])[1])
                                self.count = 0
                                self.cache_b_pixbuf(self.pixindex + 1, self.indice + 1)
                
        def last_photo(self, event):
                if not self.lock.locked():
                        with self.lock:
                                self.previous_index = self.pbindex
                                indice = self.indice
                                self.pbindex = 1
                                self.indice = len(self.list) - 1
                                format = os.path.splitext(self.list[self.indice])[1]
                                self.set_image1(format, 1, indice, (self.pbindex - 1, self.indice - 1))
                                self.add_dimensions(format, self._label)
                                self.count = 0
                                if self.expander7.get_expanded():
                                        self.set_scale_size()
                
        def last_b_photo(self, event):
                if not self.lock2.locked() and self.n > 0:
                        with self.lock2:
                                self.pixindex = 1
                                self.indice = len(self.list) - 1
                                for item in self.b_event:
                                        item.modify_bg(gtk.STATE_NORMAL, self.gallery_color)
                                self.b_event[self.indice].modify_bg(gtk.STATE_NORMAL, self.selection_color)
                                while gtk.events_pending():
                                        gtk.main_iteration(False)
                                self.set_image2(os.path.splitext(self.list[self.indice])[1])
                                self.count = 0
                                self.cache_b_pixbuf(self.pixindex - 1, self.indice - 1)
        
        def previous_photo(self, obj=None):
                if not self.lock.locked():
                        with self.lock:
                                self.count = 0
                                if self.indice > 0:
                                        self.indice -= 1
                                        self.previous_index = self.pbindex
                                        if self.pbindex > 0:
                                                self.pbindex -= 1
                                        else:
                                                self.pbindex = 2
                                        format = os.path.splitext(self.list[self.indice])[1]
                                        self.set_pixbuf(format, self.pbindex, self.indice + 1, (self.pbindex - 1, self.indice - 1))
                                        self.add_dimensions(format, self._label)
                                        if self.expander7.get_expanded():
                                                self.set_scale_size()

        def previous_b_photo(self, obj=None):
                if not self.lock2.locked():
                        with self.lock2:
                                self.count = 0
                                if self.indice > 0:
                                        self.indice -= 1
                                        for item in self.b_event:
                                                item.modify_bg(gtk.STATE_NORMAL, self.gallery_color)
                                        self.b_event[self.indice].modify_bg(gtk.STATE_NORMAL, self.selection_color)
                                        while gtk.events_pending():
                                                gtk.main_iteration(False)
                                        if self.pixindex > 0:
                                                self.pixindex -= 1
                                        else:
                                                self.pixindex = 2
                                        self.set_b_pixbuf(os.path.splitext(self.list[self.indice])[1], self.pixindex)
                                        self.cache_b_pixbuf(self.pixindex - 1, self.indice - 1)

        def next_photo(self, obj=None):
                if not self.lock.locked():
                        with self.lock:
                                self.count = 0
                                if self.indice < len(self.list) - 1:
                                        self.indice += 1
                                        self.previous_index = self.pbindex
                                        if self.pbindex < 2:
                                                self.pbindex += 1
                                        else:
                                                self.pbindex = 0
                                        format = os.path.splitext(self.list[self.indice])[1]
                                        self.set_pixbuf(format, self.pbindex, self.indice - 1, (self.pbindex + 1, self.indice + 1))
                                        self.add_dimensions(format, self._label)
                                        if self.expander7.get_expanded():
                                                self.set_scale_size()

        def next_b_photo(self, obj=None):
                if not self.lock2.locked():
                        with self.lock2:
                                self.count = 0
                                if self.indice < len(self.list) - 1:
                                        self.indice += 1
                                        for item in self.b_event:
                                                item.modify_bg(gtk.STATE_NORMAL, self.gallery_color)
                                        self.b_event[self.indice].modify_bg(gtk.STATE_NORMAL, self.selection_color)
                                        while gtk.events_pending():
                                                gtk.main_iteration(False)
                                        if self.pixindex < 2:
                                                self.pixindex += 1
                                        else:
                                                self.pixindex = 0
                                        self.set_b_pixbuf(os.path.splitext(self.list[self.indice])[1], self.pixindex)
                                        self.cache_b_pixbuf(self.pixindex + 1, self.indice + 1)

        def previous_fullphoto(self):
                if not self.lock.locked():
                        with self.lock:
                                if self.indice > 0:
                                        self.indice -= 1
                                        if self.fullpbindex > 0:
                                                self.fullpbindex -= 1
                                        else:
                                                self.fullpbindex = 2
                                        self.image1.set_from_pixbuf(self.fullpblist[self.fullpbindex])
                                        self.cache_fullpixbuf(self.fullpbindex - 1, self.indice - 1)

        def next_fullphoto(self):
                if not self.lock.locked():
                        with self.lock:
                                if self.indice < len(self.list) - 1:
                                        self.indice += 1
                                        if self.fullpbindex < 2:
                                                self.fullpbindex += 1
                                        else:
                                                self.fullpbindex = 0
                                        self.image1.set_from_pixbuf(self.fullpblist[self.fullpbindex])
                                        self.cache_fullpixbuf(self.fullpbindex + 1, self.indice + 1)                
        def zoom_in_photo(self, event, data):
                while gtk.events_pending():
                        gtk.main_iteration(False)
                if not self.n:
                        return True
                self.toolbutton6.handler_block(self.id35)
                self.toolbutton7.handler_block(self.id36)
                self.toolbutton8.handler_block(self.id37)
                self.toolbutton9.handler_block(self.id38)
                self.toolbutton16.handler_block(self.id1)
                self.toolbutton17.handler_block(self.id2)
                self.count = 0
                format = os.path.splitext(self.list[self.indice])[1]
                if not data:
                        self.rect.width -= (self.rect.width / 100) * 15
                        self.rect.height -= (self.rect.height / 100) * 15
                        if self.rect.height < 100:
                                self.rect.width = 150
                                self.rect.height = 100
                        rect_width = self.rect.width
                        rect_height = self.rect.height
                else:
                        self.rect2.width -= (self.rect2.width / 100) * 15
                        self.rect2.height -= (self.rect2.height / 100) * 15
                        if self.rect2.height < 100:
                                self.rect2.width = 150
                                self.rect2.height = 100
                        rect_width = self.rect2.width
                        rect_height = self.rect2.height
                if format.lower() in self.raw_formats:
                        if self.workflow_started :
                                pixbuf = self.wand2pixbuf(self.wand, True)
                        else:
                                if self.raw_preview:
                                        pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(os.path.join(self.temp, self.fname), rect_width, rect_height)
                                else:
                                        if self.rotated.has_key(self.indice) and self.checkbutton2.get_active():
                                                pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(self.list[self.indice][:-len(format)],\
                                                        rect_height, rect_width).rotate_simple(self.rotated[self.indice])
                                        else:
                                                pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(self.list[self.indice][:-len(format)],\
                                                                                                        rect_width, rect_height)
                else:
                        if self.workflow_started :
                                if self.rotated.has_key(self.indice) and self.checkbutton2.get_active():
                                        pixbuf = self.wand2pixbuf(self.wand, True, True).rotate_simple(self.rotated[self.indice])
                                else:
                                        pixbuf = self.wand2pixbuf(self.wand, True)
                        else:
                                if self.rotated.has_key(self.indice) and self.checkbutton2.get_active() and self.notebook2.get_current_page() == 0:
                                        pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(self.list[self.indice], \
                                                rect_height, rect_width).rotate_simple(self.rotated[self.indice])
                                else:
                                        pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(self.list[self.indice], rect_width, rect_height)
                self.pblist[self.pbindex] = self.check_icons(format, pixbuf) 
                if not data:
                        if self.expander8.get_expanded():
                                self.draw_grid(self.checkbutton24)
                        elif self.expander19.get_expanded():
                                self.draw_grid(self.checkbutton34)
                        else:
                                if self.checkbutton41.get_active():
                                        self.image28.set_from_pixbuf(self.profile_pixbuf(self.pblist[self.pbindex]))
                                else:       
                                        self.image28.set_from_pixbuf(self.pblist[self.pbindex])
                else:
                        self.swin2_image.set_from_pixbuf(self.pblist[self.pbindex])
                while gtk.events_pending():
                        gtk.main_iteration(False)
                self.calculate_percentage()
                self.toolbutton6.handler_unblock(self.id35)
                self.toolbutton7.handler_unblock(self.id36)
                self.toolbutton8.handler_unblock(self.id37)
                self.toolbutton9.handler_unblock(self.id38)
                self.toolbutton16.handler_unblock(self.id1)
                self.toolbutton17.handler_unblock(self.id2)
                
        def zoom_out_photo(self, event, data):
                while gtk.events_pending():
                        gtk.main_iteration(False)
                if not self.n:
                        return True
                self.toolbutton6.handler_block(self.id35)
                self.toolbutton7.handler_block(self.id36)
                self.toolbutton8.handler_block(self.id37)
                self.toolbutton9.handler_block(self.id38)
                self.toolbutton16.handler_block(self.id1)
                self.toolbutton17.handler_block(self.id2)
                self.count = 0
                format = os.path.splitext(self.list[self.indice])[1]
                if not data:
                        self.rect.width += (self.rect.width / 100) * 15
                        self.rect.height += (self.rect.height / 100) * 15
                        if self.rect.height < 100:
                                self.rect.width = 150
                                self.rect.height = 100
                        rect_width = self.rect.width
                        rect_height = self.rect.height
                else:
                        self.rect2.width += (self.rect2.width / 100) * 15
                        self.rect2.height += (self.rect2.height / 100) * 15
                        if self.rect2.height < 100:
                                self.rect2.width = 150
                                self.rect2.height = 100
                        rect_width = self.rect2.width
                        rect_height = self.rect2.height
                if format.lower() in self.raw_formats:
                        if self.workflow_started :
                                pixbuf = self.wand2pixbuf(self.wand, True)
                        else:
                                if self.raw_preview:
                                        pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(os.path.join(self.temp, self.fname), rect_width, rect_height)
                                else:
                                        if self.rotated.has_key(self.indice) and self.checkbutton2.get_active():
                                                pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(self.list[self.indice][:-len(format)],\
                                                rect_height, rect_width).rotate_simple(self.rotated[self.indice])
                                        else:
                                                pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(self.list[self.indice][:-len(format)],\
                                                                rect_width, rect_height)
                else:
                        if self.workflow_started :
                                if self.rotated.has_key(self.indice) and self.checkbutton2.get_active():
                                        pixbuf = self.wand2pixbuf(self.wand, True, True).rotate_simple(self.rotated[self.indice])
                                else:
                                        pixbuf = self.wand2pixbuf(self.wand, True)
                        else:
                                if self.rotated.has_key(self.indice) and self.checkbutton2.get_active() and self.notebook2.get_current_page() == 0:
                                        pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(self.list[self.indice], \
                                        rect_height, rect_width).rotate_simple(self.rotated[self.indice])
                                else:
                                        pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(self.list[self.indice], rect_width, rect_height)
                self.pblist[self.pbindex] = self.check_icons(format, pixbuf) 
                if not data:
                        if self.expander8.get_expanded():
                                self.draw_grid(self.checkbutton24)
                        elif self.expander19.get_expanded():
                                self.draw_grid(self.checkbutton34)
                        else:
                                if self.checkbutton41.get_active():
                                        self.image28.set_from_pixbuf(self.profile_pixbuf(self.pblist[self.pbindex]))
                                else:
                                        self.image28.set_from_pixbuf(self.pblist[self.pbindex])
                else:
                        self.swin2_image.set_from_pixbuf(self.pblist[self.pbindex])
                while gtk.events_pending():
                        gtk.main_iteration(False)
                self.calculate_percentage()
                self.toolbutton6.handler_unblock(self.id35)
                self.toolbutton7.handler_unblock(self.id36)
                self.toolbutton8.handler_unblock(self.id37)
                self.toolbutton9.handler_unblock(self.id38)
                self.toolbutton16.handler_unblock(self.id1)
                self.toolbutton17.handler_unblock(self.id2)

        def zoom_fit_photo(self, obj, data):
                if self.n:
                        while gtk.events_pending():
                                gtk.main_iteration(False)
                        self.toolbutton6.handler_block(self.id35)
                        self.toolbutton7.handler_block(self.id36)
                        self.toolbutton8.handler_block(self.id37)
                        self.toolbutton9.handler_block(self.id38)
                        self.toolbutton16.handler_block(self.id1)
                        self.toolbutton17.handler_block(self.id2)
                        self.count = 0
                        format = os.path.splitext(self.list[self.indice])[1]
                        if not data:
                                self.rect = self.viewport1.allocation
                                rect = self.rect
                        else:
                                self.rect2 = self.scrolledwindow2.allocation
                                rect = self.rect2
                        if format.lower() in self.raw_formats:
                                if not data:
                                        if self.workflow_started :
                                                pixbuf = self.wand2pixbuf(self.wand, True)
                                        else:
                                                if self.raw_preview:
                                                        pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(os.path.join(self.temp, self.fname), rect.width, rect.height)
                                                else:
                                                        if self.rotated.has_key(self.indice) and self.checkbutton2.get_active():
                                                                pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(self.list[self.indice][:-len(format)],\
                                                                rect.height, rect.width).rotate_simple(self.rotated[self.indice])
                                                        else:
                                                                pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(self.list[self.indice][:-len(format)],\
                                                                                rect.width, rect.height)
                                else:
                                        if self.rotated.has_key(self.indice) and self.checkbutton2.get_active():
                                                pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(self.list[self.indice][:-len(format)],\
                                                rect.height - self.swin2_label.allocation.height, rect.width)
                                                pixbuf = pixbuf.rotate_simple(self.rotated[self.indice])
                                        else:
                                                pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(self.list[self.indice][:-len(format)],\
                                                rect.width, rect.height - self.swin2_label.allocation.height)
                        else:
                                if not data:
                                        if self.workflow_started :
                                                if self.rotated.has_key(self.indice) and self.checkbutton2.get_active():
                                                        pixbuf = self.wand2pixbuf(self.wand, True, True).rotate_simple(self.rotated[self.indice])
                                                else:
                                                        pixbuf = self.wand2pixbuf(self.wand, True)
                                        else:
                                                if self.rotated.has_key(self.indice) and self.checkbutton2.get_active() and \
                                                                                        self.notebook2.get_current_page() == 0:
                                                        pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(self.list[self.indice], rect.height, rect.width)
                                                        pixbuf = pixbuf.rotate_simple(self.rotated[self.indice])
                                                else:
                                                        pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(self.list[self.indice], rect.width, rect.height)
                                else:
                                        if self.rotated.has_key(self.indice) and self.checkbutton2.get_active():
                                                pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(self.list[self.indice],\
                                                rect.height - self.swin2_label.allocation.height, rect.width)
                                                pixbuf = pixbuf.rotate_simple(self.rotated[self.indice])
                                        else:
                                                pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(self.list[self.indice],\
                                                rect.width, rect.height - self.swin2_label.allocation.height)
                        self.pblist[self.pbindex] = self.check_icons(format, pixbuf)   
                        if not data:
                                if self.expander8.get_expanded():
                                        self.draw_grid(self.checkbutton24)
                                elif self.expander19.get_expanded():
                                        self.draw_grid(self.checkbutton34)
                                elif self.expander17.get_expanded():
                                        self.set_cairo_pixbuf()
                                        self.x, self.y = self.x1, self.y1
                                else:
                                        if self.checkbutton41.get_active():
                                                self.image28.set_from_pixbuf(self.profile_pixbuf(self.pblist[self.pbindex]))
                                        else:
                                                self.image28.set_from_pixbuf(self.pblist[self.pbindex])
                        else:
                                self.swin2_image.set_from_pixbuf(self.pblist[self.pbindex])
                        self.calculate_percentage()
                        self.toolbutton6.handler_unblock(self.id35)
                        self.toolbutton7.handler_unblock(self.id36)
                        self.toolbutton8.handler_unblock(self.id37)
                        self.toolbutton9.handler_unblock(self.id38)
                        self.toolbutton16.handler_unblock(self.id1)
                        self.toolbutton17.handler_unblock(self.id2)
                
        def zoom_1_photo(self, event, data):
                if self.n:
                        while gtk.events_pending():
                                gtk.main_iteration(False)
                        self.toolbutton6.handler_block(self.id35)
                        self.toolbutton7.handler_block(self.id36)
                        self.toolbutton8.handler_block(self.id37)
                        self.toolbutton9.handler_block(self.id38)
                        self.toolbutton16.handler_block(self.id1)
                        self.toolbutton17.handler_block(self.id2)
                        self.count = 0
                        format = os.path.splitext(self.list[self.indice])[1]
                        if format.lower() in self.raw_formats:
                                if self.workflow_started :
                                        pixbuf = self.wand2pixbuf(self.wand)
                                else:
                                        if self.raw_preview:
                                                pixbuf = gtk.gdk.pixbuf_new_from_file(os.path.join(self.temp, self.fname))
                                        else:
                                                if self.rotated.has_key(self.indice) and self.checkbutton2.get_active():
                                                        pixbuf = gtk.gdk.pixbuf_new_from_file(self.list[self.indice][:-len(format)]).rotate_simple(self.rotated[self.indice])
                                                else:
                                                        pixbuf = gtk.gdk.pixbuf_new_from_file(self.list[self.indice][:-len(format)])
                        else:   
                                if self.workflow_started :
                                        pixbuf = self.wand2pixbuf(self.wand)
                                else:   
                                        if self.rotated.has_key(self.indice) and self.checkbutton2.get_active() and self.notebook2.get_current_page() == 0:
                                                pixbuf = gtk.gdk.pixbuf_new_from_file(self.list[self.indice]).rotate_simple(self.rotated[self.indice])
                                        else:
                                                pixbuf = gtk.gdk.pixbuf_new_from_file(self.list[self.indice])
                        self.pblist[self.pbindex] = self.check_icons(format, pixbuf)    
                        if not data:
                                if self.expander8.get_expanded(): 
                                        self.draw_grid(self.checkbutton24)
                                elif self.expander19.get_expanded():
                                        self.draw_grid(self.checkbutton34)
                                else:
                                        if self.checkbutton41.get_active():
                                                self.image28.set_from_pixbuf(self.profile_pixbuf(self.pblist[self.pbindex]))
                                        else:
                                                self.image28.set_from_pixbuf(self.pblist[self.pbindex])
                                self.rect.width = self.pblist[self.pbindex].get_width()
                                self.rect.height = self.pblist[self.pbindex].get_height()
                        else:
                                self.swin2_image.set_from_pixbuf(self.pblist[self.pbindex])
                                self.rect2.width = self.pblist[self.pbindex].get_width()
                                self.rect2.height = self.pblist[self.pbindex].get_height()
                        self.calculate_percentage()
                        self.toolbutton6.handler_unblock(self.id35)
                        self.toolbutton7.handler_unblock(self.id36)
                        self.toolbutton8.handler_unblock(self.id37)
                        self.toolbutton9.handler_unblock(self.id38)
                        self.toolbutton16.handler_unblock(self.id1)
                        self.toolbutton17.handler_unblock(self.id2)
                
        def sx_rotation(self, event, data):
                if self.n:
                        while gtk.events_pending():
                                gtk.main_iteration(False)
                        if self.count < 3:
                                self.count += 1
                        else:
                                self.count = 0
                        if not data:
                                rect_width = self.rect.width
                                rect_height = self.rect.height
                        else:
                                rect_width = self.rect2.width
                                rect_height = self.rect2.height
                        format = os.path.splitext(self.list[self.indice])[1]
                        if format.lower() in self.raw_formats:
                                if self.workflow_started :
                                        pixbuf = self.wand2pixbuf(self.wand, True, True)
                                else:
                                        if self.raw_preview:
                                                pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(os.path.join(self.temp, self.fname), rect_width, rect_height)
                                        else:
                                                if self.rotated.has_key(self.indice) and self.checkbutton2.get_active():
                                                        pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(self.list[self.indice][:-len(format)],rect_height,rect_width)
                                                        pixbuf = pixbuf.rotate_simple(self.rotated[self.indice])
                                                else:
                                                        pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(self.list[self.indice][:-len(format)],rect_width,rect_height)                                
                                pixbuf = pixbuf.rotate_simple(self.gradi[self.count])
                                pixbuf.save(os.path.join(self.thumbdir, os.path.split(self.list[self.indice][:-len(format)])[1]), "jpeg")
                                pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(os.path.join(self.thumbdir,\
                                        os.path.split(self.list[self.indice][:-len(format)])[1]), rect_width, rect_height)               
                        else:
                                if self.workflow_started :
                                        if self.rotated.has_key(self.indice) and self.checkbutton2.get_active():
                                                pixbuf = self.wand2pixbuf(self.wand, True, True).rotate_simple(self.rotated[self.indice])
                                        else:
                                                pixbuf = self.wand2pixbuf(self.wand, True)
                                else:
                                        if self.rotated.has_key(self.indice) and self.checkbutton2.get_active():
                                                pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(self.list[self.indice], \
                                                        rect_height, rect_width).rotate_simple(self.rotated[self.indice])
                                        else:
                                                pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(self.list[self.indice], rect_width, rect_height)
                                pixbuf = pixbuf.rotate_simple(self.gradi[self.count])
                                if pixbuf.get_has_alpha():
                                        pixbuf.save(os.path.join(self.thumbdir, os.path.split(self.list[self.indice])[1][:-len(format)] + '.png'), "png")
                                        pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(os.path.join(self.thumbdir,\
                                                os.path.split(self.list[self.indice])[1][:-len(format)] + '.png'), rect_width, rect_height)
                                else:
                                        pixbuf.save(os.path.join(self.thumbdir, os.path.split(self.list[self.indice])[1][:-len(format)]+ '.jpg'), "jpeg")
                                        pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(os.path.join(self.thumbdir,\
                                                os.path.split(self.list[self.indice])[1][:-len(format)] + '.jpg'), rect_width, rect_height)
                        if self.checkbutton41.get_active() and not data:
                                pixbuf = self.profile_pixbuf(pixbuf)
                        pixbuf = self.check_icons(format, pixbuf) 
                        if not data:
                                self.image28.set_from_pixbuf(pixbuf)
                        else:
                                self.swin2_image.set_from_pixbuf(pixbuf)
          
        def scroll_fullphoto(self, obj, event):
                if event.direction == gtk.gdk.SCROLL_UP:
                        self.previous_fullphoto()
                elif event.direction == gtk.gdk.SCROLL_DOWN:
                        self.next_fullphoto()
                
        def dx_rotation(self, event, data):
                if self.n:
                        while gtk.events_pending():
                                gtk.main_iteration(False)
                        if self.count > 0:
                                self.count -= 1
                        else:
                                self.count = 3
                        if not data:
                                rect_width = self.rect.width
                                rect_height = self.rect.height
                        else:
                                rect_width = self.rect2.width
                                rect_height = self.rect2.height
                        format = os.path.splitext(self.list[self.indice])[1]
                        if format.lower() in self.raw_formats:
                                if self.workflow_started :
                                        pixbuf = self.wand2pixbuf(self.wand, True, True)
                                else:
                                        if self.raw_preview:
                                                pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(os.path.join(self.temp, self.fname), rect_width, rect_height)
                                        else:
                                                if self.rotated.has_key(self.indice) and self.checkbutton2.get_active():
                                                        pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(self.list[self.indice][:-len(format)], \
                                                        rect_height,rect_width).rotate_simple(self.rotated[self.indice])
                                                else:
                                                        pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(self.list[self.indice][:-len(format)], rect_width,rect_height)
                                pixbuf = pixbuf.rotate_simple(self.gradi[self.count])
                                pixbuf.save(os.path.join(self.thumbdir, os.path.split(self.list[self.indice][:-len(format)])[1]), "jpeg")
                                pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(os.path.join(self.thumbdir,\
                                        os.path.split(self.list[self.indice][:-len(format)])[1]), rect_width, rect_height)           
                        else:
                                if self.workflow_started :
                                        if self.rotated.has_key(self.indice) and self.checkbutton2.get_active():
                                                pixbuf = self.wand2pixbuf(self.wand, True, True).rotate_simple(self.rotated[self.indice])
                                        else:
                                                pixbuf = self.wand2pixbuf(self.wand, True)
                                else:
                                        if self.rotated.has_key(self.indice) and self.checkbutton2.get_active():
                                                pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(self.list[self.indice], \
                                                rect_height, rect_width).rotate_simple(self.rotated[self.indice])
                                        else:
                                                pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(self.list[self.indice], rect_width, rect_height)
                                pixbuf = pixbuf.rotate_simple(self.gradi[self.count])
                                if pixbuf.get_has_alpha():
                                        pixbuf.save(os.path.join(self.thumbdir, os.path.split(self.list[self.indice])[1][:-len(format)] + '.png'), "png")
                                        pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(os.path.join(self.thumbdir,\
                                                os.path.split(self.list[self.indice])[1][:-len(format)] + '.png'), rect_width, rect_height)
                                else:
                                        pixbuf.save(os.path.join(self.thumbdir, os.path.split(self.list[self.indice])[1][:-len(format)] + '.jpg'), "jpeg")
                                        pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(os.path.join(self.thumbdir,\
                                                os.path.split(self.list[self.indice])[1][:-len(format)] + '.jpg'), rect_width, rect_height)
                        if self.checkbutton41.get_active() and not data:
                                pixbuf = self.profile_pixbuf(pixbuf)
                        pixbuf = self.check_icons(format,pixbuf) 
                        if not data:
                                self.image28.set_from_pixbuf(pixbuf)
                        else:
                                self.swin2_image.set_from_pixbuf(pixbuf)

        def open_with_gimp_(self, obj):               # Opens image with Gimp
                if self.radiobutton6.get_active() and not self.window2.get_property('visible'):
                        if self.obj in self.event:
                                self.indice = self.event.index(self.obj)
                        elif  self.obj in self.e:
                                self.indice = self.e.index(self.obj)
                filename = self.get_real_path(self.list[self.indice])
                cmd = 'gimp "%s" ' % filename
                p = Popen(cmd, shell=True, stdin=PIPE, stdout=PIPE, stderr=STDOUT)
                pid = p.pid
                p.stdin.close()
                p.stdout.close()
       
        def open_with_other_(self, obj):                 # Opens image with other software
                self.entry3.grab_focus()              
                self.dialog13.show()
                response = self.dialog13.run()
                if response == gtk.RESPONSE_CANCEL or response == gtk.RESPONSE_DELETE_EVENT:
                        self.dialog13.hide()
                elif response == gtk.RESPONSE_OK:
                        self.dialog13.hide()
                        while gtk.events_pending():
                                gtk.main_iteration(False)
                        if self.radiobutton6.get_active() and not self.window2.get_property('visible'):
                                if self.obj in self.event:
                                        self.indice = self.event.index(self.obj)
                                elif  self.obj in self.e:
                                        self.indice = self.e.index(self.obj)
                        filename = self.get_real_path(self.list[self.indice])
                        command = self.entry3.get_text()
                        cmd = ' "%s" "%s" ' % (command, filename)
                        p = Popen(cmd, shell=True, stdin=PIPE, stdout=PIPE, stderr=STDOUT)
                        pid = p.pid
                        p.stdin.close()
                        p.stdout.close()

        def load_cache_pixbuf(self, filename, w, h, rotate=None):
                try:
                        pix = gtk.gdk.pixbuf_new_from_file_at_size(filename, w, h)
                        if rotate:
                                pix.rotate_simple(rotate)
                except:
                        pix = gtk.gdk.pixbuf_new_from_file(self.get_icon_path('gtk-missing-image.png'))
                return pix

        def load_cache(self, width, height, data):         # Preloads previous and next image
                with self.lock:
                        if self.indice > 0:
                                format = os.path.splitext(self.list[self.indice - 1])[1]
                                if format.lower() in self.raw_formats:
                                        filename = self.list[self.indice - 1][:-len(format)]
                                else:
                                        filename = self.list[self.indice - 1]
                                if data == 'fullscreen':
                                        if self.rotated.has_key(self.indice - 1) and self.checkbutton2.get_active():
                                                self.fullpblist[0] = self.load_cache_pixbuf(filename, height, width, self.rotated[self.indice - 1])
                                        else:
                                                self.fullpblist[0] = self.load_cache_pixbuf(filename, width, height) 
                                else:
                                        if self.rotated.has_key(self.indice - 1) and self.checkbutton2.get_active():
                                                self.pblist[0] = self.load_cache_pixbuf(filename, height, width, self.rotated[self.indice - 1])
                                        else:
                                                self.pblist[0] = self.load_cache_pixbuf(filename, width, height)
                        if self.indice < self.n - 1:
                                format = os.path.splitext(self.list[self.indice + 1])[1]
                                if format.lower() in self.raw_formats:
                                        filename = self.list[self.indice + 1][:-len(format)]
                                else:
                                        filename = self.list[self.indice + 1]
                                if data == 'fullscreen':
                                        if self.rotated.has_key(self.indice + 1) and self.checkbutton2.get_active():
                                                self.fullpblist[2] = self.load_cache_pixbuf(filename, height, width, self.rotated[self.indice + 1])
                                        else:
                                                self.fullpblist[2] = self.load_cache_pixbuf(filename, width, height)
                                else:
                                        if self.rotated.has_key(self.indice + 1) and self.checkbutton2.get_active():
                                                self.pblist[2] = self.load_cache_pixbuf(filename, height, width, self.rotated[self.indice + 1])
                                        else:
                                                self.pblist[2] = self.load_cache_pixbuf(filename, width, height)


        def load_b_cache(self, width, height):
                with self.lock2:
                        if self.indice > 0:
                                format = os.path.splitext(self.list[self.indice - 1])[1]
                                if format.lower() in self.raw_formats:
                                        filename = self.list[self.indice - 1][:-len(format)]
                                else:
                                        filename = self.list[self.indice - 1]
                                if self.rotated.has_key(self.indice - 1) and self.checkbutton2.get_active():
                                        self.b_pixlist[0] = self.load_cache_pixbuf(filename, \
                                        height - self.swin2_label.allocation.height, width, self.rotated[self.indice - 1])
                                else:
                                        self.b_pixlist[0] = self.load_cache_pixbuf(filename, width, height - self.swin2_label.allocation.height)    
                        if self.indice < self.n - 1:
                                format = os.path.splitext(self.list[self.indice + 1])[1]
                                if format.lower() in self.raw_formats:
                                        filename = self.list[self.indice + 1][:-len(format)]
                                else:
                                        filename = self.list[self.indice + 1]
                                if self.rotated.has_key(self.indice + 1) and self.checkbutton2.get_active():
                                        self.b_pixlist[2] = self.load_cache_pixbuf(filename, \
                                        height - self.swin2_label.allocation.height, width, self.rotated[self.indice + 1])      
                                else:
                                        self.b_pixlist[2] = self.load_cache_pixbuf(filename, width, height - self.swin2_label.allocation.height)    
                                                
        def display_photo(self, obj=None):             # Opens image on details window
                self.hpaned2.handler_block(self.id22)
                self.pbindex = 1
                if self.obj in self.event:
                        self.indice = self.event.index(self.obj)
                elif  self.obj in self.e:
                        self.indice = self.e.index(self.obj)
                indice = self.indice
                self.old_indice = indice
                self.window2.maximize()
                self.window2.set_sensitive(True)
                self.window2.show_all()
                while gtk.events_pending():
                        gtk.main_iteration(False)
                self.rect = self.viewport1.allocation
                format_ = os.path.splitext(self.list[self.indice])[1]
                self.set_image1(format_)
                self.add_dimensions(format_, self._label)
                self.hpaned2.handler_unblock(self.id22)
                threading.Thread(target=self.load_cache, args=(self.rect.width, self.rect.height, None)).start()
                self.unselect()

        def eventbox1_handler(self, obj, event):
                if self.radiobutton7.get_active() and self.indice not in self.blocked_list:
                        if event.button == 3 and self.n:
                                if self.version + 'Albums/' not in self.window1.get_title():
                                        self.menuitem18.set_sensitive(True)
                                        self.menuitem19.set_sensitive(True)
                                        self.imagemenuitem20.set_sensitive(False)
                                        self.menuitem21.set_sensitive(True)
                                else:
                                        self.imagemenuitem20.set_sensitive(True)
                                        self.menuitem19.set_sensitive(False)
##                                if os.path.splitext(self.list[self.indice])[1].lower() in self.raw_formats:
##                                        self.menuitem31.set_sensitive(False)
##                                else:
##                                        self.menuitem31.set_sensitive(True)
                                self.menu12.popup(None, None, None, event.button, event.time)
                        if event.type == 5 and self.n:
                                self.open_on_window2()
                        if event.type == gtk.gdk.BUTTON_PRESS:
                                self.scrolledwindow2.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.FLEUR))
                        
        def on_scrolledwindow_release_event(self, obj, event):
                if self.expander23.get_expanded() and event.type == gtk.gdk.BUTTON_PRESS and event.button == 3:
                        self.remove_red_eye(event.window.get_pointer()[:2])
                elif self.expander23.get_expanded() and event.type == gtk.gdk.BUTTON_RELEASE and event.button == 1:
                        self.scrolledwindow3.window.set_cursor(self.make_cursor())
                elif self.expander36.get_expanded() and event.type == gtk.gdk.BUTTON_PRESS and event.button == 1:
                        if self.togglebutton3.get_active():
                                if self.color_picker_cursor:
                                        self.apply_black_balance(event.window.get_pointer()[:2])
                        else:
                                if self.color_picker_cursor:
                                        self.apply_white_balance(event.window.get_pointer()[:2])
                else:
                        self.hadj, self.vadj = obj.get_parent().get_hadjustment(), obj.get_parent().get_vadjustment()
                        self.w = self.hadj.upper - self.hadj.page_size
                        self.h = self.vadj.upper - self.vadj.page_size
                        self.hadj_lower = self.hadj.lower
                        self.vadj_lower = self.vadj.lower
                        if event.type == gtk.gdk.BUTTON_PRESS:
                                if event.button == 1:
                                        self.x, self.y = event.x, event.y
                                        obj.get_parent().window.set_cursor(gtk.gdk.Cursor(gtk.gdk.FLEUR))              
                                elif event.button == 3 and self.workflow_started:
                                        self.display_unchanged_image()
                        if event.type == gtk.gdk.BUTTON_RELEASE:
                                if event.button == 1:
                                        obj.get_parent().window.set_cursor(None)
                                elif event.button == 3 and self.workflow_started:
                                        if self.checkbutton41.get_active():
                                                self.image28.set_from_pixbuf(self.profile_pixbuf(self.pblist[self.pbindex]))
                                        else:
                                                self.image28.set_from_pixbuf(self.pblist[self.pbindex])

        def drag_image(self, obj, event, sw):
                hadj, vadj = sw.get_hadjustment(), sw.get_vadjustment()
                w = hadj.upper - hadj.page_size
                h = vadj.upper - vadj.page_size
                hadj_value = hadj.value + self.x - event.window.get_pointer()[0]
                vadj_value = vadj.value + self.y - event.window.get_pointer()[1]
                if hadj.lower < hadj_value < w:
                        hadj.value = hadj_value
                        self.x = event.window.get_pointer()[0]
                elif hadj_value >= w:
                        hadj.value = w
                if vadj.lower < vadj_value < h:
                        vadj.value = vadj_value
                        self.y = event.window.get_pointer()[1]
                elif vadj_value >= h:
                        vadj.value = h

        def display_unchanged_image(self):
                if self.image_was_changed:
                        self.window2.window.set_cursor(self.watch)
                        while gtk.events_pending():
                                gtk.main_iteration(False)
                        unchanged_pixbuf = self.wand2pixbuf(MagickImage(self.wand_list[self.wand_index].get_image()), True)
                        if self.checkbutton41.get_active():
                                self.image28.set_from_pixbuf(self.profile_pixbuf(unchanged_pixbuf))
                        else:
                                self.image28.set_from_pixbuf(unchanged_pixbuf)
                        if self.expander36.get_expanded():
                                self.make_color_picker_cursor()
                        else:
                                self.window2.window.set_cursor(None)
                        while gtk.events_pending():
                                gtk.main_iteration(False)
        
        def scroll_eventbox1(self, obj, event):
                if self.radiobutton7.get_active():
                        if event.direction == gtk.gdk.SCROLL_DOWN:
                                self.next_b_photo()
                        elif event.direction == gtk.gdk.SCROLL_UP:
                                self.previous_b_photo()
                        
        def scroll_to_selection(self, event=None):
                adj = self.scrolledwindow6.get_hadjustment()
                x, y = self.b_event[self.indice].window.get_position()
                if x > adj.value + adj.page_size:
                        adj.value = min(x, adj.upper - adj.page_size)
                elif x < adj.value:
                        adj.value = max(adj.lower, x + self.b_event[self.indice].allocation.width - adj.page_size)

        def scroll_scrolledwindow3(self, obj, event):
                if event.direction == gtk.gdk.SCROLL_DOWN:
                        if self.expander23.get_expanded():
                                self.radius += 2
                                self.radius = min(self.radius, 50)
                                self.scrolledwindow3.window.set_cursor(self.make_cursor())
                                return True
                        else:
                                self.next_photo()
                elif event.direction == gtk.gdk.SCROLL_UP:
                        if self.expander23.get_expanded():
                                self.radius -= 2
                                self.radius = max(self.radius, 5)
                                self.scrolledwindow3.window.set_cursor(self.make_cursor())
                                return True
                        else:
                                self.previous_photo()

        def open_on_window2(self, obj=None):
                self.hpaned2.handler_block(self.id22)
                indice = self.indice
                self.old_indice = indice
                self.pbindex = 1
                self.window2.maximize()
                self.window2.set_sensitive(True)
                self.window2.show()
                while gtk.events_pending():
                        gtk.main_iteration(False)
                self.rect = self.viewport1.allocation
                format = os.path.splitext(self.list[self.indice])[1]    
                self.set_image1(format)
                self.add_dimensions(format, self._label)
                threading.Thread(target=self.load_cache, args=(self.rect.width, self.rect.height, None)).start()
                self.hpaned2.handler_unblock(self.id22)
                        
        def full_exif(self, event=None):              # Populates exif panel with full exif list
                if self.hide_exif_panel or self.indice in self.blocked_list:
                        return True
                self.exifmodel.clear()
                image = self.get_real_path(self.list[self.indice])
                list_ = [exiftool, '-j', '-n', image]
                p = Popen(list_, stdin=PIPE, stdout=PIPE, stderr=PIPE)
                output = p.communicate()[0]
                p.stdout.close()
                p.stdin.close()
                p.stderr.close()
                try:
                        dct = json.loads(output)[0]
                        error = False
                except:
                        dct = json.loads(utf8conv(output))[0]
                        error = True
                if dct.has_key('ExposureTime'):
                        if dct['ExposureTime'] < 1:
                                dct['ExposureTime'] = '1/' + str(int(1.0 / dct['ExposureTime']))
                if dct.has_key('SourceFile'):
                        del dct['SourceFile']
                dct = self.format_orientation_value(dct)
                for key, value in sorted(dct.iteritems()):
                        if error and isinstance(value, unicode):
                                value = value.encode(encoding)
                        if isinstance(value, list):
                                value = ', '.join(value)
                        self.exifmodel.append([key, value])

        def display_exif(self, image, label, page=None):
                if self.hide_exif_panel:
                        return True
                if page == None:
                        page = self.notebook5.get_current_page()
                if page == 0:
                        if self.toolbutton24.get_active():
                                self.full_exif()
                                return True
                        self.exifmodel.clear()  
                        self.exifmodel.append(['Filename  ', label])
                        cmd = [exiftool, '-j', '-n'] + self.favorite_tags + [image]
                        p = Popen(cmd, stdin=PIPE, stdout=PIPE, stderr=PIPE)
                        output = p.communicate()[0]
                        p.stdout.close()
                        p.stdin.close()
                        p.stderr.close()
                        try:
                                dct = json.loads(output)[0]
                                error = False
                        except:
                                dct = json.loads(utf8conv(output))[0]
                                error = True
                        if dct.has_key('ExposureTime'):
                                if dct['ExposureTime'] < 1:
                                        dct['ExposureTime'] = '1/' + str(int(1.0 / dct['ExposureTime']))
                        if dct.has_key('SourceFile'):
                                del dct['SourceFile']
                        dct = self.format_orientation_value(dct)
                        for key, value in sorted(dct.iteritems()):
                                if error and isinstance(value, unicode):
                                        value = value.encode(encoding)
                                if isinstance(value, list):
                                        value = ', '.join(value)
                                self.exifmodel.append([key, value])
                elif page == 1:
                        cmd = [exiftool, '-j', '-n', '-exif:all', image]
                        self.get_metadata_info(cmd, self.exif_tab_model)
                elif page == 2:
                        cmd = [exiftool, '-j', '-n', '-iptc:all', image]
                        self.get_metadata_info(cmd, self.iptc_tab_model)
                elif page == 3:
                        cmd = [exiftool, '-j', '-n', '-xmp:all', image]
                        self.get_metadata_info(cmd, self.xmp_tab_model)
                elif page == 4:
                        if os.path.splitext(image)[1].lower() == '.x3f':
                                cmd = [exiftool, '-j', '-n', '-SigmaRaw:all', image]
                        else:        
                                cmd = [exiftool, '-j', '-n', '-makernotes:all', image]
                        self.get_metadata_info(cmd, self.makernotes_tab_model)

        def format_orientation_value(self, dct):
                if dct.has_key('Orientation'):
                        orientation = dct['Orientation']
                        if orientation in [8, 270]:
                                dct['Orientation'] = 'Rotated 90 CW'
                        elif orientation in [6, 90]:
                                dct['Orientation'] = 'Rotated 90 CCW'
                        else:
                                dct['Orientation'] = 'Horizontal'
                elif dct.has_key('Rotation'):
                        orientation = dct['Rotation']
                        if orientation in [8, 270]:
                                dct['Rotation'] = 'Rotated 90 CW'
                        elif orientation in [6, 90]:
                                dct['Rotation'] = 'Rotated 90 CCW'
                        else:
                                dct['Rotation'] = 'Horizontal'
                return dct

        def get_metadata_info(self, cmd, model):
                model.clear()
                p = Popen(cmd, stdin=PIPE, stdout=PIPE, stderr=PIPE)
                output = p.communicate()[0]
                p.stdout.close()
                p.stdin.close()
                p.stderr.close()
                self.stop_metadata_toggle = True
                try:
                        dct = json.loads(output)[0]
                        error = False
                except:
                        dct = json.loads(utf8conv(output))[0]
                        error = True
                if dct.has_key('SourceFile'):
                        del dct['SourceFile']
                if model == self.exif_tab_model and dct.has_key('ExposureTime'):
                        if dct['ExposureTime'] < 1:
                                dct['ExposureTime'] = '1/' + str(int(1.0 / dct['ExposureTime']))
                if model == self.exif_tab_model:
                        dct = self.format_orientation_value(dct)
                for key, value in sorted(dct.iteritems()):
                        if error and isinstance(value, unicode):
                                value = value.encode(encoding)
                        if isinstance(value, list):
                                value = ', '.join(value)
                        model.append(['-' + key in self.favorite_tags, key, value])
                self.stop_metadata_toggle = False

        def on_metadata_tab_toggled(self, cell, path, model):
                if not self.stop_metadata_toggle:
                        model[path][0] = not model[path][0]
                        if model[path][0]:
                                self.exifmodel.append([model[path][1], model[path][2]])
                                self.favorite_tags.append('-' + model[path][1])
                        else:
                                self.favorite_tags.remove('-' + model[path][1])
                                for row in self.exifmodel:
                                        if row[0] == model[path][1]:
                                                self.exifmodel.remove(row.iter)
                                                break
                        self.settings['favorite_tags'] = self.favorite_tags
                        self.store_settings_db()

        def on_notebook5_switched(self, notebook, page, page_num):
                self.window2.window.set_cursor(self.watch)
                while gtk.events_pending():
                        gtk.main_iteration(False)
                format = os.path.splitext(self.list[self.indice])[1]
                if format.lower() in self.raw_formats and self.ark not in self.list[self.indice]:
                        image = self.rawpath[self.list[self.indice]]
                elif format.lower() in self.raw_formats and self.ark in self.list[self.indice]:
                        image = self.list[self.indice].replace(homepath + 'thumb', '').replace('.thumb.jpg', '')   
                else:
                        image = self.list[self.indice]
                self.display_exif(image, self._label, page_num)
                self.window2.window.set_cursor(None)
                        
        def grep_image_size(self, lista):
                self.image_size = '0x0'
                width = height = 0
                item = lista[0]
                if item.has_key('ExifImageWidth'):
                        width = item['ExifImageWidth']
                elif item.has_key('ImageWidth'):
                        width = item['ImageWidth']
                if item.has_key('ExifImageLength'):
                        height = item['ExifImageLength']
                elif item.has_key('ExifImageHeight'):
                        height = item['ExifImageHeight']
                elif item.has_key('ImageHeight'):
                        height = item['ImageHeight']
                if isinstance(width, (str,unicode)) and width.endswith('pt'):
                        width = width[:-2].split('.')[0]
                if isinstance(height, (str,unicode)) and height.endswith('pt'):
                        height = height[:-2].split('.')[0]
                if width and height:
                        if self.rotated.has_key(self.indice) and self.list[self.indice].lower().endswith(self.raw_formats):
                                self.image_size = str(height) + 'x' + str(width)
                        else:
                                self.image_size = str(width) + 'x' + str(height)
                
        def hide_exif(self, obj):
                if not self.hide_exif_panel:
                        self.notebook2.handler_block(self.id107)
                        self.frame1.hide_all()
                        self.notebook2.handler_unblock(self.id107)
                        self.settings['hide_exif'] = True
                        self.hide_exif_panel = True
                        self.toolbutton25.set_icon_widget(gtk.image_new_from_stock(gtk.STOCK_UNINDENT, gtk.ICON_SIZE_BUTTON))
                        self.toolbutton25.show_all()
                        self.toolbutton25.set_tooltip_text('Show Sidebar')
                        self.exifmodel.clear()
                        while gtk.events_pending():
                                gtk.main_iteration(False)
                        self.zoom_fit_photo(None, 0)
                else:
                        self.settings['hide_exif'] = False
                        self.hide_exif_panel = False
                        if self.toolbutton24.get_active():
                                self.full_exif()
                        else:
                                self.add_dimensions(os.path.splitext(self.list[self.indice])[1], self._label)
                        self.frame1.show_all()
                        self.toolbutton25.set_icon_widget(gtk.image_new_from_stock(gtk.STOCK_INDENT, gtk.ICON_SIZE_BUTTON))
                        self.toolbutton25.show_all()
                        self.toolbutton25.set_tooltip_text('Hide Sidebar')
                        while gtk.events_pending():
                                gtk.main_iteration(False)
                        self.zoom_fit_photo(None, 0)
                        self.on_notebook2_switched(None)
                self.store_settings_db()

        def select_b_photo(self, obj, event):      # Displays the selected image on the single image layout
                self.pixindex = 1
                self.vpaned2.handler_block(self.id27)
                self.indice = self.b_event.index(obj)
                for item in self.b_event:
                        item.modify_bg(gtk.STATE_NORMAL, self.gallery_color)
                self.b_event[self.indice].modify_bg(gtk.STATE_NORMAL, self.selection_color)
                while gtk.events_pending():
                        gtk.main_iteration(False)
                if event.button == 1 or event.button == 3 :
                        self.rect2 = self.scrolledwindow2.allocation
                        format = os.path.splitext(self.list[self.indice])[1]
                        if format.lower() in self.raw_formats:
                                if self.rotated.has_key(self.indice) and self.checkbutton2.get_active():
                                        self.b_pixlist[self.pixindex] = gtk.gdk.pixbuf_new_from_file_at_size(self.list[self.indice]\
                                        [:-len(format)], self.rect2.height - self.swin2_label.allocation.height, self.rect2.width)
                                        self.b_pixlist[self.pixindex] = self.b_pixlist[self.pixindex].rotate_simple(self.rotated[self.indice])
                                else:
                                        self.b_pixlist[self.pixindex] = gtk.gdk.pixbuf_new_from_file_at_size(self.list[self.indice]\
                                        [:-len(format)], self.rect2.width, self.rect2.height - self.swin2_label.allocation.height)
                        else:
                                if self.rotated.has_key(self.indice) and self.checkbutton2.get_active():
                                        self.b_pixlist[self.pixindex] = gtk.gdk.pixbuf_new_from_file_at_size(self.list[self.indice],\
                                        self.rect2.height - self.swin2_label.allocation.height, self.rect2.width)
                                else:
                                        self.b_pixlist[self.pixindex] = gtk.gdk.pixbuf_new_from_file_at_size(self.list[self.indice],\
                                        self.rect2.width, self.rect2.height - self.swin2_label.allocation.height)
                        self.b_pixlist[self.pixindex] = self.check_icons(format,self.b_pixlist[self.pixindex])
                        self.swin2_image.set_from_pixbuf(self.b_pixlist[self.pixindex])
                        self.swin2_label.set_text(self.label[self.indice].get_text())
                        self.scrolledwindow2.show_all()
                        while gtk.events_pending():
                                gtk.main_iteration(False)
                        self.calculate_percentage()
                self.vpaned2.handler_unblock(self.id27)
                threading.Thread(target=self.load_b_cache, args=(self.rect2.width, self.rect2.height)).start()
                
        def check_icons(self, format, pixbuf):
                if pixbuf.get_has_alpha():
                        w = pixbuf.get_width()
                        h = pixbuf.get_height()
                        grey = self.grey.copy()
                        grey = grey.scale_simple(w, h, gtk.gdk.INTERP_BILINEAR)
                        pixbuf.composite(grey, 0, 0, w, h, 0, 0, 1.0, 1.0, gtk.gdk.INTERP_BILINEAR, 255)
                        return grey
                return pixbuf
                
        def select_photo(self, obj, event):        # Selects clicked thumbnails on layout1 & 2
                if obj in self.event:
                        indice = self.event.index(obj)
                elif obj in self.e:
                        indice = self.e.index(obj)
                if event.button == 1 and event.type != 5:
                        self.obj = obj
                        filename = self.list[indice]   
                        if filename not in self.selected:
                                self.selected.append(filename)
                                self.e[indice].modify_bg(gtk.STATE_NORMAL, self.selection_color)
                                while gtk.events_pending():
                                        gtk.main_iteration(False)
                        else:
                                del self.selected[self.selected.index(filename)]
                                self.e[indice].modify_bg(gtk.STATE_NORMAL, self.colorbutton1.get_color())
                                while gtk.events_pending():
                                        gtk.main_iteration(False)
                elif event.button == 3:
                        self.obj = obj
                        if self.list[indice] not in self.selected:     
                                self.e[indice].modify_bg(gtk.STATE_NORMAL, self.selection_color)
                                while gtk.events_pending():
                                        gtk.main_iteration(False)
                                self.selected.append(self.list[indice])       
                        if self.selected:       
                                if self.version + 'Albums/' not in self.window1.get_title():
                                        self.imagemenuitem19.set_sensitive(False)
                                        self.menuitem15.set_sensitive(True)
                                else:
                                        self.imagemenuitem19.set_sensitive(True)
                                        self.menuitem15.set_sensitive(False)
                                self.menu9.popup(None, None, None, event.button, event.time)            
                elif event.type == 5 :
                        self.obj = obj
                        self.display_photo()
                
        def selection_control(self):              #Checks for unknown files on selected images
                if self.radiobutton7.get_active():
                        self.selected = []
                        self.selected.append(self.list[self.indice])
                        for item in self.selected:
                                if os.path.isfile(self.get_real_path(item)):
                                        continue
                                else:
                                        text = _('Attention! Unknown file: %s\n The gallery will be updated!') % fname
                                        dialog6 = self.messagedialog(_("Warning!"), text, None, gtk.BUTTONS_OK, self.window1, gtk.DIALOG_MODAL, gtk.MESSAGE_WARNING, None, (400,200))
                                        dialog6.set_keep_above(True)
                                        gtk.gdk.beep()
                                        response = dialog6.run()
                                        if response == gtk.RESPONSE_OK or response == gtk.RESPONSE_DELETE_EVENT:
                                                dialog6.destroy()
                                                if self.version + 'Albums/' not in self.window1.get_title() :
                                                        self.scrolledwindow2.get_child().destroy()
                                                        if self.radiobutton7.get_active():
                                                                try:
                                                                        self.hbox1.destroy()
                                                                except:
                                                                        pass
                                                        self.n = self.count_images()
                                                        if self.n:
                                                                self.make_table()
                                                                self.attach_images()
                                                                self.statusbar_pop(self.statusbar3, self.context_id3)
                                                                self.statusbar_push(self.statusbar3, self.context_id3,\
                                                                _('There are %s images     Total size %f MB') %(self.n, self.total_size / 1000000.0))
                                                                if self.radiobutton7.get_active():
                                                                        self.make_bottom_panel(None, self.n)
                                                                else:
                                                                        self.make_thumbnails(self.width, self.height, self.n)

                                                        else:
                                                                self.statusbar_pop(self.statusbar3, self.context_id3)
                                                                self.statusbar_push(self.statusbar3, self.context_id3,\
                                                                _('There are %s images') % self.n)
                                                        self.selected = []
                                                        return False
                                                else:
                                                        del self.albums[self.album]
                                                        lista = []
                                                        for item in os.listdir(os.path.join(self.ark, self.album)):
                                                                format = os.path.splitext(item)[1]
                                                                if format.lower() in self.raw_formats:
                                                                        fname = os.path.join(self.ark, self.album, 'thumb', item[:-len(format)] +\
                                                                        '.thumb.jpg' + format)
                                                                        lista += [fname]
                                                                elif format.lower() in self.formats:
                                                                        fname = os.path.join(self.ark, self.album, item)
                                                                        lista += [fname]
                                                        self.albums[self.album] = lista
                                                        self.make_album_gallery()
                                                        self.selected = []
                                                        self.store_albums_db()
                                                        return False
                return True

        def pre_add_selection(self, event=None):
                if self.selected or self.radiobutton7.get_active():
                        self.add_selection()
                else:
                        dialog9 = self.messagedialog(_("Info!"), _("No file selected!"), None, gtk.BUTTONS_CLOSE, self.window1, gtk.DIALOG_MODAL, gtk.MESSAGE_INFO, None, (230,120))
                        dialog9.set_keep_above(True)
                        gtk.gdk.beep()
                        response = dialog9.run()
                        if response == gtk.RESPONSE_CLOSE or response == gtk.RESPONSE_DELETE_EVENT:
                                dialog9.destroy()
                                                
        def add_selection(self, event=None):                 # Adds selected images to album
                self.progressbar2.set_text(_('Saving to album...'))
                if self.selection_control():
                        images = 0
                        selected = self.selected
                        self.dialog4.show()
                        response = self.dialog4.run()
                        if response == gtk.RESPONSE_CANCEL or response == gtk.RESPONSE_DELETE_EVENT:
                                self.dialog4.hide()
                                return True
                        elif response == gtk.RESPONSE_APPLY:      
                                if not self.add_album(self, 'button'):
                                        return True
                                self.window1.window.set_cursor(self.watch)
                                self.dialog4.hide()
                                if self.radiobutton6.get_active():
                                        for item in self.e:
                                                item.modify_bg(gtk.STATE_NORMAL, self.gallery_color)
                                fraction = 0.99 / len(selected)
                                self.window5.show()
                                while gtk.events_pending():
                                        gtk.main_iteration(False)
                                for item in self.selected:
                                        format = os.path.splitext(item)[1]
                                        source = self.get_real_path(item)
                                        dest = os.path.join(self.ark, self.added_album)
                                        if self.temp in item:
                                                fname = item.replace(self.temp, os.path.join(dest, 'thumb'))
                                                raw = True
                                        elif self.ark in item:
                                                fname = os.path.join(dest, 'thumb', os.path.split(item)[1])
                                                raw = True
                                        else:
                                                fname = os.path.join(dest, os.path.split(item)[1])
                                                raw = False
                                        if fname in self.albums[self.added_album]:
                                                continue
                                        else:
                                                shutil.copy2(source, dest)
                                                if raw:
                                                        shutil.copy(item[:-len(format)], os.path.join(dest, 'thumb'))
                                        self.albums[self.added_album] += [fname]
                                        self.progressbar2.set_fraction(self.progressbar2.get_fraction() + fraction)
                                        while gtk.events_pending():
                                                gtk.main_iteration(False)
                                self.window5.hide()
                                self.selected = []
                                self.progressbar2.set_fraction(0)
                                album = self.added_album[:]
                                self.dialog4.hide()
                                self.window1.window.set_cursor(None)
                                self.store_albums_db()
                        elif response == gtk.RESPONSE_OK:
                                album = self.combobox29.get_active_text()
                                if album not in self.albums:
                                        self.dialog4.hide()
                                        return True
                                else:
                                        self.album = album[:]   
                                if self.version + 'Albums/' + self.album in self.window1.get_title():
                                        self.dialog4.hide()
                                        return True
                                self.window1.window.set_cursor(self.watch)
                                self.dialog4.hide()
                                images = len(self.albums[self.album])
                                if self.radiobutton6.get_active():
                                        for item in self.e:
                                                item.modify_bg(gtk.STATE_NORMAL, self.gallery_color)
                                fraction = 0.99 / len(selected)
                                self.window5.show()
                                for item in self.selected:
                                        format = os.path.splitext(item)[1]
                                        source = self.get_real_path(item)
                                        dest = os.path.join(self.ark, self.album)
                                        if self.temp in item:
                                                fname = item.replace(self.temp, os.path.join(dest, 'thumb'))
                                                raw = True
                                        elif self.ark in item:
                                                fname = os.path.join(dest, 'thumb', os.path.split(item)[1])
                                                raw = True
                                        else:
                                                fname = os.path.join(dest, os.path.split(item)[1])
                                                raw = False
                                        if fname in self.albums[self.album]:
                                                continue
                                        else:
                                                shutil.copy2(source, dest)
                                                if raw:
                                                        shutil.copy(item[:-len(format)], os.path.join(dest, 'thumb'))
                                        self.albums[self.album] += [fname]
                                        self.progressbar2.set_fraction(self.progressbar2.get_fraction() + fraction)
                                        while gtk.events_pending():
                                                gtk.main_iteration(False)
                                self.window5.hide()
                                self.selected = []
                                self.progressbar2.set_fraction(0)
                                self.window1.window.set_cursor(None)
                                self.store_albums_db()
                        if not images :
                                self.make_preview(album, self.albums[album][0])

        def copy_and_move(self, obj):            # Copies or Moves selected files
                if self.radiobutton6.get_active():
                        if not self.selected:
                                return True
                if obj in [self.menuitem14, self.menuitem18, self.menuitem26]:
                        command = shutil.copy2
                        title = 'Copying to...'
                        self.filechooserdialog3.set_title(_('Choose a destination directory where to copy'))
                elif obj in [self.menuitem15, self.menuitem19, self.menuitem27]:
                        command = shutil.move
                        title = 'Moving to...'
                        self.filechooserdialog3.set_title(_('Choose a destination directory where to move'))
                if self.selection_control():
                        self.filechooserdialog3.show()
                        response = self.filechooserdialog3.run()
                        if response == gtk.RESPONSE_CANCEL or response == gtk.RESPONSE_DELETE_EVENT:
                                self.filechooserdialog3.hide()
                                while gtk.events_pending():
                                        gtk.main_iteration(False)
                                return True
                        elif response == gtk.RESPONSE_OK:
                                self.filechooserdialog3.hide()
                                while gtk.events_pending():
                                        gtk.main_iteration(False)
                                dest = self.filechooserdialog3.get_current_folder()
                                self.progressbar2.set_text(title)
                                fraction = 0.99 / len(self.selected)
                                self.window5.show()
                                if self.version + 'TAG:' in self.window1.get_title():
                                        tagged = True
                                        tagged_list = self.list[:]
                                else:
                                        tagged = False
                                rotated = {}
                                for key in self.rotated.iterkeys():
                                        rotated[key] = (self.list[key], self.rotated[key])
                                for item in self.selected:
                                        if title == 'Moving to...' and self.ark in item: # moving album items is not permitted!
                                                text29 = _('Moving album images is not permitted!')
                                                gtk.gdk.beep()
                                                dialog29 = self.messagedialog(_("Warning!"), text29, None, gtk.BUTTONS_CLOSE, None,
                                                                              gtk.DIALOG_MODAL, gtk.MESSAGE_WARNING, None, (500,230))
                                                dialog29.set_keep_above(true)
                                                response = dialog29.run()
                                                if response == gtk.RESPONSE_CLOSE or response == gtk.RESPONSE_DELETE_EVENT:
                                                        dialog29.destroy()
                                                self.progressbar2.set_fraction(self.progressbar2.get_fraction() + fraction)
                                                while gtk.events_pending():
                                                        gtk.main_iteration(False)
                                                continue
                                        source = self.get_real_path(item)
                                        if title == 'Moving to...':
                                                self.total_size -= os.path.getsize(source)
                                                self.remove_thumbnails([source])
                                        outfile = os.path.join(dest, os.path.basename(source))
                                        bul = False
                                        if os.path.isfile(outfile):
                                                if not self.overwrite_all:
                                                        if self.overwrite_dialog2(os.path.basename(source)):
                                                                try:
                                                                        command(source, dest)
                                                                        bul = True
                                                                except:
                                                                        self.show_dialog15(_('Operation terminated with errors!\nOpen the Error Log for details.'))
                                                                        break
                                                else:
                                                        try:
                                                                command(source, dest)
                                                                bul = True
                                                        except:
                                                                self.show_dialog15(_('Operation terminated with errors!\nOpen the Error Log for details.'))
                                                                break
                                        else:
                                                try:
                                                        command(source, dest)
                                                        bul = True
                                                except:
                                                        self.show_dialog15(_('Operation terminated with errors!\nOpen the Error Log for details.'))
                                                        break
                                        if bul and title == 'Moving to...' and self.radiobutton6.get_active():
                                                self.image[self.list.index(item)] = 0
                                                self.label[self.list.index(item)] = 0
                                                self.list[self.list.index(item)]  = 0       
                                        if self.temp in item:
                                                keywords = Sqlite.get_keywords(source)
                                        else:
                                                keywords = Sqlite.get_keywords(item)
                                        if keywords:
                                                if title == 'Moving to...':
                                                        Sqlite.update_row((outfile, dest, source))
                                                        if tagged:
                                                                tagged_list.remove(item)
                                        self.progressbar2.set_fraction(self.progressbar2.get_fraction() + fraction)
                                        while gtk.events_pending():
                                                gtk.main_iteration(False)
                                if obj in [self.menuitem14, self.menuitem18, self.menuitem26] and self.radiobutton6.get_active():
                                        for item in self.e:
                                                item.modify_bg(gtk.STATE_NORMAL, self.gallery_color)
                                elif obj in [self.menuitem15, self.menuitem19, self.menuitem27]:        # move to
                                        if self.radiobutton6.get_active():
                                                images = [i.get_pixbuf().copy() for i in self.image[:self.n] if i != 0]
                                                labels = [i.get_text() for i in self.label[:self.n] if i != 0]
                                                for key in self.rotated.keys():
                                                        if self.list[key] == 0:
                                                                del rotated[key]
                                                self.list = [i for i in self.list if i != 0]
                                                self.rotated.clear()
                                                for value in rotated.itervalues():
                                                        self.rotated[self.list.index(value[0])] = value[1]  
                                        self.scrolledwindow2.get_child().destroy()
                                        if self.radiobutton7.get_active():
                                                try:
                                                        self.hbox1.destroy()
                                                except:
                                                        pass
                                        if tagged:
                                                self.list = tagged_list[:]
                                                self.n = len(tagged_list)
                                        else:
                                                if self.radiobutton7.get_active():
                                                        self.n = self.count_images()
                                                else:
                                                        self.n = len(self.list)
                                        if self.n:
                                                if self.radiobutton7.get_active():
                                                        self.make_bottom_panel(None, self.n)
                                                else:
                                                        self.make_table()
                                                        self.label = [gtk.Label() for i in xrange(self.n)]
                                                        for text, label in zip(labels, self.label):
                                                                label.set_text(text)
                                                        self.attach_images(images)
                                                        for i in self.label[:self.n]:
                                                                i.set_size_request(self.width, -1)
                                                self.statusbar_pop(self.statusbar3, self.context_id3)
                                                self.statusbar_push(self.statusbar3, self.context_id3,\
                                                _('There are %s images     Total size %f MB') % (self.n, self.total_size / 1000000.0))
                                                self.window1.show_all()
                                                self.button5.hide()
                                        else:
                                                self.statusbar_pop(self.statusbar3, self.context_id3)
                                                self.statusbar_push(self.statusbar3, self.context_id3, _('There are %s images') % self.n)
                                self.selected = []
                                self.window5.hide()
                                self.progressbar2.set_fraction(0)
                                self.overwrite_all = False

        def get_image_pattern(self, text):
                l = list(text)
                l.reverse()
                n = []
                for i in l:
                        if i.isdigit():
                                n.insert(0, i)
                        else:
                                break
                if n:
                        basename = text[:-len(n)]
                else:
                        basename = text[:]
                        if basename[-1] != '_':
                                basename += '_'
                zeros = []
                while 1:
                        if n and n[0] == '0':
                                zeros.append('0')
                                del n[0]
                        else:
                                break
                pn = 0
                if n:
                        pn = int(''.join(n))
                pn +=1
                pattern = zeros + [item for item in str(pn)]
                return basename + ''.join(pattern)
                
        def rename_image(self, obj=None):
                if self.radiobutton6.get_active():
                        if not self.selected:
                                return True
                        if self.obj in self.event:
                                indice = self.event.index(self.obj)
                        elif  self.obj in self.e:
                                indice = self.e.index(self.obj)
                        text, format = os.path.splitext(os.path.basename(self.get_real_path(self.selected[0])))
                else:
                        indice = self.indice
                        self.selected = [self.list[indice]]
                        text, format = os.path.splitext(os.path.basename(self.get_real_path(self.list[indice])))
                self.entry10.set_text(text)
                self.dialog23.show()
                response = self.dialog23.run()
                if response == gtk.RESPONSE_CANCEL or response == gtk.RESPONSE_DELETE_EVENT:
                        self.dialog23.hide()
                        self.unselect()
                        return True
                elif response == gtk.RESPONSE_OK:
                        self.dialog23.hide()
                        new_text = self.entry10.get_text().strip()
                        if not new_text:
                                gtk.gdk.beep()
                                self.rename_image()
                        elif new_text in [text, text + format] and len(self.selected) == 1:
                                self.dialog23.hide()
                        else:
                                self.progressbar2.set_text('renaming...')
                                fraction = 0.99 / len(self.selected)
                                self.window5.show()
                                for item in self.selected:
                                        if new_text in [text, text + format] and self.selected.index(item) == 0:
                                                self.progressbar2.set_fraction(self.progressbar2.get_fraction() + fraction)
                                                while gtk.events_pending():
                                                        gtk.main_iteration(False)
                                                continue
                                        self.overwrite = False
                                        text_copy = None
                                        indice = self.list.index(item)
                                        original_name = self.list[indice]
                                        format = os.path.splitext(self.list[indice])[1]
                                        if not new_text.endswith(format):
                                                if len(self.selected) > 1 and self.selected.index(item) != 0:
                                                        new_text = self.get_image_pattern(new_text) + format
                                                else:
                                                        new_text += format
                                        else:
                                                if len(self.selected) > 1 and self.selected.index(item) != 0:
                                                        new_text = self.get_image_pattern(new_text[:-len(format)]) + format
                                        if format.lower() in self.raw_formats:
                                                if self.ark not in self.list[indice]:  
                                                        source = self.get_real_path(self.list[indice])
                                                        if len(self.selected) == 1:
                                                                if not self.overwrite_dialog(os.path.dirname(source), new_text):
                                                                        self.window5.hide()
                                                                        self.progressbar2.set_fraction(0)
                                                                        self.progressbar2.set_text('')
                                                                        return True
                                                        dest = os.path.join(os.path.dirname(source), new_text)
                                                        if len(self.selected) > 1 and os.path.isfile(dest):
                                                                if source == dest:
                                                                        self.progressbar2.set_fraction(self.progressbar2.get_fraction() + fraction)
                                                                        while gtk.events_pending():
                                                                                gtk.main_iteration(False)
                                                                        continue        
                                                                dest = self.make_outname(dest)
                                                                text_copy = new_text[:]
                                                                new_text = os.path.basename(dest)
                                                        del self.rawpath[self.list[indice]]
                                                        os.rename(self.list[indice][:-len(format)], os.path.join(os.path.dirname(self.list[indice]),\
                                                        os.path.splitext(new_text)[0] + '.thumb.jpg'))
                                                        self.list[indice] = os.path.join(os.path.dirname(self.list[indice]),\
                                                        os.path.splitext(new_text)[0] + '.thumb.jpg' + format)
                                                        self.rawpath[self.list[indice]] = dest
                                                        keys = Sqlite.get_keywords(source)
                                                        if keys:
                                                                for tag in keys[:]:
                                                                        if Sqlite.has_filename(new_text, tag):
                                                                                keys.remove(tag)
                                                                if keys:
                                                                        Sqlite.update_filename((dest, source))
                                                                        Sqlite.update_keywords(dest, keys)
                                                                else:
                                                                        Sqlite.remove_row(source)
                                                else:                                  
                                                        source = self.get_real_path(self.list[indice])
                                                        if len(self.selected) == 1:
                                                                if not self.overwrite_dialog(os.path.dirname(source), new_text):
                                                                        self.window5.hide()
                                                                        self.progressbar2.set_fraction(0)
                                                                        self.progressbar2.set_text('')
                                                                        return True
                                                        dest = os.path.join(os.path.dirname(source), new_text)
                                                        if len(self.selected) > 1 and os.path.isfile(dest):
                                                                if source == dest:
                                                                        self.progressbar2.set_fraction(self.progressbar2.get_fraction() + fraction)
                                                                        while gtk.events_pending():
                                                                                gtk.main_iteration(False)
                                                                        continue 
                                                                dest = self.make_outname(dest)
                                                                text_copy = new_text[:]
                                                                new_text = os.path.basename(dest)
                                                        os.rename(self.list[indice][:-len(format)], os.path.join(os.path.dirname(self.list[indice]),\
                                                        os.path.splitext(new_text)[0] + '.thumb.jpg'))
                                                        self.list[indice] = os.path.join(os.path.dirname(self.list[indice]),\
                                                        os.path.splitext(new_text)[0] + '.thumb.jpg' + format)
                                                        album = os.path.basename(os.path.dirname(source))
                                                        if self.overwrite:
                                                                self.albums[album].remove(original_name)
                                                        else:
                                                                self.albums[album][self.albums[album].index(original_name)] = self.list[indice]
                                                        keys = Sqlite.get_keywords(original_name)
                                                        if keys:
                                                                for tag in keys[:]:
                                                                        if Sqlite.has_filename(os.path.basename(self.list[indice]), tag):
                                                                                keys.remove(tag)
                                                                if keys:
                                                                        Sqlite.update_filename((self.list[indice], original_name))
                                                                        Sqlite.update_keywords(self.list[indice], keys)
                                                                else:
                                                                        Sqlite.remove_row(original_name)
                                        else:
                                                source = self.list[indice]
                                                dest = os.path.join(os.path.dirname(source), new_text)
                                                if len(self.selected) == 1:
                                                        if not self.overwrite_dialog(os.path.dirname(source), new_text):
                                                                self.window5.hide()
                                                                self.progressbar2.set_fraction(0)
                                                                self.progressbar2.set_text('')
                                                                return True
                                                if len(self.selected) > 1 and os.path.isfile(dest):
                                                        if source == dest:
                                                                self.progressbar2.set_fraction(self.progressbar2.get_fraction() + fraction)
                                                                while gtk.events_pending():
                                                                        gtk.main_iteration(False)
                                                                continue 
                                                        dest = self.make_outname(dest)
                                                        text_copy = new_text[:]
                                                        new_text = os.path.basename(dest)
                                                self.list[indice] = os.path.join(os.path.dirname(self.list[indice]), new_text)
                                                if self.ark in source:
                                                        album = os.path.basename(os.path.dirname(source))
                                                        if self.overwrite:
                                                                self.albums[album].remove(original_name)
                                                        else:
                                                                self.albums[album][self.albums[album].index(original_name)] = self.list[indice]
                                                keys = Sqlite.get_keywords(original_name)
                                                if keys:        
                                                        for tag in keys[:]:
                                                                if Sqlite.has_filename(os.path.basename(self.list[indice]), tag):
                                                                        keys.remove(tag)
                                                        if keys:
                                                                Sqlite.update_filename((self.list[indice], original_name))
                                                                Sqlite.update_keywords(self.list[indice], keys)
                                                        else:
                                                                Sqlite.remove_row(original_name)
                                        self.remove_thumbnails([source, dest])
                                        os.rename(source, dest)
                                        if self.radiobutton6.get_active():
                                                self.label[indice].set_text(new_text)
                                                for item in self.e:
                                                        item.modify_bg(gtk.STATE_NORMAL, self.gallery_color)
                                        else:
                                                self.swin2_label.set_text(new_text)
                                                self.label[indice].set_text(new_text)
                                        self.progressbar2.set_fraction(self.progressbar2.get_fraction() + fraction)
                                        while gtk.events_pending():
                                                gtk.main_iteration(False)
                                        if text_copy:
                                                new_text = text_copy[:]
                                        new_text = new_text[:-len(format)]
                                self.window5.hide()
                                while gtk.events_pending():
                                        gtk.main_iteration(False)
                                self.progressbar2.set_fraction(0)
                                self.progressbar2.set_text('')
                                self.selected = []
                                if self.overwrite:      
                                        if self.version + 'TAG:' in self.window1.get_title():
                                                self.find_tagged(None)
                                        elif self.version + 'Albums/' in self.window1.get_title():
                                                self.make_album_gallery()
                                        else:
                                                self.scrolledwindow2.get_child().destroy()
                                                self.n = self.count_images()
                                                if self.n:
                                                        if self.radiobutton7.get_active():
                                                                self.make_bottom_panel(None, self.n)
                                                        else:
                                                                self.make_table()
                                                                self.attach_images()
                                                                self.make_thumbnails(self.width, self.height, self.n)
                                                        self.statusbar_pop(self.statusbar3, self.context_id3)
                                                        self.statusbar_push(self.statusbar3, self.context_id3,\
                                                        _('There are %s images     Total size %f MB') % (self.n, self.total_size / 1000000.0))
                                                else:
                                                        self.statusbar_pop(self.statusbar3, self.context_id3)
                                                        self.statusbar_push(self.statusbar3, self.context_id3,\
                                                        _('There are %s images') % self.n)
                                                        if self.radiobutton7.get_active():
                                                                try:
                                                                        self.hbox1.destroy()
                                                                except:
                                                                        pass
                                self.store_albums_db()     

        def remove_thumbnails(self, thumblist):
                for item in thumblist:
                        thumb = str(hash(item)) + '.jpeg'    
                        if thumb in self.cache_list:
                                self.cache_list.remove(thumb)
                                os.remove(os.path.join(self.cache, thumb))
                
        def delete_files(self, obj):                 # Deletes selected files from a directory.
                if self.selection_control():
                        if self.radiobutton7.get_active():
                                self.delete_from_layout3()
                        else:  
                                dialog5 = self.messagedialog(_("Attention!"), _("Are you sure to delete selected images?"), None,
                                                             gtk.BUTTONS_OK_CANCEL, self.window1, gtk.DIALOG_MODAL, gtk.MESSAGE_WARNING, None, (380,200))
                                dialog5.set_keep_above(True)
                                gtk.gdk.beep()
                                response = dialog5.run()
                                if response == gtk.RESPONSE_CANCEL or response == gtk.RESPONSE_DELETE_EVENT:
                                        dialog5.destroy()
                                        return True
                                elif response == gtk.RESPONSE_OK:
                                        dialog5.destroy()
                                        while gtk.events_pending():
                                                gtk.main_iteration(False)
                                        self.progressbar2.set_text(_('Deleting...'))
                                        fraction = 0.99 / len(self.selected)
                                        self.window5.show()
                                        if self.version + 'TAG:' in self.window1.get_title():
                                                tagged = True
                                                tagged_list = self.list[:]
                                        else:
                                                tagged = False
                                        rotated = {}
                                        for key in self.rotated.iterkeys():
                                                rotated[key] = (self.list[key], self.rotated[key])
                                        for item in self.selected:
                                                self.image[self.list.index(item)] = 0
                                                self.label[self.list.index(item)] = 0
                                                self.list[self.list.index(item)]  = 0
                                                source = self.get_real_path(item)
                                                self.total_size -= os.path.getsize(source)
                                                os.remove(source)
                                                self.remove_thumbnails([source])
                                                if self.ark in item:    # probable when 'TAG:' in window title
                                                        format = os.path.splitext(item)[1]
                                                        if format.lower() in self.raw_formats:
                                                                os.remove(item[:-len(format)])
                                                                album = os.path.basename(item.split(homepath + 'thumb' + homepath)[0])
                                                        else:
                                                                album = os.path.basename(os.path.dirname(item))
                                                        self.albums[album].remove(item)
                                                        if not self.albums[album]:
                                                                iter0 = self.model.get_iter_first()
                                                                value=''
                                                                while value != album:
                                                                        value = self.model.get_value(iter0, 1)
                                                                        if value == album:
                                                                                break
                                                                        iter0 = self.model.iter_next(iter0)
                                                                self.model.set_value(iter0, 0, gtk.gdk.pixbuf_new_from_file(self.get_icon_path('no_preview.png')))
                                                                os.remove(os.path.join(self.ark, album, '_preview_.png'))
                                                if self.temp in item:
                                                        f = source[:]
                                                else:
                                                        f = item[:]
                                                if Sqlite.get_keywords(f):
                                                        Sqlite.remove_row(f)
                                                        if tagged and item in tagged_list:
                                                                tagged_list.remove(item)
                                                self.progressbar2.set_fraction(self.progressbar2.get_fraction() + fraction)
                                                while gtk.events_pending():
                                                        gtk.main_iteration(False)

                                        images = [i.get_pixbuf().copy() for i in self.image[:self.n] if i != 0]
                                        labels = [i.get_text() for i in self.label[:self.n] if i != 0]
                                        for key in rotated.keys():
                                                if self.list[key] == 0:
                                                        del rotated[key]
                                        self.list = [i for i in self.list if i != 0]
                                        self.rotated.clear()
                                        for value in rotated.itervalues():
                                                self.rotated[self.list.index(value[0])] = value[1]        
                                        self.scrolledwindow2.get_child().destroy()
                                        if tagged:
                                                self.n = len(tagged_list)
                                                self.list = tagged_list[:]
                                        else:
                                                self.n = len(self.list)
                                        if self.n:
                                                self.make_table()
                                                self.label = [gtk.Label() for i in xrange(self.n)]
                                                for text, label in zip(labels, self.label):
                                                        label.set_text(text)
                                                self.attach_images(images)
                                                for i in self.label[:self.n]:
                                                        i.set_size_request(self.width, -1)
                                                self.statusbar_pop(self.statusbar3, self.context_id3)
                                                self.statusbar_push(self.statusbar3, self.context_id3,\
                                                _('There are %s images     Total size %f MB') % (self.n, self.total_size / 1000000.0))
                                                self.window1.show_all()
                                                self.button5.hide()
                                        else:
                                                self.statusbar_pop(self.statusbar3, self.context_id3)
                                                self.statusbar_push(self.statusbar3, self.context_id3,\
                                                _('There are %s images') % self.n)
                                        self.selected = []
                                        self.window5.hide()
                                        self.progressbar2.set_fraction(0)
                                        self.store_albums_db()

        def delete_from_layout3(self):
                text = _('Delete %s ?') % self.label[self.indice].get_text()
                dialog26 = self.messagedialog(_("Attention!"), text, None, gtk.BUTTONS_OK_CANCEL, None, gtk.DIALOG_MODAL, gtk.MESSAGE_QUESTION, None, (350,165))
                dialog26.set_keep_above(True)
                response = dialog26.run()
                if response == gtk.RESPONSE_CANCEL or response == gtk.RESPONSE_DELETE_EVENT:
                        dialog26.destroy()
                elif response == gtk.RESPONSE_OK:
                        rotated = {}
                        for key in self.rotated.iterkeys():
                                rotated[key] = (self.list[key], self.rotated[key])
                        if rotated.has_key(self.indice):
                                del rotated[self.indice]
                        dialog26.destroy()
                        format = os.path.splitext(self.list[self.indice])[1]
                        if self.indice < len(self.list) - 1:
                                self.next_b_photo()
                                self.b_event[self.indice - 1].destroy() 
                                del self.b_event[self.indice - 1], self.b_img[self.indice - 1]
                                self.remove_img_from_win2(self.list[self.indice - 1], self.indice - 1)
                                del self.list[self.indice - 1], self.label[self.indice - 1]
                                self.indice -= 1
                                self.n = len(self.list)
                                self.cache_b_pixbuf(self.pixindex - 1, self.indice - 1)
                        elif self.indice == len(self.list) - 1:   
                                if len(self.list) > 1:
                                        self.previous_b_photo()
                                        self.b_event[self.indice + 1].destroy()
                                        self.remove_img_from_win2(self.list[self.indice + 1], self.indice + 1)
                                        del self.list[self.indice + 1], self.label[self.indice + 1], self.b_img[self.indice + 1]
                                        self.n = len(self.list)
                                else:
                                        self.remove_img_from_win2(self.list[self.indice], self.indice)
                                        del self.list[self.indice], self.label[self.indice], self.b_img[self.indice]
                                        self.n = len(self.list)
                                        self.pblist = range(3)
                                        self.b_event[self.indice].destroy()
                                        self.swin2_image.clear()
                                        self.swin2_label.set_text('')
                        self.rotated.clear()
                        for value in rotated.itervalues():
                                self.rotated[self.list.index(value[0])] = value[1]  
                self.selected = []

        def delete_image_from_window2(self, obj):
                text = _('Delete %s ?') % self.label[self.indice].get_text()
                dialog26 = self.messagedialog(_("Attention!"), text, None, gtk.BUTTONS_OK_CANCEL, None, gtk.DIALOG_MODAL, gtk.MESSAGE_QUESTION, None, (350,165))
                dialog26.set_keep_above(True)
                response = dialog26.run()
                if response == gtk.RESPONSE_CANCEL or response == gtk.RESPONSE_DELETE_EVENT:
                        dialog26.destroy()
                elif response == gtk.RESPONSE_OK:
                        rotated = {}
                        for key in self.rotated.iterkeys():
                                rotated[key] = (self.list[key], self.rotated[key])
                        if rotated.has_key(self.indice):
                                del rotated[self.indice]
                        dialog26.destroy()
                        if self.workflow_started:
                                if self.show_dialog21():
                                        return True
                        format = os.path.splitext(self.list[self.indice])[1]
                        if self.indice < len(self.list) - 1:
                                self.next_photo()
                                if self.radiobutton7.get_active():
                                        if self.indice - 1 == self.old_indice:
                                                self.indice -= 1
                                                self.next_b_photo()
                                                self.cache_b_pixbuf(self.pixindex - 1, self.indice - 2)
                                        elif self.indice - 1 == self.old_indice + 1:
                                                self.cache_b_pixbuf(self.pixindex + 1, self.indice)
                                        elif self.indice - 1 == self.old_indice - 1:
                                                self.cache_b_pixbuf(self.pixindex - 1, self.indice - 2)
                                                self.old_indice -= 1
                                        self.b_event[self.indice - 1].destroy() 
                                        del self.b_event[self.indice - 1], self.b_img[self.indice -1]
                                        self.removed_from_win2 = False
                                else:
                                        self.removed_from_win2 = True
                                self.remove_img_from_win2(self.list[self.indice - 1], self.indice - 1)
                                del self.list[self.indice - 1], self.label[self.indice - 1]
                                self.indice -= 1
                                self.n = len(self.list)
                                self.cache_pixbuf(self.pbindex - 1, self.indice - 1)            
                        elif self.indice == len(self.list) - 1:
                                if len(self.list) > 1:
                                        self.previous_photo()
                                        if self.radiobutton7.get_active():
                                                if self.indice + 1 == self.old_indice:
                                                        self.indice += 1
                                                        self.previous_b_photo()
                                                        self.old_indice -= 1
                                                self.b_event[self.indice + 1].destroy()
                                                del self.b_img[self.indice + 1]
                                                self.removed_from_win2 = False
                                        else:
                                                self.removed_from_win2 = True
                                        self.remove_img_from_win2(self.list[self.indice + 1], self.indice + 1)
                                        del self.list[self.indice + 1], self.label[self.indice + 1]
                                        self.n = len(self.list)
                                else:
                                        self.remove_img_from_win2(self.list[self.indice], self.indice)
                                        del self.list[self.indice], self.label[self.indice]
                                        self.n = len(self.list)
                                        self.pblist = range(3)
                                        self.image28.clear()
                                        self.window2.set_sensitive(False)
                                        if self.radiobutton7.get_active():
                                                self.b_event[self.indice].destroy()
                                                del self.b_img[self.indice]
                                                self.swin2_image.clear()
                                                self.swin2_label.set_text('')
                                                self.removed_from_win2 = False
                                        else:
                                                self.removed_from_win2 = True
                        self.rotated.clear()
                        for value in rotated.itervalues():
                                self.rotated[self.list.index(value[0])] = value[1]  

        def remake_gallery(self):
                if self.removed_from_win2:
                        self.removed_from_win2 = False
                        self.scrolledwindow2.get_child().destroy()
                        if self.n:
                                self.total_size = 0
                                for item in self.list:
                                        self.total_size += os.path.getsize(self.get_real_path(item))
                                self.make_table()
                                self.attach_images()
                                self.statusbar_pop(self.statusbar3, self.context_id3)
                                self.statusbar_push(self.statusbar3, self.context_id3,\
                                _('There are %s images     Total size %f MB') % (self.n, self.total_size / 1000000.0))
                                self.make_thumbnails(self.width, self.height, self.n)
                        else:
                                self.statusbar_pop(self.statusbar3, self.context_id3)
                                self.statusbar_push(self.statusbar3, self.context_id3,\
                                _('There are %s images') % self.n)      
                                self.image28.clear()
                                self.window2.set_sensitive(False)
                
        def remove_img_from_win2(self, img, index):
                source = self.get_real_path(img)
                self.total_size -= os.path.getsize(source)
                os.remove(source)
                self.remove_thumbnails([source])
                format = os.path.splitext(img)[1]
                if self.ark in img:    # probable when 'TAG:' in window title
                        if format.lower() in self.raw_formats:
                                os.remove(img[:-len(format)])
                                album = os.path.basename(img.split(homepath + 'thumb' + homepath)[0])
                        else:
                                album = os.path.basename(os.path.dirname(img))
                        self.albums[album].remove(img)
                        if not self.albums[album]:
                                iter0 = self.model.get_iter_first()
                                value=''
                                while value != album:
                                        value = self.model.get_value(iter0, 1)
                                        if value == album:
                                                break
                                        iter0 = self.model.iter_next(iter0)
                                self.model.set_value(iter0, 0, gtk.gdk.pixbuf_new_from_file(self.get_icon_path('no_preview.png')))
                                os.remove(os.path.join(self.ark, album, '_preview_.png'))
                        self.store_albums_db()
                else:
                        if format.lower() in self.raw_formats:
                                del self.rawpath[img]
                        if self.rotated.has_key(index):
                                del self.rotated[index]
                if self.temp in img:
                        img = source[:]
                if Sqlite.get_keywords(img):
                        Sqlite.remove_row(img)
                self.statusbar_pop(self.statusbar3, self.context_id3)
                self.statusbar_push(self.statusbar3, self.context_id3,\
                        _('There are %s images     Total size %f MB') % (self.n -1 , self.total_size / 1000000.0))

        def overwrite_dialog2(self, outfile):
                self.label94.set_text(_('A file named "%s" \nalready exists! Overwrite it?') % outfile)
                self.dialog22.show()
                gtk.gdk.beep()
                response = self.dialog22.run()
                if response == gtk.RESPONSE_CANCEL or response == gtk.RESPONSE_DELETE_EVENT:
                        self.dialog22.hide()
                        while gtk.events_pending():
                                gtk.main_iteration(False)
                        return False
                elif response == gtk.RESPONSE_OK:
                        self.dialog22.hide()
                        while gtk.events_pending():
                                gtk.main_iteration(False)
                        return True
                elif response == gtk.RESPONSE_APPLY:
                        self.dialog22.hide()
                        while gtk.events_pending():
                                gtk.main_iteration(False)
                        self.overwrite_all = True
                        return True

        def set_album_preview(self, obj):
                while gtk.events_pending():
                        gtk.main_iteration(False)
                if self.radiobutton6.get_active():
                        if self.obj in self.event:
                                self.indice = self.event.index(self.obj)
                        elif  self.obj in self.e:
                                self.indice = self.e.index(self.obj)
                self.make_preview(self.album, self.list[self.indice])
                self.unselect()

        def make_preview(self,album, infile):
                format = os.path.splitext(infile)[1]
                if format.lower() in self.raw_formats:
                        outfile = infile[:-len(format)]
                else:
                        outfile = infile
                pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(outfile, 100, 100)
                w, h = pixbuf.get_width(), pixbuf.get_height()
                l = min(w,h)
                pixbuf= pixbuf.subpixbuf((w-l)/2,(h-l)/2,l,l).scale_simple(53, 53, gtk.gdk.INTERP_BILINEAR)
                rotation = self.autorotate_preview(self.get_real_path(infile))
                if rotation:
                        pixbuf = pixbuf.rotate_simple(rotation)
                pixbuf = self.add_thumbnail_border(pixbuf, 0x00000000)
                pixbuf.save(os.path.join(self.ark, album, '_preview_.png'), "png")
                iter0 = self.model.get_iter_first()
                value = ''
                while value != album:
                        value = self.model.get_value(iter0, 1)
                        if value == album:
                                break
                        iter0 = self.model.iter_next(iter0)
                self.model.set_value(iter0, 0, pixbuf)
                
        def autorotate_preview(self, infile):               # Autorotates album preview
                f = open(infile, 'rb')
                tags = exif.process_file(f, details=False)
                f.close()
                rotation = False
                if tags and tags.has_key('Image Orientation'):
                        if 'Rotated 90 CCW' in str(tags['Image Orientation']):
                                rotation = gtk.gdk.PIXBUF_ROTATE_COUNTERCLOCKWISE       
                        elif 'Rotated 90 CW' in str(tags['Image Orientation']):
                                rotation = gtk.gdk.PIXBUF_ROTATE_CLOCKWISE
                else:
                        list_ = [exiftool, '-j', '-n', '-Orientation', '-Rotation', infile]
                        p = Popen(list_, stdin=PIPE, stdout=PIPE, stderr=PIPE)
                        output = p.communicate()[0]
                        p.stdout.close()
                        p.stdin.close()
                        p.stderr.close()
                        output_ = None
                        dct = json.loads(output)[0]
                        if dct.has_key('Orientation'):
                                output_ = dct['Orientation']
                        elif dct.has_key('Rotation'):
                                output_ = dct['Rotation']
                        if output_:
                                if output_ in [8, 270]:
                                        rotation = gtk.gdk.PIXBUF_ROTATE_COUNTERCLOCKWISE
                                elif output_ in [6, 90]:
                                        rotation = gtk.gdk.PIXBUF_ROTATE_CLOCKWISE
                return rotation

        def make_raw(self, infile, format, i):
                preview = os.path.join(self.temp, os.path.splitext(os.path.split(infile)[1])[0] + '.thumb.jpg')
                if not os.path.exists(preview):
                        if format.lower() in ('.x3f', '.dng'):
                                cmd = '%s -c -e "%s" | convert -quality 100 -  "%s"' % (dcraw, infile, preview)
                        else:
                                cmd = '%s -c -e "%s" > "%s"' % (dcraw, infile, preview)
                        retcode = call(cmd, shell=True)
                thumb = preview + format  
                self.list[i] = thumb
                self.rawpath[thumb] = infile
                if self.checkbutton2.get_active():
                        self.autorotate(infile, i)
                return thumb

        def autorotate(self, infile, i):               # Autorotates thumbnails
                f = open(infile, 'rb')
                tags = exif.process_file(f, details=False)
                f.close()
                if tags and tags.has_key('Image Orientation'):
                        if 'Rotated 90 CCW' in str(tags['Image Orientation']):
                                self.rotated[i] = gtk.gdk.PIXBUF_ROTATE_COUNTERCLOCKWISE
                                self.rotate = gtk.gdk.PIXBUF_ROTATE_COUNTERCLOCKWISE    
                        elif 'Rotated 90 CW' in str(tags['Image Orientation']):
                                self.rotated[i] = gtk.gdk.PIXBUF_ROTATE_CLOCKWISE
                                self.rotate = gtk.gdk.PIXBUF_ROTATE_CLOCKWISE
                        else:
                                self.rotate = False
                else:
                        list_ = [exiftool, '-j', '-n', '-Orientation', '-Rotation', infile]
                        p = Popen(list_, stdin=PIPE, stdout=PIPE, stderr=PIPE)
                        output = p.communicate()[0]
                        p.stdout.close()
                        p.stdin.close()
                        p.stderr.close()
                        dct = json.loads(output)[0]
                        if dct.has_key('Orientation'):
                                orientation = dct['Orientation']
                        elif dct.has_key('Rotation'):
                                orientation = dct['Rotation']
                        else:
                                orientation = False
                                self.rotate = False
                        if orientation:
                                if orientation in [8, 270]:
                                        self.rotated[i] = gtk.gdk.PIXBUF_ROTATE_COUNTERCLOCKWISE
                                        self.rotate = gtk.gdk.PIXBUF_ROTATE_COUNTERCLOCKWISE
                                elif orientation in [6, 90]:
                                        self.rotated[i] = gtk.gdk.PIXBUF_ROTATE_CLOCKWISE
                                        self.rotate = gtk.gdk.PIXBUF_ROTATE_CLOCKWISE
                                else:
                                        self.rotate = False
                        
        def stop_thumbnails(self, event):
                self.stop_thread3.set()
                
        def sort_files(self, filelist):
                if self.radiomenuitem1.get_active():
                        return sorted(filelist, cmp = lambda x, y: cmp(os.path.basename(x).lower(), os.path.basename(y).lower()))
                elif self.radiomenuitem2.get_active():
                        filelist.sort(cmp = lambda x, y: cmp(os.path.basename(x).lower(), os.path.basename(y).lower()))
                        return sorted(filelist, cmp = lambda x, y : cmp(os.path.splitext(x)[1].lower(), os.path.splitext(y)[1].lower()))
                elif self.radiomenuitem3.get_active():
                        return sorted(filelist, cmp = lambda x, y: cmp(os.path.basename(x).lower(), os.path.basename(y).lower()), reverse=True)
                elif self.radiomenuitem4.get_active():
                        return filelist
                elif self.menuitem30.get_active():
                        gobject.idle_add(self.window1.window.set_cursor, self.watch)
                        self.statusbar_pop(self.statusbar4, self.context_id4)
                        self.statusbar_push(self.statusbar4, self.context_id4, _("Calculating timestamp..."))
                        while gtk.events_pending():
                                gtk.main_iteration(False)
                        self._d = {}
                        for item in filelist:
                                self._d[self.get_real_path(item)] = item
                        cmd = [exiftool , '-j', '-n', '-DateTimeOriginal'] + self._d.keys()
                        p = Popen(cmd, stdin=PIPE, stdout=PIPE, stderr=PIPE)
                        output = p.communicate()[0]
                        p.stdout.close()
                        p.stdin.close()
                        p.stderr.close()
                        dct = json.loads(utf8conv(output))
                        l = [i for i in dct if i.has_key('DateTimeOriginal')]
                        lista = [i['SourceFile'] for i in sorted(l, lambda x,y: cmp(x['DateTimeOriginal'], y['DateTimeOriginal']))]
                        for item in range(len(lista)):
                                lista[item]=os.path.normpath(lista[item])
                        outlist = [self._d[s] for s in lista]
                        gobject.idle_add(self.window1.window.set_cursor, None)
                        self.statusbar_push(self.statusbar4, self.context_id4, "")
                        while gtk.events_pending():
                                gtk.main_iteration(False)
                        return outlist + list(set(self._d.values()) - set(outlist))
                elif self.radiomenuitem5.get_active():
                        return sorted(filelist, cmp = lambda x, y: cmp(os.path.getsize(self.get_real_path(y)), os.path.getsize(self.get_real_path(x))))
                
        def show_window9(self,obj):
                self.image80.set_from_stock(gtk.STOCK_FIND, 4)
                self.button86.set_tooltip_text(_('Scan'))
                self.tv7_model.clear()
                self.label163.set_text('')
                self.window9.show()

        def on_abort_button_clicked(self, obj):
                self.stop_progressbar5.set()
                while gtk.events_pending():
                        gtk.main_iteration(False)
                                                        
        def count_images(self):
                self.list = []
                self.total_size = 0
                get_size = os.path.getsize
                get_format = os.path.splitext
                formats_ = self.formats + self.raw_formats
                files = glob.glob(self.cartella + homepath + '*')
                for item in files:
                        if get_format(item)[1].lower() in formats_ :
                                self.list += [item]
                                self.total_size += get_size(item)
                return len(self.list)
        
        def scan_dir(self, dirname):
                filelist = glob.glob(dirname + '/*')
                files = [f for f in filelist if os.path.splitext(f)[1].lower() in self.formats + self.raw_formats]
                if not files:
                        return True
                list_ = [exiftool, '-j', '-n', '-1IPTC:keywords', '-SupplementalCategories', '-XMP:LastKeywordIPTC',
                '-XMP:LastKeywordXMP', '-XMP-dc:Subject'] + files
                p = Popen(list_, stdin=PIPE, stdout=PIPE, stderr=PIPE)
                exiftool_output = p.communicate()[0]
                p.stdout.close()
                p.stdin.close()
                p.stderr.close()
                try:
                        output_list = json.loads(exiftool_output)
                except:
                        return True
                for item in output_list:
                        lista = []
                        filename = item['SourceFile']
                        if not filename.lower().endswith(self.formats + self.raw_formats):
                                continue
                        for key in ['Keywords', 'Subject', 'LastKeywordXMP', 'LastKeywordIPTC', 'SupplementalCategories']:
                                if item.has_key(key):
                                        if isinstance(item[key], (unicode, str)):
                                                lista += item[key].split(',')
                                        elif isinstance(item[key], list):
                                                lista += item[key]
                        lista = [s.strip() for s in lista]
                        if lista:
                                found = False
                                keywords = Sqlite.get_keywords(filename)
                                diff = []
                                for tag in lista:
                                        if tag not in keywords and not Sqlite.has_filename(os.path.basename(filename), tag):
                                                found = True
                                                diff += [tag]
                                        if tag not in self.settings['tags']:
                                                self.treeview5.get_model().append([False, tag])
                                                self.treeview6.get_model().append([False, tag])
                                                self.tagmodel3.append([tag])
                                                self.settings['tags'].append(tag)
                                if keywords:
                                        Sqlite.update_keywords(filename, keywords + diff)
                                else:
                                        Sqlite.insert_row(filename, diff, os.path.dirname(filename))
                                if found:
                                        found = False
                                        for row in self.treeview7.get_model():
                                                if row[0] == filename:
                                                        found = True
                                                        row[1] = ' '.join(diff)
                                        if not found:
                                                self.treeview7.get_model().append([filename, ' '.join(diff)])
                        
        def synchronize_tag_database(self):
                if self.image80.get_stock()[0] == 'gtk-find':
                        basename = os.path.basename
                        join = os.path.join
                        self.tv7_model.clear()
                        self.label163.set_text('')
                        self.label165.set_text(_('Scanning...'))
                        self.button86.set_tooltip_text(_('Stop'))
                        self.image80.set_from_stock(gtk.STOCK_MEDIA_STOP, 4)
                        self.progressbar5.show()
                        self.progressbar5.pulse()
                        while gtk.events_pending():
                                gtk.main_iteration(False)
                        root_dir = self.filechooserbutton8.get_current_folder()
                        self.scan_dir(root_dir)
                        self.progressbar5.set_text(root_dir)
                        self.progressbar5.pulse()
                        while gtk.events_pending():
                                gtk.main_iteration(False)
                        for root, dirs, files in os.walk(root_dir):
                                if basename(root)[0] != '.' :
                                        for name in dirs:
                                                if homepath + '.' not in join(root, name):
                                                        self.progressbar5.set_text(utf8conv(join(root, name)))
                                                        self.progressbar5.pulse()
                                                        while gtk.events_pending():
                                                                gtk.main_iteration(False)
                                                        self.scan_dir(join(root, name))
                                                if self.stop_progressbar5.isSet():
                                                        break
                                        if self.stop_progressbar5.isSet():
                                                break
                        self.label163.set_text(_('Found: %s') % len(self.treeview7.get_model()))
                        self.progressbar5.set_text('')
                        self.progressbar5.set_fraction(0)
                        self.progressbar5.hide()
                        self.label165.set_text(_('This tool will try to find tagged images\ninto the filesystem updating the tags database.'))
                        self.image80.set_from_stock(gtk.STOCK_FIND, 4)
                        self.button86.set_tooltip_text(_('Scan'))
                        self.stop_progressbar5.clear()
                        self.store_settings_db()
                else:
                        self.stop_progressbar5.set()
                        while gtk.events_pending():
                                gtk.main_iteration(False)
                        
        def on_window9_delete_event(self, obj, event=None):
                if self.image80.get_stock()[0] == 'gtk-find':
                        self.window9.hide()
                        return True
                else:
                        self.stop_progressbar5.set() 

        def find_tagged(self, obj):
                self.window1.window.set_cursor(self.watch)
                while gtk.events_pending():
                        gtk.main_iteration(False)
                tag = self.combobox15.get_active_text()
                tagged_list = []
                total_size = 0
                get_size = os.path.getsize
                os_path_join = os.path.join
                tuple_ = Sqlite.get_tagged_from_database(tag)
                if tuple_:
                        if not self.checkbutton30.get_active():
                                for item in tuple_:
                                        if os.path.isfile(self.get_real_path(item[0])):
                                                tagged_list += [item[0]]
                                                total_size += get_size(self.get_real_path(item[0]))
                                        else:
                                                Sqlite.remove_row(item[0])
                        else:
                                if self.radiobutton4.get_active():
                                        folder = self.filechooserbutton9.get_current_folder()
                                        if self.checkbutton31.get_active():
                                                for item in tuple_:
                                                        if folder in item[1]:
                                                                if os.path.isfile(item[0]):
                                                                        tagged_list += [item[0]]
                                                                        total_size += get_size(item[0])
                                                                else:
                                                                        Sqlite.remove_row(item[0])
                                        else:
                                                for item in tuple_:
                                                        if folder == item[1]:
                                                                if os.path.isfile(item[0]):
                                                                        tagged_list += [item[0]]
                                                                        total_size += get_size(item[0])
                                                                else:
                                                                        Sqlite.remove_row(item[0])
                                else:
                                        album = self.combobox16.get_active_text()
                                        if album:
                                                if album == 'All Albums':
                                                        for item in tuple_:
                                                                if self.ark in item[1]:
                                                                        if os.path.isfile(self.get_real_path(item[0])):
                                                                                tagged_list += [item[0]]
                                                                                total_size += get_size(self.get_real_path(item[0]))
                                                                        else:
                                                                                Sqlite.remove_row(item[0])
                                                else:
                                                        for item in tuple_:
                                                                if os_path_join(self.ark, album) in item[1]:
                                                                        if os.path.isfile(self.get_real_path(item[0])):
                                                                                tagged_list += [item[0]]
                                                                                total_size += get_size(self.get_real_path(item[0]))
                                                                        else:
                                                                                Sqlite.remove_row(item[0])
                self.window1.window.set_cursor(None)
                while gtk.events_pending():
                        gtk.main_iteration(False)
                self.total_size = total_size
                self.list = tagged_list[:]
                self.n = len(tagged_list)
                self.vpaned2.handler_block(self.id27)
                self.window1.set_title(self.version + 'TAG: ' + tag)
                viewport = self.scrolledwindow2.get_child()
                if viewport:
                        viewport.destroy()
                if self.n:
                        if self.radiobutton7.get_active():
                                self.make_bottom_panel(None, self.n)
                        else:
                                self.make_table()
                                self.attach_images()
                                self.make_thumbnails(self.width, self.height, self.n)
                        self.statusbar_pop(self.statusbar3, self.context_id3)
                        self.statusbar_push(self.statusbar3, self.context_id3,\
                        _('There are %d images     Total size %f MB') % (self.n, self.total_size / 1000000.0))
                else:
                        self.statusbar_pop(self.statusbar3, self.context_id3)
                        self.statusbar_push(self.statusbar3, self.context_id3, _('There are %d images') % self.n)
                        if self.radiobutton7.get_active():
                                try:
                                        self.hbox1.destroy()
                                except:
                                        pass
                self.add_selection1.set_sensitive(True)
                self.selected = []
                self.vpaned2.handler_unblock(self.id27)

        def make_table(self):
                if self.scrolledwindow6:
                        self.scrolledwindow6.destroy()
                while gtk.events_pending():
                        gtk.main_iteration(False)
                self.rect2 = self.scrolledwindow2.get_allocation()
                self.n_columns = (self.rect2.width - 20) / (self.width + 20)
                self.rows = int(math.ceil(self.n / float(self.n_columns)))
                self.table = gtk.Table(self.rows, self.n_columns, False)
                self.table.set_row_spacings(20)
                self.table.set_col_spacings(20)
                self.table.set_border_width(20)
                self.eb.add(self.table)
                self.eb.modify_bg(gtk.STATE_NORMAL, self.colorbutton1.get_color())
                self.scrolledwindow2.add_with_viewport(self.eb)
                self.table.show()
                self.window1.show_all()
                while gtk.events_pending():
                        gtk.main_iteration(False)
                
        def attach_images(self, images=None):
                if self.checkbutton360.get_active():
                        self.height = self.width
                img_n = self.n
                self.image = [ gtk.Image() for i in xrange(img_n) ]
                self.e = [ gtk.EventBox() for i in xrange(img_n) ]
                self.event = [ gtk.EventBox() for i in xrange(img_n) ]
                if self.rows > 1:
                        n = 0
                        for y in xrange(img_n / self.n_columns):
                                for x in xrange(self.n_columns):
                                        self.vbox = gtk.VBox()
                                        self.event[n].add(self.image[n])
                                        self.event[n].add_events(gtk.gdk.BUTTON_PRESS_MASK | gtk.gdk.ENTER_NOTIFY_MASK)
                                        self.event[n].connect("button_press_event", self.select_photo)
                                        self.event[n].modify_bg(gtk.STATE_NORMAL, self.gallery_color)
                                        self.e[n].add_events(gtk.gdk.BUTTON_PRESS_MASK)
                                        self.e[n].connect("button_press_event", self.select_photo)
                                        self.e[n].modify_bg(gtk.STATE_NORMAL, self.gallery_color)
                                        self.event[n].set_size_request(self.width + 2, self.height + 3)
                                        if images:
                                                self.e[n].add(self.label[n])
                                                self.image[n].set_from_pixbuf(images[n])
                                                if n in self.blocked_list:
                                                        self.event[n].handler_block_by_func(self.select_photo)
                                                        self.e[n].handler_block_by_func(self.select_photo)
                                        self.table.attach(self.vbox, x, x + 1, y, y + 1, xoptions = 0, yoptions = 0)
                                        self.vbox.pack_start(self.event[n], expand=True, fill=True)
                                        self.vbox.pack_start(self.e[n], expand=False, fill=False)
                                        n += 1
                        resto = img_n % self.n_columns
                        if resto > 0:
                                for x in xrange(resto):
                                        self.vbox = gtk.VBox()
                                        self.event[n].add(self.image[n])
                                        self.event[n].add_events(gtk.gdk.BUTTON_PRESS_MASK | gtk.gdk.ENTER_NOTIFY_MASK)
                                        self.event[n].connect("button_press_event", self.select_photo)
                                        self.event[n].modify_bg(gtk.STATE_NORMAL, self.gallery_color)
                                        self.e[n].add_events(gtk.gdk.BUTTON_PRESS_MASK)
                                        self.e[n].connect("button_press_event", self.select_photo)
                                        self.e[n].modify_bg(gtk.STATE_NORMAL, self.gallery_color)
                                        self.event[n].set_size_request(self.width + 2, self.height + 3)
                                        if images:
                                                self.e[n].add(self.label[n])
                                                self.image[n].set_from_pixbuf(images[n])
                                                if n in self.blocked_list:
                                                        self.event[n].handler_block_by_func(self.select_photo)
                                                        self.e[n].handler_block_by_func(self.select_photo)
                                        self.table.attach(self.vbox, x, x + 1, y + 1, y + 2, xoptions = 0, yoptions = 0)
                                        self.vbox.pack_start(self.event[n], expand=True, fill=True)
                                        self.vbox.pack_start(self.e[n], expand=False, fill=False)
                                        n += 1
                else:
                        for x in xrange(img_n):
                                self.vbox = gtk.VBox()
                                self.event[x].add(self.image[x])
                                self.event[x].add_events(gtk.gdk.BUTTON_PRESS_MASK | gtk.gdk.ENTER_NOTIFY_MASK)
                                self.event[x].connect("button_press_event", self.select_photo)
                                self.event[x].modify_bg(gtk.STATE_NORMAL, self.gallery_color)
                                self.e[x].add_events(gtk.gdk.BUTTON_PRESS_MASK)
                                self.e[x].connect("button_press_event", self.select_photo)
                                self.e[x].modify_bg(gtk.STATE_NORMAL, self.gallery_color)
                                self.event[x].set_size_request(self.width + 2, self.height + 3)
                                if images:
                                        self.e[x].add(self.label[x])
                                        self.image[x].set_from_pixbuf(images[x])
                                        if x in self.blocked_list:
                                                self.event[x].handler_block_by_func(self.select_photo)
                                                self.e[x].handler_block_by_func(self.select_photo)
                                self.table.attach(self.vbox, x, x + 1, 0, 1, xoptions = 0, yoptions = 0)
                                self.vbox.pack_start(self.event[x])
                                self.vbox.pack_start(self.e[x], expand=False, fill=False)
                                
        def make_thumbnails(self, width, height, img_n):
                self.gallery_thread = threading.Thread(target=self.make_thumbnails_thread, args=(width, height, img_n))
                self.gallery_thread.start()
               
        def make_thumbnails_thread(self, width, height, img_n):
                self.blocked_list = []
                gobject.idle_add(self.button7.set_sensitive, False)
                self.rotated.clear()
                cache_list = self.cache_list
                get_filename = self.get_filename
                os_path_join = os.path.join
                add_thumbnail_border = self.add_thumbnail_border
                border = self.checkbutton4.get_active()
                gobject.idle_add(self.radiobutton6.set_active, True)
                self.settings['radiobutton6'] = True
                self.settings['radiobutton7'] = False
                self.toolbuttons(False, True)    
                self.b_pixlist = range(3)       
                gobject.idle_add(self.button5.show)
                self.toolbutton16.handler_block(self.id1)
                self.toolbutton17.handler_block(self.id2)
                self.toolbutton14.handler_block(self.id9)
                self.toolbutton15.handler_block(self.id10)
                self.toolbutton18.handler_block(self.id11)
                self.toolbutton19.handler_block(self.id12)
                self.toolbutton20.handler_block(self.id13)
                self.toolbutton21.handler_block(self.id19)
                self.toolbutton26.handler_block(self.id23)
                self.toolbutton27.handler_block(self.id24)
                self.toolbutton28.handler_block(self.id39)
                self.toolbutton29.handler_block(self.id26)
                self.toolbutton30.handler_block(self.id30)
                self.hpaned1.handler_block(self.id28)
                self.new1.handler_block(self.id14)
                self.delete2.handler_block(self.id15)
                self.add_selection1.handler_block(self.id16)
                self.i = 0
                fraction = 0.99 / img_n
                self.label = range(img_n)
                self.list = self.sort_files(self.list)
                list_ = self.list
                thumblist = []
                for infile in list_:    
                        if self.stop_thread3.isSet():
                                self.total_size = 0
                                for item in list_[:self.i]:
                                        self.total_size += os.path.getsize(self.get_real_path(item))
                                self.statusbar_pop(self.statusbar3, self.context_id3)
                                self.statusbar_push(self.statusbar3, self.context_id3,\
                                _('There are %s images     Total size %f MB') % (self.i, self.total_size / 1000000.0))
                                break
                        if self.stop_gallery_thread.isSet():
                                return True
                        infile, filename = get_filename(infile, self.i)
                        preview = str(hash(self.get_real_path(filename))) + '.jpeg'
                        if self.checkbutton7.get_active():
                                if preview not in cache_list:
                                        thumblist += [(infile, filename, preview, self.rotate)]
                                        pixbuf = self.load_image(infile)
                                else:
                                        if self.thumbnail_was_modified(preview, filename):
                                                thumblist += [(infile, filename, preview, self.rotate)]
                                                pixbuf = self.load_image(infile)
                                        else:
                                                if self.checkbutton360.get_active():
                                                        pixbuf = self.load_image(infile)
                                                else:
                                                        pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(os_path_join(self.cache, preview), self.width, self.height)
                        else:
                                if preview in cache_list:
                                        if self.thumbnail_was_modified(preview, filename):
                                                thumblist += [(infile, filename, preview, self.rotate)]
                                                pixbuf = self.load_image(infile)
                                        else:
                                                if self.checkbutton360.get_active():
                                                        pixbuf = self.load_image(infile)
                                                else:
                                                        pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(os_path_join(self.cache, preview), self.width, self.height)
                                else:
                                        pixbuf = self.load_image(infile)
                        if border:
                                pixbuf = add_thumbnail_border(pixbuf, self.border_color)
                        gobject.idle_add(self.image[self.i].set_from_pixbuf, pixbuf)
                        gobject.idle_add(self.label[self.i].modify_fg, gtk.STATE_NORMAL, self.label_color)
                        gobject.idle_add(self.label[self.i].set_size_request, self.width, -1)
                        self.e[self.i].add(self.label[self.i])
                        self.event[self.i].connect("enter_notify_event", self.add_image_info)
                        gobject.idle_add(self.progressbar3.set_fraction, self.progressbar3.get_fraction() + fraction)
                        gobject.idle_add(self.window1.show_all)
                        self.i += 1
                if self.stop_gallery_thread.isSet():
                        return True
                gobject.idle_add(self.progressbar3.set_fraction, 0)
                self.toolbutton16.handler_unblock(self.id1)
                self.toolbutton17.handler_unblock(self.id2)
                self.toolbutton14.handler_unblock(self.id9)
                self.toolbutton15.handler_unblock(self.id10)
                self.toolbutton18.handler_unblock(self.id11)
                self.toolbutton19.handler_unblock(self.id12)
                self.toolbutton20.handler_unblock(self.id13)
                self.toolbutton21.handler_unblock(self.id19)
                self.toolbutton26.handler_unblock(self.id23)
                self.toolbutton27.handler_unblock(self.id24)
                self.toolbutton28.handler_unblock(self.id39)
                self.toolbutton29.handler_unblock(self.id26)
                self.hpaned1.handler_unblock(self.id28)
                self.toolbutton30.handler_unblock(self.id30)
                self.new1.handler_unblock(self.id14)
                self.delete2.handler_unblock(self.id15)
                self.add_selection1.handler_unblock(self.id16)
                gobject.idle_add(self.button5.hide)
                gobject.idle_add(self.statusbar_pop, self.statusbar4, self.context_id4)
                gobject.idle_add(self.statusbar_push, self.statusbar4, self.context_id4, '')
                self.list = list_[:self.i]
                self.n = len(self.list)
                gc.collect()
                gobject.idle_add(self.button7.set_sensitive, True)
                if thumblist:
                        self.generate_thumbnails_thread = threading.Thread(target=self.generate_thumbnails, args=(thumblist,))
                        self.generate_thumbnails_thread.start()
                if self.stop_thread3.isSet():
                        self.stop_thread3.clear()

        def generate_thumbnails(self, thumblist):
                thumblist2 = []
                def save_thumbnail(cache, infile, filename, preview, rotation):
                        pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(infile, 260, 174)
                        if rotation:
                                pixbuf = pixbuf.rotate_simple(rotation)
                        pixbuf.save(os.path.join(cache, preview), "jpeg", {"quality" : "100"})     
                def generate(thumblist, thumblist2, cache):
                        for item in thumblist:
                                if self.stop_thumb_generator.is_set():
                                        self.stop_thumb_generator.clear()
                                        break
                                save_thumbnail(cache, *item)
                                thumblist2.append(item)
                generate(thumblist, thumblist2, self.cache)
                for item in thumblist2:
                        self.mtimes[item[2]] = os.stat(self.get_real_path(item[1])).st_mtime        
                self.store_mtimes_db()
                self.cache_list = os.listdir(self.cache)

        def square_pixbuf(self, pixbuf):
                w, h = pixbuf.get_width(), pixbuf.get_height()
                l = min(w,h)
                return pixbuf.subpixbuf((w-l)/2,(h-l)/2,l,l).scale_simple(self.width, self.width, gtk.gdk.INTERP_BILINEAR)
                
        def load_image(self, infile):
                try:
                        if self.checkbutton360.get_active():
                                pixbuf = gtk.gdk.pixbuf_new_from_file(infile)
                                pixbuf = self.square_pixbuf(pixbuf)
                                if self.rotated:
                                        pixbuf = pixbuf.rotate_simple(self.rotate)
                        else:
                                if not self.rotate:
                                        pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(infile, self.width, self.height)
                                else:
                                        pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(infile, self.height, self.width).rotate_simple(self.rotate)
                except:
                        gobject.idle_add(sys.stderr.write, traceback.format_exc())
                        pixbuf = gtk.gdk.pixbuf_new_from_file(self.get_icon_path('gtk-missing-image.png'))
                        self.blocked_list.append(self.i)
                        if self.radiobutton6.get_active():
                                self.event[self.i].handler_block_by_func(self.select_photo)
                                self.e[self.i].handler_block_by_func(self.select_photo)
                return pixbuf

        def thumbnail_was_modified(self, preview, filename):
                if os.path.splitext(filename)[1] in self.formats:
                        if self.mtimes.has_key(preview) and self.mtimes[preview] != os.stat(filename).st_mtime:
                                return True
                return False
                
        def make_bottom_panel(self, event, img_n=None, fname=None, images=None):
                self.make_bottom_panel_thread(event, img_n, fname, images)
                self.gallery_thread = threading.Thread(target=self.load_bottom_gallery, args=(images, fname, img_n))
                self.gallery_thread.start()
                
        def make_bottom_panel_thread(self, event, img_n=None, fname=None, images=None):
                self.button7.set_sensitive(False)
                self.toolbuttons(True, False)
                self.toolbutton14.handler_block(self.id9)
                self.toolbutton15.handler_block(self.id10)
                self.toolbutton16.handler_block(self.id1)
                self.toolbutton17.handler_block(self.id2)
                self.toolbutton18.handler_block(self.id11)
                self.toolbutton19.handler_block(self.id12)
                self.toolbutton20.handler_block(self.id13)
                self.toolbutton21.handler_block(self.id19)
                self.toolbutton26.handler_block(self.id23)
                self.toolbutton27.handler_block(self.id24)
                self.toolbutton28.handler_block(self.id39)
                self.toolbutton29.handler_block(self.id26)
                self.toolbutton30.handler_block(self.id30)
                self.toolbutton31.handler_block(self.id31)
                self.toolbutton32.handler_block(self.id32)
                self.toolbutton33.handler_block(self.id33)
                self.toolbutton34.handler_block(self.id34)
                self.vpaned2.handler_block(self.id27)
                self.hpaned1.handler_block(self.id28)
                if not img_n:
                       img_n = self.n
                cache_list = self.cache_list
                os_path_join = os.path.join
                get_filename = self.get_filename
                add_thumbnail_border = self.add_thumbnail_border
                border = self.checkbutton4.get_active()
                if self.scrolledwindow6:
                        self.scrolledwindow6.destroy()
                if self.radiobutton6.get_active() and not self.close_panel:
                        self.hpaned1.set_position(int(self.screen0.get_width() * 0.2))
                        self.close_panel = False
                        while gtk.events_pending():
                               gtk.main_iteration(False)
                elif self.radiobutton6.get_active() and self.close_panel:
                        self.hpaned1.set_position(0)
                        self.close_panel = False
                        while gtk.events_pending():
                               gtk.main_iteration(False)
                viewport = self.scrolledwindow2.get_child()  
                if viewport:
                       viewport.destroy()
                self.radiobutton7.set_active(True)
                self.settings['radiobutton7'] = True
                self.settings['radiobutton6'] = False
                gobject.idle_add(self.button5.show)
                self.vbox1 = gtk.VBox(False, 0)
                self.swin2_image = gtk.Image()
                self.swin2_label = gtk.Label()
                self.swin2_label.modify_fg(gtk.STATE_NORMAL, self.label_color)
                self.vbox1.pack_start(self.swin2_image, True, True, 0)
                self.vbox1.pack_start(self.swin2_label, False, False, 0)
                self.scrolledwindow2.add_with_viewport(self.vbox1)
                viewport = self.scrolledwindow2.get_child()
                viewport.add_events(gtk.gdk.BUTTON_RELEASE_MASK | gtk.gdk.BUTTON_PRESS_MASK | gtk.gdk.BUTTON1_MOTION_MASK | gtk.gdk.POINTER_MOTION_HINT_MASK)
                viewport.connect('button_release_event', self.on_scrolledwindow_release_event)
                viewport.connect('button_press_event', self.on_scrolledwindow_release_event)
                viewport.connect('motion_notify_event', self.drag_image, self.scrolledwindow2)
                if img_n:
                        self.scrolledwindow2.get_child().modify_bg(gtk.STATE_NORMAL, self.image_background_color)
                else:
                        self.scrolledwindow2.get_child().modify_bg(gtk.STATE_NORMAL, self.gallery_color)
                self.rect = self.scrolledwindow2.allocation
                self.scrolledwindow2.set_size_request(-1, self.rect.height - self.height + 30)
                self.eventbox9 = gtk.EventBox()
                self.eventbox9.set_size_request(-1, self.height + 30)
                self.eventbox9.modify_bg(gtk.STATE_NORMAL, self.gallery_color)
                self.eventbox9.add_events(gtk.gdk.SCROLL_MASK)
                self.eventbox9.connect("scroll_event", self.scroll_bottom_panel)
                self.scrolledwindow6 = gtk.ScrolledWindow()
                self.hbox1 = gtk.HBox(False, 0)
                self.eventbox9.add(self.hbox1)
                self.scrolledwindow6.set_policy(gtk.POLICY_ALWAYS, gtk.POLICY_NEVER)
                self.scrolledwindow6.add_with_viewport(self.eventbox9)
                self.vpaned2.pack2(self.scrolledwindow6, False, False)
                self.vpaned2.set_position(int(self.screen0.get_height()) - self.height + 30)
                if not images:
                        self.label = range(img_n)
                        self.rotated.clear()
                self.b_img = [ gtk.Image() for i in xrange(img_n) ]
                self.b_event = [ gtk.EventBox() for i in xrange(img_n) ]
                for i in xrange(img_n):
                        self.b_event[i].modify_bg(gtk.STATE_NORMAL, self.gallery_color)
                        self.b_event[i].add_events(gtk.gdk.BUTTON_PRESS_MASK | gtk.gdk.ENTER_NOTIFY_MASK)
                        self.b_event[i].add(self.b_img[i])
                        self.hbox1.pack_start(self.b_event[i], True, True, 0)
                self.window1.show_all()
                while gtk.events_pending():
                       gtk.main_iteration(False)
     
        def load_bottom_gallery(self, images, fname, img_n):
                cache_list = self.cache_list
                os_path_join = os.path.join
                get_filename = self.get_filename
                add_thumbnail_border = self.add_thumbnail_border
                border = self.checkbutton4.get_active()
                if img_n:
                        if not fname:
                                gobject.idle_add(self.b_event[0].modify_bg, gtk.STATE_NORMAL, self.selection_color)
                fraction = 0.99 / img_n
                self.i = 0
                self.indice = 0
                self.pixindex = 1
                self.b_pixlist = range(3)
                if not images:
                        self.list = self.sort_files(self.list)
                list_ = self.list
                thumblist = []
                if fname:
                        self.indice = list_.index(fname)
                        infile, filename = get_filename(fname, self.indice)
                        gobject.idle_add(self.swin2_label.set_text, self.label[self.indice].get_text())
                        self.rect2 = self.scrolledwindow2.get_allocation()
                        if self.rotated.has_key(self.indice) and self.checkbutton2.get_active():
                                self.b_pixlist[self.pixindex] = self.load_cache_pixbuf(infile,\
                                self.rect2.height - self.swin2_label.allocation.height, self.rect2.width, self.rotated[self.indice])
                        else:   
                                self.b_pixlist[self.pixindex] = self.load_cache_pixbuf(infile, self.rect2.width,self.rect2.height - self.swin2_label.allocation.height)
                        format = os.path.splitext(infile)[1]
                        self.b_pixlist[self.pixindex] = self.check_icons(format, self.b_pixlist[self.pixindex])
                        gobject.idle_add(self.swin2_image.set_from_pixbuf, self.b_pixlist[self.pixindex])
                        gobject.idle_add(self.swin2_image.show)
                        if self.indice > 0:
                                get_filename(list_[self.indice - 1], self.indice - 1)
                        if self.indice < img_n - 1:
                                get_filename(list_[self.indice + 1], self.indice + 1)
                        threading.Thread(target=self.load_b_cache, args=(self.rect2.width, self.rect2.height)).start()
                n = 0
                if not images:
                        self.blocked_list = []
                for infile in list_:
                        if self.stop_thread3.isSet():
                                self.total_size = 0
                                for item in list_[:self.i]:
                                        self.total_size += os.path.getsize(self.get_real_path(item))
                                gobject.idle_add(self.statusbar_pop, self.statusbar3, self.context_id3)
                                gobject.idle_add(self.statusbar_push, self.statusbar3, self.context_id3,\
                                _('There are %s images     Total size %f MB') % (self.i, self.total_size / 1000000.0))
                                break
                        if self.stop_gallery_thread.isSet():
                                return True
                        if images:
                                pixbuf = images[n]
                                n += 1
                        else:
                                infile, filename = get_filename(infile, self.i)
                                preview = str(hash(self.get_real_path(filename))) + '.jpeg'
                                if self.checkbutton7.get_active():
                                        if preview not in cache_list:
                                                thumblist += [(infile, filename, preview, self.rotate)]
                                                pixbuf = self.load_image(infile)
                                        else:
                                                if self.thumbnail_was_modified(preview, filename):
                                                        thumblist += [(infile, filename, preview, self.rotate)]
                                                        pixbuf = self.load_image(infile)
                                                else:
                                                        if self.checkbutton360.get_active():
                                                                pixbuf = self.load_image(infile)
                                                        else:
                                                                pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(os_path_join(self.cache, preview), self.width, self.height)
                                else:
                                        if preview in cache_list:
                                                if self.thumbnail_was_modified(preview, filename):
                                                        thumblist += [(infile, filename, preview, self.rotate)]
                                                        pixbuf = self.load_image(infile)
                                                else:
                                                        if self.checkbutton360.get_active():
                                                                pixbuf = self.load_image(infile)
                                                        else:
                                                                pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(os_path_join(self.cache, preview), self.width, self.height)
                                        else:
                                                pixbuf = self.load_image(infile)
                        if border and not images:
                                pixbuf = add_thumbnail_border(pixbuf, self.border_color)
                        if self.i not in self.blocked_list:
                                self.b_event[self.i].connect("button_press_event", self.select_b_photo)
                                self.b_event[self.i].connect("enter_notify_event", self.add_image_info)
                        gobject.idle_add(self.b_event[self.i].set_size_request, pixbuf.get_width() + 20, self.height + 30)
                        gobject.idle_add(self.b_img[self.i].set_from_pixbuf, pixbuf)
                        if self.i == 0 and not fname:
                                if images:
                                        infile, filename = get_filename(infile, self.i)
                                gobject.idle_add(self.swin2_label.set_text, self.label[0].get_text())
                                self.rect2 = self.scrolledwindow2.get_allocation()
                                if self.rotated.has_key(self.i) and self.checkbutton2.get_active():
                                        self.b_pixlist[self.pixindex] = self.load_cache_pixbuf(infile,\
                                        self.rect2.height - self.swin2_label.allocation.height, self.rect2.width, self.rotated[self.i])
                                else:   
                                        self.b_pixlist[self.pixindex] = self.load_cache_pixbuf(infile,\
                                        self.rect2.width,self.rect2.height - self.swin2_label.allocation.height)
                                format = os.path.splitext(infile)[1]
                                self.b_pixlist[self.pixindex] = self.check_icons(format, self.b_pixlist[self.pixindex])
                                gobject.idle_add(self.swin2_image.set_from_pixbuf, self.b_pixlist[self.pixindex])
                        if self.i == 1 and not fname:
                                threading.Thread(target=self.load_b_cache, args=(self.rect2.width, self.rect2.height)).start()
                        gobject.idle_add(self.progressbar3.set_fraction, self.progressbar3.get_fraction() + fraction)
                        gobject.idle_add(self.window1.show_all)
                        self.i += 1
                if self.stop_gallery_thread.isSet():
                        return True
                gobject.idle_add(self.progressbar3.set_fraction, 0)
                gobject.idle_add(self.button5.hide)
                self.toolbutton14.handler_unblock(self.id9)
                self.toolbutton15.handler_unblock(self.id10)
                self.toolbutton16.handler_unblock(self.id1)
                self.toolbutton17.handler_unblock(self.id2)
                self.toolbutton18.handler_unblock(self.id11)
                self.toolbutton19.handler_unblock(self.id12)
                self.toolbutton20.handler_unblock(self.id13)
                self.toolbutton21.handler_unblock(self.id19)
                self.toolbutton26.handler_unblock(self.id23)
                self.toolbutton27.handler_unblock(self.id24)
                self.toolbutton28.handler_unblock(self.id39)
                self.toolbutton29.handler_unblock(self.id26)
                self.toolbutton30.handler_unblock(self.id30)
                self.toolbutton31.handler_unblock(self.id31)
                self.toolbutton32.handler_unblock(self.id32)
                self.toolbutton33.handler_unblock(self.id33)
                self.toolbutton34.handler_unblock(self.id34)
                self.vpaned2.handler_unblock(self.id27)
                self.hpaned1.handler_unblock(self.id28)
                self.list = list_[:self.i]
                self.n = len(self.list) 
                self.calculate_percentage()
                gobject.idle_add(self.button7.set_sensitive, True)
                gc.collect()
                if thumblist:
                        self.generate_thumbnails_thread = threading.Thread(target=self.generate_thumbnails, args=(thumblist,))
                        self.generate_thumbnails_thread.start()
                if self.stop_thread3.isSet():
                        self.stop_thread3.clear()

        def add_image_info(self, obj, event):
                threading.Thread(target=self.add_image_info_thread, args=(obj, event)).start()
                
        def add_image_info_thread(self, obj, event):
                if self.n:
                        try:
                                if self.radiobutton7.get_active():
                                        i = self.b_event.index(obj)
                                else:
                                        i = self.event.index(obj)
                                try:
                                        filename = self.get_real_path(self.list[i])
                                        if not os.path.isfile(filename):
                                                return True
                                        dirname = os.path.dirname(filename)
                                        list_ = [exiftool, '-j', '-n', '-CreateDate', '-DateTimeOriginal', '-ModifyDate', '-Model', '-Software',
                                                '-CameraModelID', '-FileSize', '-ISO', '-ExposureTime', '-FNumber', filename]
                                        p = Popen(list_, stdin=PIPE, stdout=PIPE, stderr=PIPE)
                                        output = p.communicate()[0]
                                        p.stdout.close()
                                        p.stderr.close()
                                        if output:
                                                outputlist = json.loads(utf8conv(output))
                                                text = 'Filename: ' + self.label[i].get_text() + '\nDirectory: ' + dirname
                                                if outputlist:
                                                        item = outputlist[0]
                                                        if item.has_key('DateTimeOriginal'):
                                                                text += '\nCreated: ' + item['DateTimeOriginal']
                                                        elif item.has_key('CreateDate') and 'Created:' not in text:
                                                                text += '\nCreated: ' + item['CreateDate']
                                                        if item.has_key('ModifyDate'):
                                                                text += '\nModified: ' + item['ModifyDate']
                                                        if item.has_key('Model'):
                                                                text += '\nCamera: ' + item['Model']
                                                        if item.has_key('Software'):
                                                                text += '\nSoftware: ' + item['Software']
                                                        if item.has_key('FileSize'):
                                                                text += '\nSize: ' + str(item['FileSize']/1000.0) + ' Kb'
                                                        if item.has_key('ISO'):
                                                                text += '\nISO: ' + str(item['ISO'])
                                                        if item.has_key('ExposureTime'):
                                                                time = item['ExposureTime']
                                                                if time < 1:
                                                                        string = '1/' + str(int(1.0/time))
                                                                elif time == 1:
                                                                        string = 1
                                                                else:
                                                                        string = str(time)
                                                                text += '\nExposureTime: %s' % string
                                                        if item.has_key('FNumber'):
                                                                text += '\nFNumber: %.1f' % item['FNumber']
                                                info = ''
                                                if self.temp in self.list[i]:
                                                        keywords = Sqlite.get_keywords(filename)
                                                else:
                                                        keywords = Sqlite.get_keywords(self.list[i])
                                                for tag in keywords:
                                                        info += tag + ' '
                                                if info:
                                                        text += '\nTags: ' + info
                                                if self.radiobutton7.get_active():
                                                        gobject.idle_add(self.b_event[i].set_tooltip_text, text)
                                                else:
                                                        gobject.idle_add(self.e[i].set_tooltip_text, text)
                                                        gobject.idle_add(self.event[i].set_tooltip_text, text)
                                except:
                                        if self.radiobutton7.get_active():
                                                gobject.idle_add(self.b_event[i].set_tooltip_text, 'Filename:  ' + self.label[i].get_text())
                                        else:
                                                gobject.idle_add(self.e[i].set_tooltip_text, 'Filename:  ' + self.label[i].get_text())  
                                                gobject.idle_add(self.event[i].set_tooltip_text, 'Filename:  ' + self.label[i].get_text()) 
                        except:
                                pass
                
        def add_thumbnail_border(self, pixbuf, border_color):
                pwidth = pixbuf.get_width()
                pheight = pixbuf.get_height()
                if border_color == 0x00000000:
                        new_pixbuf = self.thumb_background.copy().scale_simple(pwidth + 2, pheight + 2, gtk.gdk.INTERP_NEAREST)
                        if pixbuf.get_has_alpha():
                                new_pixbuf = new_pixbuf.add_alpha(False, chr(0), chr(0), chr(0))
                        pixbuf.copy_area(0, 0, pwidth, pheight, new_pixbuf, 1, 1)
                else:
                        new_pixbuf = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB, True, 8, pwidth + 2, pheight + 2)
                        new_pixbuf.fill(border_color)
                        pixbuf.copy_area(0, 0, pwidth, pheight, new_pixbuf, 1, 1)
                return new_pixbuf
                
        def get_filename(self, infile, i):
                source = infile[:]
                self.rotate = False
                format = os.path.splitext(infile)[1]
                if self.ark in infile:
                        if format.lower() in self.raw_formats:
                                f = os.path.split(infile)[1].replace('.thumb.jpg', '')
                                self.label[i] = gtk.Label(f)
                                filename = infile.replace(homepath + 'thumb', '').replace('.thumb.jpg', '')
                                infile = os.path.splitext(infile)[0]
                                if self.checkbutton2.get_active():
                                        self.autorotate(filename, i)
                        else:
                                self.label[i] = gtk.Label(os.path.split(infile)[1])
                                filename = infile[:]
                elif self.ark not in infile and format.lower() in self.raw_formats and self.temp not in infile:
                        infile = self.make_raw(infile, format, i)
                        filename = infile[:]
                else:
                        self.label[i] = gtk.Label(os.path.split(infile)[1])
                        filename = infile[:]
                if self.temp in infile:
                        f = os.path.split(infile)[1].replace('.thumb.jpg', '')          
                        self.label[i] = gtk.Label(f)
                        infile = os.path.splitext(infile)[0]
                        if self.temp in source and self.checkbutton2.get_active():
                                self.autorotate(self.rawpath[source], i)
                if format.lower() in ['.jpg', '.jpeg'] and self.checkbutton2.get_active():
                        self.autorotate(filename, i)
                return infile, filename
                
        def cache_control(self):
                if hasattr(self, 'generate_thumbnails_thread') and self.generate_thumbnails_thread.isAlive():
                        self.generate_thumbnails_thread.join()
                if hasattr(self, 'delete_cached_thread') and self.delete_cached_thread.isAlive():
                        self.delete_cached_thread.join()
                filelist = glob.glob(os.path.join(self.cache, '*'))
                if filelist:
                        total_size = 0
                        _dict = {}
                        for item in filelist:
                                total_size += os.path.getsize(item)
                                _dict[os.stat(item)[7]] = item
                        keys = sorted(_dict.iterkeys())
                        while total_size / 1000000.0 > self.cache_size:
                                total_size -= os.path.getsize(_dict[keys[0]])
                                os.remove(_dict[keys[0]])
                                del keys[0]
                        gobject.idle_add(self.progressbar1.set_fraction, (total_size / 1000000.0) / self.cache_size)
        
        def clear_cache(self, obj):
                text = _('Are you sure to delete\nall cached thumbnails?')
                dialog8 = self.messagedialog(_("Attention!"), text, None, gtk.BUTTONS_OK_CANCEL, self.dialog12, gtk.DIALOG_MODAL, gtk.MESSAGE_WARNING, None, (285,180))
                dialog8.set_keep_above(True)
                response = dialog8.run()
                if response == gtk.RESPONSE_CANCEL or response == gtk.RESPONSE_DELETE_EVENT:
                        dialog8.destroy()
                        self.get_options()
                elif response == gtk.RESPONSE_OK:
                        dialog8.destroy()
                        self.progressbar1.set_fraction(0)
                  #      self.get_options()
                        self.delete_cached_thread = threading.Thread(target=self.delete_cached, args=())
                        self.delete_cached_thread.start()
                        
        def delete_cached(self):
                if hasattr(self, 'cache_control_thread') and self.cache_control_thread.isAlive():
                        self.cache_control_thread.join()
                if hasattr(self, 'generate_thumbnails_thread') and self.generate_thumbnails_thread.isAlive():
                        self.stop_thumb_generator.set()
                        self.generate_thumbnails_thread.join()
                        if self.stop_thumb_generator.is_set():
                                self.stop_thumb_generator.clear()
                for item in os.listdir(self.cache):
                        os.remove(os.path.join(self.cache, item))
                self.cache_list = []
                self.mtimes.clear()
                self.store_mtimes_db()
                gobject.idle_add(self.progressbar1.set_fraction, 0)
                
        def close_window2(self, obj):
                self.on_delete_event(self.window2, obj)

        def on_notebook2_switched(self, *event):
                format = os.path.splitext(self.list[self.indice])[1]
                if event[0]  == self.notebook2 and event[2] in [2, 3, 4, 5] and self.notebook2.get_current_page() in [0,1,6] and not self.raw_preview:
                        if format.lower() in self.raw_formats:
                                gobject.idle_add(self.load_raw_preview, self.rect.width, self.rect.height)
                        else:
                                if self.rotated.has_key(self.indice):
                                        self.pblist[self.pbindex] = gtk.gdk.pixbuf_new_from_file_at_size(self.list[self.indice],\
                                                                    self.rect.width, self.rect.height)
                                        if self.checkbutton41.get_active():
                                                self.image28.set_from_pixbuf(self.profile_pixbuf(self.pblist[self.pbindex]))
                                        else:
                                                self.image28.set_from_pixbuf(self.pblist[self.pbindex])
                                        self.image28.show()      
                elif len(event) == 1:
                        if self.notebook2.get_current_page() in [2, 3, 4, 5]:
                                gobject.idle_add(self.load_raw_preview, self.rect.width, self.rect.height)
                if event[0] == self.notebook2:
                        if event[2] == 3 and format.lower() in self.formats:
                                self.get_channel()
                        elif event[2] == 1:
                                self.show_tags()
                        elif event[2] == 0:
                                self.add_dimensions(format, self._label)
                        if self.expander23.get_expanded():
                                self.expander23.set_expanded(False)
                         
        def make_surface(self, text):
                width, height = 260, 150
                self.surface = cairo.ImageSurface(cairo.FORMAT_ARGB32, width, height)
                self.cr = cairo.Context(self.surface)
                self.cr.rectangle(0, 0, width, height)
                self.cr.clip()
                self.cr.set_source_rgb(0, 0, 0)
                self.cr.rectangle(0, 0, 259, 122)
                self.cr.fill()
                self.cr.set_source_rgb(1.0, 1.0, 1.0)
                self.cr.rectangle(1, 1, 257, 120)
                self.cr.fill()
                self.cr.stroke()
                self.cr.set_source_rgb(0.5, 0.5, 0.5)
                n = 51
                self.cr.set_line_width(1)
                for item in range(4):
                        self.cr.move_to(0.5 + n, height - 29)
                        self.cr.line_to(0.5 + n, 1)
                        n += 51
                self.cr.stroke()
                if text:
                        self.cr.set_source_rgb(0, 0, 0)
                        self.cr.select_font_face("Sans", cairo.FONT_SLANT_NORMAL, cairo.FONT_WEIGHT_NORMAL)
                        self.cr.set_font_size(10)
                        self.cr.move_to(80, 70)
                        self.cr.show_text(_('Loading histogram...'))
                while gtk.events_pending(): 
                        gtk.main_iteration(False)
                return width, height
                
        def draw_gradient(self, r, g, b):
                height = 150
                linear = cairo.LinearGradient(0, height - 20, 259, 0)
                linear.add_color_stop_rgb(0, 0, 0, 0)
                linear.add_color_stop_rgb(0.7, r, g, b)
                self.cr.set_source_rgb(0, 0, 0)
                self.cr.rectangle(0, height - 23 ,259, 10)
                self.cr.fill()
                self.cr.set_source(linear)
                self.cr.rectangle(1, height - 22, 257, 8)
                self.cr.fill()
                self.cr.stroke()
                data = self.surface.get_data()  
                stride = self.surface.get_stride()
                pixbuf = gtk.gdk.pixbuf_new_from_data(data, gtk.gdk.COLORSPACE_RGB, True, 8, 259, height, stride)
                self.image22.set_from_pixbuf(pixbuf)
                while gtk.events_pending(): 
                        gtk.main_iteration(False)

        def on_togglebutton1_toggled(self, obj):
                if obj == self.togglebutton1:
                        self.togglebutton2.handler_block(self.id64)
                        self.togglebutton2.set_active(False)
                        self.togglebutton2.handler_unblock(self.id64)
                        self.togglebutton1.set_active(True)
                elif obj == self.togglebutton2:
                        self.togglebutton1.handler_block(self.id63)
                        self.togglebutton1.set_active(False)
                        self.togglebutton1.handler_unblock(self.id63)
                        self.togglebutton2.set_active(True)
                self.get_channel()

        def pixbuf2image(self, data):
                assert(self.pblist[self.pbindex].get_colorspace() == gtk.gdk.COLORSPACE_RGB)
                if not pygtk_has_numpy_support:
                        dimensions = self.pblist[self.pbindex].get_width(), self.pblist[self.pbindex].get_height()
                        stride = self.pblist[self.pbindex].get_rowstride()
                        pixels = self.pblist[self.pbindex].get_pixels()
                        mode = self.pblist[self.pbindex].get_has_alpha() and "RGBA" or "RGB"
                        im = Image.frombuffer(mode, dimensions, pixels, "raw", mode, stride, 1)
                        if data:
                                im = im.convert(data)
                        hist = im.histogram()
                else:
                        if data:
                                pixbuf = self.pblist[self.pbindex].copy()
                                pixbuf.saturate_and_pixelate(pixbuf, 0, False)
                                with warnings.catch_warnings():
                                        warnings.simplefilter("ignore")
                                        array = pixbuf.get_pixels_array()
                                hist = numpy.histogram(array[:,:,0], bins=256, range=(0,256.0))[0].tolist()
                        else:
                                with warnings.catch_warnings():
                                        warnings.simplefilter("ignore")
                                        array = self.pblist[self.pbindex].get_pixels_array()
                                hist = []
                                for i in range(array.shape[2]):
                                        hist += numpy.histogram(array[:,:,i], bins=256, range=(0,256.0))[0].tolist()
                if self.togglebutton2.get_active():
                        return map(self.logarithmic_histogram, hist)
                return hist
                                
        def logarithmic_histogram(self, x):
                if x > 1 :
                        return math.log10(x)
                return 0
        
        def get_channel(self):
                width = 260
                height = 150
                self.frame1.window.set_cursor(self.watch)
                channel = self.combobox1.get_active()
                self.make_surface(True)
                if channel == 0:
                        self.r = 1; self.g = 1; self.b = 1; i = 0;
                        self.draw_gradient(1, 1, 1)
                        self.make_surface(False)
                        self.draw_gradient(1, 1, 1)
                        hist = self.pixbuf2image('L')
                        self.draw_channel(hist, None, 0, 0, 0, i, 1)
                elif channel == 1:
                        self.r = 0; self.g = 0; self.b = 1; i = 0;
                        self.draw_gradient(self.r, self.g, self.b)
                        self.make_surface(False)
                        self.draw_gradient(self.r, self.g, self.b)
                        hist = self.pixbuf2image(None)
                        self.draw_channel(hist[:256], None, self.r, self.g, self.b, i, 1)
                elif channel == 2:
                        self.r = 0; self.g = 1; self.b = 0; i = 1;
                        self.draw_gradient(self.r, self.g, self.b)
                        self.make_surface(False)
                        self.draw_gradient(self.r, self.g, self.b)
                        hist = self.pixbuf2image(None)
                        self.draw_channel(hist[256:512], None, self.r, self.g, self.b, i, 1)
                elif channel == 3:
                        self.r = 1; self.g = 0; self.b = 0; i = 2;
                        self.draw_gradient(self.r, self.g, self.b)
                        self.make_surface(False)
                        self.draw_gradient(self.r, self.g, self.b)
                        hist = self.pixbuf2image(None)
                        self.draw_channel(hist[512:768], None, self.r, self.g, self.b, i, 1)
                elif channel == 4:
                        self.r = 0; self.g = 1; self.b = 1 ; i = 3;
                        self.draw_gradient(self.r, self.g, self.b)
                        self.make_surface(False)
                        self.draw_gradient(self.r, self.g, self.b)
                        hist = self.pixbuf2image(None)
                        self.draw_channel(hist[768:], None, self.r, self.g, self.b, i, 2)
                elif channel == 5:
                        self.r = 1; self.g = 1; self.b = 1;
                        self.draw_gradient(self.r, self.g, self.b)
                        self.make_surface(False)
                        self.draw_gradient(self.r, self.g, self.b)
                        hist = self.pixbuf2image(None)
                        colors_hist = [max(hist[item], hist[item + 256], hist[item + 512]) for item in xrange(256)]
                        self.draw_channel(hist[:256], colors_hist, 0, 0, 1, 0, 0.7)
                        self.draw_channel(hist[256:512], colors_hist, 0, 1, 0, 1, 0.7)
                        self.draw_channel(hist[512:768], colors_hist, 1, 0, 0, 2, 0.7)
                self.frame1.window.set_cursor(None)

        def draw_channel(self, hist, colors_hist, r, g, b, i, w):
                height = 150
                if i == 3 and not self.pblist[self.pbindex].get_has_alpha():
                        return True
                if colors_hist:
                        h = max(colors_hist)
                else:
                        h = max(hist)
                self.cr.set_source_rgb(r, g, b)
                self.cr.set_line_width(w)
                n = 0
                for item in hist:
                        y=(item * (height - 30)) / h
                        self.cr.move_to(n + 2, height - 29)
                        self.cr.line_to(n + 2, height - 29 - y)
                        n += 1  
                self.cr.stroke()
                data = self.surface.get_data()  
                stride = self.surface.get_stride()
                pixbuf = gtk.gdk.pixbuf_new_from_data(data, gtk.gdk.COLORSPACE_RGB, True, 8, 259, height, stride)
                self.image22.set_from_pixbuf(pixbuf)
                if colors_hist:
                        hist = colors_hist[:]
                self.label48.set_text('Mean: %.1f' % numpy.mean(hist))
                self.label49.set_text('Std Dev: %.1f' % numpy.std(hist))
                self.label53.set_text('Median: %.1f' % numpy.median(hist))
                try:
                        dimensions = self.image_size.split('x')
                        self.label35.set_text('Pixels: %d' % (int(dimensions[0]) * int(dimensions[1])))
                except:
                        self.label35.set_text('Pixels: ')
                
        def on_window8_delete_event(self, *obj):
                self.window8.hide()
                self.unselect()
                while gtk.events_pending():
                        gtk.main_iteration(False)
                return True
                
        def on_delete_event(self, obj, event):
                self.loop = 0
                while gtk.events_pending():
                        gtk.main_iteration(False)
                if obj == self.window2 and self.workflow_started:
                        if self.show_dialog21(True): 
                                return True
                obj.hide()
                self.image22.clear()
                if obj != self.window7:
                        for item in self.expander_list:
                                if item.get_expanded():
                                        item.set_expanded(False)
                                        if item == self.expander17:
                                                self.destroy_drawingarea()
                else:
                        self.stop_batch_process = True
                if obj == self.window2 and self.radiobutton7.get_active():
                        indice = self.old_indice
                        self.indice = indice
                        self.pblist = range(3)
                        self.fullpblist = range(3)
                        gc.collect()
                if obj == self.window4:
                        indice = self.old_indice
                        self.indice = indice
                        if self.checkbutton6.get_active() and self.pid:
                                try:
                                        self.kill_process(self.pid)
                                except:
                                        gobject.idle_add(sys.stderr.write, traceback.format_exc())
                self.pid = self.count = 0
                if obj == self.window2:
                        self.remake_gallery()
                        self.combobox1.handler_block(self.id50) 
                        self.combobox1.set_active(0)
                        self.combobox1.handler_unblock(self.id50)
                        self.image28.clear()
                        for row in self.tagmodel2:
                                row[0] = False
                return True

        def kill_process(self, pid):
                if sys.platform.startswith('win'):
                        from ctypes import windll
                        handle = windll.kernel32.OpenProcess(1, False, pid)
                        windll.kernel32.TerminateProcess(handle, -1)
                        windll.kernel32.CloseHandle(handle)
                else:
                        os.kill(pid, 9)

        def on_delete_window3(self, obj, event):
                obj.hide()
                self.fullpblist = range(3)
                gc.collect()
                indice = self.old_indice
                self.indice = indice
                
        def hide_win6(self, obj):
                self.window6.hide()
                self.stop_thread.set()
                self.unselect()
                return True

        def main(self):
                gtk.main()

        def store_batch_queue(self):
                self.batch_list[0] = [[row[0], row[2], row[3], row[4]] for row in self.batchmodel]
                self.batch_list[1] = self.batch_queue
                f = open(os.path.join(self.ark, 'batch.db'), 'w')
                cPickle.dump(self.batch_list, f)
                f.close()

        def store_albums_db(self):
                f = open(os.path.join(self.ark, 'albums.db'), 'w')
                cPickle.dump(self.albums, f)
                f.close()

        def store_templates_db(self):
                f = open(os.path.join(self.ark, 'templates.db'), 'w')
                cPickle.dump(self.templates, f)
                f.close()

        def store_mtimes_db(self):
                f = open(os.path.join(self.ark, 'thumbs_mtimes.db'), 'w')
                cPickle.dump(self.mtimes, f)
                f.close()

        def store_settings_db(self):
                f = open(os.path.join(self.ark, 'settings'), 'w')
                cPickle.dump(self.settings, f)
                f.close()
                
        def destroy(self, *widget):
                print "Saving settings..."
                self.stop_gallery_thread.set()
                if hasattr(self, 'gallery_thread') and self.gallery_thread.isAlive():
                        self.gallery_thread.join()
                self.settings['filechooserbutton7'] = self.filechooserbutton7.get_current_folder()
                self.settings['combobox10'] = self.combobox10.get_active()
                self.settings['hscale23'] = self.hscale23.get_adjustment().value 
                self.settings['window1_size'] = self.window1.get_size()
                self.settings['in_profile_icc'] = self.input_profile_name
                self.settings['out_profile_icc'] = self.output_profile_name
                self.settings['display_profile_icc'] = self.display_profile_name
                self.store_dcraw_settings()
                self.store_settings_db()
                self.store_albums_db()
                self.store_batch_queue()
                self.store_templates_db()
                self.store_mtimes_db()
                shutil.rmtree(self.thumbdir, ignore_errors=True)
                shutil.rmtree(self.temp, ignore_errors=True)
                print "Bye!"
                gtk.main_quit()
                
if __name__ == "__main__":
        gtkrawgallery = Gallery()
        gtkrawgallery.main()

        
